/*----------------------------------------------------------------------------
  Project: cxxx <http://cxplorer.sourceforge.net>
  File   : CBuddy.cpp
  Info   : $Id: CBuddy.cpp,v 1.11 2012/06/26 08:05:52 gkraeuter Exp $
----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
  Copyright © 2005 - 2012 Gerhard Kräuter <gkraeuter@users.sourceforge.net>

  This  software  is  provided  'as-is',  without  any  express  or  implied
  warranty.  In  no  event  will  the authors be held liable for any damages
  arising from the use of this software.

  Permission  is  granted  to  anyone  to use this software for any purpose,
  including  commercial  applications,  and  to alter it and redistribute it
  freely, subject to the following restrictions:

  1. The  origin  of  this software must not be misrepresented; you must not
     claim  that  you  wrote the original software. If you use this software
     in  a  product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.

----------------------------------------------------------------------------*/

// cxxx headers
#include <cxxx/CBuddy.h>

// X11 headers
#include <X11/cursorfont.h>

// namespace declarations

CBuddy::CBuddy(CWindow* parent, unsigned msgid)
: CWindow(parent, msgid),
  mState(NO_BUTTON),
  mPos(0),
  mMin(0),
  mMax(100),
  mStep(1),
  mTimer(this, IDTIMER)
{
	Resize(WIDTH, HEIGHT);
	mAttributes.event_mask |= ButtonPressMask | ButtonReleaseMask;
	ChangeAttributes(CWEventMask);
	mIconUp.Create(CIcon::ARROW_UP, 0);
	mIconDown.Create(CIcon::ARROW_DOWN, 0);
	mCursor = XCreateFontCursor(cxxx::cxDpy, XC_left_ptr);
	XDefineCursor(cxxx::cxDpy, GetXID(), mCursor);
}	

CBuddy::~CBuddy()
{
}

void CBuddy::OnExpose(XExposeEvent& e __attribute__ ((unused)))
{
	unsigned h = mHeight / 2;
	if (mState == UP_BUTTON) DrawFrame(0, 0, mWidth, h, FS_SUNKEN, false);
	else DrawFrame(0, 0, mWidth, h, FS_RAISED, false);
	if (mState == DOWN_BUTTON) DrawFrame(0, h, mWidth, h, FS_SUNKEN, false);
	else DrawFrame(0, h, mWidth, h, FS_RAISED, false);
	CColor col(TC_FRAME_TEXT);
	DrawIcon((mWidth - mIconUp.GetWidth()) / 2, (h - mIconUp.GetHeight()) / 2, mIconUp, col);
	DrawIcon((mWidth - mIconDown.GetWidth()) / 2, h + (h - mIconDown.GetHeight()) / 2, mIconDown, col);
}

void CBuddy::OnLButtonDown(XButtonEvent& e)
{
	if (e.y < (int)mHeight / 2) {
		mState = UP_BUTTON;
		Increment();
		DrawFrame(0, 0, mWidth, mHeight / 2, FS_SUNKEN, false);
	} else {
		mState = DOWN_BUTTON;
		Decrement();
		DrawFrame(0, mHeight / 2, mWidth, mHeight / 2, FS_SUNKEN, false);
	}
	mTimer.SetTime(500);
}

void CBuddy::OnMouseWheelUp(XButtonEvent& e)
{
	if (e.type == ButtonPress) Increment();
}

void CBuddy::OnMouseWheelDown(XButtonEvent& e)
{
	if (e.type == ButtonPress) Decrement();
}

void CBuddy::OnButtonUp(XButtonEvent& e __attribute__ ((unused)))
{
	mTimer.Stop();
	if (mState == UP_BUTTON) DrawFrame(0, 0, mWidth, mHeight / 2, FS_RAISED, false);
	if (mState == DOWN_BUTTON) DrawFrame(0, mHeight / 2, mWidth, mHeight / 2, FS_RAISED, false);
	mState = NO_BUTTON;
}

void CBuddy::OnMessage(unsigned msgid, TMessage type __attribute__ ((unused)), unsigned value __attribute__ ((unused)))
{
	if (msgid == IDTIMER) {
		if (mState == UP_BUTTON) Increment();
		else if (mState == DOWN_BUTTON) Decrement();
		else return;
	}
	mTimer.SetTime(100);
}
