/*----------------------------------------------------------------------------
  Project: cxxx <http://cxplorer.sourceforge.net>
  File   : CBuddy.h
  Info   : $Id: CBuddy.h,v 1.11 2012/08/06 17:36:40 gkraeuter Exp $
----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
  Copyright © 2005 - 2012 Gerhard Kräuter <gkraeuter@users.sourceforge.net>

  This  software  is  provided  'as-is',  without  any  express  or  implied
  warranty.  In  no  event  will  the authors be held liable for any damages
  arising from the use of this software.

  Permission  is  granted  to  anyone  to use this software for any purpose,
  including  commercial  applications,  and  to alter it and redistribute it
  freely, subject to the following restrictions:

  1. The  origin  of  this software must not be misrepresented; you must not
     claim  that  you  wrote the original software. If you use this software
     in  a  product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.

----------------------------------------------------------------------------*/

#ifndef CBuddy_h
#define CBuddy_h

// C++X headers
#include <cxxx/CWindow.h>

class CBuddy : public CWindow
{
private:
	enum
	{
		// State
		NO_BUTTON,
		UP_BUTTON,
		DOWN_BUTTON,

		// Geometry
		WIDTH = 11,
		HEIGHT = 16,

		// Message IDs
		IDTIMER = cxxx::msgid::MSGID_CBuddy
	};

public:
	// Constructor
	CBuddy(CWindow* parent, unsigned msgid);

	// Destructor
	virtual ~CBuddy();

	// X event processing 
	virtual void OnExpose(XExposeEvent& e);
	virtual void OnLButtonDown(XButtonEvent& e);
	virtual void OnButtonUp(XButtonEvent& e);
	virtual void OnMouseWheelUp(XButtonEvent& e);
	virtual void OnMouseWheelDown(XButtonEvent& e);

	// C++X message processing
	virtual void OnMessage(unsigned msgid, TMessage type, unsigned value);

	// Sets position to pos
	void SetPosition(int pos)
	{
		if (pos != mPos) {
			// range check
			if (pos < mMin) pos = mMin;
			if (pos > mMax) pos = mMax;
			// round
			pos = ((2 * pos + mStep) / (2 * mStep)) * mStep;
			mPos = pos;
		}
	}

	// Sets minimum and maximum values for position
	void SetRange(int min, int max)
	{
		mMin = std::min(min, max);
		mMax = std::max(min, max);
		if (mPos < mMin) SetPosition(mMin);
		else if (mPos > mMax) SetPosition(mMax);
	}

	// Sets step width
	void SetStep(int step)
	{
		cxAssert(step >= 1);
		mStep = step;
	}

	// Returns current position
	int GetPosition() const
	{
		return mPos;
	}

protected:
	// Adds mStep to position
	void Increment()
	{
		SetPosition(mPos + mStep);
		SendMessage(MT_SCROLL, mPos);
	}

	// Subtracts mStep from position
	void Decrement()
	{
		SetPosition(mPos - mStep);
		SendMessage(MT_SCROLL, mPos);
	}

protected:
	unsigned mState;
	int mPos;
	int mMin;
	int mMax;
	int mStep;
	CIcon mIconUp;
	CIcon mIconDown;
	CTimer mTimer;
	Cursor mCursor;
};

#endif // CBuddy_h
