/*----------------------------------------------------------------------------
  Project: cxxx <http://cxplorer.sourceforge.net>
  File   : CButton.cpp
  Info   : $Id: CButton.cpp,v 1.19 2012/06/26 08:05:52 gkraeuter Exp $
----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
  Copyright © 2002 - 2012 Gerhard Kräuter <gkraeuter@users.sourceforge.net>

  This  software  is  provided  'as-is',  without  any  express  or  implied
  warranty.  In  no  event  will  the authors be held liable for any damages
  arising from the use of this software.

  Permission  is  granted  to  anyone  to use this software for any purpose,
  including  commercial  applications,  and  to alter it and redistribute it
  freely, subject to the following restrictions:

  1. The  origin  of  this software must not be misrepresented; you must not
     claim  that  you  wrote the original software. If you use this software
     in  a  product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.

----------------------------------------------------------------------------*/

// C++ headers
#include <cctype>

// cxxx headers
#include <cxxx/CButton.h>

// namespace declarations
using std::toupper;
using std::string;
using std::wstring;
using cxxx::cxWiden;

CButton::CButton(CWindow *parent, unsigned msgid)
: CWindow(parent, msgid),
  mAccelerator(L'\0')
{
	mAttributes.event_mask |= ButtonPressMask | ButtonReleaseMask | KeyPressMask;
	ChangeAttributes(CWEventMask);
	SetMessageTarget(parent);
	SetFocusable();
}

/* virtual */ CButton::~CButton()
{
	if (mAccelerator) GetTopLevelWindow()->ReleaseKey(mAccelerator);
}

/* virtual */void CButton::OnAccelerator(wchar_t key __attribute__ ((unused)))
{
	SetFocus();
	PostMessage(MT_ACCELERATOR);
}

/* virtual */ void CButton::Enable(bool enable)
{
	if (IsEnabled() == enable) return ;
	CWindow::Enable(enable);
	SetFocusable(enable);
	if (!IsEnabled() && IsFocused()) GetTopLevelWindow()->SetFocusWindow(NULL);
	Redraw();
}

/* virtual */ void CButton::SetName(const string& name)
{
	mName = cxWiden(name);
	string::size_type pos = mName.find(L'&');
	if(pos < mName.size() - 1) {
        mName.erase(pos, 1);
		wchar_t hc = mName[pos];
		wstring tmp = mName;
		if (cxxx::cxRTL) {
			tmp = cxxx::cxBidi->Convert(mName);
			pos = tmp.rfind(hc);
		} else pos = tmp.find(hc);
		mHotCharPos.mBegin = GetTextWidth(tmp, pos);
		mHotCharPos.mEnd = mHotCharPos.mBegin + GetCharWidth(tmp[pos]) - 1;
        mAccelerator = toupper(hc);
		GetTopLevelWindow()->BindKey(mAccelerator, this);
	} else {
		mHotCharPos.mBegin = 0;
		mHotCharPos.mEnd = 0;
		mAccelerator = L'\0';
	}
}

/* virtual */ void CButton::UpdateState()
{}
