/*----------------------------------------------------------------------------
  Project: cxxx <http://cxplorer.sourceforge.net>
  File   : CCaret.h
  Info   : $Id: CCaret.h,v 1.12 2012/05/06 18:59:35 gkraeuter Exp $
----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
  Copyright © 2002 - 2012 Gerhard Kräuter <gkraeuter@users.sourceforge.net>

  This  software  is  provided  'as-is',  without  any  express  or  implied
  warranty.  In  no  event  will  the authors be held liable for any damages
  arising from the use of this software.

  Permission  is  granted  to  anyone  to use this software for any purpose,
  including  commercial  applications,  and  to alter it and redistribute it
  freely, subject to the following restrictions:

  1. The  origin  of  this software must not be misrepresented; you must not
     claim  that  you  wrote the original software. If you use this software
     in  a  product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.

----------------------------------------------------------------------------*/

#ifndef CCaret_h
#define CCaret_h

// C++X headers
#include <cxxx/CWindow.h>

class CCaret : public CWindow
{
	enum
	{
		// Message IDs
		IDTIMER = cxxx::msgid::MSGID_CCaret
	};

public:
	// Constructor
	CCaret(CWindow* parent);

	// X event processing
	virtual void OnExpose(XExposeEvent& e);
	virtual void OnMessage(unsigned msgid, TMessage type, unsigned value);
	virtual void Recolor();

	// Sets the height of the caret
	void SetCaretHeight(unsigned h)
	{
		Resize(1, h);
	}

	// Sets the blinking frequency of the caret
	void SetBlinkRate(unsigned rate)
	{
		if (rate == 0) {
			HideCaret();
			mBlinkRate = 0;
		} else {
			cxAssert(rate < 1000);
			mBlinkRate = rate;
		}
	}

	// Makes caret visible
	void ShowCaret()
	{
		Show();
		mVisible = true;
		if (mBlinkRate) mTimer.SetTime(mBlinkRate);
	}

	// Makes caret invisible
	void HideCaret()
	{
		mTimer.Stop();
		mVisible = false;
		Hide();
	}

protected:
	CTimer mTimer;
	unsigned mBlinkRate;
	bool mVisible;
};





#endif // CCaret_h
