/*----------------------------------------------------------------------------
  Project: cxxx <http://cxplorer.sourceforge.net>
  File   : CCheckButton.h
  Info   : $Id: CCheckButton.h,v 1.12 2012/05/06 18:57:36 gkraeuter Exp $
----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
  Copyright © 2002 - 2012 Gerhard Kräuter <gkraeuter@users.sourceforge.net>

  This  software  is  provided  'as-is',  without  any  express  or  implied
  warranty.  In  no  event  will  the authors be held liable for any damages
  arising from the use of this software.

  Permission  is  granted  to  anyone  to use this software for any purpose,
  including  commercial  applications,  and  to alter it and redistribute it
  freely, subject to the following restrictions:

  1. The  origin  of  this software must not be misrepresented; you must not
     claim  that  you  wrote the original software. If you use this software
     in  a  product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.

----------------------------------------------------------------------------*/

#ifndef CCheckButton_h
#define CCheckButton_h

// C++X headers
#include <cxxx/CButton.h>

class CCheckButton : public CButton
{
public:
	// Constructor
	CCheckButton(CWindow* parent, unsigned msgid);
	
	// X event processing
	virtual void OnExpose(XExposeEvent& e);
	virtual void OnLButtonDown(XButtonEvent& e);
	virtual void OnFocus(bool in);
	virtual void OnAccelerator(wchar_t key);
	
	// Assigns a name to the check button
	virtual void SetName(const std::string& name);

	// Sets or clears check mark
	void SetCheck(bool check)
	{
		if (mChecked == check) return ;
		mChecked = check;
		UpdateState();
	}

	// Returns wether button is checked
	bool IsChecked() const
	{
		return mChecked;
	}

protected:
	// Makes changes visible
	virtual void UpdateState();

	// Toggles check mark
    void ToggleCheck()
	{
		if (IsChecked()) {
			SetCheck(false);
			PostMessage(MT_UNCHECK);
		} else {
			SetCheck(true);
			PostMessage(MT_CHECK);
		}
	}

	// Draws or deletes focusing rectangle
	void DrawFocusRect();

protected:
	bool mChecked;
	CIcon mCheck;
};

#endif // CCheckButton_h
