/*----------------------------------------------------------------------------
  Project: cxxx <http://cxplorer.sourceforge.net>
  File   : CColor.cpp
  Info   : $Id: CColor.cpp,v 1.18 2012/07/01 11:32:36 gkraeuter Exp $
----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
  Copyright © 2003 - 2012 Gerhard Kräuter <gkraeuter@users.sourceforge.net>

  This  software  is  provided  'as-is',  without  any  express  or  implied
  warranty.  In  no  event  will  the authors be held liable for any damages
  arising from the use of this software.

  Permission  is  granted  to  anyone  to use this software for any purpose,
  including  commercial  applications,  and  to alter it and redistribute it
  freely, subject to the following restrictions:

  1. The  origin  of  this software must not be misrepresented; you must not
     claim  that  you  wrote the original software. If you use this software
     in  a  product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.

----------------------------------------------------------------------------*/

// cxxx headers
#include <cxxx/CColor.h>

// namespace declarations
using cxxx::cxDpy;
using std::string;
using std::vector;

unsigned CColor::mVisualClass;
unsigned CColor::mRedMask;
unsigned CColor::mGreenMask;
unsigned CColor::mBlueMask;
int CColor::mRedShift;
int CColor::mGreenShift;
int CColor::mBlueShift;
unsigned CColor::mBitmapPad;
vector<XColor> CColor::mColormap;
CColor CColor::mTable[TC_COLORCOUNT];

void CColor::Init(const XVisualInfo& vis)
{
	mVisualClass = vis.c_class;
	mRedMask = vis.red_mask;
	mGreenMask = vis.green_mask;
	mBlueMask = vis.blue_mask;
	mBitmapPad = 8;
	while (mBitmapPad < (unsigned)vis.depth) mBitmapPad <<= 1;
	mRedShift = 0;
	mGreenShift = 0;
	mBlueShift = 0;
	if (mVisualClass == TrueColor) {
		while ((mRedMask & (1 << mRedShift)) == 0) ++mRedShift;
	   	while ((mRedMask & (1 << mRedShift)) != 0) ++mRedShift;
	   	while ((mGreenMask & (1 << mGreenShift)) == 0) ++mGreenShift;
	   	while ((mGreenMask & (1 << mGreenShift)) != 0) ++mGreenShift;
	   	while ((mBlueMask & (1 << mBlueShift)) == 0) ++mBlueShift;
	   	while ((mBlueMask & (1 << mBlueShift)) != 0) ++mBlueShift;
	    mRedShift -= 8;
	    mGreenShift -= 8;
	    mBlueShift -= 8;
	} else {
		mColormap.resize(vis.colormap_size);
		for (unsigned i = 0; i < mColormap.size(); ++i) mColormap[i].pixel = i;
		XQueryColors(cxDpy, DefaultColormap(cxDpy, DefaultScreen(cxDpy)), &mColormap[0], mColormap.size());
		cxAssert(mColormap.size() == (unsigned)vis.colormap_size);
	}
}

unsigned CColor::GetIndexedPixel() const
{
	unsigned delta = 768;
	unsigned idx = 0;
	for (unsigned i = 0; i < mColormap.size(); ++i) {
		XColor col = mColormap[i];
		unsigned tmp = abs((col.red / 257) - GetRed()) + abs((col.green / 257) - GetGreen()) + abs((col.blue / 257) - GetBlue());
		if (tmp < delta) {
			delta = tmp;
			idx = i;
		}
	}
	return idx;
}
