/*----------------------------------------------------------------------------
  Project: cxxx <http://cxplorer.sourceforge.net>
  File   : CColor.h
  Info   : $Id: CColor.h,v 1.32 2012/08/06 17:36:40 gkraeuter Exp $
----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
  Copyright © 2003 - 2012 Gerhard Kräuter <gkraeuter@users.sourceforge.net>

  This  software  is  provided  'as-is',  without  any  express  or  implied
  warranty.  In  no  event  will  the authors be held liable for any damages
  arising from the use of this software.

  Permission  is  granted  to  anyone  to use this software for any purpose,
  including  commercial  applications,  and  to alter it and redistribute it
  freely, subject to the following restrictions:

  1. The  origin  of  this software must not be misrepresented; you must not
     claim  that  you  wrote the original software. If you use this software
     in  a  product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.

----------------------------------------------------------------------------*/

#ifndef CColor_h
#define CColor_h

// C++X headers
#include <cxxx/Cxxx.h>

// C++ headers
#include <vector>

// X11 headers
#include <X11/Xutil.h>

class CColor
{
public:
	// Constructors
	CColor()
	{
		mColor.mARGB = 0xff000000;
	} 

	CColor(unsigned char r, unsigned char g, unsigned char b, unsigned char a = 0xff)
	{
#ifdef CXXX_BIGENDIAN
		mColor.mChannel[0] = a;
		mColor.mChannel[1] = r;
		mColor.mChannel[2] = g;
		mColor.mChannel[3] = b;
#else // CXXX_BIGENDIAN
		mColor.mChannel[3] = a;
		mColor.mChannel[2] = r;
		mColor.mChannel[1] = g;
		mColor.mChannel[0] = b;
#endif // CXXX_BIGENDIAN
	}

    CColor(const std::string& name)
	{
		XColor col;
		XParseColor(cxxx::cxDpy, DefaultColormap(cxxx::cxDpy, DefaultScreen(cxxx::cxDpy)), name.c_str(), &col);
#ifdef CXXX_BIGENDIAN
		mColor.mChannel[0] = 0xff;
		mColor.mChannel[1] = col.red / 257;
		mColor.mChannel[2] = col.green / 257;
		mColor.mChannel[3] = col.blue / 257;
#else // CXXX_BIGENDIAN
		mColor.mChannel[3] = 0xff;
		mColor.mChannel[2] = col.red / 257;
		mColor.mChannel[1] = col.green / 257;
		mColor.mChannel[0] = col.blue / 257;
#endif // CXXX_BIGENDIAN
	}
	
	CColor(TColor idx)
	{
		mColor = mTable[idx].mColor;
	}
	
	// Initializing routine
    static void Init(const XVisualInfo& vis);

	// Access functions
    unsigned char GetAlpha() const
	{
#ifdef CXXX_BIGENDIAN
		return mColor.mChannel[0];
#else // CXXX_BIGENDIAN
		return mColor.mChannel[3];
#endif // CXXX_BIGENDIAN
	}

	unsigned char GetRed() const
	{
#ifdef CXXX_BIGENDIAN
		return mColor.mChannel[1];
#else // CXXX_BIGENDIAN
		return mColor.mChannel[2];
#endif // CXXX_BIGENDIAN
	}

	unsigned char GetGreen() const
	{
#ifdef CXXX_BIGENDIAN
		return mColor.mChannel[2];
#else // CXXX_BIGENDIAN
		return mColor.mChannel[1];
#endif // CXXX_BIGENDIAN
	}

	unsigned char GetBlue() const
	{
#ifdef CXXX_BIGENDIAN
		return mColor.mChannel[3];
#else // CXXX_BIGENDIAN
		return mColor.mChannel[0];
#endif // CXXX_BIGENDIAN
	}

	unsigned long GetPixel() const
	{
		if (mVisualClass == TrueColor) {
			unsigned r = GetRed();
			unsigned g = GetGreen();
			unsigned b = GetBlue();
			if (mRedShift > 0) r <<= mRedShift;
			else r >>= -mRedShift;
			if (mGreenShift > 0) g <<= mGreenShift;
			else g >>= -mGreenShift;
			if (mBlueShift > 0) b <<= mBlueShift;
			else b >>= -mBlueShift;
			r &= mRedMask;
			g &= mGreenMask;
			b &= mBlueMask;
			return r | g | b;
		} else if (mVisualClass == PseudoColor) return GetIndexedPixel();
		else return 0;
	}

    unsigned GetARGB() const
	{
		return mColor.mARGB;
	}

    unsigned GetLuminance() const
	{
		unsigned lum = GetRed() * 77 + GetGreen() * 150 + GetBlue() * 29;
		lum >>= 8;
		return lum;
	}
	
	// Brighten or darken color by percent percents
    void Brighten(int percent)
	{
		if(percent > 1000) percent = 1000;
		if(percent < -100) percent = -100;
		unsigned mult = 100 + percent;
		unsigned r = GetRed();
		unsigned g = GetGreen();
		unsigned b = GetBlue();
		r *= mult;
		g *= mult;
		b *= mult;
		r /= 100;
		g /= 100;
		b /= 100;
		if (r > 0xff) r = 0xff;
		if (g > 0xff) g = 0xff;
		if (b > 0xff) b = 0xff;
#ifdef CXXX_BIGENDIAN
		mColor.mChannel[1] = r;
		mColor.mChannel[2] = g;
		mColor.mChannel[3] = b;
#else // CXXX_BIGENDIAN
		mColor.mChannel[2] = r;
		mColor.mChannel[1] = g;
		mColor.mChannel[0] = b;
#endif // CXXX_BIGENDIAN
	}
	
	// Assignment operator
	CColor& operator=(const XColor& xc)
	{
#ifdef CXXX_BIGENDIAN
		mColor.mChannel[0] = 0xff;
		mColor.mChannel[1] = xc.red / 257;
		mColor.mChannel[2] = xc.green / 257;
		mColor.mChannel[3] = xc.blue / 257;
#else // CXXX_BIGENDIAN
		mColor.mChannel[3] = 0xff;
		mColor.mChannel[2] = xc.red / 257;
		mColor.mChannel[1] = xc.green / 257;
		mColor.mChannel[0] = xc.blue / 257;
#endif // CXXX_BIGENDIAN
		return *this;
	}

	// Converting function
	static CColor PixelToColor(unsigned pixel)
	{
		CColor ret;
		if (mVisualClass == TrueColor)
		{
			unsigned r, g, b;
			if (mRedShift > 0) r = pixel >> mRedShift;
			else r = pixel << -mRedShift;
			if (mGreenShift > 0) g = pixel >> mGreenShift;
			else g = pixel << -mGreenShift;
			if (mBlueShift > 0) b = pixel >> mBlueShift;
			else b = pixel << -mBlueShift;
			ret = CColor(r & 0xff, g & 0xff, b & 0xff);
		} else if (mVisualClass == PseudoColor) {
			cxAssert(pixel < mColormap.size());
			ret = mColormap[pixel];
		}
		return ret;
	}
	
	// Conversion to XRenderColor, convenience function
	operator XRenderColor() const
	{
		XRenderColor col;
		col.alpha = GetAlpha() * 257;
		col.red = GetRed() * 257;
		col.green = GetGreen() * 257;
		col.blue = GetBlue() * 257;
		return col;
	}
		
	void SetColor(TColor idx)
	{
		unsigned save = GetARGB();
		cxAssert(idx < TC_COLORCOUNT);
		mTable[idx].mColor = mColor;
		if (idx == TC_FRAME) {
			Brighten(-25);
			SetColor(TC_DARK);
			mColor.mARGB = save;
			Brighten(25);
			SetColor(TC_BRIGHT);
			mColor.mARGB = save;
			unsigned r = GetRed();
			unsigned g = GetGreen();
			unsigned b = GetBlue();
			r += 0x100;
			g += 0x100;
			b += 0x100;
			r /= 2;
			g /= 2;
			b /= 2;
#ifdef CXXX_BIGENDIAN
			mColor.mChannel[1] = r;
			mColor.mChannel[2] = g;
			mColor.mChannel[3] = b;
#else // CXXX_BIGENDIAN
			mColor.mChannel[2] = r;
			mColor.mChannel[1] = g;
			mColor.mChannel[0] = b;
#endif // CXXX_BIGENDIAN
			SetColor(TC_SCROLL);
			mColor.mARGB = save;
		}
		if (idx < TC_TEXT) {
			if (GetLuminance() < 0x80) mColor = mTable[TC_WHITE].mColor;
			else mColor = mTable[TC_BLACK].mColor;
			SetColor((TColor)(idx + TC_TEXT));
		}
		mColor.mARGB = save;
	}

protected:
	// PseudoColor visual
    unsigned GetIndexedPixel() const;

protected:
	union
	{
		std::uint8_t mChannel[4];
		std::uint32_t mARGB;
	} mColor;
	static unsigned mVisualClass;
	static unsigned mRedMask;
	static unsigned mGreenMask;
	static unsigned mBlueMask;
	static int mRedShift;
	static int mGreenShift;
	static int mBlueShift;
	static unsigned mBitmapPad;
	static std::vector<XColor> mColormap;
	static CColor mTable[TC_COLORCOUNT];
};

#endif // CColor_h
