/*----------------------------------------------------------------------------
  Project: cxxx <http://cxplorer.sourceforge.net>
  File   : CColorDialog.cpp
  Info   : $Id: CColorDialog.cpp,v 1.23 2012/06/26 08:05:52 gkraeuter Exp $
----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
  Copyright © 2005 - 2012 Gerhard Kräuter <gkraeuter@users.sourceforge.net>

  This  software  is  provided  'as-is',  without  any  express  or  implied
  warranty.  In  no  event  will  the authors be held liable for any damages
  arising from the use of this software.

  Permission  is  granted  to  anyone  to use this software for any purpose,
  including  commercial  applications,  and  to alter it and redistribute it
  freely, subject to the following restrictions:

  1. The  origin  of  this software must not be misrepresented; you must not
     claim  that  you  wrote the original software. If you use this software
     in  a  product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.

----------------------------------------------------------------------------*/

// cxxx headers
#include <cxxx/CColorDialog.h>

// X headers
#include <X11/cursorfont.h>

// namespace declarations
using cxxx::lib::cxgettext;
using cxxx::cxDpy;
using cxxx::cxError;

CColorDialog::CColorDialog(CWindow* parent, const CColor& color)
: CTopLevelWindow(),
  mRed(this, IDRED),
  mGreen(this, IDGREEN),
  mBlue(this, IDBLUE),
  mRedEdit(this,IDRED_EDIT),
  mGreenEdit(this, IDGREEN_EDIT),
  mBlueEdit(this, IDBLUE_EDIT),
  mColorPicker(this, IDCOLOR_PICKER, CIcon::COLORPICKER, _("Pick a color", 404)),
  mButtonOK(this, IDOK),
  mButtonCancel(this, IDCANCEL),
  mCrossHairCursor(None),
  mPicking(false)
{
	TransientFor(parent->GetTopLevelWindow());
	Resize(310, 125);
	FixSize();
	SetCaption(_("Choose Color", 440));
	mCrossHairCursor = XCreateFontCursor(cxDpy, XC_crosshair);
	mRed.SetRange(0, 255);
	mRed.SetPosition(color.GetRed());
	mGreen.SetRange(0, 255);
	mGreen.SetPosition(color.GetGreen());
	mBlue.SetRange(0, 255);
	mBlue.SetPosition(color.GetBlue());
	mRedEdit.SetRange(0, 255);
	mRedEdit.SetValue(color.GetRed());
	mGreenEdit.SetRange(0, 255);
	mGreenEdit.SetValue(color.GetGreen());
	mBlueEdit.SetRange(0, 255);
	mBlueEdit.SetValue(color.GetBlue());
	mRed.Resize(145, 20);
	mRed.Move(50, 10);
	mGreen.Resize(145, 20);
	mGreen.Move(50, 35);
	mBlue.Resize(145, 20);
	mBlue.Move(50, 60);
	mRedEdit.Move(200, 10);
	mGreenEdit.Move(200, 35);
	mBlueEdit.Move(200, 60);
	mColorPicker.Move(278, 58);
	mButtonOK.SetName(_("&OK", 25));
	mButtonOK.Move(10, 90);
	mButtonCancel.SetName(_("&Cancel", 29));
	mButtonCancel.Move(220, 90);
	mRed.Show();
	mGreen.Show();
	mBlue.Show();
	mRedEdit.Show();
	mGreenEdit.Show();
	mBlueEdit.Show();
	mButtonOK.Show();
	mButtonCancel.Show();
}

/* virtual */ CColorDialog::~CColorDialog()
{
	XFreeCursor(cxDpy, mCrossHairCursor);	
}

/* virtual */ void CColorDialog::OnExpose(XExposeEvent& e __attribute__ ((unused)))
{
	unsigned off = (20 - GetTextHeight()) / 2;
	CColor col(TC_FRAME_TEXT);
	DrawString(10, 10 + off, col, _("Red", 402));
	DrawString(10, 35 + off, col, _("Green", 399)); 
	DrawString(10, 60 + off, col, _("Blue", 403));
	DrawFrame(255, 10, 45, 45, FS_SUNKEN_THICK, false);
	DrawColorRectangle(CColor(mRed.GetPosition(), mGreen.GetPosition(), mBlue.GetPosition()));
}

/* virtual */ void CColorDialog::OnButtonDown(XButtonEvent& e)
{
	XUngrabPointer(cxDpy, CurrentTime);
	if (mPicking && e.button == 1) {
		mPicking = false;
		XImage* img = XGetImage(cxDpy, e.root, e.x_root, e.y_root, 1, 1, AllPlanes, ZPixmap);
		if (img == NULL) {
			cxError << _("Failed to read pixel value", 405) << CError::Error;
			return;
		}
		unsigned pixel = XGetPixel(img, 0, 0);
		XDestroyImage(img);
		CColor col = CColor::PixelToColor(pixel);
		mRedEdit.SetValue(col.GetRed());
		mRed.SetPosition(col.GetRed());
		mGreenEdit.SetValue(col.GetGreen());
		mGreen.SetPosition(col.GetGreen());
		mBlueEdit.SetValue(col.GetBlue());
		mBlue.SetPosition(col.GetBlue());
		DrawColorRectangle(col);
	}
}

/* virtual */ void CColorDialog::OnKey(wchar_t key, unsigned state)
{
	CTopLevelWindow::OnKey(key, state);
	if (key == L'\n') {
		mReturn = OK;
		mShutDown = true;
	} else if (key == L'\x1b') OnQuit();
}

/* virtual */ void CColorDialog::OnMessage(unsigned msgid, TMessage type __attribute__ ((unused)), unsigned value)
{
	switch (msgid) {
	case IDRED:
		mRedEdit.SetValue(value);
		break;
	case IDRED_EDIT:
		mRed.SetPosition(value);
		break;
	case IDGREEN:
		mGreenEdit.SetValue(value);
		break;
	case IDGREEN_EDIT:
		mGreen.SetPosition(value);
		break;
	case IDBLUE:
		mBlueEdit.SetValue(value);
		break;
	case IDBLUE_EDIT:
		mBlue.SetPosition(value);
		break;
	case IDCOLOR_PICKER:
		OnColorPick();
		break;
	case IDOK:
		mReturn = OK;
		mShutDown = true;
		break;
	case IDCANCEL:
		OnQuit();
		break;
	default:
		break;
	}		
	DrawColorRectangle(CColor(mRed.GetPosition(), mGreen.GetPosition(), mBlue.GetPosition()));
}

/* virtual */ bool CColorDialog::OnUpdate(unsigned msgid) const
{
	if (msgid == IDCOLOR_PICKER) return true;
	else return false;
}

/* virtual */ void CColorDialog::OnQuit()
{
	mReturn = CANCEL;
	mShutDown = true;
}
