/*----------------------------------------------------------------------------
  Project: cxxx <http://cxplorer.sourceforge.net>
  File   : CColorDialog.h
  Info   : $Id: CColorDialog.h,v 1.13 2009/02/23 16:48:49 gkraeuter Exp $
----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
  Copyright © 2005 - 2009 Gerhard Kräuter <gkraeuter@users.sourceforge.net>

  This  software  is  provided  'as-is',  without  any  express  or  implied
  warranty.  In  no  event  will  the authors be held liable for any damages
  arising from the use of this software.

  Permission  is  granted  to  anyone  to use this software for any purpose,
  including  commercial  applications,  and  to alter it and redistribute it
  freely, subject to the following restrictions:

  1. The  origin  of  this software must not be misrepresented; you must not
     claim  that  you  wrote the original software. If you use this software
     in  a  product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.

----------------------------------------------------------------------------*/

#ifndef CColorDialog_h
#define CColorDialog_h

// C++X headers
#include <cxxx/CTopLevelWindow.h>
#include <cxxx/CSlider.h>
#include <cxxx/CNumEdit.h>
#include <cxxx/CToolButton.h>
#include <cxxx/CPushButton.h>

class CColorDialog : public CTopLevelWindow
{
public:
	// return values
	enum 
	{
		OK,
		CANCEL
	};

private:
	// Message IDs
	enum
	{
		IDRED = cxxx::msgid::MSGID_CColorDialog,
		IDGREEN,
		IDBLUE,
		IDRED_EDIT,
		IDGREEN_EDIT,
		IDBLUE_EDIT,
		IDCOLOR_PICKER,
		IDOK,
		IDCANCEL
	};

public:
	// Constructor
    CColorDialog(CWindow* parent, const CColor& color);

	// Destructor
	virtual ~CColorDialog();

	// X event processing
	virtual void OnExpose(XExposeEvent& e);
	virtual bool OnUpdate(unsigned msgid) const;
	virtual void OnButtonDown(XButtonEvent& e);
	virtual void OnKey(wchar_t key, unsigned state);

	// C++X message processing
	virtual void OnMessage(unsigned msgid, TMessage type, unsigned value);

	// Exit
	virtual void OnQuit();

	// Get the value
	CColor GetColor()
	{
		return CColor(mRed.GetPosition(), mGreen.GetPosition(), mBlue.GetPosition());
	}

protected:
	// Provide feedback
	void DrawColorRectangle(const CColor& color)
	{
		FillRectangle (257, 12, 41, 41, color);
	}

	// Activate color picker
	void OnColorPick()
	{
		if (XGrabPointer(cxxx::cxDpy, GetXID(), False, ButtonPressMask, GrabModeAsync, GrabModeAsync, None, mCrossHairCursor, CurrentTime ) == GrabSuccess ) mPicking = true;
	}

protected:
	CSlider mRed;
	CSlider mGreen;
	CSlider mBlue;
	CNumEdit mRedEdit;
	CNumEdit mGreenEdit;
	CNumEdit mBlueEdit;
	CToolButton mColorPicker;
	CPushButton mButtonOK;
	CPushButton mButtonCancel;
	Cursor mCrossHairCursor;
	bool mPicking;
};



#endif // CColorDialog_h
