/*----------------------------------------------------------------------------
  Project: cxxx <http://cxplorer.sourceforge.net>
  File   : CColorSheet.cpp
  Info   : $Id: CColorSheet.cpp,v 1.25 2012/08/06 17:36:40 gkraeuter Exp $
----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
  Copyright © 2005 - 2012 Gerhard Kräuter <gkraeuter@users.sourceforge.net>

  This  software  is  provided  'as-is',  without  any  express  or  implied
  warranty.  In  no  event  will  the authors be held liable for any damages
  arising from the use of this software.

  Permission  is  granted  to  anyone  to use this software for any purpose,
  including  commercial  applications,  and  to alter it and redistribute it
  freely, subject to the following restrictions:

  1. The  origin  of  this software must not be misrepresented; you must not
     claim  that  you  wrote the original software. If you use this software
     in  a  product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.

----------------------------------------------------------------------------*/

// C++X headers
#include <cxxx/CColorSheet.h>
#include <cxxx/CColorDialog.h>
#include <cxxx/CProfile.h>

// namespace declarations
using std::min;
using std::max;
using cxxx::lib::cxgettext;
using cxxx::lib::cxProfile;

CColorSheet::CColorSheet(CWindow* parent)
: CWindow(parent),
  mFrameChoose(this, IDFRAME),
  mTitleChoose(this, IDTITLE),
  mTooltipChoose(this, IDTOOL),
  mBackgroundChoose(this, IDBACK)
{
	unsigned col = cxProfile->Read("Frame Color", 0xffc0c0c0);
	mFrameColor = CColor((col >> 16) & 0xff, (col >> 8) & 0xff, col & 0xff, (col >> 24) & 0xff);
	col = cxProfile->Read("Title Color", 0xff000000);
	mTitleColor = CColor((col >> 16) & 0xff, (col >> 8) & 0xff, col & 0xff, (col >> 24) & 0xff);
	col = cxProfile->Read("Tooltip Color", 0xffffffff);
	mTooltipColor = CColor((col >> 16) & 0xff, (col >> 8) & 0xff, col & 0xff, (col >> 24) & 0xff);
	col = cxProfile->Read("Background Color", 0xffffffff);
	mBackgroundColor = CColor((col >> 16) & 0xff, (col >> 8) & 0xff, col & 0xff, (col >> 24) & 0xff);
	mFrameChoose.SetName(_("Change...", 415));
	mTitleChoose.SetName(_("Change...", 415));
	mTooltipChoose.SetName(_("Change...", 415));
	mBackgroundChoose.SetName(_("Change...", 415));
	mFrameChoose.Show();
	mTitleChoose.Show();
	mTooltipChoose.Show();
	mBackgroundChoose.Show();
}

/* virtual */ void CColorSheet::OnConfigure(XConfigureEvent& e __attribute__ ((unused)))
{
	mFrameChoose.Move(200, 20);
	mTitleChoose.Move(200, 55);
	mTooltipChoose.Move(200, 90);
	mBackgroundChoose.Move(200, 125);
}

/* virtual */ void CColorSheet::OnExpose(XExposeEvent& e)
{
	unsigned h = mFrameChoose.GetHeight();
	unsigned w = 160;
	int x = 20;
	int x1 = max(e.x, x);
	int x2 = min(e.x + e.width, (int)(x + w));
	if (x2 > x1) {
		int y = 20;
		int y1 = max(e.y, y);
		int y2 = min(e.y + e.height, (int)(y + h));
		if (y2 > y1) DrawColorRect(x, y, w, h, mFrameColor, _("Frame color", 416));
		y = 55;
		y1 = max(e.y, y);
		y2 = min(e.y + e.height, (int)(y + h));
		if (y2 > y1) DrawColorRect(x, y, w, h, mTitleColor, _("Title color", 417));
		y = 90;
		y1 = max(e.y, y);
		y2 = min(e.y + e.height, (int)(y + h));
		if (y2 > y1) DrawColorRect(x, y, w, h, mTooltipColor, _("Tooltip color", 414));
		y = 125;
		y1 = max(e.y, y);
		y2 = min(e.y + e.height, (int)(y + h));
		if (y2 > y1) DrawColorRect(x, y, w, h, mBackgroundColor, _("Background color", 373));
	
	}
	DrawFrame(0, 0, GetWidth(), GetHeight(), FS_SHEET, false);
}

/* virtual */ void CColorSheet::OnMessage(unsigned msgid, TMessage type __attribute__ ((unused)), unsigned value __attribute__ ((unused)))
{
	switch (msgid) {
	case IDFRAME:
		{
			CColorDialog cd(this, mFrameColor);
			int x = 30;
			int y = 30;
			TranslateCoordinates(x, y, NULL);
			cd.Move(x, y);
			cd.Show();
			if (cd.DoModal() == CColorDialog::OK) mFrameColor = cd.GetColor();
		}
		DrawColorRect(20, 20, 160, mFrameChoose.GetHeight(), mFrameColor, _("Frame color", 416));
		break;
	case IDTITLE:
		{
			CColorDialog cd(this, mTitleColor);
			int x = 30;
			int y = 30;
			TranslateCoordinates(x, y, NULL);
			cd.Move(x, y);
			cd.Show();
			if (cd.DoModal() == CColorDialog::OK) mTitleColor = cd.GetColor();
		}
		DrawColorRect(20, 55, 160, mTitleChoose.GetHeight(), mTitleColor, _("Title color", 417));
		break;
	case IDTOOL:
		{
			CColorDialog cd(this, mTooltipColor);
			int x = 30;
			int y = 30;
			TranslateCoordinates(x, y, NULL);
			cd.Move(x, y);
			cd.Show();
			if (cd.DoModal() == CColorDialog::OK) mTooltipColor = cd.GetColor();
		}
		DrawColorRect(20, 90, 160, mTooltipChoose.GetHeight(), mTooltipColor, _("Tooltip color", 414));
		break;
	case IDBACK:
		{
			CColorDialog cd(this, mBackgroundColor);
			int x = 30;
			int y = 30;
			TranslateCoordinates(x, y, NULL);
			cd.Move(x, y);
			cd.Show();
			if (cd.DoModal() == CColorDialog::OK) mBackgroundColor = cd.GetColor();
		}
		DrawColorRect(20, 125, 160, mBackgroundChoose.GetHeight(), mBackgroundColor, _("Background color", 373));
		break;
	default:
		break;
	}
}

bool CColorSheet::DoOK()
{
	cxProfile->Write("Frame Color", mFrameColor.GetARGB());
	mFrameColor.SetColor(TC_FRAME);
	cxProfile->Write("Title Color", mTitleColor.GetARGB());
	mTitleColor.SetColor(TC_TITLE);
	cxProfile->Write("Tooltip Color", mTooltipColor.GetARGB());
	mTooltipColor.SetColor(TC_TOOLTIP);
	cxProfile->Write("Background Color", mBackgroundColor.GetARGB());
	mBackgroundColor.SetColor(TC_BACKGND);
	return true;
}

void CColorSheet::DrawColorRect(int x, int y, unsigned w, unsigned h, CColor color, const std::string& s)
{
	DrawFrame(x, y, w, h, FS_SUNKEN_THICK, false);
	FillRectangle (x + 2, y + 2, w - 4, h - 4, color);
	CColor col;
	if (color.GetLuminance() >= 0x80) col = CColor(TC_BLACK);
	else col = CColor(TC_WHITE);
	x += 7;
	y += (h - GetTextHeight()) / 2;
	DrawString(x, y, col, s, false);
}
