/*----------------------------------------------------------------------------
  Project: cxxx <http://cxplorer.sourceforge.net>
  File   : CControl.cpp
  Info   : $Id: CControl.cpp,v 1.25 2012/08/06 17:36:40 gkraeuter Exp $
----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
  Copyright © 2005 - 2012 Gerhard Kräuter <gkraeuter@users.sourceforge.net>

  This  software  is  provided  'as-is',  without  any  express  or  implied
  warranty.  In  no  event  will  the authors be held liable for any damages
  arising from the use of this software.

  Permission  is  granted  to  anyone  to use this software for any purpose,
  including  commercial  applications,  and  to alter it and redistribute it
  freely, subject to the following restrictions:

  1. The  origin  of  this software must not be misrepresented; you must not
     claim  that  you  wrote the original software. If you use this software
     in  a  product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.

----------------------------------------------------------------------------*/

// C++X headers
#include <cxxx/CControl.h>

CControl::CControl(CWindow* parent, unsigned msgid)
: CDnDWindow( parent, msgid),
  mHScrollBar(this, IDHSCROLL),
  mVScrollBar(this, IDVSCROLL),
  mScrollTimer(this, IDTIMER),
  mMultipleSelection(false),
  mXOffset(0),
  mYOffset(0),
  mLastSel(-1u),
  mDocWidth(0),
  mDocHeight(0),
  mViewWidth(0),
  mViewHeight(0),
  mIconWidth(ICON_WIDTH),
  mIconHeight(ICON_HEIGHT),
  mIconSize(16)
{
    SetFocusable();
	CColor col(TC_BACKGND);
	mAttributes.background_pixel = col.GetPixel();
	mAttributes.backing_store = Always;
    mAttributes.event_mask |= ButtonReleaseMask;
   	ChangeAttributes(CWBackPixel | CWEventMask | CWBackingStore);
	mHScrollBar.SetMessageTarget(this);
	mVScrollBar.SetMessageTarget(this);
}

/* virtual */ void CControl::Recolor()
{
	CColor col(TC_BACKGND);
	mAttributes.background_pixel = col.GetPixel();
	ChangeAttributes(CWBackPixel);
	Redraw();
}

/* virtual */ void CControl::OnMessage(unsigned msgid, TMessage type, unsigned value)
{
    switch (msgid) {
	case IDHSCROLL:
		if (type == MT_SCROLL) {
			value -= mXOffset;
			mXOffset += value;
			Scroll(value * SCROLL_STEP, 0);
		}
		break;
	case IDVSCROLL:
		if (type ==  MT_SCROLL) {
			value -= mYOffset;
			mYOffset += value;
			Scroll(0, value * SCROLL_STEP);
		}
		break;
	case IDTIMER:
		if (type == MT_TIMER && GetSelCount() == 1) ScrollIn(GetSelection().front());
		break;
	default:
		CDnDWindow::OnMessage(msgid, type, value);
		break;
	}
}

/* virtual */ void CControl::OnConfigure(XConfigureEvent& e __attribute__ ((unused)))
{
	Layout();
}

/* virtual */ void CControl::OnFocus(bool in __attribute__ ((unused)))
{
	if (mSelection.empty()) return;
	for (std::vector<unsigned>::iterator i = mSelection.begin(); i != mSelection.end(); ++i) DrawItem(*i);
	DrawFrame();
}

/* virtual */ void CControl::OnDblClick(XButtonEvent& e)
{
    if (e.button == 1) PostMessage(MT_CONFIRM_SELECTION);
    CancelDrag();
}

/* virtual */ void CControl::OnButtonUp(XButtonEvent& e)
{
	if (e.button == 3 && !IsDragging()) OnContext();
	StopDrag(e);
}

/* virtual */ void CControl::OnMouseWheelUp(XButtonEvent& e __attribute__ ((unused)))
{
	if (mVScrollBar.IsEnabled()) mVScrollBar.SetPosition(mVScrollBar.GetPosition() - 1);
}

/* virtual */ void CControl::OnMouseWheelDown(XButtonEvent& e __attribute__ ((unused)))
{
	if (mVScrollBar.IsEnabled()) mVScrollBar.SetPosition(mVScrollBar.GetPosition() + 1);
}

/* virtual */ void CControl::OnKey(wchar_t key, unsigned state __attribute__ ((unused)))
{
	if (key == CK_Left) {
		if (mHScrollBar.IsEnabled()) mHScrollBar.SetPosition(mHScrollBar.GetPosition() - 1);
	} else if (key == CK_Right) {
		if (mHScrollBar.IsEnabled()) mHScrollBar.SetPosition(mHScrollBar.GetPosition() + 1);
	} else if (key == CK_Up) {
		if (mVScrollBar.IsEnabled()) mVScrollBar.SetPosition(mVScrollBar.GetPosition() - 1);
	} else if (key == CK_Down) {
		if (mVScrollBar.IsEnabled()) mVScrollBar.SetPosition(mVScrollBar.GetPosition() + 1);
	}
}

/* virtual */ void CControl::OnContext()
{
    // derived classes may implement
}

void CControl::Scroll(int x, int y)
{
	int x1 = 0;
	int y1 = 0;
	int w = mViewWidth;
	int h = mViewHeight;

	if (x < 0) {
		x1 = -x;
		x = 0;
	} 
	if (y < 0) {
		y1 = -y;
		y = 0;
	}
	w -= x + x1;
	h -= y + y1;
    if ((w <= 0) || (h <= 0)) Redraw();
    else {
		if (cxxx::cxRTL) XCopyArea(cxxx::cxDpy, GetXID(), GetXID(), XDefaultGC(cxxx::cxDpy, XDefaultScreen(cxxx::cxDpy)), mWidth - w - x - BORDER_WIDTH, y + BORDER_WIDTH, w, h, mWidth - w - x1 - BORDER_WIDTH, y1 + BORDER_WIDTH); 
		else XCopyArea(cxxx::cxDpy, GetXID(), GetXID(), XDefaultGC(cxxx::cxDpy, XDefaultScreen(cxxx::cxDpy)), x + BORDER_WIDTH, y + BORDER_WIDTH, w, h, x1 + BORDER_WIDTH, y1 + BORDER_WIDTH); 
		if (x1) RedrawArea(BORDER_WIDTH, BORDER_WIDTH, x1, mViewHeight);
		else RedrawArea(w + BORDER_WIDTH, BORDER_WIDTH, x, mViewHeight);
        if (y1) RedrawArea(BORDER_WIDTH, BORDER_WIDTH, mViewWidth, y1);
        else RedrawArea(BORDER_WIDTH, h + BORDER_WIDTH, mViewWidth, y);
    }
}

void CControl::Select(unsigned lo, unsigned hi)
{
    if (hi < lo) {
        unsigned tmp = hi;
        hi = lo;
        lo = tmp;
    }
    if (hi == -1u) hi = lo;
  	cxAssert(hi < mList.size());
    mSelection.clear();
    ++hi;
    for (unsigned i = 0; i < lo; ++i) {
        if (!mList[i].IsSelected()) continue;
        mList[i].Select(false);
        DrawItem(i);
    }
    for (unsigned i = lo; i < hi; ++i) {
        mSelection.push_back(i);
        if (mList[i].IsSelected()) continue;
        mList[i].Select(true);
        DrawItem(i);
    }
    for (unsigned i = hi; i < mList.size(); ++i) {
        if (!mList[i].IsSelected()) continue;
        mList[i].Select(false);
        DrawItem(i);
    }
    DrawFrame();
}

void CControl::Layout()
{
	mViewHeight = GetHeight() - 2 * BORDER_WIDTH;
	mViewWidth = GetWidth() - 2 * BORDER_WIDTH;
	mDocHeight = mViewHeight;
	mDocWidth = mViewWidth;
	bool hsb = false;
	bool vsb = false;
	bool done = false;
	CalculateDocSize();
	while (!done) {
		done = true;
		if (!hsb && mDocWidth > mViewWidth) {
			mViewHeight -= mHScrollBar.GetHeight();
			mDocHeight = mViewHeight;
			done = false;
			hsb = true;
			CalculateDocSize();
		}
		if (!vsb && mDocHeight > mViewHeight) {
			mViewWidth -= mVScrollBar.GetWidth();
			mDocWidth = mViewWidth;
			done = false;
			vsb = true;
			CalculateDocSize();
		}
	}
    if (hsb) {
        mHScrollBar.Resize(mViewWidth, mHScrollBar.GetHeight());
       	mHScrollBar.Move(BORDER_WIDTH, mViewHeight + BORDER_WIDTH);
	    mHScrollBar.SetRange((mDocWidth + SCROLL_STEP - 1) / SCROLL_STEP);
       	mHScrollBar.SetPage(mViewWidth / SCROLL_STEP);
        mHScrollBar.Enable(true);
	    mHScrollBar.SetPosition(mHScrollBar.GetPosition());
      	mHScrollBar.Show();
        mHScrollBar.Redraw();
   	} else if (mHScrollBar.IsEnabled()) {
      	mHScrollBar.SetPosition(0);
        mHScrollBar.Enable(false);
   		mHScrollBar.Hide();
    }
    if (vsb) {
		mVScrollBar.Resize(mVScrollBar.GetWidth(), mViewHeight);
       	mVScrollBar.Move(mViewWidth + BORDER_WIDTH, BORDER_WIDTH);
 	    mVScrollBar.SetRange((mDocHeight + SCROLL_STEP - 1) / SCROLL_STEP);
       	mVScrollBar.SetPage(mViewHeight / SCROLL_STEP);
        mVScrollBar.Enable(true);
	    mVScrollBar.SetPosition(mVScrollBar.GetPosition());
      	mVScrollBar.Show();
        mVScrollBar.Redraw();
   	} else if (mVScrollBar.IsEnabled()) {
       	mVScrollBar.SetPosition(0);
        mVScrollBar.Enable(false);
   		mVScrollBar.Hide();
    }
	Redraw();
}
