/*----------------------------------------------------------------------------
  Project: cxxx <http://cxplorer.sourceforge.net>
  File   : CControlItem.h
  Info   : $Id: CControlItem.h,v 1.14 2012/08/06 17:36:40 gkraeuter Exp $
----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
  Copyright © 2005 - 2012 Gerhard Kräuter <gkraeuter@users.sourceforge.net>

  This  software  is  provided  'as-is',  without  any  express  or  implied
  warranty.  In  no  event  will  the authors be held liable for any damages
  arising from the use of this software.

  Permission  is  granted  to  anyone  to use this software for any purpose,
  including  commercial  applications,  and  to alter it and redistribute it
  freely, subject to the following restrictions:

  1. The  origin  of  this software must not be misrepresented; you must not
     claim  that  you  wrote the original software. If you use this software
     in  a  product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.

----------------------------------------------------------------------------*/

#ifndef CControlItem_h
#define CControlItem_h

class CControlItemInfo
{
public:
	enum
	{
		SELECTED = 1
	};
		
public:
	// Constructor
	CControlItemInfo(const std::string& text, bool sel)
	: mText(text),
	  mRefCount(1),
	  mState(0)
	{
		if (sel) mState = SELECTED;
	}

	// Destructor
	virtual ~CControlItemInfo()
	{
	}

public:
	std::string mText;
	unsigned mRefCount;
	unsigned mState;
};

class CControlItem
{
protected:
	enum
	{
		SELECTED = CControlItemInfo::SELECTED
	};
		
public:
	// Constructor
	CControlItem()
	: mInfo(NULL)
	{
	}

	CControlItem(const CControlItem& it)
	: mInfo(it.mInfo)
	{
		if (mInfo != NULL) ++mInfo->mRefCount;
	}

	// Destructor
	virtual ~CControlItem()
	{
		if (mInfo != NULL && --mInfo->mRefCount == 0) {
			delete mInfo;
			mInfo = NULL;
		}
	}
	
	// Assignment operator
	CControlItem& operator=(const CControlItem& it)
	{
		if (mInfo != it.mInfo) {
			if (mInfo != NULL && --mInfo->mRefCount == 0) delete mInfo;
			mInfo = it.mInfo;
			if (mInfo != NULL) ++mInfo->mRefCount;
		}
		return *this;
	}

	// Comparing operators
	bool operator==(const CControlItem& it) const
	{
		return mInfo == it.mInfo;
	}

	bool operator!=(const CControlItem& it) const
	{
		return mInfo != it.mInfo;
	}

	// Returns the name / text element
	const std::string& GetText() const
	{
		return mInfo->mText;
	}

	// Returns true if the item is selected
	bool IsSelected() const
	{
		return mInfo->mState & SELECTED;
	}

	// Select or deselect item
	void Select(bool sel)
	{
		if (sel) mInfo->mState |= SELECTED;
		else mInfo->mState  &= ~SELECTED;
	}

protected:
	CControlItemInfo* mInfo;
};

#endif // CControlItem_h
