/*----------------------------------------------------------------------------
  Project: cxxx <http://cxplorer.sourceforge.net>
  File   : CCursor.h
  Info   : $Id: CCursor.h,v 1.1 2012/09/14 18:32:41 gkraeuter Exp $
----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
  Copyright © 2012 Gerhard Kräuter <gkraeuter@users.sourceforge.net>

  This  software  is  provided  'as-is',  without  any  express  or  implied
  warranty.  In  no  event  will  the authors be held liable for any damages
  arising from the use of this software.

  Permission  is  granted  to  anyone  to use this software for any purpose,
  including  commercial  applications,  and  to alter it and redistribute it
  freely, subject to the following restrictions:

  1. The  origin  of  this software must not be misrepresented; you must not
     claim  that  you  wrote the original software. If you use this software
     in  a  product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.

----------------------------------------------------------------------------*/

#ifndef CCursor_h
#define CCursor_h

// C++X headers
#include <cxxx/Cxxx.h>

// C++ headers
#include <map>
#include <cstring>

// X11 headers
#include <X11/extensions/Xrender.h>

class CCursor
{
public:	
	// Constructor
	CCursor()
	: mCursor(None)
	{}

	CCursor(const std::string& name)
	{
		Create(name);
	}

	bool Create(const std::string& name);
	
	// Destructor
	~CCursor()
	{
		if (cxxx::cxMainWin) {
			XFreeCursor(cxxx::cxDpy, mCursor);
			mCursor = None;
		}
	}
	
	// returns true if the cursor is valid
	bool IsValid() const
	{
		return mCursor != None;
	}

	operator Cursor() const
	{
		return mCursor;
	}
	
private:
	// Initializes the cursor cache
	static void Init()
	{
		AddTheme("default");
	}

	// Actual creation of the picture
	bool DoCreate(const std::string& name);

	// Cursor Themes
	static void AddTheme(const std::string& theme);
	static void AddThemeDir(const std::string& dir, const std::string& theme);

	// Returns lowercase of s
	static std::string ToLower(const std::string& s);
	
private:
	
	Cursor mCursor;

	// These static members are used to provide caching and theme import
	static bool mInitialized;
	static std::vector<std::string> mThemeDirs;
};

#endif // CCursor_h
