/*----------------------------------------------------------------------------
  Project: cxxx <http://cxplorer.sourceforge.net>
  File   : CDeskTopEntry.cpp
  Info   : $Id: CDeskTopEntry.cpp,v 1.29 2012/08/06 17:36:40 gkraeuter Exp $
----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
  Copyright © 2002 - 2012 Gerhard Kräuter <gkraeuter@users.sourceforge.net>

  This  software  is  provided  'as-is',  without  any  express  or  implied
  warranty.  In  no  event  will  the authors be held liable for any damages
  arising from the use of this software.

  Permission  is  granted  to  anyone  to use this software for any purpose,
  including  commercial  applications,  and  to alter it and redistribute it
  freely, subject to the following restrictions:

  1. The  origin  of  this software must not be misrepresented; you must not
     claim  that  you  wrote the original software. If you use this software
     in  a  product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.

----------------------------------------------------------------------------*/

// C++ headers
#include <fstream>
#include <cerrno>
#include <cctype>

// C++X headers
#include <cxxx/CDeskTopEntry.h>
#include <cxxx/CApplication.h>
#include <cxxx/CError.h>

// namespace declarations
using std::ifstream;
using std::string;
using cxxx::cxError;
using cxxx::cxLocale;
using cxxx::cxIconv;
using cxxx::lib::cxgettext;

CDeskTopEntry::CDeskTopEntry()
: mVersion(0),
  mType(UNKNOWN),
  mTerm(false)
{
}

CDeskTopEntry::CDeskTopEntry(const string& file)
: mVersion(0),
  mType(UNKNOWN),
  mTerm(false)
{
	Read(file);
}

void CDeskTopEntry::Read(const string& file)
{
	cxAssert(mFile.empty());
	mFile = file;
	ifstream in;
	in.open(mFile.c_str());
	if (!in) {
		cxError << _("Failed to open file {1}.\n{2}", 28) << file << strerror(errno) << CError::Error;
		return ;
	}
	string buf;
	while (in) {
		getline(in, buf);
		if (buf.empty()) continue; // empty
		if (buf[0] == '#') continue; // comment
		if (buf[0] == '[') {
			if ((buf == "[Desktop Entry]") || (buf == "[KDE Desktop Entry]")) {
				ReadAction(in);
				return ;
			}
		}
	}
}

void CDeskTopEntry::ReadAction(ifstream& in)
{
	string buf;
	string longloc;
	longloc = cxLocale.name();
	longloc.resize(5);
	string shortloc(longloc);
	shortloc.resize(2);
	while (in.good()) {
		getline(in, buf);
		if(buf.empty()) continue;
		switch (buf[0]) {
		case '#':
			break;
		case '[':
			//TODO add actions
			break;
		case 'C':
			if (buf.substr(1, 6) == "omment") {
				if (isalnum(buf.at(7))) break; // not a comment
				if (buf[7] == '[') {
					if ((buf.substr(8, 5) == longloc) || (buf.substr(8, 2) == shortloc)) mComment = GetValue(buf);
				} else if (mComment.empty()) mComment = GetValue(buf);
			}
			break;
		case 'E':
			if (buf.substr(1, 7) == "ncoding") {
				if (isalnum(buf.at(8))) break; // not an encoding
				mEncoding = GetValue(buf);
			} else if (buf.substr(1, 3) == "xec") {
				if (isalnum(buf.at(4))) break; // not an exec entry
				mExec = GetValue(buf);
			}
			break;
		case 'I':
			if (buf.substr(1, 3) ==  "con") {
				if (isalnum(buf.at(4))) break;
				mIcon = GetValue(buf);
			}
			break;
		case 'N':
			if (buf.substr(1, 3) == "ame") {
				if (isalnum(buf.at(4))) break;
				if (buf[4] == '[') {
					if ((buf.substr(5, 5) == longloc) || (buf.substr(5, 2) == shortloc)) mName = GetValue(buf);
				} else if (mName.empty()) mName = GetValue(buf);
			}
			break;
		case 'T':
			if (buf.substr(1, 3) == "ype") {
				if (isalnum(buf.at(4))) break;
				string s = GetValue(buf);
				if (s.empty()) break;
				if (s == "Application") mType = APPLICATION;
				else if (s == "KAPP") mType = APPLICATION;
				else if (s == "XAPP") mType = APPLICATION;
				else if (s == "Link") mType = LINK;
				else if (s == "FSDevice") mType = DEVICE;
				else if (s == "Mime Type") mType = MIME;
				else if (s == "Directory") mType = DIRECTORY;
				else if (s == "Service") mType = SERVICE;
				else if (s == "Service Type") mType = SERVICETYPE;
			} else if (buf.substr(1, 7) == "erminal") {
				if (isalnum(buf.at(8))) break;
				string s = GetValue(buf);
				if (buf.empty()) break;
				if ((buf == "1") || (buf == "true")) mTerm = true;
			}
			break;
		case 'V':
			if (buf.substr(1, 6) == "ersion") {
				if (isalnum(buf.at(7))) break;
				mVersion = GetVersion(buf);
			}
			break;
		default:
			break;
		}
	}
}

string CDeskTopEntry::GetValue(const string& s)
{
	string::size_type len = s.size();
	if (len == 0) return "";
	string::size_type pos =	s.find('=');
	if (pos == string::npos) return "";
	++pos;
	if (pos >= len) return "";
	pos = s.find_first_not_of(' ', pos);
	if (pos == string::npos) return "";
	else return s.substr(pos);
}

unsigned CDeskTopEntry::GetVersion(const string& s)
{
	string tmp = GetValue(s);
	if (tmp.empty()) return 0;
	unsigned ret = s[0] - '0';
	if (ret > 9) ret = 0;
	return ret;
}

