/*----------------------------------------------------------------------------
  Project: cxxx <http://cxplorer.sourceforge.net>
  File   : CDeskTopEntry.h
  Info   : $Id: CDeskTopEntry.h,v 1.18 2012/08/06 17:36:40 gkraeuter Exp $
----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
  Copyright © 2002 - 2012 Gerhard Kräuter <gkraeuter@users.sourceforge.net>

  This  software  is  provided  'as-is',  without  any  express  or  implied
  warranty.  In  no  event  will  the authors be held liable for any damages
  arising from the use of this software.

  Permission  is  granted  to  anyone  to use this software for any purpose,
  including  commercial  applications,  and  to alter it and redistribute it
  freely, subject to the following restrictions:

  1. The  origin  of  this software must not be misrepresented; you must not
     claim  that  you  wrote the original software. If you use this software
     in  a  product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.

----------------------------------------------------------------------------*/

#ifndef CDesktopEntry_h
#define CDesktopEntry_h

// C++X headers
#include <cxxx/Cxxx.h>

// C++ headers
#include <iosfwd>
#include <vector>

class CDeskTopEntry
{
public:
	// Type definition
    typedef std::vector<CDeskTopEntry> TContainer;

	enum
	{
		// Types
		UNKNOWN,
		APPLICATION,
		LINK,
		DEVICE,
		MIME,
		DIRECTORY,
		SERVICE,
		SERVICETYPE
	};

public:
	// Constructor
	CDeskTopEntry();
	CDeskTopEntry(const std::string& file);

	// Read desktop entry file
	void Read(const std::string& file);

	// Returns the encoding of the file
	const std::string& GetEncoding() const
	{
		return mEncoding;
	}

	// Returns the version of the desktop entry standard
	unsigned GetVersion() const
	{
		return mVersion;
	}

	// Returns the value of the name field
	std::string GetName() const
	{
		unsigned conv = cxxx::cxIconv.GetConverter(mEncoding, "");
		return cxxx::cxIconv.Convert(mName, conv);
	}

	// Returns the type
	unsigned GetType() const
	{
		return mType;
	}

	// Returns the value of the comment field
	std::string GetComment() const
	{
		unsigned conv = cxxx::cxIconv.GetConverter(mEncoding, "");
		return cxxx::cxIconv.Convert(mComment, conv);
	}
	
	// Returns the value of the exec field
	std::string GetExec() const
	{
		unsigned conv = cxxx::cxIconv.GetConverter(mEncoding, "");
		return cxxx::cxIconv.Convert(mExec, conv);
	}
	
	// Returns possible actions
	TContainer& GetActions()
	{
		return mActions;
	}
	
	// Returns the icon file name
    const std::string& GetIcon() const
	{
		return mIcon;
	}
	
	// Returns the value of the path field
	const std::string& GetPath() const
	{
		return mPath;
	}
	
	// Returns true if a terminal is needed
	bool NeedTerm() const
	{
		return mTerm;
	}
	
	// Returns the value of the URL field
	const std::string& GetURL()
	{
		return mURL;
	}

protected:
	// Reads an action
	void ReadAction(std::ifstream& ist);

	// Returns the value part of a desktop entry line
	std::string GetValue(const std::string& s);

	// Not implemented; returns always 0
	unsigned GetVersion(const std::string& s);

protected:
	std::string mFile;
	std::string mEncoding;
	unsigned mVersion;
	std::string mName;
	unsigned mType;
	std::string mComment;
	TContainer mActions;
	std::string mIcon;
	std::string mPath;
	std::string mExec;
	bool mTerm;
	std::string mURL;
};

#endif
