/*----------------------------------------------------------------------------
  Project: cxxx <http://cxplorer.sourceforge.net>
  File   : CDnDAskDialog.cpp
  Info   : $Id: CDnDAskDialog.cpp,v 1.18 2012/08/06 17:36:40 gkraeuter Exp $
----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
  Copyright © 2003 - 2012 Gerhard Kräuter <gkraeuter@users.sourceforge.net>

  This  software  is  provided  'as-is',  without  any  express  or  implied
  warranty.  In  no  event  will  the authors be held liable for any damages
  arising from the use of this software.

  Permission  is  granted  to  anyone  to use this software for any purpose,
  including  commercial  applications,  and  to alter it and redistribute it
  freely, subject to the following restrictions:

  1. The  origin  of  this software must not be misrepresented; you must not
     claim  that  you  wrote the original software. If you use this software
     in  a  product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.

----------------------------------------------------------------------------*/

// cxxx headers
#include <cxxx/CDnDAskDialog.h>
#include <cxxx/CRadioButton.h>

// namespace declarations
using cxxx::cxMainWin;
using std::string;
using std::vector;
using cxxx::lib::cxgettext;

CDnDAskDialog::CDnDAskDialog(const vector<string>& list)
: CTopLevelWindow(),
  mGroup(this),
  mPushOK(this, IDOK),
  mPushCancel(this, IDCANCEL)
{
	unsigned h = list.size() * 20;
	h += 24 + 45;
	unsigned w = 0;
	for (unsigned i = 0; i < list.size(); ++i) {
		CRadioButton* b = new CRadioButton(&mGroup, IDRADIO + i);
		b->SetName(list[i]);
		if (b->GetWidth() > w) w = b->GetWidth();
		mButton.push_back(b);
	}
	w += 40;
	if (w < 190) w = 190;
	Resize(w, h);
	FixSize();
	for (unsigned i = 0; i < mButton.size(); ++i) {
		if (cxxx::cxRTL) mButton[i]->Move(w - 20 - mButton[i]->GetWidth(), 20 * (i + 1));
		else mButton[i]->Move(20, 20 * (i + 1));
	}
	mPushOK.SetName(_("&OK", 25));
	mPushCancel.SetName(_("&Cancel", 29));
	mPushOK.Move(10, h - 29);
	mPushCancel.Move(100, h - 29);
	TransientFor(static_cast<CTopLevelWindow*>(cxMainWin));
	SetCaption(_("Choose action", 30));
	mButton[0]->SetCheck(true);
	for (unsigned i = 0; i < mButton.size(); ++i) mButton[i]->Show();
	mPushOK.Show();
	mPushCancel.Show();
	mReturn = 0;
}

/* virtual */ CDnDAskDialog::~CDnDAskDialog()
{
	for(vector<CRadioButton*>::iterator i = mButton.begin(); i != mButton.end(); ++i) delete *i;
}

/* virtual */ void CDnDAskDialog::OnExpose(XExposeEvent& e __attribute__ ((unused)))
{
	DrawFrame(10, 10, GetWidth() - 20, GetHeight() - 49, FS_ENGRAVED, false);
}

/* virtual */ void CDnDAskDialog::OnMessage(unsigned msgid, TMessage type, unsigned value __attribute__ ((unused)))
{
	switch(msgid) {
	case IDOK:
		switch (type) {
		case MT_RELEASED:
		case MT_ACCELERATOR:
			mShutDown = true;
			break;
		default:
			break;
		}
		break;
	case IDCANCEL:
		switch (type) {
		case MT_RELEASED:
		case MT_ACCELERATOR:
			mReturn = IDCANCEL;
			mShutDown = true;
			break;
		default:
			break;
		}
		break;
	default:
		mReturn = msgid - IDRADIO;
		break;
	}
}

/* virtual */ void CDnDAskDialog::OnQuit()
{
	mReturn = IDCANCEL;
	mShutDown = true;
	return;
}
