/*----------------------------------------------------------------------------
  Project: cxxx <http://cxplorer.sourceforge.net>
  File   : CDnDWindow.cpp
  Info   : $Id: CDnDWindow.cpp,v 1.20 2012/09/14 18:31:01 gkraeuter Exp $
----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
  Copyright © 2003 - 2012 Gerhard Kräuter <gkraeuter@users.sourceforge.net>

  This  software  is  provided  'as-is',  without  any  express  or  implied
  warranty.  In  no  event  will  the authors be held liable for any damages
  arising from the use of this software.

  Permission  is  granted  to  anyone  to use this software for any purpose,
  including  commercial  applications,  and  to alter it and redistribute it
  freely, subject to the following restrictions:

  1. The  origin  of  this software must not be misrepresented; you must not
     claim  that  you  wrote the original software. If you use this software
     in  a  product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.

----------------------------------------------------------------------------*/

// C++X headers
#include <cxxx/CDnDWindow.h>
#include <cxxx/CTopLevelWindow.h>

// X11 headers
#include <X11/Xatom.h>

// namespace declarations
using std::vector;
using cxxx::cxDnDManager;
using cxxx::cxDpy;
using cxxx::cxError;
using std::string;

// static members
int CDnDWindow::mXDragStart;
int CDnDWindow::mYDragStart;
Atom CDnDWindow::DND_MOVE(None);
Atom CDnDWindow::DND_COPY(None);
Atom CDnDWindow::DND_LINK(None);
Atom CDnDWindow::DND_ASK(None);
Atom CDnDWindow::DND_PRIVATE(None);
Atom CDnDWindow::DND_SELECTION(None);

CDnDWindow::CDnDWindow(CWindow* parent, unsigned msgid)
: CWindow(parent, msgid),
  mButton(0),
  mDragging(false)
{
}

/* virtual */ void CDnDWindow::OnEvent(XEvent& e)
{
	switch(e.xany.type) {
		case MotionNotify:
		OnMotion(e.xmotion);
		break;
	default:
		break;
	}
}

/* virtual */ void CDnDWindow::OnMotion(XMotionEvent& e)
{
	if (!mDragging &&((abs(mXDragStart - e.x) > DRAG_THRESHOLD) || (abs(mYDragStart - e.y) > DRAG_THRESHOLD))) {
		mDragging = cxDnDManager->StartDrag(*this, e, mButton);
		if (!mDragging) XChangeActivePointerGrab(cxDpy, ButtonReleaseMask, None, CurrentTime);
	}
}

/* virtual */ bool CDnDWindow::OnDragInit(unsigned button __attribute__ ((unused)), Atom& action __attribute__ ((unused)))
{
#ifdef CXXX_DEBUG
	if (IsDnDAware()) cxError<<"CDnDWindow::OnDragInit not implemented"<<CError::Error;
#endif // CXXX_DEBUG
	return false;
}

/* virtual */ bool CDnDWindow::CanAcceptAction(Atom action __attribute__ ((unused)))
{
#ifdef CXXX_DEBUG
	if (IsDnDAware()) cxError<<"CDnDWindow::CanAcceptAction not implemented"<<CError::Error;
#endif // CXXX_DEBUG
	return false;
}

/* virtual */ bool CDnDWindow::CanPerformAction(Atom action __attribute__ ((unused)))
{
#ifdef CXXX_DEBUG
	if (IsDnDAware()) cxError<<"CDnDWindow::CanPerformAction not implemented"<<CError::Error;
#endif // CXXX_DEBUG
	return false;
}

/* virtual */ bool CDnDWindow::OnDrop(Atom action __attribute__ ((unused)))
{
#ifdef CXXX_DEBUG
	if (IsDnDAware()) cxError<<"CDnDWindow::OnDrop not implemented"<<CError::Error;
#endif // CXXX_DEBUG
	return false;
}

/* virtual */ Cursor CDnDWindow::GetDnDCursor(Atom action)
{
    if (action == None) return cxDnDManager->GetNotCursor();
    if (action == DND_MOVE) return cxDnDManager->GetMoveCursor();
    if (action == DND_ASK) return cxDnDManager->GetAskCursor();
    if (action == DND_COPY) return cxDnDManager->GetCopyCursor();
    if (action == DND_LINK) return cxDnDManager->GetLinkCursor();
    if (action == DND_PRIVATE) return cxDnDManager->GetNotCursor();
    return cxDnDManager->GetAskCursor();
}

/* virtual */ Atom CDnDWindow::GetDnDAction(int x __attribute__ ((unused)), int y __attribute__ ((unused)), Atom action __attribute__ ((unused)))
{
#ifdef CXXX_DEBUG
	if (IsDnDAware()) cxError<<"CDnDWindow::GetDnDAction not implemented"<<CError::Error;
#endif // CXXX_DEBUG
	return None;
}

/* virtual */ bool CDnDWindow::GetActionList(vector<Atom>& actions __attribute__ ((unused)), vector<string>& desc __attribute__ ((unused)))
{
#ifdef CXXX_DEBUG
	if (IsDnDAware()) cxError << "CDnDWindow::GetActionList not implemented" << CError::Error;
#endif // CXXX_DEBUG
	return false;
}

/* virtual */ bool CDnDWindow::OnDropInit(std::vector<Atom> mime __attribute__ ((unused)))
{
#ifdef CXXX_DEBUG
	if (IsDnDAware()) cxError<<"CDnDWindow::OnDropInit not implemented"<<CError::Error;
#endif // CXXX_DEBUG
	return false;
}

/* virtual */ void CDnDWindow::OnDropFinished()
{
#ifdef CXXX_DEBUG
	if (IsDnDAware()) cxError<<"CDnDWindow::OnDropFinished not implemented"<<CError::Error;
#endif // CXXX_DEBUG
}
