/*----------------------------------------------------------------------------
  Project: cxxx <http://cxplorer.sourceforge.net>
  File   : CDropDownList.h
  Info   : $Id: CDropDownList.h,v 1.20 2012/05/06 18:57:36 gkraeuter Exp $
----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
  Copyright © 2002 - 2012 Gerhard Kräuter <gkraeuter@users.sourceforge.net>

  This  software  is  provided  'as-is',  without  any  express  or  implied
  warranty.  In  no  event  will  the authors be held liable for any damages
  arising from the use of this software.

  Permission  is  granted  to  anyone  to use this software for any purpose,
  including  commercial  applications,  and  to alter it and redistribute it
  freely, subject to the following restrictions:

  1. The  origin  of  this software must not be misrepresented; you must not
     claim  that  you  wrote the original software. If you use this software
     in  a  product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.

----------------------------------------------------------------------------*/

#ifndef CDropDownList_h
#define CDropDownList_h

// C++X headers
#include <cxxx/CWindow.h>
#include <cxxx/CListItem.h>
#include <cxxx/CDropDownPopup.h>

class CDropDownList : public CWindow
{
private:
	enum
	{
		// Message IDs
		IDPOPUP = cxxx::msgid::MSGID_CDropDownList
	};

	// Type definitions
	typedef std::vector<std::string> TList;

public:
	// Constructor
	CDropDownList(CWindow* parent, unsigned msgid);

	// Destructor
	virtual ~CDropDownList();

	// X event processing
	virtual void OnExpose(XExposeEvent& e);

	virtual void OnLButtonDown(XButtonEvent& e);

	// C++X message processing
	virtual void OnMessage(unsigned msgid, TMessage type, unsigned value);

	// Updates the look of the window
	virtual void Recolor();

	// Adds item to list
	void AppendItem(const std::string& text)
	{
		mList.push_back(text);
	}
	
	// Deletes item at index idx
	void DeleteAt(unsigned idx)
	{
		TList::iterator i = mList.begin() + idx;
		cxAssert(i < mList.end());
		mList.erase(i);
	}
	
	// Returns the item count
	unsigned GetItemCount() const
	{
		return mList.size();
	}
	
	// Selects item at index idx
	void Select(unsigned idx)
	{
		if (idx != -1u) {
			cxAssert(idx < mList.size());
			mSelection = idx;
		}
		Redraw();
	}
	
	// Opens the drop down popup	
	void DropDown()
	{
		if (mList.empty()) return;
		int x = 0;
		int y = 0;
		TranslateCoordinates(x, y, NULL);
		CDropDownPopup* p = new CDropDownPopup(this, IDPOPUP, mList);
		p->Position(x, y, GetWidth(), GetHeight());
		p->Select(mSelection);
		p->Show();
	}
	
	// Returns the text of the selected item
	const std::string& GetText() const
	{
		if (mSelection == -1u) return cxxx::cxEmptyString;
		cxAssert(mSelection < mList.size());
		return mList[mSelection];
	}
	
	// Returns the index of the selected item
	unsigned GetSelection() const
	{
		return mSelection;
	}
	
	// Clears the list
	void Clear()
	{
		mList.clear();
		mSelection = -1u;
		mButtonDown = false;
	}
	
protected:
	CIcon mArrow;
	TList mList;
	unsigned mSelection;
	bool mButtonDown;
};

#endif // CDropDownList_h
