/*----------------------------------------------------------------------------
  Project: cxxx <http://cxplorer.sourceforge.net>
  File   : CDropDownPopup.h
  Info   : $Id: CDropDownPopup.h,v 1.16 2009/12/28 13:27:55 gkraeuter Exp $
----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
  Copyright © 2002 - 2009 Gerhard Kräuter <gkraeuter@users.sourceforge.net>

  This  software  is  provided  'as-is',  without  any  express  or  implied
  warranty.  In  no  event  will  the authors be held liable for any damages
  arising from the use of this software.

  Permission  is  granted  to  anyone  to use this software for any purpose,
  including  commercial  applications,  and  to alter it and redistribute it
  freely, subject to the following restrictions:

  1. The  origin  of  this software must not be misrepresented; you must not
     claim  that  you  wrote the original software. If you use this software
     in  a  product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.

----------------------------------------------------------------------------*/

#ifndef CDropDownPopup_h
#define CDropDownPopup_h

// C++X headers
#include <cxxx/CScrollBar.h>

class CDropDownPopup : public CWindow
{
private:
	enum
	{
		// Message IDs
		IDSCROLL = cxxx::msgid::MSGID_CDropDownPopup
	};

	typedef std::vector<std::wstring> TList;

public:
	// Constructor
	CDropDownPopup(CWindow* parent, unsigned msgid, const std::vector<std::string>& list);

	// Destructor
	virtual ~CDropDownPopup();

	// X event processing
	virtual void OnExpose(XExposeEvent& e);
	virtual void OnLButtonDown(XButtonEvent& e);
	virtual void OnButtonUp(XButtonEvent& e);
	virtual void OnEvent(XEvent& e);

	// C++X message processing
	virtual void OnMessage(unsigned msgid, TMessage type, unsigned value);

	// Updates the look of the window
	virtual void Recolor();

	// Positions the window around the given rectangle
	void Position(int x, int y, int w, int h);

	// Selects item at index idx
	void Select(unsigned idx)
	{
		if (mSelected == idx) return;
		if (mSelected != -1u) DrawItem(mSelected, false);
		mSelected = idx;
		if (mSelected != -1u) DrawItem(mSelected, true);
		if (mSelected != -1u && mScrolling) {
			if ((mSelected < mOffset / 20) || (mSelected > (mOffset + mHeight - 5) / 20)) mScrollbar.SetPosition(mSelected - mHeight / 40);
		}
	}
	
protected:
	// Draws item at index idx
	void DrawItem(unsigned idx, bool selected);

	// Returns item index of item at screen position x, y
	unsigned GetItemFromPos(int x, int y)
	{
		if (mScrolling && x > (int)(mWidth - mScrollbar.GetWidth())) return -1u;
		if (y < 2) return -1u;
		if (y >= (int)mHeight - 2) return -1u;
		unsigned ret = (y - 2 + mOffset) / 20;
		if (ret < mList.size()) return ret;
		else return -1u;
	}

protected:
	TList mList;
	unsigned mSelected;
	unsigned mOffset;
	Cursor mCursor;
	bool mScrolling;
	CVScrollBar mScrollbar;
};

#endif // CDropDownPopup_h
