/*----------------------------------------------------------------------------
  Project: cxxx <http://cxplorer.sourceforge.net>
  File   : CDupFNDialog.cpp
  Info   : $Id: CDupFNDialog.cpp,v 1.37 2012/08/06 17:36:40 gkraeuter Exp $
----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
  Copyright © 2002 - 2012 Gerhard Kräuter <gkraeuter@users.sourceforge.net>

  This  software  is  provided  'as-is',  without  any  express  or  implied
  warranty.  In  no  event  will  the authors be held liable for any damages
  arising from the use of this software.

  Permission  is  granted  to  anyone  to use this software for any purpose,
  including  commercial  applications,  and  to alter it and redistribute it
  freely, subject to the following restrictions:

  1. The  origin  of  this software must not be misrepresented; you must not
     claim  that  you  wrote the original software. If you use this software
     in  a  product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.

----------------------------------------------------------------------------*/

// C++ headers
#include <cstdio>

// cxxx headers
#include <cxxx/CDupFNDialog.h>
#include <cxxx/CFileHandler.h>

// C headers
#include <unistd.h>

// namespace declarations
using cxxx::cxDpy;
using std::string;
using std::wstring;
using cxxx::lib::cxgettext;
using cxxx::cxFileManager;

CDupFNDialog::CDupFNDialog(const string& path, const string& name, bool overwrite)
: CTopLevelWindow(),
  mStatic(this),
  mEdit(this),
  mAction(this),
  mRename(&mAction, IDRENAME),
  mSkip(&mAction, IDSKIP),
  mAutoSkip(&mAction, IDAUTOSKIP),
  mOver(&mAction, IDOVER),
  mOverAll(&mAction, IDOVERALL),
  mOK(this, IDOK),
  mCancel(this, IDCANCEL),
  mPath(path)
{
	mName = name;
	Resize(390, 230);
	Move((DisplayWidth(cxDpy, DefaultScreen(cxDpy)) - mWidth) / 2, (DisplayHeight(cxDpy, DefaultScreen(cxDpy)) - mHeight) / 2);
	FixSize();
	SetCaption(_("File exists", 34));
	mStatic.Resize(330, 60);
 	mStatic.Move(50, 10);
	string text = _("An item named {1} already exists in {2}.", 35);
	string::size_type pos = text.find("{1}");
	cxAssert(pos != string::npos);
	text.replace(pos, 3, name);
	pos = text.find("{2}");
	cxAssert(pos != string::npos);
	text.replace(pos, 3, path);
	mStatic.SetText(text);
	mEdit.Resize(370, 20);
	mEdit.Move(10, 75);
	mEdit.SetText(mName);
	mIcon.Create(CIcon::WARNING, 32);
	mRename.SetName(_("&Rename", 36));
	mRename.Move(20, 120);
	mRename.Enable(false);
	mSkip.SetName(_("&Skip", 37));
	mSkip.Move(20, 140);
	mAutoSkip.SetName(_("A&uto Skip", 38));
	mAutoSkip.Move(20, 160);
	mOver.SetName(_("O&verwrite", 39));
	mOver.Move(200, 120);
	mOver.Enable(overwrite);
	mOverAll.SetName(_("Overwrite &All", 40));
	mOverAll.Move(200, 140);
	mOverAll.Enable(overwrite);
	mOK.SetName(_("&OK", 25));
	mOK.Move(10, 200);
	mOK.Enable(false);
	mCancel.SetName(_("&Cancel", 29));
	mCancel.Move(300, 200);
	mStatic.Show();
	mEdit.Show();
	mRename.Show();
	mSkip.Show();
	mAutoSkip.Show();
	mOver.Show();
	mOverAll.Show();
	mOK.Show();
	mCancel.Show();
	mPath += '/';
	SetFocusWindow(&mEdit);
	mReturn = IDCANCEL;
	Show();
}

/* virtual */ void CDupFNDialog::OnExpose(XExposeEvent& e __attribute__ ((unused)))
{
	ClearArea(10, 10, mIcon.GetWidth(), mIcon.GetHeight());
	DrawIcon(10, 10, mIcon);
	DrawFrame(10, 110, 370, 80, FS_ENGRAVED, false);
}

/* virtual */ void CDupFNDialog::OnMessage(unsigned msgid, TMessage type, unsigned value __attribute__ ((unused)))
{
	switch (msgid) {
	case IDRENAME:
	case IDSKIP:
	case IDAUTOSKIP:
	case IDOVER:
	case IDOVERALL:
		if (type == MT_CHECK) {
			mReturn = msgid;
			mOK.Enable(true);
		}
		break;
	case IDOK:
		mShutDown = true;
		break;
	case IDCANCEL:
		OnQuit();
		break;
	default:
		break;
	}
}

/* virtual */ void CDupFNDialog::OnFocus(bool in)
{
	if (in) SetFocusWindow(&mEdit);
}

/* virtual */ void CDupFNDialog::OnQuit()
{
	mReturn = IDCANCEL;
	mShutDown = true;
}

/* virtual */ void CDupFNDialog::OnKey(wchar_t key, unsigned state)
{
	if (key == '\x1b') OnQuit();
	else CTopLevelWindow::OnKey(key, state);
	bool en = mRename.IsEnabled();
	if (mEdit.GetText() != mName) {
		unsigned len = mPath.size();
		mPath += mEdit.GetText();
		if (access(mPath.c_str(), F_OK) != 0) en = true;
		else en = false;
		mPath.resize(len);
	}
	else en = false;
	if (en && !mRename.IsEnabled()) {
		mRename.Enable(true);
		mRename.SetCheck(true);
		mReturn = IDRENAME;
		mOK.Enable(true);
	} else if (!en && mRename.IsEnabled()) {
		mRename.SetCheck(false);
		mRename.Enable(false);
		mReturn = IDCANCEL;
		mOK.Enable(false);
	}
	if (key == L'\n' && mOK.IsEnabled()) mShutDown = true;
}
