/*----------------------------------------------------------------------------
  Project: cxxx <http://cxplorer.sourceforge.net>
  File   : CError.h
  Info   : $Id: CError.h,v 1.10 2012/08/06 17:36:40 gkraeuter Exp $
----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
  Copyright © 2002 - 2012 Gerhard Kräuter <gkraeuter@users.sourceforge.net>

  This  software  is  provided  'as-is',  without  any  express  or  implied
  warranty.  In  no  event  will  the authors be held liable for any damages
  arising from the use of this software.

  Permission  is  granted  to  anyone  to use this software for any purpose,
  including  commercial  applications,  and  to alter it and redistribute it
  freely, subject to the following restrictions:

  1. The  origin  of  this software must not be misrepresented; you must not
     claim  that  you  wrote the original software. If you use this software
     in  a  product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.

----------------------------------------------------------------------------*/

#ifndef CError_h
#define CError_h

// C++ headers
#include <string>
#include <sstream>

class CError
{
public:	
	// Contructor
	CError()
	: mCount(0)
	{
	}

	// Replaces a placeholder
	CError& operator<<(const std::string& s)
	{
		if(mCount == 0) mText += s;
		else {
			std::string tmp("{0}");
			tmp[1] += mCount;
			std::string::size_type i = mText.find(tmp);
			if (i != std::string::npos) mText.replace(i, tmp.size(), s);
			else mText += s;
		}
		++mCount;
		return *this;
	}

	// Replaces a placeholder
	CError& operator<<(int i)
	{
		std::ostringstream os;
		os << i;
		return operator<<(os.str());
	}
	
	// Adaptor for the following functions
	void operator<<(void (*op)(CError&))
	{
		(*op)(*this);
	}
	
	// Makes this an informational note
	static void Info(CError& e);

	// Makes this a warning
	static void Warn(CError& e);
	
	// Makes this an error
	static void Error(CError& e);
	
	// Makes this a fatal error and aborts the program
	static void Fatal(CError& e);

private:
	std::string mText;
	unsigned mCount;
};

#endif // CError_h
