/*----------------------------------------------------------------------------
  Project: cxxx <http://cxplorer.sourceforge.net>
  File   : CFileOpenDialog.cpp
  Info   : $Id: CFileOpenDialog.cpp,v 1.22 2012/08/06 17:36:40 gkraeuter Exp $
----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
  Copyright © 2003 - 2012 Gerhard Kräuter <gkraeuter@users.sourceforge.net>

  This  software  is  provided  'as-is',  without  any  express  or  implied
  warranty.  In  no  event  will  the authors be held liable for any damages
  arising from the use of this software.

  Permission  is  granted  to  anyone  to use this software for any purpose,
  including  commercial  applications,  and  to alter it and redistribute it
  freely, subject to the following restrictions:

  1. The  origin  of  this software must not be misrepresented; you must not
     claim  that  you  wrote the original software. If you use this software
     in  a  product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.

----------------------------------------------------------------------------*/


// C++X headers
#include <cxxx/CFileOpenDialog.h>
#include <cxxx/CFileListControl.h>
#include <cxxx/CFileHandler.h>

// C headers
#include <unistd.h>

// Namespace declarations
using std::string;
using std::vector;
using cxxx::cxFileManager;
using cxxx::filesort::ASCENDING;
using cxxx::filesort::NAME;
using cxxx::filesort::DIRFIRST;
using cxxx::lib::cxgettext;

CFileOpenDialog::CFileOpenDialog(CWindow* parent, const string& dir, const vector<unsigned>& filter)
: CTopLevelWindow(),
  mFileList(this, IDFILELIST),
  mLevelUp(this, IDLEVELUP, CIcon::UP, _("One level up", 377)),
  mOK(this, IDOK),
  mCancel(this, IDCANCEL),
  mEdit(this)
{
	TransientFor(parent->GetTopLevelWindow());
	SetCaption(_("Open file", 294));
	Resize(400, 225);
	FixSize();
	for (vector<unsigned>::const_iterator i = filter.begin(); i < filter.end(); ++i)
	{
		mFileList.AddFilter(*i);
	}
	mFileList.Resize(400, 145);
	mFileList.Move(0, 35);
	mFileList.SetSortKey(ASCENDING | NAME | DIRFIRST);
	mFileList.SetDirectory(dir);
	mLevelUp.Move(10, 5);
	mOK.SetName(_("&Open", 376));
	mOK.Move(10, 195);
	mOK.Enable(false);
	mCancel.SetName(_("&Cancel", 29));
	mCancel.Move(310, 195);
	mEdit.Resize(355, mEdit.GetHeight());
	mEdit.SetText(dir);
	mEdit.Move(40, mLevelUp.GetYPos() + (mLevelUp.GetHeight() - mEdit.GetHeight()) / 2);
	mLevelUp.Show();
	mFileList.Show();
	mOK.Show();
	mCancel.Show();
	mEdit.Show();
	SetFocusWindow(&mEdit);
}

/* virtual */ void CFileOpenDialog::OnMessage(unsigned msgid, TMessage type, unsigned value __attribute__ ((unused)))
{
	switch (msgid) {
	case IDFILELIST:
		switch (type) {
		case MT_CONFIRM_SELECTION:
			OnSelectionConfirmed();
			break;
		case MT_SELECTION_CHANGED:
			if (mFileList.GetSelCount() != 1) mOK.Enable(false);
			else {
				mEdit.SetText(mFileList.GetPath(mFileList.GetSelection().front()));
				mOK.Enable(true);
			}
		default:
			break;
		}
		break;
	case IDLEVELUP:
		mFileList.SetDirectory(cxFileManager->GetFilePath(mFileList.GetPath(-1u)));
		{
			string dir = mFileList.GetPath(-1u);
			if (dir[dir.size() - 1] != '/') dir += '/';
			mEdit.SetText(dir);
		}
		mFileList.Reload();
		break;
	case IDOK:
		OnOK();
		break;
	case IDCANCEL:
		OnQuit();
		break;
	default:
		break;
	}
}

void CFileOpenDialog::OnSelectionConfirmed()
{
	CMagic mag;
	string fn = mEdit.GetText();
	if (mag.GetFileType(fn) == TMagic::DIRECTORY) {
		mFileList.SetDirectory(fn);
		mFileList.Reload();
		if (fn.empty() || fn[fn.size() - 1] != '/') mEdit.SetText(mEdit.GetText() + '/');
	} else OnOK();
}

/* virtual */ void CFileOpenDialog::OnOK()
{
	mReturn = OK;
	mShutDown = true;
}

/* virtual */ bool CFileOpenDialog::OnUpdate(unsigned msgid) const
{
	bool ret = false;
	switch (msgid) {
	case IDLEVELUP:
		ret = !mFileList.GetPath(-1u).empty() && mFileList.GetPath(-1u) != "/";
		break;
	default:
		break;
	}
	const_cast<CPushButton&>(mOK).Enable(access(mEdit.GetText().c_str(), R_OK) == 0);
	return ret;
}

/* virtual */ void CFileOpenDialog::OnKey(wchar_t key, unsigned mask)
{
	CTopLevelWindow::OnKey(key, mask);
	if (mFocusWindow == &mEdit || mFocusWindow == &mFileList) {
		if (key == L'\n') OnOK();
	}
}	

/* virtual */ void CFileOpenDialog::OnQuit()
{
	mReturn = CANCEL;
	mShutDown = true;
}
