/*----------------------------------------------------------------------------
  Project: cxxx <http://cxplorer.sourceforge.net>
  File   : CFileOpenDialog.h
  Info   : $Id: CFileOpenDialog.h,v 1.13 2012/07/17 12:16:42 gkraeuter Exp $
----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
  Copyright © 2003 - 2012 Gerhard Kräuter <gkraeuter@users.sourceforge.net>

  This  software  is  provided  'as-is',  without  any  express  or  implied
  warranty.  In  no  event  will  the authors be held liable for any damages
  arising from the use of this software.

  Permission  is  granted  to  anyone  to use this software for any purpose,
  including  commercial  applications,  and  to alter it and redistribute it
  freely, subject to the following restrictions:

  1. The  origin  of  this software must not be misrepresented; you must not
     claim  that  you  wrote the original software. If you use this software
     in  a  product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.

----------------------------------------------------------------------------*/

#ifndef CFileOpenDialog_h
#define CFileOpenDialog_h

// C++X headers
#include <cxxx/CTopLevelWindow.h>
#include <cxxx/CFileListControl.h>
#include <cxxx/CToolButton.h>
#include <cxxx/CPushButton.h>
#include <cxxx/CEdit.h>
#include <cxxx/TMagic.h>

class CFileOpenDialog : public CTopLevelWindow
{
public:
	enum
	{
		// Return values
		OK,
		CANCEL
	};

private:
	enum
	{
		// Message IDs
		IDOK = cxxx::msgid::MSGID_CFileOpenDialog,
		IDCANCEL,
		IDNEXT,
		IDFILELIST,
		IDLEVELUP
	};
public:
	// Constructor
	CFileOpenDialog(CWindow* parent, const std::string& dir, const std::vector<unsigned>& filter);

	// C++X message processing
	virtual void OnMessage(unsigned msgid, TMessage type, unsigned value);
	virtual bool OnUpdate(unsigned msgid) const;

	// X event processing
	virtual void OnKey(wchar_t key, unsigned mask);

	// Exit functions
	virtual void OnQuit();
	virtual void OnOK();

	// Access function
	std::string GetPath() const
	{
		return mEdit.GetText();
	}
 
private:
	// C++X message processing
	void OnSelectionConfirmed();

private:
	CFileListControl mFileList;
	CToolButton mLevelUp;
	CPushButton mOK;
	CPushButton mCancel;
	CEdit mEdit;
};


#endif // CFileOpenDialog_h
