/*----------------------------------------------------------------------------
  Project: cxxx <http://cxplorer.sourceforge.net>
  File   : CFileTree.h
  Info   : $Id: CFileTree.h,v 1.35 2012/08/06 17:36:40 gkraeuter Exp $
----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
  Copyright © 2002 - 2012 Gerhard Kräuter <gkraeuter@users.sourceforge.net>

  This  software  is  provided  'as-is',  without  any  express  or  implied
  warranty.  In  no  event  will  the authors be held liable for any damages
  arising from the use of this software.

  Permission  is  granted  to  anyone  to use this software for any purpose,
  including  commercial  applications,  and  to alter it and redistribute it
  freely, subject to the following restrictions:

  1. The  origin  of  this software must not be misrepresented; you must not
     claim  that  you  wrote the original software. If you use this software
     in  a  product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.

----------------------------------------------------------------------------*/

#ifndef CFileTree_h
#define CFileTree_h

// C++X headers
#include <cxxx/CTreeControl.h>

struct CMountTableEntry
{
	std::string mDevice;
	std::string mMountPoint;
	bool mValid;
};

typedef std::vector<CMountTableEntry> CMountTable;

class CFileTree : public CTreeControl
{
private:
	enum
	{
		// Message IDs
		IDFILEWATCH = cxxx::msgid::MSGID_CFileTree
	};

public:
	enum
	{
		// return values of GetStatus
		NOSUBDIR,
		SUBDIRS,
		ISLOCKED
	};

public:
	// Constructor
	CFileTree(CWindow *parent, unsigned msgid);
		
	// C++X message processing
	virtual void OnMessage(unsigned msgid, TMessage type, unsigned value);

	// Opens tree item with index idx
	virtual void Open(unsigned idx);

	// Makes dir the current directory and openes it
	void SetDirectory(const std::string& dir);

	// Sets the sort critereon to key
	void SetSortKey(unsigned key)
	{
		mFlags = key;
		mFileTime = 0;
		Reload();
	}

	// Reloads the tree
	void Reload();

	// Returns the absolute path of the item with index idx
	std::string GetPath(unsigned idx)
	{
		if (idx < mList.size()) return GetPath(GetItem(idx));
		cxAssert(!mList.empty());
		return mTree.GetText();
	}

	// Inserts an item into the tree
	virtual void Append(CTreeItem& parent, CIcon::TIconName closed, CIcon::TIconName opened, const std::string& text);

protected:
	// Draws item with index idx
	virtual void DrawItem(unsigned idx);

	// Returns the absolute path of item
	std::string GetPath(const CTreeItem& item) const
	{
		std::string ret;
		if (!item.IsTree()) {
			ret = GetPath(item.GetParent());
			if (ret[ret.size() - 1] != '/') ret += '/';
		}
		ret += item.GetText();
		return ret;
	}

	// Reads directory item
	void ReadDir(CTreeItem& item);
		
	// Updates directory item
	bool UpdateDir(CTreeItem& item, bool update);

	// Returns whether dir has subdirectories
	unsigned GetStatus(const std::string& dir);

	// Comparisons
	static bool CompNameA(const CTreeItem& i1, const CTreeItem& i2)
	{
		return cxxx::cxLocale(i1.GetText(), i2.GetText());
	}

	static bool CompNameD(const CTreeItem& i1, const CTreeItem& i2)
	{
		return CompNameA(i2, i1);
	}

	// Sorts the directory item
	void Sort(CTreeItem& item)
	{
		if (mFlags & cxxx::filesort::ASCENDING) std::stable_sort(item.GetChildren().begin(), item.GetChildren().end(), CompNameA);
		else std::stable_sort(item.GetChildren().begin(), item.GetChildren().end(), CompNameD);
	}
		
	bool UpdateMountPoints();
		
protected:
	time_t mFileTime;
	CTimer mFileWatch;
	unsigned mFlags;
	unsigned mIconSize;
	CIcon mLockOverlay;
	CIcon mLinkOverlay;
};

#endif // CFileTree_h
