/*----------------------------------------------------------------------------
  Project: cxxx <http://cxplorer.sourceforge.net>
  File   : CFontSheet.h
  Info   : $Id: CFontSheet.h,v 1.19 2012/08/06 17:36:40 gkraeuter Exp $
----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
  Copyright © 2005 - 2012 Gerhard Kräuter <gkraeuter@users.sourceforge.net>

  This  software  is  provided  'as-is',  without  any  express  or  implied
  warranty.  In  no  event  will  the authors be held liable for any damages
  arising from the use of this software.

  Permission  is  granted  to  anyone  to use this software for any purpose,
  including  commercial  applications,  and  to alter it and redistribute it
  freely, subject to the following restrictions:

  1. The  origin  of  this software must not be misrepresented; you must not
     claim  that  you  wrote the original software. If you use this software
     in  a  product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.

----------------------------------------------------------------------------*/

#ifndef CFontSheet_h
#define CFontSheet_h

// C++X headers
#include <cxxx/CPushButton.h>
#include <cxxx/CDropDownList.h>
#include <cxxx/CLabel.h>

// C++ headers
#include <map>

typedef std::map<std::string, std::map<std::string, unsigned> > CFontList;

struct CFontStyleNames
{
	const char* mOriginal;
	int mTranslation;
};

class CFontSheet : public CWindow
{
private:
	enum
	{
		// Message ids
		IDTEXTFAMILY = cxxx::msgid::MSGID_CFontSheet,
		IDTEXTSTYLE,
		IDTEXTSIZE,
		IDTITLE,
		IDTITLEFAMILY,
		IDTITLESTYLE,
		IDTITLESIZE,

		// Array sizes
		WEIGHTCOUNT = 8,
		SLANTCOUNT = 2,
		WIDTHCOUNT = 8
	};

public:
	// Constructor
	CFontSheet(CWindow* parent);
	
	// X event processing
	virtual void OnExpose(XExposeEvent& e);
	virtual void OnConfigure(XConfigureEvent& e);

	// C++X message processing
	virtual void OnMessage(unsigned msgid, TMessage type, unsigned value);

	// Prepares window to close down
	bool DoOK();

protected:
	// Composes a xlfd, title switches between standard and title font
	std::string ComposeXlfd(bool title)
	{
		std::string fnt;
		if (title) {
			fnt = mTitleFontFamily.GetText();
			fnt += '-';
			fnt += DeLocalize(mTitleFontStyle.GetText());
			fnt += '-';
			fnt += mTitleFontSize.GetText();
		} else {
			fnt = mTextFontFamily.GetText();
			fnt += '-';
			fnt += DeLocalize(mTextFontStyle.GetText());
			fnt += '-';
			fnt += mTextFontSize.GetText();
		}
		return fnt;
	}

	// Fills m into d and selects preset
	void FillStyle(CDropDownList& d, const std::map<std::string, unsigned>& m, const std::string& preset);

	// Fills s into d and selects preset
	// Each bit in s represents a size between 6 and 36
	void FillSize(CDropDownList& d, unsigned s, unsigned preset);

	// Lists the fonts into mFontList
	void CreateFontList();

	// Returns the translation of s
	std::string Localize(const std::string& s);

	// Returns the original string of the translation s
	std::string DeLocalize(const std::string& s);

protected:
	CFontList mFontList;
	CDropDownList mTextFontFamily;
	CDropDownList mTitleFontFamily;
	CDropDownList mTextFontStyle;
	CDropDownList mTitleFontStyle;
	CDropDownList mTextFontSize;
	CDropDownList mTitleFontSize;
	CLabel mTextFontPreview;
	CLabel mTitleFontPreview;
	CFont mTextFont;
	CFont mTitleFont;
	static const CFontStyleNames mWeightNames[WEIGHTCOUNT];
	static const CFontStyleNames mSlantNames[SLANTCOUNT];
	static const CFontStyleNames mWidthNames[WIDTHCOUNT];
};

#endif // CFontSheet_h
