/*----------------------------------------------------------------------------
  Project: cxxx <http://cxplorer.sourceforge.net>
  File   : CHSplitterWindow.h
  Info   : $Id: CHSplitterWindow.h,v 1.8 2012/08/06 17:36:40 gkraeuter Exp $
----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
  Copyright © 2002 - 2012 Gerhard Kräuter <gkraeuter@users.sourceforge.net>

  This  software  is  provided  'as-is',  without  any  express  or  implied
  warranty.  In  no  event  will  the authors be held liable for any damages
  arising from the use of this software.

  Permission  is  granted  to  anyone  to use this software for any purpose,
  including  commercial  applications,  and  to alter it and redistribute it
  freely, subject to the following restrictions:

  1. The  origin  of  this software must not be misrepresented; you must not
     claim  that  you  wrote the original software. If you use this software
     in  a  product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.

----------------------------------------------------------------------------*/

#ifndef CHSplitterWindow_h
#define CHSplitterWindow_h

// C++X headers
#include <cxxx/CWindow.h>

class CHSplitterWindow : public CWindow
{
public:
	// Constructor
	CHSplitterWindow(CWindow* parent, unsigned msgid);

	// Destructor
	virtual ~CHSplitterWindow()
	{
		XFreeCursor(cxxx::cxDpy, mCursor);
	}
	
	// X event processing
	virtual void OnLButtonDown(XButtonEvent& e);
	virtual void OnButtonUp(XButtonEvent& e);
	virtual void OnEvent(XEvent& e);
	void OnMotion(XMotionEvent& e)
	{
		if (mMin >= mMax) return;
		int x = e.x - mXOffset;
		int y = e.y;
		TranslateCoordinates(x, y, GetParent());
		if (x < mMin) x = mMin;
		if (x > mMax) x = mMax;
		PostMessage(MT_MOTION, x);
	}

    // Sets the range
	void SetRange(int min, int max)
    {
        mMin = min;
        mMax = max;
        if (mMin < 0) mMin = 0;
        if (mMax < 0) mMax = 0;
        if (max > (int)GetParent()->GetWidth()) max = GetParent()->GetWidth();
    }

protected:
	Cursor mCursor;
	int mXOffset;
	int mMin;
	int mMax;
};

#endif // CHSplitterWindow_h
