/*----------------------------------------------------------------------------
  Project: cxxx <http://cxplorer.sourceforge.net>
  File   : CIconv.cpp
  Info   : $Id: CIconv.cpp,v 1.13 2009/12/24 17:08:21 gkraeuter Exp $
----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
  Copyright © 2007 - 2009 Gerhard Kräuter <gkraeuter@users.sourceforge.net>

  This  software  is  provided  'as-is',  without  any  express  or  implied
  warranty.  In  no  event  will  the authors be held liable for any damages
  arising from the use of this software.

  Permission  is  granted  to  anyone  to use this software for any purpose,
  including  commercial  applications,  and  to alter it and redistribute it
  freely, subject to the following restrictions:

  1. The  origin  of  this software must not be misrepresented; you must not
     claim  that  you  wrote the original software. If you use this software
     in  a  product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.

----------------------------------------------------------------------------*/

// C++X headers
#include <cxxx/CxxxFeatures.h>
#include <cxxx/CIconv.h>

// C++ headers
#include <map>

// namespace declarations
using std::string;
using std::map;

string CIconv::Convert(const string& in, unsigned conv)
{
	if (conv >= mConverterTable.size()) return in;
	iconv_t ic = mConverterTable[conv].mConverter;		
	string ret;
	size_t inleft = in.size();
	size_t outleft = CHAR_BUFFER_SIZE;
	
	cxxx::iconvstring_t inbegin = const_cast<cxxx::iconvstring_t>(in.c_str());
	char* outbegin = mBuffer;
	bool done = false;
	bool error = false;
	while (!done && !error) {
		error = iconv(ic, &inbegin, &inleft, &outbegin, &outleft) == (size_t)(-1);
		if (!error) done = true;
		else if (errno == E2BIG) {
			ret.append(mBuffer, CHAR_BUFFER_SIZE - outleft );
			outleft = CHAR_BUFFER_SIZE;
			outbegin = mBuffer;
			error = false;
		} else if (errno == EILSEQ) {
			ret.append(mBuffer, CHAR_BUFFER_SIZE - outleft);
			outleft = CHAR_BUFFER_SIZE;
			outbegin = mBuffer;
			ret.append("?");
			++inbegin;
			--inleft;
			error = false;
		}
	}
	if (error) {
		inleft = 0;
		// reset state
		iconv(ic, NULL, &inleft, NULL, &inleft);
	} else {
		ret.append(mBuffer, CHAR_BUFFER_SIZE - outleft);
	}
	return ret;
}
