/*----------------------------------------------------------------------------
  Project: cxxx <http://cxplorer.sourceforge.net>
  File   : CIconv.h
  Info   : $Id: CIconv.h,v 1.16 2012/08/06 17:40:54 gkraeuter Exp $
----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
  Copyright © 2007 - 2012 Gerhard Kräuter <gkraeuter@users.sourceforge.net>

  This  software  is  provided  'as-is',  without  any  express  or  implied
  warranty.  In  no  event  will  the authors be held liable for any damages
  arising from the use of this software.

  Permission  is  granted  to  anyone  to use this software for any purpose,
  including  commercial  applications,  and  to alter it and redistribute it
  freely, subject to the following restrictions:

  1. The  origin  of  this software must not be misrepresented; you must not
     claim  that  you  wrote the original software. If you use this software
     in  a  product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.

----------------------------------------------------------------------------*/

#ifndef CIconv_h
#define CIconv_h

// C++ headers
#include <string>
#include <vector>
#include <cerrno>

// C headers
#include <iconv.h>

class CIconv
{
	struct TConverter
	{
		std::string mFrom;
		std::string mTo;
		iconv_t mConverter;
	};

	typedef std::vector<TConverter> TConverterTable;
	
	enum
	{
		// Buffer size
		CHAR_BUFFER_SIZE = 256
	};

public:
	// Constructor
	CIconv()
	{
		*mBuffer = 0;
	}

	// Destructor
	~CIconv()
	{
		for (TConverterTable::iterator i = mConverterTable.begin(); i != mConverterTable.end(); ++i) {
			iconv_close(i->mConverter);
		}
	}

	// Returns an index to a converter
	unsigned GetConverter(const std::string& from, const std::string& to)
	{
		std::string ftmp = ToLower(from);
		std::string ttmp = ToLower(to);
		if (ftmp == ttmp) return -1u;
		for (unsigned i = 0; i < mConverterTable.size(); ++i) {
			if (mConverterTable[i].mFrom == ftmp && mConverterTable[i].mTo == ttmp) return i;
		}
		if (CreateConverter(ftmp, ttmp)) return mConverterTable.size() - 1;
		return -1u;
	}

	// Does the conversion
	std::string Convert(const std::string& in, unsigned conv);

private:
	// Creates a converter
	bool CreateConverter(const std::string& from, const std::string& to)
	{
		TConverter c;
		c.mFrom = from;
		c.mTo = to;
		c.mConverter = iconv_open(c.mTo.c_str(), c.mFrom.c_str());
		if (c.mConverter != (iconv_t)(-1)) {
			mConverterTable.push_back(c);
			return true;
		}
		return false;
	}

	// Returns the lower case converted string of in
	std::string ToLower(const std::string& in)
	{
		std::string ret;
		const char* beg = in.c_str();
		const char* end = beg + in.size();
		for (const char* i = beg; i < end; ++i) ret += tolower(*i);
		return ret;
	}

private:
	TConverterTable mConverterTable;
	char mBuffer[CHAR_BUFFER_SIZE];
};

#endif
