/*----------------------------------------------------------------------------
  Project: cxxx <http://cxplorer.sourceforge.net>
  File   : CListControl.cpp
  Info   : $Id: CListControl.cpp,v 1.37 2012/08/06 17:36:40 gkraeuter Exp $
----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
  Copyright © 2005 - 2012 Gerhard Kräuter <gkraeuter@users.sourceforge.net>

  This  software  is  provided  'as-is',  without  any  express  or  implied
  warranty.  In  no  event  will  the authors be held liable for any damages
  arising from the use of this software.

  Permission  is  granted  to  anyone  to use this software for any purpose,
  including  commercial  applications,  and  to alter it and redistribute it
  freely, subject to the following restrictions:

  1. The  origin  of  this software must not be misrepresented; you must not
     claim  that  you  wrote the original software. If you use this software
     in  a  product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.

----------------------------------------------------------------------------*/

// C++X headers
#include <cxxx/CListControl.h>

// namespace declarations
using cxxx::cxLocale;
using cxxx::cxWiden;
using std::string;
using std::wstring;
using std::vector;
using std::max;

/* virtual */ void CListControl::OnExpose(XExposeEvent& e)
{
	if (!mList.empty()) {
		int x = e.x;
		int y = e.y;
		ScreenToWindow(x, y);
		unsigned cl = GetColumn(x);
		unsigned rl = GetRow(y);
		if (cl != -1u && rl != -1u) {
			x += e.width;
			y += e.height;
			unsigned ch = GetColumn(x);
			unsigned rh = GetRow(y);
			if (rh == -1u) rh = mRow.size() - 1;
			if (ch == -1u) ch = mColumn.size() - 1;
			unsigned idx;
			if (mStyle == MULTI_COLUMN_VIEW) {
				for (unsigned c = cl; c <= ch; c++) {
					for (unsigned r = rl; r <= rh; r++) {
						idx = c * mRow.size() + r;
						if (idx >= mList.size()) break;
						DrawItem(idx);
					}
				}
			} else {
				for (unsigned c = cl; c <= ch; c++) {
					for (unsigned r = rl; r <= rh; r++) {
						idx = r * mColumn.size() + c;
						if (idx >= mList.size()) break;
						DrawItem(idx);
					}
				}
			}
		}
	}
	// restore the Frame
	DrawFrame();
}

/* virtual */ void CListControl::OnLButtonDown(XButtonEvent& e)
{
	if (IsDragging()) CancelDrag();
	unsigned idx = ItemFromPos(e.x, e.y);
	if (idx >= mList.size()) {
   		DeselectAll();
   		PostMessage(MT_SELECTION_CHANGED);
		StopDrag(e);
		mLastSel = -1u;
		return ;
   	}
	if (mMultipleSelection) {
		if ((e.state & ControlMask) != 0) ToggleSelection(idx);
		else if((e.state & ShiftMask) != 0) Select(idx, mLastSel);
		else if (!mList[idx].IsSelected()) {
			DeselectAll();
			Select(idx);
		}
		PostMessage(MT_SELECTION_CHANGED);
	} else if (!mList[idx].IsSelected()) {
		DeselectAll();
		Select(idx);
		PostMessage(MT_SELECTION_CHANGED);
	}
	StartDrag(e);
	mLastSel = idx;
}

/* virtual */ void CListControl::OnRButtonDown(XButtonEvent& e)
{
	if (IsDragging()) CancelDrag();
	unsigned idx = ItemFromPos(e.x, e.y);
	if (idx >= mList.size()) {
   		DeselectAll();
   		PostMessage(MT_SELECTION_CHANGED);
	StopDrag(e);
	mLastSel = -1u;
	return;
   	}
	if (!mList[idx].IsSelected()) {
		DeselectAll();
		Select(idx);
	}
	PostMessage(MT_SELECTION_CHANGED);
	StartDrag(e);
	mLastSel = idx;
}

/* virtual */ void CListControl::OnKey(wchar_t key, unsigned state)
{
	if (isalnum(key) && !mList.empty()) {
		unsigned idx = mList.size() - 1;
		if (!mSelection.empty()) idx = mSelection.front();
		unsigned i = idx;
		int tmp = tolower(key);
		if (++i == mList.size()) i = 0;
		while (i != idx) {
			if (!mList[i].GetText().empty() && tmp == tolower(mList[i].GetText()[0])) break;
			if (++i == mList.size()) i = 0;
		}
		if (i != idx) {
			DeselectAll();
			Select(i);
			PostMessage(MT_SELECTION_CHANGED);
		}
	} else CControl::OnKey(key, state);
}
	
/* virtual */ void CListControl::CalculateDocSize()
{
	mColumn.clear();
	mRow.clear();
	if (mStyle == MULTI_COLUMN_VIEW) {
		for (unsigned r = mIconHeight; r < mDocHeight; r += mIconHeight) mRow.push_back(r);
		if (mList.empty()) return;
		unsigned colmax = 1 + (mList.size() - 1) / mRow.size();
		unsigned w;
		unsigned tmp;
		unsigned beg;
		unsigned end;
		for (unsigned c = 0; c < colmax; ++c) {
			w = MIN_ITEM_WIDTH;
			beg = c * mRow.size();
			end = beg + mRow.size();
			if (end > mList.size()) end = mList.size();
			for(unsigned idx = beg; idx < end; ++idx) {
				tmp = GetItem(idx).GetWidth();
				if (w < tmp) w = tmp;
			}
			w += INTER_SPACE;
			if (!mColumn.empty()) w += mColumn.back();
			mColumn.push_back(w);
		}
		mDocWidth = mColumn.back();
	} else {
		unsigned colwidth = ICON_ITEM_WIDTH + 2 * INTER_SPACE;
		if (mDocWidth < 3 * colwidth) mDocWidth = 3 * colwidth;
		for (unsigned c = colwidth; c <= mDocWidth; c += colwidth) mColumn.push_back(c);
		if (mList.empty()) return;
		unsigned rowmax = 1 + (mList.size() - 1) / mColumn.size();
		unsigned h;
		unsigned tmp;
		unsigned beg;
		unsigned end;
		for (unsigned r = 0; r < rowmax; ++r) {
			h = 0;
			beg = r * mColumn.size();
			end = beg + mColumn.size();
			if (end > mList.size()) end = mList.size();
			for(unsigned idx = beg; idx < end; ++idx) {
				tmp = GetItem(idx).GetHeight();
				if (h < tmp) h = tmp;
			}
			h += INTER_SPACE;
			if (!mRow.empty()) h += mRow.back();
			mRow.push_back(h);
		}
		mDocWidth = mColumn.back();
		mDocHeight = mRow.back();
	}
}

/* virtual */ void CListControl::DrawItem(unsigned idx)
{
	cxAssert(idx < mList.size());
	CListItem item = GetItem(idx);
	unsigned row;
	unsigned col;
	// translate index to row and column
	if (mStyle == MULTI_COLUMN_VIEW) {
		col = idx / mRow.size();
		row = idx % mRow.size();
	} else {
		col = idx % mColumn.size();
		row = idx / mColumn.size();
	}
	int x;
	int y;
	if (col) x = mColumn[col - 1];
	else x = 0;
	if (row) y = mRow[row - 1];
	else y = 0;
	unsigned w = mColumn[col] - x;
	unsigned h = mRow[row] - y;
	WindowToScreen(x, y);
	ClearArea(x, y, w, h);
	int xi, yi;
	unsigned wi, hi;
	// Multi column view
	if (mStyle == MULTI_COLUMN_VIEW) {
		xi = x + ICON_BORDER + (mIconWidth - item.GetIcon().GetWidth()) / 2;
		yi = y + (h - item.GetIcon().GetHeight()) / 2;
		if (item.GetIcon().IsValid()) {
			DrawIcon(xi, yi, item.GetIcon());
		}
		xi = x + mIconWidth;
		yi = y + (h - GetTextHeight()) / 2 -TEXT_BORDER;
		wi = item.GetWidth() - mIconWidth;
		hi = GetTextHeight() + 2 * TEXT_BORDER;
		CColor col;
		if (item.IsSelected()) {
			if (IsFocused()) {
				FillRectangle(xi, yi, wi, hi, CColor(TC_TITLE));
				col = CColor(TC_TITLE_TEXT);
			} else {
				col = CColor(TC_BACKGND_TEXT);
				DrawDottedRectangle(xi, yi, wi, hi, col);
			}
		} else 	col = CColor(TC_BACKGND_TEXT);
		xi += TEXT_BORDER;
		yi += TEXT_BORDER;
		if (!item.GetText().empty()) DrawString(xi, yi, col, item.GetText(), !item.IsSelected());
	} else {
		// Icon view
		xi = x + (w - item.GetIcon().GetWidth()) / 2;
		yi = y + ICON_BORDER + (mIconHeight - item.GetIcon().GetHeight()) / 2;
		if (item.GetIcon().IsValid()) DrawIcon(xi, yi, item.GetIcon());
		yi = y + mIconHeight;
		hi = GetTextHeight() + 2 * TEXT_BORDER;
		unsigned pos = 0;
		unsigned line = 0;
		CTextSizeHint hint = item.GetTextHint(0);
		wstring wtxt = cxWiden(item.GetText());
		while (hint.mLength != 0) {
			xi = x + (w - hint.mWidth) / 2;
			wi = hint.mWidth;
			CColor col;
			if (item.IsSelected()) {
				if (IsFocused()) {
					FillRectangle(xi, yi, wi, hi, CColor(TC_TITLE));
					col = CColor(TC_TITLE_TEXT);
				} else {
					col = CColor(TC_BACKGND_TEXT);
					DrawDottedRectangle(xi, yi, wi, hi, col);
				}
			} else col = CColor(TC_BACKGND_TEXT);
			xi += TEXT_BORDER;
			yi += TEXT_BORDER;
			DrawString(xi, yi, col, wtxt.substr(pos, hint.mLength), !item.IsSelected());
			yi += GetTextHeight() + TEXT_BORDER;
			pos += hint.mLength;
			hint = item.GetTextHint(++line);
		}
	}
	// restore the border
	if ((x < BORDER_WIDTH) || (x + w + BORDER_WIDTH >= GetWidth()) || (y + h + BORDER_WIDTH >= GetHeight())) {
		DrawFrame();
	}		
}

void CListControl::GetBounds(int& x, int& y, unsigned& w, unsigned& h)
{
    ScreenToWindow(x, y);
	unsigned row = GetRow(y);
	unsigned col = GetColumn(x);
	if (row == -1u) {
		x = 0;
		y = mRow.back();
		w = mDocWidth + mViewWidth;
		h = mDocHeight + mViewHeight;
	} else if (col == -1u) {
		x = mColumn.back();
		y = 0;
		w = mDocWidth + mViewWidth;
		h = mDocHeight + mViewHeight;
	} else {
		int xsave(x);
		int ysave(y);
		unsigned idx;
		if (mStyle == MULTI_COLUMN_VIEW) idx = row + mRow.size() * col;
		else idx = col + mColumn.size() * row;
		if (col == 0) x = 0;
		else x = mColumn[col - 1];
		w = mColumn[col] - x;
		if (row == 0) y = 0; 
		else y = mRow[row - 1];
		h = mRow[row] - y;
		if (idx < mList.size()) {
			if (mStyle == MULTI_COLUMN_VIEW) {
				if (xsave - x > (int)GetItem(idx).GetWidth()) {
					x += GetItem(idx).GetWidth();
					w -= GetItem(idx).GetWidth();
				} else {
					w = GetItem(idx).GetWidth();
				}
			} else {
				// Icon view
				if (ysave - y < (int)mIconHeight) { 
					int xl = x + (w - mIconWidth) / 2;
					int xh = x + (w + mIconWidth) / 2;				
					if (xsave < xl) {
						w = xl - x;
						h = mIconHeight;
					} else if (xsave > xh) {
						w = w - xh + x;
						x = xh;
						h = mIconHeight;
					} else {
						x = xl;
						w = xh - xl;
						h = mIconHeight;
					}
				} else {
					unsigned line = (ysave - y - mIconHeight) / (GetTextHeight() + 2 * TEXT_BORDER);
					int xl = x + (w - GetItem(idx).GetTextHint(line).mWidth) / 2;
					int xh = x + (w + GetItem(idx).GetTextHint(line).mWidth) / 2;
					if (xsave < xl) {
						y += mIconHeight + (GetTextHeight() + 2 * TEXT_BORDER) * line;
						w = xl - x;
					} else if (xsave > xh) {
						w = w - xh + x;
						x = xh;
						y += mIconHeight + (GetTextHeight() + 2 * TEXT_BORDER) * line;
					} else {
						x = xl;
						y += mIconHeight + (GetTextHeight() + 2 * TEXT_BORDER) * line;
						w = xh - xl;
					}
					if (mIconHeight + (GetTextHeight() + 2 * TEXT_BORDER) * (line + 1) > h) h -= mIconHeight + (GetTextHeight() + 2 * TEXT_BORDER) * line;
					else h = GetTextHeight() + 2 * TEXT_BORDER;
				}
			}
		}
	}
   	WindowToScreen(x, y);
	if (x < BORDER_WIDTH) {
		w += x - BORDER_WIDTH;
		x = BORDER_WIDTH;
	}
	if (y < BORDER_WIDTH) {
		h += y - BORDER_WIDTH;
		y = BORDER_WIDTH;
	}
	if ((x + w) > mViewWidth + BORDER_WIDTH) w = mViewWidth + BORDER_WIDTH - x;
	if ((y + h) > mViewHeight + BORDER_WIDTH) h = mViewHeight + BORDER_WIDTH - y;
}


unsigned CListControl::ItemFromPos(int x, int y)
{
	ScreenToWindow(x, y);
	unsigned row = GetRow(y);
	if (row >= mRow.size()) return -1u;
	unsigned col = GetColumn(x);
	if (col >= mColumn.size()) return -1u;
	unsigned idx;	
	if (mStyle == MULTI_COLUMN_VIEW) idx = row + mRow.size() * col;
	else idx = col + mColumn.size() * row; 
	if (idx >= mList.size()) return -1u;
	// inside the item's square, see if it's hit
	if (mStyle == MULTI_COLUMN_VIEW) {
		int xh;
		if (col) xh = mColumn[--col];
		else xh = 0;
		xh += GetItem(idx).GetWidth();
		if(x > xh) return -1u;
		else return idx;
	} else {
		int xh = mColumn[col];
		int xl;
		if (col) xl = mColumn[--col];
		else xl = 0;
		int w = xh - xl;
		int y0;
		if (row) y0 = mRow[--row];
		else y0 = 0;
		if (y - y0 > (int)GetItem(idx).GetHeight()) return -1u;
		y0 = y - y0 - mIconHeight;
		if (y0 < 0) {
			xh = xl + (w + mIconWidth) / 2;
			xl += (w - mIconWidth) / 2;				
		} else {
			y0 /= GetTextHeight() + 2 * TEXT_BORDER;
			xh = xl + (w + GetItem(idx).GetTextHint(y0).mWidth) / 2;
			xl += (w - GetItem(idx).GetTextHint(y0).mWidth) / 2;
		}
		if (x < xl || x > xh) return -1u;
		else return idx;
	}
}

/* virtual */ void CListControl::ScrollIn(unsigned idx)
{
	unsigned col;
	unsigned row;
	// translate index to row and column
	if (mStyle == MULTI_COLUMN_VIEW) {
		col = idx / mRow.size();
		row = idx % mRow.size();
	} else {
		col = idx % mColumn.size();
		row = idx / mColumn.size();
	}
	int xi;
	if (col) xi = mColumn[col - 1];
	else xi = 0;
	xi += GetItem(idx).GetWidth();
	int yi;
	if (row) yi = mRow[row - 1];
	else yi = 0;
	yi += GetItem(idx).GetHeight();
	int xw = BORDER_WIDTH + mViewWidth;
	int yw = BORDER_WIDTH + mViewHeight;
	int scrlx = 0;
	int scrly = 0;
	WindowToScreen(xi, yi);
	if (yi > yw) scrly = (yi - yw + SCROLL_STEP - 1) / SCROLL_STEP;
	yi -= GetItem(idx).GetHeight();
	yw -= mViewHeight;
	if (yi < yw) scrly += (yi - yw - SCROLL_STEP + 1) / SCROLL_STEP;
	if (xi > xw) scrlx = (xi - xw + SCROLL_STEP - 1) / SCROLL_STEP;
	xi -= GetItem(idx).GetWidth();
	xw -= mViewWidth;
	if (xi < xw) scrlx += (xi - xw - SCROLL_STEP + 1) / SCROLL_STEP;
	if (scrlx != 0) mHScrollBar.SetPosition(mHScrollBar.GetPosition() + scrlx);
	if (scrly != 0) mVScrollBar.SetPosition(mVScrollBar.GetPosition() + scrly);
}
