/*----------------------------------------------------------------------------
  Project: cxxx <http://cxplorer.sourceforge.net>
  File   : CListControl.h
  Info   : $Id: CListControl.h,v 1.30 2012/08/06 17:36:40 gkraeuter Exp $
----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
  Copyright © 2003 - 2012 Gerhard Kräuter <gkraeuter@users.sourceforge.net>

  This  software  is  provided  'as-is',  without  any  express  or  implied
  warranty.  In  no  event  will  the authors be held liable for any damages
  arising from the use of this software.

  Permission  is  granted  to  anyone  to use this software for any purpose,
  including  commercial  applications,  and  to alter it and redistribute it
  freely, subject to the following restrictions:

  1. The  origin  of  this software must not be misrepresented; you must not
     claim  that  you  wrote the original software. If you use this software
     in  a  product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.

----------------------------------------------------------------------------*/

#ifndef CListControl_h
#define CListControl_h

// C++X headers
#include <cxxx/CListItem.h>
#include <cxxx/CControl.h>

class CListControl : public CControl
{
protected:
	enum
	{
		// Geometry
		TEXT_BORDER = 2,
		MIN_ITEM_WIDTH = 40,
		INTER_SPACE = 6,
		ICON_ITEM_WIDTH = 80
	};
public:
	
	enum
	{
		// Style
		MULTI_COLUMN_VIEW = 0,
		ICON_VIEW
	};

public:
	// Constructor
	CListControl(CWindow* parent, unsigned msgid)
	: CControl(parent, msgid),
	  mStyle(MULTI_COLUMN_VIEW)
	{
		mIconWidth = ICON_WIDTH;
		mIconHeight = ICON_HEIGHT;
	}

	// X event processing
	virtual void OnExpose(XExposeEvent& e);
	virtual void OnLButtonDown(XButtonEvent& e);
	virtual void OnRButtonDown(XButtonEvent& e);
	virtual void OnKey(wchar_t key, unsigned state);

	// Sets the style of the list control
	virtual void SetStyle(unsigned style, unsigned wico, unsigned hico)
	{
		mStyle = style;
		SetIconSize(wico, hico);
	}

	// Returns the item with index idx
	CListItem& GetItem(unsigned idx)
	{
		cxAssert(idx < mList.size());
		return (CListItem&)mList[idx];
	}

protected:
	// Calculate width and height of the document 
	virtual void CalculateDocSize();

	// Draws item with index idx
	virtual void DrawItem(unsigned idx);

	// Brings item with index idx into view
	virtual void ScrollIn(unsigned idx);

	// Returns the index of screen position x, y
	unsigned ItemFromPos(int x, int y);

	// Returns the column index if screen position x
	unsigned GetColumn(int x)
	{
		if (x <= 0) return 0;
		std::vector<unsigned>::iterator i = std::lower_bound(mColumn.begin(), mColumn.end(), (unsigned) x);
		if (i == mColumn.end()) return -1u;
		return i - mColumn.begin();
	}

	// Returns the row index of screen position y
	unsigned GetRow(int y)
	{
		if (y <= 0) return 0;
		std::vector<unsigned>::iterator i = std::lower_bound(mRow.begin(), mRow.end(), (unsigned) y);
		if (i == mRow.end()) return -1u;
		return i - mRow.begin();
	}

	// Returns a rectangle around x, y
	void GetBounds(int& x, int& y, unsigned& w, unsigned& h);

protected:
	unsigned mStyle;
};

#endif // CListControl_h
