/*----------------------------------------------------------------------------
  Project: cxxx <http://cxplorer.sourceforge.net>
  File   : CMagic.cpp
  Info   : $Id: CMagic.cpp,v 1.83 2012/09/16 17:49:40 gkraeuter Exp $
----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
  Copyright © 2002 - 2012 Gerhard Kräuter <gkraeuter@users.sourceforge.net>

  This  software  is  provided  'as-is',  without  any  express  or  implied
  warranty.  In  no  event  will  the authors be held liable for any damages
  arising from the use of this software.

  Permission  is  granted  to  anyone  to use this software for any purpose,
  including  commercial  applications,  and  to alter it and redistribute it
  freely, subject to the following restrictions:

  1. The  origin  of  this software must not be misrepresented; you must not
     claim  that  you  wrote the original software. If you use this software
     in  a  product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.

----------------------------------------------------------------------------*/

// C++X headers
#include <cxxx/CApplication.h>
#include <cxxx/CError.h>
#include <cxxx/CMagic.h>
#include <cxxx/CZStreambuf.h>

// C++ headers
#include <fstream>
#include <cctype>
#include <cerrno>

// Namespace declarations
using std::string;
using std::streampos;
using std::filebuf;
using std::lower_bound;
using std::find;
using std::ios;
using cxxx::cxError;
using cxxx::cxFileManager;
using cxxx::lib::cxgettext;

const CMagic::CMagInfo CMagic::mMagInfo[TMagic::MAGINFOCOUNT] = 
{
	/* ABIWORD    */ {CIcon::ABIWORD, "application/x-abiword", cxxx::noop::_("AbiWord Document", 4)},
	/* ADASRC     */ {CIcon::TXT, "text/x-adasrc", cxxx::noop::_("Ada sources", 368)},
	/* AIFFAU     */ {CIcon::SOUND, "audio/aiff", cxxx::noop::_("AIFF audio", 371)},
	/* AIFFCAU    */ {CIcon::SOUND, "audio/aifc", cxxx::noop::_("AIFC audio", 369)},
	/* AMIPRO     */ {CIcon::DOCUMENT, "application/x-amipro", cxxx::noop::_("Lotus AmiPro document", 370)},
	/* APPLIX     */ {CIcon::APPLIX, "application/x-applix-unknown", cxxx::noop::_("Unknown Applix document", 354)},
	/* ARARCH     */ {CIcon::ARCHIVE, "application/x-arc", cxxx::noop::_("Ar archive", 355)},
	/* ARJARCH    */ {CIcon::ARCHIVE, "application/x-arj", cxxx::noop::_("ARJ archive", 356)},
	/* ASCII      */ {CIcon::TXT, "text/plain-ascii", cxxx::noop::_("ASCII encoded text", 357)},
	/* ASP        */ {CIcon::HTML, "application/x-asp", cxxx::noop::_("Active server page", 358)},
	/* ASHSCT     */ {CIcon::SHELLSCRIPT, "application/x-ash", cxxx::noop::_("Ash script", 359)},
	/* AUTHORS    */ {CIcon::INFO, "text/x-authors", cxxx::noop::_("Authors list", 360)},
	/* AUTOCAD    */ {CIcon::VECTORGFX, "image/vnd.dwg", cxxx::noop::_("AutoCAD image", 361)},
	/* AWKSCT     */ {CIcon::SHELLSCRIPT, "application/x-awk", cxxx::noop::_("AWK script", 362)},
	/* AXSPREAD   */ {CIcon::APPLIX, "application/x-applix-spreadsheet", cxxx::noop::_("Applix Spreadsheets document", 363)},
	/* AXGRAPH    */ {CIcon::APPLIX, "image/x-applix-graphics", cxxx::noop::_("Applix Graphics document", 364)},
	/* AXWORD     */ {CIcon::APPLIX, "application/x-applix-word", cxxx::noop::_("Applix Words document", 365)},
	/* BACKUP     */ {CIcon::RECYCLED, "application/x-trash", cxxx::noop::_("Backup file", 366)},
	/* BASHSCT    */ {CIcon::SHELLSCRIPT, "application/x-sh", cxxx::noop::_("Bourne again shell script", 367)},
	/* BCPIO      */ {CIcon::DOCUMENT, "application/x-bcpio", cxxx::noop::_("BCPIO document", 103)},
	/* BDFFONT    */ {CIcon::FONT, "application/x-font-sunos-news", cxxx::noop::_("SunOS News font", 104)},
	/* BERKELEYDB */ {CIcon::DATABASE, "application/x-dbm", cxxx::noop::_("Berkeley database", 105)},
	/* BINHEX     */ {CIcon::BINARY, "application/mac-binhex40", cxxx::noop::_("Macintosh BinHex-encoded file", 106)},
	/* BITTORRENT */ {CIcon::DOCUMENT, "application/x-bittorrent", cxxx::noop::_("BitTorrent file", 107)},
	/* BLOCKDEV   */ {CIcon::BLOCKDEVICE, "inode/blockdevice", cxxx::noop::_("Block device", 108)},
	/* BZIP2ED    */ {CIcon::COMPRESS, "application/x-bzip", cxxx::noop::_("Bzip file", 109)},
	/* CGIPRG     */ {CIcon::BINARY, "application/x-cgi", cxxx::noop::_("CGI program", 110)},
	/* CERT509    */ {CIcon::HTML, "application/x-x509-ca-cert", cxxx::noop::_("X.509 certificate", 111)},
	/* CHARDEV    */ {CIcon::CHARDEVICE, "inode/chardevice", cxxx::noop::_("Character device", 112)},
	/* CONFIG     */ {CIcon::CONFIG, "text/x-config", cxxx::noop::_("Configuration file", 113)},
	/* COPYING    */ {CIcon::FILE_IMPORTANT, "text/x-copying", cxxx::noop::_("Software license terms", 114)},
	/* CPIOARCH   */ {CIcon::ARCHIVE, "application/x-cpio", cxxx::noop::_("CPIO archive", 115)},
	/* CPPHDR     */ {CIcon::SOURCE_H, "text/x-c++hdr", cxxx::noop::_("C++ header file", 116)},
	/* CPPSRC     */ {CIcon::SOURCE_CPP, "text/x-c++src", cxxx::noop::_("C++ source file", 117)},
	/* CREAMUSIC  */ {CIcon::SOUND, "audio/x-creative-music", cxxx::noop::_("Creative music file", 118)},
	/* CREAVOICE  */ {CIcon::SOUND, "audio/x-creative-voice", cxxx::noop::_("Creative voice file", 119)},
	/* CREDITS    */ {CIcon::TXT, "text/x-credits", cxxx::noop::_("Software author credits", 120)},
	/* CRUNCHED   */ {CIcon::COMPRESS, "application/octet-stream", cxxx::noop::_("Crunched file", 121)},
	/* CSEPTEXT   */ {CIcon::TXT, "text/x-comma-separated-values", cxxx::noop::_("Text file with comma separated values", 122)},
	/* CSHSCT     */ {CIcon::SHELLSCRIPT, "application/x-csh", cxxx::noop::_("C-shell script", 123)},
	/* CSRC       */ {CIcon::SOURCE_C, "text/x-csrc", cxxx::noop::_("C source file", 124)},
	/* CSSHEET    */ {CIcon::SOURCE_CSS, "text/css", cxxx::noop::_("Cascading Style Sheet", 125)},
	/* DECAU      */ {CIcon::SOUND, "audio/x-dec-basic", cxxx::noop::_("DEC audio", 126)},
	/* DBASE      */ {CIcon::DATABASE, "application/x-dbase", cxxx::noop::_("dBASE document", 127)},
	/* DEBIAN     */ {CIcon::DEB, "application/x-deb", cxxx::noop::_("Debian package", 128)},
	/* DESKTOP    */ {CIcon::DESKTOP, "application/x-desktop", cxxx::noop::_("Desktop configuration file", 129)},
	/* DIFFOUT    */ {CIcon::PATCH, "text/x-diff", cxxx::noop::_("Differences between files", 130)},
	/* DIRECTORY  */ {CIcon::FOLDER, "inode/directory", cxxx::noop::_("Directory", 131)},
	/* DOCTDEF    */ {CIcon::TXT, "text/x-dtd", cxxx::noop::_("Document type definition", 132)},
	/* DOLBYDA    */ {CIcon::SOUND, "audio/ac3", cxxx::noop::_("Dolby Digital audio", 133)},
	/* DOSEXE     */ {CIcon::MSDOS, "application/x-executable-dos", cxxx::noop::_("DOS executable", 134)},
	/* DOSFONT    */ {CIcon::FONT_BITMAP, "application/x-font-dos", cxxx::noop::_("DOS font", 135)},
	/* DVI        */ {CIcon::DVI, "application/x-dvi", cxxx::noop::_("TeX Device Independent file", 136)},
	/* DXFGRAPH   */ {CIcon::VECTORGFX, "image/vnd.dxf", cxxx::noop::_("DXF vector graphic", 137)},
	/* ELFCORE    */ {CIcon::CORE, "application/x-core-elf", cxxx::noop::_("Program crash data", 138)},
	/* ELFEXE     */ {CIcon::EXEC, "application/x-executable-elf", cxxx::noop::_("Elf executable", 139)},
	/* ELFOBJ     */ {CIcon::SOURCE_O, "application/x-relocatable-elf", cxxx::noop::_("Elf relocatable", 140)},
	/* ELFSOBJ    */ {CIcon::LIB, "application/x-shared-library-elf", cxxx::noop::_("Elf shared library", 141)},
	/* ELISP      */ {CIcon::SOURCE_L, "text/x-emacs-lisp", cxxx::noop::_("Emacs Lisp source code", 142)},
	/* EMPTY      */ {CIcon::EMPTY, "application/x-zerosize", cxxx::noop::_("Empty Document", 143)},
	/* EPOSTSCR   */ {CIcon::POSTSCRIPT, "image/x-djvu", cxxx::noop::_("Encapsulated PostScript image", 144)},
	/* ETHEME     */ {CIcon::IMAGE, "application/x-e-theme", cxxx::noop::_("Enlightenment theme", 145)},
	/* EXTMOD     */ {CIcon::SOUND, "audio/mod", cxxx::noop::_("Extended MOD file", 146)},
	/* FGFIMG     */ {CIcon::IMAGE, "image/x-fgf", cxxx::noop::_("FGF image", 147)},
	/* FMAKFONT   */ {CIcon::FONT, "application/x-font-framemaker", cxxx::noop::_("Adobe FrameMaker font", 148)},
	/* FLASH      */ {CIcon::VIDEO, "application/x-flv", cxxx::noop::_("Adobe Flash Video", 479)},
	/* FONT       */ {CIcon::FONT, "application/x-font", cxxx::noop::_("Font", 149)},
	/* FONTMETR   */ {CIcon::FONT_BITMAP, "application/x-font-afm", cxxx::noop::_("Adobe font metrics", 150)},
	/* FORTRAN    */ {CIcon::SOURCE_F, "text/x-fortran", cxxx::noop::_("Fortran source code", 151)},
	/* FRAMEMAK   */ {CIcon::DOCUMENT, "application/x-frame", cxxx::noop::_("FrameMaker document", 152)},
	/* FUZZYBMP   */ {CIcon::IMAGE, "image/fuzzy-bitmap", cxxx::noop::_("Fuzzy bitmap", 153)},
	/* G3FAX      */ {CIcon::IMAGE, "image/g3fax", cxxx::noop::_("Group 3 fax", 154)},
	/* GAWKSCT    */ {CIcon::SHELLSCRIPT, "application/x-gawk", cxxx::noop::_("Gawk script", 155)},
	/* GDBM, */ {CIcon::DATABASE, "application/x-gdbm", cxxx::noop::_("Gnu database", 156)},
	/* GLADE      */ {CIcon::DOCUMENT, "application/x-glade", cxxx::noop::_("Glade project", 157)},
	/* CDDBTEXT   */ {CIcon::CDBO_LIST, "text/xmcd", cxxx::noop::_("CD database text", 158)},
	/* GENMIDI    */ {CIcon::MIDI, "audio/midi", cxxx::noop::_("Generic MIDI file", 159)},
	/* GETTEXT    */ {CIcon::GETTEXT, "text/x-gettext-translation", cxxx::noop::_("Translated messages", 160)},
	/* GETTEXTT   */ {CIcon::GETTEXT, "text/x-gettext-translation-template", cxxx::noop::_("Message translation template", 161)},
	/* GIF        */ {CIcon::IMAGE, "image/gif", cxxx::noop::_("GIF image", 162)},
	/* GIF1       */ {CIcon::IMAGE, "image/x-gif94", cxxx::noop::_("GIF94 image", 163)},
	/* GIMPBRUSH  */ {CIcon::IMAGE, "application/x-gimp.brush", cxxx::noop::_("Gimp brush data", 164)},
	/* GIMPGRAD   */ {CIcon::IMAGE, "application/x-gimp.gradient", cxxx::noop::_("Gimp gradient data", 165)},
	/* GIMPPAL    */ {CIcon::IMAGE, "application/x-gimp.palette", cxxx::noop::_("Gimp palette data", 166)},
	/* GIMPPAT    */ {CIcon::IMAGE, "application/x-gimp.pattern", cxxx::noop::_("Gimp pattern data", 167)},
	/* GNUCASH    */ {CIcon::SPREADSHEET, "application/x-gnucash", cxxx::noop::_("GnuCash Workbook", 168)},
	/* GNUMERIC   */ {CIcon::SPREADSHEET, "application/x-gnumeric", cxxx::noop::_("GNUmeric spreadsheet", 169)},
	/* GNUMOCAT   */ {CIcon::GETTEXT, "application/x-gettext-translation", cxxx::noop::_("Translated messages", 160)},
	/* GNUTAR     */ {CIcon::TAR, "application/x-tar-gnu", cxxx::noop::_("GNU tape archive", 170)},
	/* GRXFONT    */ {CIcon::FONT, "application/x-font-libgrx", cxxx::noop::_("LIBGRX font", 171)},
	/* GTKRC      */ {CIcon::CONFIG, "text/x-gtkrc", cxxx::noop::_("GTK configuration", 172)},
	/* GTKTALOG   */ {CIcon::DOCUMENT, "application/x-gtktalog", cxxx::noop::_("Gtktalog Catalogue", 173)},
	/* GZIPED     */ {CIcon::COMPRESS, "application/x-gzip", cxxx::noop::_("Gzip file", 174)},
	/* HEADER     */ {CIcon::SOURCE_H, "text/x-chdr", cxxx::noop::_("Header file", 175)},
	/* HUFFMAN    */ {CIcon::COMPRESS, "application/octet-stream", cxxx::noop::_("Huffman compressed file", 176)},
	/* HTML       */ {CIcon::HTML, "text/html", cxxx::noop::_("HTML page", 177)},
	/* HSIJPEG    */ {CIcon::IMAGE, "image/jpeg-hsi", cxxx::noop::_("JPEG image (HSI)", 178)},
	/* IFF2D      */ {CIcon::IMAGE, "image/iff-2d", cxxx::noop::_("IFF 2D object", 179)},
	/* IFF3D      */ {CIcon::IMAGE, "image/iff-3d", cxxx::noop::_("IFF 3D rendering", 180)},
	/* IFFIMAG    */ {CIcon::IMAGE, "image/x-iff", cxxx::noop::_("IFF RGB image", 181)},
	/* IFFTXT     */ {CIcon::TXT, "text/x-iff", cxxx::noop::_("IFF text", 182)},
	/* ILEAVEIM   */ {CIcon::IMAGE, "image/x-ilbm", cxxx::noop::_("Interleaved image", 183)},
	/* ILLUSTR    */ {CIcon::KPRESENTER, "application/illustrator", cxxx::noop::_("Adobe Illustrator Document", 184)},
	/* INSTALL    */ {CIcon::INSTALL, "text/x-install", cxxx::noop::_("software installation instructions", 185)},
	/* INTLIZE    */ {CIcon::GETTEXT, "application/x-intlize", cxxx::noop::_("Translated messages", 160)},
	/* JAVAARCH   */ {CIcon::JAVA_JAR, "application/x-jar", cxxx::noop::_("Java arcive", 186)},
	/* JAVACLAS   */ {CIcon::SOURCE_JAVA, "application/x-java", cxxx::noop::_("Java Class", 187)},
	/* JAVASRC    */ {CIcon::SOURCE_JAVA, "text/x-java", cxxx::noop::_("Java sources", 188)},
	/* JBUILDER   */ {CIcon::DOCUMENT, "application/x-jbuilder-project", cxxx::noop::_("JBuilder Project", 189)},
	/* JNG        */ {CIcon::VIDEO, "video/jng", cxxx::noop::_("JNG video", 190)},
	/* JPEG       */ {CIcon::IMAGE, "image/jpeg", cxxx::noop::_("JPEG image", 191)},
	/* KARBON     */ {CIcon::KARBON, "application/x-karbon", cxxx::noop::_("Karbon14 document", 192)},
	/* KCHART     */ {CIcon::KCHART, "application/x-kchart", cxxx::noop::_("KChart diagram", 193)},
	/* KDEDESK    */ {CIcon::DESKTOP, "application/x-desktop", cxxx::noop::_("Desktop entry file", 194)},
	/* KDEVELOP   */ {CIcon::CONFIG, "text/x-kdevelop", cxxx::noop::_("KDevelop project file", 195)},
	/* KFORMULA   */ {CIcon::KFORMULA, "application/x-kformula", cxxx::noop::_("KFormula formula", 196)},
	/* KILLUST    */ {CIcon::KOFFICE, "application/x-killustrator", cxxx::noop::_("KIllustrator document", 197)},
	/* KIVIO      */ {CIcon::KIVIO, "application/x-kivio", cxxx::noop::_("Kivio file", 198)},
	/* KONTOUR    */ {CIcon::KOFFICE, "application/x-kontour", cxxx::noop::_("Kontour drawing", 199)},
	/* KPOVRAY    */ {CIcon::KPOVMODELER, "application/x-kpovmodeler", cxxx::noop::_("KPovModeler file", 200)},
	/* KPRESENT   */ {CIcon::KPRESENTER, "application/x-kpresenter", cxxx::noop::_("KPresenter presentation", 201)},
	/* KRITA      */ {CIcon::KRITA, "application/x-krita",cxxx::noop::_("Krita document", 202)},
	/* KSHSCT     */ {CIcon::SHELLSCRIPT, "application/x-ksh", cxxx::noop::_("Korn shell script", 203)},
	/* KSPREAD    */ {CIcon::KSPREAD, "application/x-kspread", cxxx::noop::_("KSpread document", 204)},
	/* KUGAR      */ {CIcon::KUGARDATA, "application/x-ksysv-package", cxxx::noop::_("Kugar Data File", 205)},
	/* KWORD      */ {CIcon::KWORD, "application/x-kword", cxxx::noop::_("KWord document", 206)},
	/* LHAARCH    */ {CIcon::ARCHIVE, "application/x-lha", cxxx::noop::_("LHA archive", 207)},
	/* LOCKDIR    */ {CIcon::FOLDER, "inode/directory", cxxx::noop::_("Locked directory", 208)},
	/* LOCKED     */ {CIcon::FILE_LOCKED, "unknown/locked", cxxx::noop::_("Locked file", 209)},
	/* LOGFILE    */ {CIcon::LOG, "text/x-log", cxxx::noop::_("Application log file", 210)},
	/* LOTUS123   */ {CIcon::SPREADSHEET, "application/x-123", cxxx::noop::_("Lotus 1-2-3 document", 211)},
	/* LSM        */ {CIcon::LINUX, "text/x-lsm", cxxx::noop::_("Linux software map", 212)},
	/* LXKERN     */ {CIcon::LINUX, "application/linux-kernel", cxxx::noop::_("Linux kernel", 213)},
	/* LYX        */ {CIcon::LYX, "application/x-lyx", cxxx::noop::_("LyX Document", 214)},
	/* LZHCOMPR   */ {CIcon::COMPRESS, "application/x-lzh", cxxx::noop::_("LZH compressed file", 215)}, 
	/* MAGICKIM   */ {CIcon::IMAGE, "image/x-image-magick", cxxx::noop::_("Image Magick file", 216)},
	/* MAKEFILE   */ {CIcon::MAKE, "text/x-makefile", cxxx::noop::_("Makefile", 217)},
	/* MANPAGE    */ {CIcon::MAN, "application/x-troff-man", cxxx::noop::_("Manual", 218)},
	/* MATHML     */ {CIcon::KFORMULA, "text/mathml", cxxx::noop::_("MathML document", 219)},
	/* MIME       */ {CIcon::HTML, "text/mime", cxxx::noop::_("MIME encoded file", 220)},
	/* MNG        */ {CIcon::VIDEO, "video/mng", cxxx::noop::_("MNG video", 221)},
	/* MPEG1      */ {CIcon::SOUND, "audio/x-mp1", cxxx::noop::_("MPEG layer 1 audio", 441)},
	/* MPEG2      */ {CIcon::SOUND, "audio/x-mp2", cxxx::noop::_("MPEG layer 2 audio", 406)},
	/* MPEG3      */ {CIcon::SOUND, "audio/x-mp3", cxxx::noop::_("MPEG layer 3 audio", 222)},
	/* MPEGS      */ {CIcon::VIDEO, "video/mpeg-system", cxxx::noop::_("MPEG system file", 223)},
	/* MPEGV      */ {CIcon::VIDEO, "video/mpeg", cxxx::noop::_("MPEG video", 224)},
	/* MSASF      */ {CIcon::VIDEO, "video/x-ms-asf", cxxx::noop::_("Microsoft streaming video", 374)},
	/* MSBMP      */ {CIcon::IMAGE, "image/bmp", cxxx::noop::_("Windows BMP image", 225)},
	/* MSEXCEL    */ {CIcon::SPREADSHEET, "application/vnd.ms-excel", cxxx::noop::_("Microsoft Excel spreadsheet", 226)},
	/* MSGNEWS    */ {CIcon::MESSAGE, "message/news", cxxx::noop::_("News text", 227)},
	/* MSICO      */ {CIcon::IMAGE, "image/x-ico", cxxx::noop::_("MS icon", 228)},
	/* MSPPT      */ {CIcon::KPRESENTER, "application/vnd.ms-powerpoint", cxxx::noop::_("Microsoft PowerPoint document", 229)},
	/* MSWORD     */ {CIcon::WORDPROCESSING, "application/msword", cxxx::noop::_("Microsoft Word document", 230)},
	/* MSACCESS   */ {CIcon::DATABASE, "application/vnd.ms-access", cxxx::noop::_("Microsoft Access Document", 231)},
	/* NAUTLINK   */ {CIcon::DESKTOP, "application/x-nautilus-link", cxxx::noop::_("Nautilus link", 232)},
	/* NAWKSCT    */ {CIcon::SHELLSCRIPT, "application/x-nawk", cxxx::noop::_("Nawk script", 233)},
	/* NIFF       */ {CIcon::IMAGE, "image/x-niff", cxxx::noop::_("Navy interchange file", 234)},
	/* OBJCSRC    */ {CIcon::SOURCE_C, "text/x-objcsrc", cxxx::noop::_("Objective-C sources", 235)},
	/* OPENCALC   */ {CIcon::OOO_CALC, "application/vnd.sun.xml.calc", cxxx::noop::_("OpenOffice spreadsheet", 236)},
	/* OPENDRAT   */ {CIcon::OOO_DRAW_TPL, "application/vnd.sun.xml.draw-template", cxxx::noop::_("OpenOffice drawing template", 237)},
	/* OPENDRAW   */ {CIcon::OOO_DRAW, "application/vnd.sun.xml.draw", cxxx::noop::_("OpenOffice drawing", 238)},
	/* OPENIMPR   */ {CIcon::OOO_IMPRESS, "application/vnd.sun.xml.impress", cxxx::noop::_("OpenOffice Impress presentation", 239)},
	/* OPENIMPT   */ {CIcon::OOO_IMPRESS_TPL, "application/vnd.sun.xml.impress-template", cxxx::noop::_("OpenOffice Impress presentation template", 240)},
	/* OPENMATH   */ {CIcon::KFORMULA, "application/vnd.sun.xml.math", cxxx::noop::_("OpenOffice Math document", 241)},
	/* OPENTMPL   */ {CIcon::OOO_CALC_TPL, "application/vnd.sun.xml.calc-template", cxxx::noop::_("OpenOffice spreadsheet template", 242)},
	/* OPENWRIG   */ {CIcon::OOO_WRITER_TPL, "application/vnd.sun.xml.writer-global", cxxx::noop::_("OpenOffice Writer global document", 243)},
	/* OPENWRIT   */ {CIcon::OOO_WRITER, "application/vnd.sun.xml.writer", cxxx::noop::_("OpenOffice Writer document", 244)},
	/* OPENWRTT   */ {CIcon::OOO_WRITER_TPL, "application/vnd.sun.xml.writer-template", cxxx::noop::_("OpenOffice Writer template", 245)},
	/* PAM        */ {CIcon::IMAGE, "image/x-portable-pixmap-pam", cxxx::noop::_("Portable arbitrary map", 246)},
	/* CPIO       */ {CIcon::ARCHIVE, "application/x-cpio", cxxx::noop::_("Cpio archive", 247)},
	/* PASCAL     */ {CIcon::SOURCE_P, "text/x-pascal", cxxx::noop::_("Pascal sources", 248)},
	/* PBFIMAGE   */ {CIcon::IMAGE, "image/x-pbf", cxxx::noop::_("PBF image", 249)},
	/* PBM        */ {CIcon::IMAGE, "image/x-portable-bitmap", cxxx::noop::_("Portable bitmap raw", 250)},
	/* PBMT       */ {CIcon::IMAGE, "image/x-portable-bitmap-ascii", cxxx::noop::_("Portable bitmap ASCII", 251)},
	/* PCFFONT    */ {CIcon::FONT_BITMAP, "application/x-font-pcf", cxxx::noop::_("PCF font", 252)},
	/* PDF        */ {CIcon::PDF, "application/pdf", cxxx::noop::_("PDF document", 253)},
	/* PERLMODULE */ {CIcon::SOURCE_PL, "text/perl", cxxx::noop::_("Perl 5 module source text", 254)},
	/* PERLSCT    */ {CIcon::SHELLSCRIPT, "application/x-perl", cxxx::noop::_("Perl program", 255)},
	/* PGM        */ {CIcon::IMAGE, "image/x-portable-greymap", cxxx::noop::_("Portable graymap raw", 256)},
	/* PGMT       */ {CIcon::IMAGE, "image/x-portable-greymap-ascii", cxxx::noop::_("Portable graymap ASCII", 257)},
	/* PGPMSG     */ {CIcon::DOCUMENT, "application/pgp", cxxx::noop::_("PGP message", 258)},
	/* PHOTOSHOP  */ {CIcon::IMAGE, "image/x-photoshop", cxxx::noop::_("Adobe Photoshop image", 259)},
	/* PHPSCT     */ {CIcon::SOURCE_PHP, "application/x-php", cxxx::noop::_("PHP script", 260)},
	/* PIPE       */ {CIcon::PIPE, "inode/fifo", cxxx::noop::_("Pipe", 261)},
	/* PKCS12     */ {CIcon::DOCUMENT, "application/x-pkcs12", cxxx::noop::_("PKCS#12 Certificate Bundle", 262)},
	/* PNG        */ {CIcon::IMAGE, "image/png", cxxx::noop::_("Portable network graphics", 263)},
	/* POSIXTAR   */ {CIcon::TAR, "application/x-tar-posix", cxxx::noop::_("POSIX tape archive", 264)},
	/* POSTSCR    */ {CIcon::POSTSCRIPT, "application/postscript", cxxx::noop::_("PostScript document", 265)},
	/* POSTSCTPD  */ {CIcon::POSTSCRIPT, "application/x-ppd", cxxx::noop::_("PostScript printer description", 266)},
	/* PPM        */ {CIcon::IMAGE, "image/x-portable-pixmap", cxxx::noop::_("Portable pixmap raw", 267)},
	/* PPMT       */ {CIcon::IMAGE, "image/x-portable-pixmap-ascii", cxxx::noop::_("Portable pixmap ASCII", 268)},
	/* PS1CFONT   */ {CIcon::FONT, "application/x-font-ps1", cxxx::noop::_("PS1 font", 269)},
	/* PS1TFONT   */ {CIcon::FONT, "application/x-font-ps1-source", cxxx::noop::_("PS1 font source", 270)},
	/* PSFFONT    */ {CIcon::FONT, "application/x-font-linux-psf", cxxx::noop::_("Linux PSF console font", 271)},
	/* PYTHON     */ {CIcon::SOURCE_PY, "application/x-python", cxxx::noop::_("Python program", 272)},
	/* PYTHONBC   */ {CIcon::SOURCE_PY, "application/x-python-bytecode", cxxx::noop::_("Python bytecode", 273)},
	/* QTDESIGN   */ {CIcon::WIDGET_DOC, "application/x-designer", cxxx::noop::_("Qt Designer File", 274)},
	/* QTMETA     */ {CIcon::SOURCE_MOC, "text/x-moc", cxxx::noop::_("Qt Meta Object File", 275)},
	/* QUICKTIME  */ {CIcon::QUICKTIME, "video/quicktime", cxxx::noop::_("Quicktime Video", 276)},
	/* RARARCH    */ {CIcon::ARCHIVE, "application/x-rar", cxxx::noop::_("Rar archive", 277)},
	/* RAWCD      */ {CIcon::CDIMAGE, "application/x-cd-image", cxxx::noop::_("Raw CD image", 278)},
	/* README     */ {CIcon::FILE_IMPORTANT, "text/plain", cxxx::noop::_("Package information", 372)},
	/* REALAU     */ {CIcon::REAL, "audio/x-rn-realaudio", cxxx::noop::_("RealAudio broadcast", 279)},
	/* REALMED    */ {CIcon::REAL, "video/x-rn-realaudio", cxxx::noop::_("RealMedia broadcast", 378)},
	/* REJPATCH   */ {CIcon::PATCH, "application/x-reject", cxxx::noop::_("rejected patch file", 280)},
	/* RFC822     */ {CIcon::MESSAGE, "message/rfc822", cxxx::noop::_("News or mail text", 281)},
	/* RICHTEXT   */ {CIcon::ABIWORD, "text/enriched", cxxx::noop::_("Rich text document", 282)},
	/* RIFFACUR   */ {CIcon::IMAGE, "image/x-ani", cxxx::noop::_("MS animated cursor", 283)},
	/* RIFFAVI    */ {CIcon::VIDEO, "video/x-msvideo", cxxx::noop::_("MS video", 284)},
	/* RIFFDIB    */ {CIcon::IMAGE, "image/x-dib", cxxx::noop::_("MS device independant bitmap", 285)},
	/* RIFFMIDI   */ {CIcon::MIDI, "audio/x-midi", cxxx::noop::_("MS MIDI file", 286)},
	/* RIFFMOV    */ {CIcon::VIDEO, "video/x-ms-wmv", cxxx::noop::_("MS movie", 287)},
	/* RIFFPAL    */ {CIcon::IMAGE, "application/x-palette", cxxx::noop::_("MS palette", 288)},
	/* RIFFWAVE   */ {CIcon::SOUND, "audio/x-wav", cxxx::noop::_("WAV audio", 289)},
	/* RPMARCH    */ {CIcon::RPM, "application/x-rpm", cxxx::noop::_("RPM package file", 290)},
	/* SAMPAU     */ {CIcon::SOUND, "audio/aiff-sampled-audio", cxxx::noop::_("AIFF sampled audio", 291)},
	/* SAMPVOX    */ {CIcon::SOUND, "audio/aiff-sampled-voice", cxxx::noop::_("AIFF sampled voice", 292)},
	/* SBINSTR    */ {CIcon::SOUND, "audio/unknown-sb-instrument", cxxx::noop::_("Sound Blaster instrument file", 293)},
	/* SGIMOV     */ {CIcon::VIDEO, "video/sgi", cxxx::noop::_("SGI video", 295)},
	/* SGML       */ {CIcon::HTML, "text/sgml", cxxx::noop::_("SGML page", 296)},
	/* SHLIBSCR   */ {CIcon::ARCHIVE, "application/x-shared-library-la", cxxx::noop::_("Shared library", 297)},
	/* SHOCKWAV   */ {CIcon::DOCUMENT, "application/x-shockwave-flash", cxxx::noop::_("Shockwave Flash Media", 298)},
	/* SHSCT      */ {CIcon::SHELLSCRIPT, "application/x-sh", cxxx::noop::_("Bourne shell script", 299)},
	/* SMP        */ {CIcon::IMAGE, "image/smp", cxxx::noop::_("Small pixmap", 300)},
	/* SOCKET     */ {CIcon::SOCKET, "inode/socket", cxxx::noop::_("Socket", 301)},
	/* SPDFONT    */ {CIcon::FONT, "application/x-font-speedo", cxxx::noop::_("Speedo font", 302)},
	/* SQL        */ {CIcon::DOCUMENT, "text/x-sql", cxxx::noop::_("SQL code", 303)},
	/* SQLITE2    */ {CIcon::DATABASE, "application/x-sqlite2", cxxx::noop::_("SQLite v2 database", 478)},
	/* SQLITE3    */ {CIcon::DATABASE, "application/x-sqlite3", cxxx::noop::_("SQLite v3 database", 477)},	
	/* SQUEEZED   */ {CIcon::COMPRESS, "application/octet-stream", cxxx::noop::_("Squeezed file", 304)},
	/* STARCALC   */ {CIcon::SOFFICE, "application/vnd.stardivision.calc", cxxx::noop::_("StarCalc spreadsheet", 305)},
	/* STARCHRT   */ {CIcon::SOFFICE, "application/vnd.stardivision.chart", cxxx::noop::_("StarChart chart", 306)},
	/* STARDRAW   */ {CIcon::SOFFICE, "application/vnd.stardivision.draw", cxxx::noop::_("StarDraw drawing", 307)},
	/* STARIMPR   */ {CIcon::SOFFICE, "application/vnd.stardivision.impress", cxxx::noop::_("StarImpress presentation", 308)},
	/* STARMAIL   */ {CIcon::SOFFICE, "application/vnd.stardivision.mail", cxxx::noop::_("StarMail file", 309)},
	/* STARMATH   */ {CIcon::SOFFICE, "application/vnd.stardivision.math", cxxx::noop::_("StarMath document", 310)},
	/* STARWRIT   */ {CIcon::SOFFICE, "application/vnd.stardivision.writer", cxxx::noop::_("StarWriter document", 311)},
	/* STRMP3     */ {CIcon::SOUND, "audio/x-mpegurl", cxxx::noop::_("Streaming MPEG layer 3 audio", 312)},
	/* STUFFIT    */ {CIcon::IMAGE, "application/x-stuffit", cxxx::noop::_("Macintosh StuffIt archive", 313)},
	/* SUNAU      */ {CIcon::SOUND, "audio/basic", cxxx::noop::_("ULAW (Sun) audio", 314)},
	/* SVGIMG     */ {CIcon::IMAGE, "image/svg", cxxx::noop::_("Scalable vector graphics", 315)},
	/* SVR4PACK   */ {CIcon::ARCHIVE, "application/x-svr4-package", cxxx::noop::_("SVR 4 package", 316)},
	/* SVRARCH    */ {CIcon::ARCHIVE, "application/x-archive-svr1", cxxx::noop::_("SVR1 archive", 317)},
	/* SYMLINK    */ {CIcon::LINK, "inode/symlink", cxxx::noop::_("Symbolic Link", 318)},
	/* TABTEXT    */ {CIcon::TXT, "text/tab-separated-values", cxxx::noop::_("tab-separated text document", 319)},
	/* TARGA      */ {CIcon::IMAGE, "image/x-tga", cxxx::noop::_("Targa image", 320)},
	/* TARDIR     */ {CIcon::FOLDER_TAR, "inode/directory", cxxx::noop::_("Directory", 131)},
	/* TCL        */ {CIcon::SOURCE, "text/x-tcl", cxxx::noop::_("Tcl file", 321)},
	/* TEX        */ {CIcon::TEX, "text/x-tex", cxxx::noop::_("TeX file", 322)},
	/* TEXFONT    */ {CIcon::FONT, "application/x-font-tex", cxxx::noop::_("TeX font", 323)},
	/* TEXFONTM   */ {CIcon::FONT, "application/x-font-tex-tfm", cxxx::noop::_("TeX font metrics", 324)},
	/* TEXINFO    */ {CIcon::INFO, "text/x-texinfo", cxxx::noop::_("TeXInfo document", 325)},
	/* TEXT       */ {CIcon::TXT, "text/plain", cxxx::noop::_("Plain text", 326)},
	/* TIFF       */ {CIcon::IMAGE, "image/tiff", cxxx::noop::_("Tagged image file", 327)},
	/* TROFF      */ {CIcon::MAN, "application/x-troff", cxxx::noop::_("Troff document", 328)},
	/* TSHSCT     */ {CIcon::SHELLSCRIPT, "application/x-tcsh", cxxx::noop::_("Tcsh script", 329)},
	/* TTFFONT    */ {CIcon::FONT_TRUETYPE, "application/x-font-ttf", cxxx::noop::_("Truetype Font", 330)},
	/* UNKARCH    */ {CIcon::ARCHIVE, "application/x-unknown-archive", cxxx::noop::_("Unknown archive", 331)},
	/* UNKELF     */ {CIcon::EXEC, "application/x-unknown-elf", cxxx::noop::_("Unknown elf file", 332)},
	/* UNKIFF     */ {CIcon::UNKNOWN, "unknown/iff", cxxx::noop::_("Unknown IFF file", 333)},
	/* UNKNOWN    */ {CIcon::UNKNOWN, "unknown", cxxx::noop::_("Unknown file", 334)},
	/* UNKRIFF    */ {CIcon::UNKNOWN, "unknown/x-riff", cxxx::noop::_("Unknown RIFF file", 335)},
	/* UNKTAR     */ {CIcon::TAR, "application/x-tar-unknown", cxxx::noop::_("Unknown tape archive", 336)},
	/* URLOC      */ {CIcon::HTML, "text/x-uri", cxxx::noop::_("the location of some resource", 337)},
	/* UTF8       */ {CIcon::TXT, "text/plain-utf8", cxxx::noop::_("UTF8 encoded text", 338)}, 
	/* UXCOMPR    */ {CIcon::COMPRESS, "application/x-compressed", cxxx::noop::_("UNIX compressed file", 339)},
	/* VFONT      */ {CIcon::FONT, "application/x-font-vfont", cxxx::noop::_("V font", 340)},
	/* VORBISAU   */ {CIcon::SOUND, "application/ogg", cxxx::noop::_("Ogg Vorbis audio", 341)},
	/* WINEXE     */ {CIcon::EXEC_WINE, "application/x-executable-windows", cxxx::noop::_("Windows executable", 342)},
	/* WORDPERF   */ {CIcon::WORDPROCESSING, "application/wordperfect", cxxx::noop::_("WordPerfect document", 343)},
	/* XCF        */ {CIcon::IMAGE, "image/x-xcf", cxxx::noop::_("GIMP native image format", 344)},
	/* XBM        */ {CIcon::IMAGE, "image/x-xbitmap", cxxx::noop::_("X BitMap image", 345)},
	/* XHTML      */ {CIcon::HTML, "application/xhtml+xml", cxxx::noop::_("XHTML Page", 346)},
	/* XML        */ {CIcon::HTML, "text/xml", cxxx::noop::_("eXtensible Markup Language document", 347)},
	/* XPM        */ {CIcon::IMAGE, "image/xpm", cxxx::noop::_("X pixmap", 348)},
	/* XSLT       */ {CIcon::HTML, "text/x-xslt", cxxx::noop::_("XSLT Stylesheet file", 349)},
	/* XWDUMP     */ {CIcon::IMAGE, "image/x-xwindowdump", cxxx::noop::_("X window image", 350)},
	/* ZIPARCH    */ {CIcon::ARCHIVE, "application/zip", cxxx::noop::_("PKZIP archive", 351)},
	/* ZOOARCH    */ {CIcon::ARCHIVE, "application/x-zoo", cxxx::noop::_("Zoo archive", 352)},
	/* ZSHSCT     */ {CIcon::SHELLSCRIPT, "application/x-zsh", cxxx::noop::_("Z-shell script", 353)},
	/* OPENTYPE   */ {CIcon::FONT_TRUETYPE, "font/opentype", cxxx::noop::_("OpenType Font", 449)},
	/* ODTEXT     */ {CIcon::OOO_WRITER, "application/vnd.oasis.opendocument.text", cxxx::noop::_("OpenDocument text document", 462)},
	/* ODTEXTT    */ {CIcon::OOO_WRITER_TPL, "application/vnd.oasis.opendocument.text-template", cxxx::noop::_("OpenDocument text template", 470)},
	/* ODSPREAD   */ {CIcon::OOO_CALC, "application/vnd.oasis.opendocument.spreadsheet", cxxx::noop::_("OpenDocument spreadsheet", 463)},
	/* ODSPREADT  */ {CIcon::OOO_CALC_TPL, "application/vnd.oasis.opendocument.spreadsheet-template", cxxx::noop::_("OpenDocument spreadsheet template", 469)},
	/* ODPRES     */ {CIcon::OOO_IMPRESS, "application/vnd.oasis.opendocument.presentation", cxxx::noop::_("OpenDocument presentation", 464)},
	/* ODPREST    */ {CIcon::OOO_IMPRESS_TPL, "application/vnd.oasis.opendocument.presentation-template", cxxx::noop::_("OpenDocument presentation template", 468)},
	/* ODGRAPH    */ {CIcon::OOO_DRAW, "application/vnd.oasis.opendocument.graphics", cxxx::noop::_("OpenDocument graphics", 461)},
	/* ODGRAPHT   */ {CIcon::OOO_DRAW_TPL, "application/vnd.oasis.opendocument.graphics-template", cxxx::noop::_("OpenDocument graphics template", 465)},
	/* ODCHART    */ {CIcon::KCHART, "application/vnd.oasis.opendocument.chart", cxxx::noop::_("OpenDocument chart", 2)},
	/* ODFORM     */ {CIcon::KFORMULA, "application/vnd.oasis.opendocument.formula", cxxx::noop::_("OpenDocument formula", 466)},
	/* ODIMAGE    */ {CIcon::IMAGE, "application/vnd.oasis.opendocument.image", cxxx::noop::_("OpenDocument image", 467)},
	/* ODGLOBAL   */ {CIcon::OOO_WRITER_TPL, "application/vnd.oasis.opendocument.text-master", cxxx::noop::_("OpenDocument global document", 472)},
	/* CLAMAVDB   */ {CIcon::DATABASE, "application/x-clamav-database", cxxx::noop::_("ClamAV database", 475)},
	/* CABINET    */ {CIcon::ARCHIVE, "application/vnd.ms-cab-compressed", cxxx::noop::_("MS Cabinet archive", 487)},
	/* WINHELP    */ {CIcon::MAN, "application/vnd.ms-htmlhelp", cxxx::noop::_("MS help file", 476)},
	/* TNEF	      */ {CIcon::MESSAGE, "application/vnd.ms-tnef", cxxx::noop::_("TNEF message", 490)},
	/* ZIP7	      */ {CIcon::COMPRESS, "application/x-7z-compressed", cxxx::noop::_("7-zip archive", 482)},
	/* XZIPED     */ {CIcon::COMPRESS, "application/x-xz", cxxx::noop::_("xz archive", 489)},
	/* DLL 	      */ {CIcon::LIB_WINE, "application/x-shared-library-windows", cxxx::noop::_("Windows shared library", 486)},
	/* XCUR       */ {CIcon::IMAGE, "image/x-xcursor", cxxx::noop::_("X cursor", 494)}
};

const CMagic::CGlob CMagic::mEnding[ENDINGSIZE] =
{
	{"ac3", TMagic::DOLBYDA},       {"adb", TMagic::ADASRC},      {"ads", TMagic::ADASRC},
	{"afm", TMagic::FONTMETR},      {"ai", TMagic::ILLUSTR},      {"al", TMagic::PERLMODULE},
	{"asc", TMagic::ASCII},         {"asp", TMagic::ASP},         {"bak", TMagic::BACKUP},
	{"bcpio", TMagic::BCPIO},       {"c", TMagic::CSRC},          {"c++", TMagic::CPPSRC},
	{"cc", TMagic::CPPSRC},         {"cer", TMagic::CERT509},     {"cert", TMagic::CERT509},
	{"cfg", TMagic::CONFIG},        {"cgi", TMagic::CGIPRG},      {"cls", TMagic::TEX},
	{"conf", TMagic::CONFIG},       {"config", TMagic::CONFIG},   {"cpp", TMagic::CPPSRC},
	{"crt", TMagic::CERT509},       {"css", TMagic::CSSHEET},     {"cssl", TMagic::CSSHEET},
	{"csv", TMagic::CSEPTEXT},      {"cxx", TMagic::CPPSRC},      {"dbf", TMagic::DBASE},
	{"der", TMagic::CERT509},       {"desktop", TMagic::DESKTOP}, {"dwg", TMagic::AUTOCAD},
	{"dxf", TMagic::DXFGRAPH},      {"etheme", TMagic::ETHEME},   {"f", TMagic::FORTRAN},
	{"glade", TMagic::GLADE},       {"gmo", TMagic::GNUMOCAT},    {"gnc", TMagic::GNUCASH},
	{"gnucash", TMagic::GNUCASH},   {"h", TMagic::HEADER},        {"h++", TMagic::CPPHDR},
	{"hh", TMagic::CPPHDR},         {"htm", TMagic::HTML},        {"html", TMagic::HTML},
	{"ico", TMagic::MSICO},         {"iso", TMagic::RAWCD},       {"java", TMagic::JAVASRC},
	{"jpr", TMagic::JBUILDER},      {"jpx", TMagic::JBUILDER},    {"kdelnk", TMagic::KDEDESK},
	{"kdevelop", TMagic::KDEVELOP}, {"kpm", TMagic::KPOVRAY},     {"kud", TMagic::KUGAR},
	{"la", TMagic::SHLIBSCR},       {"log", TMagic::LOGFILE},     {"ltx", TMagic::TEX},
	{"m", TMagic::OBJCSRC},         {"m3u", TMagic::STRMP3},      {"man", TMagic::MANPAGE},
	{"mda", TMagic::MSACCESS},      {"mdb", TMagic::MSACCESS},    {"mdc", TMagic::MSACCESS},
	{"mml", TMagic::MATHML},        {"moc", TMagic::QTMETA},      {"old", TMagic::BACKUP},
	{"p", TMagic::PASCAL},          {"p12", TMagic::PKCS12},      {"pas", TMagic::PASCAL},
	{"pem", TMagic::CERT509},       {"perl", TMagic::PERLSCT},    {"pfx", TMagic::PKCS12},
	{"pgp", TMagic::PGPMSG},        {"pl", TMagic::PERLSCT},      {"pnm", TMagic::PAM},
	{"pot", TMagic::MSPPT},         {"pps", TMagic::MSPPT},       {"ppt", TMagic::MSPPT},
	{"ppz", TMagic::MSPPT},         {"pyc", TMagic::PYTHONBC},    {"pyo", TMagic::PYTHONBC},
	{"rc", TMagic::CONFIG},         {"rej", TMagic::REJPATCH},    {"sam", TMagic::AMIPRO},
	{"sda", TMagic::STARDRAW},      {"sdc", TMagic::STARCALC},    {"sdd", TMagic::STARIMPR},
	{"sdp", TMagic::STARIMPR},      {"sds", TMagic::STARCHRT},    {"sdw", TMagic::STARWRIT},
	{"sgl", TMagic::STARWRIT},      {"sgm", TMagic::SGML},        {"sgml", TMagic::SGML},
	{"sik", TMagic::BACKUP},        {"smd", TMagic::STARMAIL},    {"smf", TMagic::STARMATH},
	{"sql", TMagic::SQL},           {"sty", TMagic::TEX},         {"sxg", TMagic::OPENWRIG},
	{"tcl", TMagic::TCL},           {"tex", TMagic::TEX},         {"texi", TMagic::TEXINFO},
	{"texinfo", TMagic::TEXINFO},   {"tk", TMagic::TCL},          {"tsv", TMagic::TABTEXT},
	{"txt", TMagic::TEXT},          {"ui", TMagic::QTDESIGN},     {"uri", TMagic::URLOC},
	{"url", TMagic::URLOC},         {"vor", TMagic::STARWRIT},    {"xac", TMagic::GNUCASH},
	{"xbm", TMagic::XBM},           {"xhtml", TMagic::XHTML},     {"xla", TMagic::MSEXCEL},
	{"xlc", TMagic::MSEXCEL},       {"xld", TMagic::MSEXCEL},     {"xll", TMagic::MSEXCEL},
	{"xlm", TMagic::MSEXCEL},       {"xls", TMagic::MSEXCEL},     {"xlt", TMagic::MSEXCEL},
	{"xlw", TMagic::MSEXCEL},       {"xsl", TMagic::XSLT},        {"xslt", TMagic::XSLT},
	{"xwd", TMagic::XWDUMP}
};

const CMagic::CGlob CMagic::mKOffice[KOFFICESIZE] =
{
	// KOffice
	{"karbon", TMagic::KARBON},        {"kchart", TMagic::KCHART}, {"kformula", TMagic::KFORMULA},		
	{"killustrator", TMagic::KILLUST}, {"kivio", TMagic::KIVIO},   {"kontour", TMagic::KONTOUR},
	{"kpresenter", TMagic::KPRESENT},  {"krita", TMagic::KRITA},   {"kspread", TMagic::KSPREAD},
	{"kword", TMagic::KWORD}
};

const CMagic::CGlob CMagic::mOpenDoc[OPENDOCSIZE] =
{
	// OpenDocument
	{"text-template", TMagic::ODTEXTT}, {"text-master", TMagic::ODGLOBAL},
	{"text", TMagic::ODTEXT},           {"spreadsheet-template", TMagic::ODSPREADT},
	{"spreadsheet", TMagic::ODSPREAD},  {"graphics-template", TMagic::ODGRAPHT},
	{"graphics", TMagic::ODGRAPH},      {"presentation-template", TMagic::ODPREST},
	{"presentation", TMagic::ODPRES},   {"chart", TMagic::ODCHART},
	{"formula", TMagic::ODFORM},        {"image", TMagic::ODIMAGE}
};

const CMagic::CGlob CMagic::mOpenOffice[OPENOFFICESIZE] =
{
	// OpenOffice
	{"calc", TMagic::OPENCALC}, {"draw", TMagic::OPENDRAW}, {"impress", TMagic::OPENIMPR},
	{"math", TMagic::OPENMATH}, {"writer", TMagic::OPENWRIT}
};

const CMagic::CGlob CMagic::mName[NAMESIZE] =
{
	{".gtkrc", TMagic::GTKRC},      {"AUTHORS", TMagic::AUTHORS},   {"COPYING", TMagic::COPYING},
	{"INSTALL", TMagic::INSTALL},   {"Makefile", TMagic::MAKEFILE}, {"README", TMagic::README},
	{"algorithm", TMagic::CPPHDR},  {"bitset", TMagic::CPPHDR},     {"cassert", TMagic::CPPHDR},
	{"cctype", TMagic::CPPHDR},     {"cerrno", TMagic::CPPHDR},     {"cfloat", TMagic::CPPHDR},	
	{"ciso646", TMagic::CPPHDR},    {"climits", TMagic::CPPHDR},    {"clocale", TMagic::CPPHDR},
	{"cmath", TMagic::CPPHDR},      {"complex", TMagic::CPPHDR},    {"csetjmp", TMagic::CPPHDR},
	{"csignal", TMagic::CPPHDR},    {"cstdarg", TMagic::CPPHDR},    {"cstddef", TMagic::CPPHDR},
	{"cstdio", TMagic::CPPHDR},     {"cstdlib", TMagic::CPPHDR},    {"cstring", TMagic::CPPHDR},
	{"ctime", TMagic::CPPHDR},      {"cwchar", TMagic::CPPHDR},     {"cwctype", TMagic::CPPHDR},
	{"deque", TMagic::CPPHDR},      {"exception", TMagic::CPPHDR},  {"fstream", TMagic::CPPHDR},
	{"functional", TMagic::CPPHDR}, {"gtkrc", TMagic::GTKRC},       {"iomanip", TMagic::CPPHDR},
	{"ios", TMagic::CPPHDR},        {"iosfwd", TMagic::CPPHDR},     {"iostream", TMagic::CPPHDR},
	{"istream", TMagic::CPPHDR},    {"iterator", TMagic::CPPHDR},   {"limits", TMagic::CPPHDR},
	{"list", TMagic::CPPHDR},       {"locale", TMagic::CPPHDR},     {"makefile", TMagic::MAKEFILE},
	{"map", TMagic::CPPHDR},        {"memory", TMagic::CPPHDR},     {"new", TMagic::CPPHDR},
	{"numeric", TMagic::CPPHDR},    {"ostream", TMagic::CPPHDR},    {"queue", TMagic::CPPHDR},
	{"set", TMagic::CPPHDR},        {"sstream", TMagic::CPPHDR},    {"stack", TMagic::CPPHDR},
	{"stdexcept", TMagic::CPPHDR},  {"streambuf", TMagic::CPPHDR},  {"string", TMagic::CPPHDR},
	{"typeinfo", TMagic::CPPHDR},   {"utility", TMagic::CPPHDR},    {"valarray", TMagic::CPPHDR},
	{"vector", TMagic::CPPHDR}
};

const CMagic::CGlob CMagic::mMagic0[MAGIC0SIZE] =
{
	{"\x04""\x00\x00\x01\xb3", TMagic::MPEGV},         {"\x04""\x00\x00\x01\xba", TMagic::MPEGS},
	{"\x04""\x00\x00\x02\x00\xf4", TMagic::LOTUS123},  {"\x04""\x00\x00\x1a\x00", TMagic::LOTUS123},
	{"\x05""\x00\x01\x00\x00\x00", TMagic::TTFFONT},   {"\x04""\x00\x05\x31\x62", TMagic::BERKELEYDB},
	{"\x04""\x00\x06\x15\x61", TMagic::BERKELEYDB},    {"\x04""\x01""fcp", TMagic::PCFFONT},
	{"\x03""\x04%!\xff", TMagic::POSTSCR},             {"\x02""\x0a(", TMagic::ELISP},
	{"\x06""\n<html", TMagic::HTML},                   {"\x08""\npackage", TMagic::PERLMODULE},
	{"\x07""\x0d\x0a<html", TMagic::HTML},             {"\x04""\x13\x57\x9a\xce", TMagic::GDBM},
	{"\x03""\x13\x7a\x29", TMagic::BDFFONT},           {"\x04""\x14\x02\x59\x19", TMagic::GRXFONT},
	{"\x02""\x1f\x1e", TMagic::HUFFMAN},               {"\x02""\x1f\x8b\xfe", TMagic::GZIPED},
	{"\x02""\x1f\x9d", TMagic::UXCOMPR},               {"\x07""!<arch>\xfc", TMagic::ARARCH},
	{"\x04""!=ar", TMagic::UNKARCH},                   {"\x05""!\\<ar", TMagic::SVRARCH},
	{"\x03""\"\"\"", TMagic::PYTHON},                  {"\x0d""# Config File", TMagic::DESKTOP},
	{"\x11""# KDE Config File", TMagic::KDEDESK},      {"\x14""# PaCkAgE DaTaStReAm", TMagic::SVR4PACK},
	{"\x06""# xmcd", TMagic::CDDBTEXT},                {"\x0b""#! /bin/ash", TMagic::ASHSCT},
	{"\x0b""#! /bin/awk", TMagic::AWKSCT},             {"\x0c""#! /bin/bash", TMagic::BASHSCT},
	{"\x0b""#! /bin/csh", TMagic::CSHSCT},             {"\x0c""#! /bin/gawk", TMagic::GAWKSCT},
	{"\x0b""#! /bin/ksh", TMagic::KSHSCT},             {"\x0c""#! /bin/nawk", TMagic::NAWKSCT},
	{"\x0c""#! /bin/perl", TMagic::PERLSCT},           {"\x0a""#! /bin/sh", TMagic::SHSCT},
	{"\x0c""#! /bin/tcsh", TMagic::TSHSCT},            {"\x0f""#! /usr/bin/awk", TMagic::AWKSCT},
	{"\x14""#! /usr/bin/env bash", TMagic::BASHSCT},   {"\x14""#! /usr/bin/env perl", TMagic::PERLSCT},
	{"\x16""#! /usr/bin/env python", TMagic::PYTHON},  {"\x10""#! /usr/bin/gawk", TMagic::GAWKSCT},
	{"\x10""#! /usr/bin/nawk", TMagic::NAWKSCT},       {"\x10""#! /usr/bin/perl", TMagic::PERLSCT},
	{"\x12""#! /usr/bin/python", TMagic::PYTHON},      {"\x15""#! /usr/local/bin/ash", TMagic::ASHSCT},
	{"\x16""#! /usr/local/bin/bash", TMagic::BASHSCT}, {"\x16""#! /usr/local/bin/gawk", TMagic::GAWKSCT},
	{"\x16""#! /usr/local/bin/nawk", TMagic::NAWKSCT}, {"\x16""#! /usr/local/bin/perl", TMagic::PERLSCT},
	{"\x16""#! /usr/local/bin/tcsh", TMagic::TSHSCT},  {"\x15""#! /usr/local/bin/zsh", TMagic::ZSHSCT},
	{"\x12""#! /usr/local/tcsh", TMagic::TSHSCT},      {"\x07""#! perl", TMagic::PERLSCT},
	{"\x08""#! rnews", TMagic::RFC822},                {"\x0a""#!/bin/ash", TMagic::ASHSCT},
	{"\x0a""#!/bin/awk", TMagic::AWKSCT},              {"\x0b""#!/bin/bash", TMagic::BASHSCT},
	{"\x0a""#!/bin/csh", TMagic::CSHSCT},              {"\x0b""#!/bin/gawk", TMagic::GAWKSCT},
	{"\x0a""#!/bin/ksh", TMagic::KSHSCT},              {"\x0b""#!/bin/nawk", TMagic::NAWKSCT},
	{"\x0b""#!/bin/perl", TMagic::PERLSCT},            {"\x09""#!/bin/sh", TMagic::SHSCT},
	{"\x0b""#!/bin/tcsh", TMagic::TSHSCT},             {"\x0a""#!/bin/zsh", TMagic::ZSHSCT},
	{"\x13""#!/lib/klibc/bin/sh", TMagic::SHSCT},      {"\x0e""#!/usr/bin/awk", TMagic::AWKSCT},
	{"\x13""#!/usr/bin/env bash", TMagic::BASHSCT},    {"\x13""#!/usr/bin/env perl", TMagic::PERLSCT},
	{"\x15""#!/usr/bin/env python", TMagic::PYTHON},   {"\x0f""#!/usr/bin/gawk", TMagic::GAWKSCT},
	{"\x0f""#!/usr/bin/nawk", TMagic::NAWKSCT},        {"\x0f""#!/usr/bin/perl", TMagic::PERLSCT},
	{"\x11""#!/usr/bin/python",TMagic:: PYTHON},       {"\x0e""#!/usr/bin/zsh", TMagic::ZSHSCT},
	{"\x14""#!/usr/local/bin/ash", TMagic::ASHSCT},    {"\x15""#!/usr/local/bin/bash", TMagic::BASHSCT},
	{"\x15""#!/usr/local/bin/gawk", TMagic::GAWKSCT},  {"\x15""#!/usr/local/bin/nawk", TMagic::NAWKSCT},
	{"\x15""#!/usr/local/bin/perl", TMagic::PERLSCT},  {"\x15""#!/usr/local/bin/tcsh", TMagic::TSHSCT},
	{"\x14""#!/usr/local/bin/zsh", TMagic::ZSHSCT},    {"\x11""#!/usr/local/tcsh", TMagic::TSHSCT},	
	{"\x06""#!perl", TMagic::PERLSCT},                 {"\x04""#LyX", TMagic::LYX},				
	{"\x02""%!\xff", TMagic::POSTSCR},                 {"\x05""%PDF-", TMagic::PDF},				
	{"\x04""\'.\\\"", TMagic::TROFF},                  {"\x03""\'\\\"", TMagic::TROFF},
	{"\x1f""** This file contains an SQLite", TMagic::SQLITE2},
	{"\x06""*BEGIN\xfd", TMagic::APPLIX},              {"\x0b""*PPD-Adobe:", TMagic::POSTSCTPD},		
	{"\x04""+++ ", TMagic::DIFFOUT},                   {"\x04""--- ", TMagic::DIFFOUT},			
	{"\x04"".RMF", TMagic::REALMED},                   {"\x03"".\\\"", TMagic::TROFF},	
	{"\x05"".ra\xfd\x00", TMagic::REALAU},             {"\x04"".sd\x00", TMagic::DECAU},
	{"\x04"".snd", TMagic::SUNAU},                     {"\x06""/* XPM", TMagic::XPM},
	{"\x06""\x30\x26\xb2\x75\x8e\x66", TMagic::MSASF}, {"\x06""7z\xbc\xaf\x27\x1c", TMagic::ZIP7},
	{"\x04""8BPS", TMagic::PHOTOSHOP},                 {"\x08"";ELC\x13\x00\x00\x00", TMagic::ELISP},
	{"\x04""<!--", TMagic::HTML},                      {"\x0e""<!DOCTYPE HTML", TMagic::HTML},
	{"\x0e""<!DOCTYPE html", TMagic::HTML},            {"\x0e""<!doctype HTML", TMagic::HTML},
	{"\x0e""<!doctype html", TMagic::HTML},            {"\x05""<?xml\xf3", TMagic::XML},
	{"\x04""<AR>", TMagic::ARARCH},                    {"\x05""<Book", TMagic::FRAMEMAK},
	{"\x03""<H1", TMagic::HTML},                       {"\x05""<HEAD", TMagic::HTML},
	{"\x05""<HTML\xf5", TMagic::HTML},                 {"\x08""<MIFFile", TMagic::FRAMEMAK},
	{"\x04""<MML", TMagic::FRAMEMAK},                  {"\x06""<Maker", TMagic::FRAMEMAK},
	{"\x06""<TITLE", TMagic::HTML},                    {"\x03""<h1", TMagic::HTML},
	{"\x05""<head", TMagic::HTML},                     {"\x05""<html\xf5", TMagic::HTML},
	{"\x06""<title", TMagic::HTML},                    {"\x05""=<ar>", TMagic::UNKARCH},
	{"\x07""Article", TMagic::MSGNEWS},                {"\x05""BEGIN", TMagic::AWKSCT},
	{"\x02""BM\xfb", TMagic::MSBMP},                   {"\x03""BZh", TMagic::BZIP2ED},
	{"\x06""Begin3", TMagic::LSM},                     {"\x04""CTMF", TMagic::CREAMUSIC},
	{"\x0b""ClamAV-VDB:", TMagic::CLAMAVDB},           {"\x17""Common subdirectories: ", TMagic::DIFFOUT},
	{"\x0d""Content-Type:", TMagic::MIME},             {"\x13""Creative Voice File", TMagic::CREAVOICE},
	{"\x05""D1.0\x0d", TMagic::SPDFONT},               {"\x04""EMOD", TMagic::EXTMOD},
	{"\x04""FFIL", TMagic::TTFFONT},                   {"\x06""FGF95a", TMagic::FGFIMG},
	{"\x04""FLV\x01", TMagic::FLASH},                  {"\x04""FONT", TMagic::VFONT},
	{"\x04""FORM\xfa", TMagic::UNKIFF},                {"\x03""FWS", TMagic::SHOCKWAV},
	{"\x0a""Forward to", TMagic::RFC822},              {"\x05""From:", TMagic::RFC822},
	{"\x04""GDBM", TMagic::GDBM},                      {"\x04""GIF8", TMagic::GIF},
	{"\x06""GIF94z", TMagic::GIF1},                    {"\x0d""GIMP Gradient", TMagic::GIMPGRAD},
	{"\x0c""GIMP Palette", TMagic::GIMPPAL},           {"\x03""ID3", TMagic::MPEG3},
	{"\x04""II\x2a\x00", TMagic::TIFF},                {"\x04""IIN1", TMagic::NIFF},
	{"\x10""ITSF\x03\x00\x00\x00\x60\x00\x00\00\x01\x00\x00\x00", TMagic::WINHELP},
	{"\x07""Index: ", TMagic::DIFFOUT},                {"\x04""LWFN", TMagic::FONT},
	{"\x04""MM\x00\x2a", TMagic::TIFF},                {"\x04""MOVI", TMagic::SGIMOV},
	{"\x08""MSCF\x00\x00\x00\x00", TMagic::CABINET},   {"\x04""MThd", TMagic::GENMIDI},
	{"\x02""MZ\xf9", TMagic::DOSEXE},                  {"\x0f""MakerScreenFont", TMagic::FMAKFONT},
	{"\x09""N#! rnews", TMagic::RFC822},               {"\x04""OTTO", TMagic::OPENTYPE},
	{"\x04""OggS", TMagic::VORBISAU},                  {"\x08""Only in ", TMagic::DIFFOUT},
	{"\x02""P1", TMagic::PBMT},                        {"\x02""P2", TMagic::PGMT},
	{"\x02""P3", TMagic::PPMT},                        {"\x02""P4", TMagic::PBM},
	{"\x02""P5", TMagic::PGM},                         {"\x02""P6", TMagic::PPM},
	{"\x02""P7", TMagic::PAM},                         {"\x03""PBF", TMagic::PBFIMAGE},
	{"\x04""PK\x03\x04\xf8", TMagic::ZIPARCH},         {"\x05""Path:", TMagic::MSGNEWS},
	{"\x07""Pipe to", TMagic::RFC822},                 {"\x04""RIFF\xf7", TMagic::UNKRIFF},
	{"\x04""Rar!", TMagic::RARARCH},                   {"\x0e""Relay-Version:", TMagic::RFC822},
	{"\x0c""Return-Path:", TMagic::RFC822},            {"\x03""SBI", TMagic::SBINSTR},
	{"\x04""SIT!", TMagic::STUFFIT},                   {"\x07""SMALLPM", TMagic::SMP},	
	{"\x0f""SQLite format 3", TMagic::SQLITE3},        {"\x0a""STARTFONT ", TMagic::BDFFONT},
	{"\x08""StuffIt ", TMagic::STUFFIT},               {"\x04""Xcur", TMagic::XCUR},
	{"\x05""Xref:", TMagic::MSGNEWS},
	{"\x0f""[Desktop Action", TMagic::DESKTOP},        {"\x0f""[Desktop Entry]", TMagic::DESKTOP},
	{"\x13""[KDE Desktop Entry]", TMagic::KDEDESK},    {"\x02""\\\"", TMagic::TROFF},
	{"\x02""\x60\xea", TMagic::ARJARCH},               {"\x04""\x61\x15\x06\x00", TMagic::BERKELEYDB},
	{"\x04""\x62\x31\x05\x00", TMagic::BERKELEYDB},    {"\x0b""d8:announce", TMagic::BITTORRENT},
	{"\x05""diff ", TMagic::DIFFOUT},                  {"\x14""eval \"exec /bin/perl", TMagic::PERLSCT},
	{"\x18""eval \"exec /usr/bin/perl", TMagic::PERLSCT}, {"\x1e""eval \"exec /usr/local/bin/perl", TMagic::PERLSCT},
	{"\x20""eval \"exec /usr/local/bin/python", TMagic::PYTHON}, {"\x0d""gimp xcf file", TMagic::XCF},
	{"\x04""hsi1", TMagic::HSIJPEG},                   {"\x0e""id=ImageMagick", TMagic::MAGICKIM},
	{"\x08""intlize\x00", TMagic::INTLIZE},            {"\x07""package", TMagic::PERLMODULE},
	{"\x02""\x71\xc7", TMagic::CPIO},                  {"\x02""v\xfd", TMagic::LZHCOMPR},
	{"\x02""v\xfe", TMagic::CRUNCHED},                 {"\x02""v\xff", TMagic::SQUEEZED},
	{"\x04""x\x9f>\"", TMagic::TNEF},                  {"\x05""{\\rtf", TMagic::RICHTEXT},
	{"\x04""\x7f""ELF\xf6", TMagic::UNKELF},           {"\x04""\x89PNG", TMagic::PNG},
	{"\x04""\x8aJNG", TMagic::JNG},                    {"\x04""\x8aMNG", TMagic::MNG},
	{"\x06""\xb8\xc0\x07\x8e\xd8\xb8", TMagic::LXKERN}, {"\x02""\xc7\x71", TMagic::CPIO},
	{"\x04""\xca\xfe\xba\xbe", TMagic::JAVACLAS},      {"\x04""\xce\x9a\x57\x13", TMagic::GDBM},
	{"\x04""\xde\x12\x04\x95", TMagic::GNUMOCAT},      {"\x04""\xed\xab\xee\xdb", TMagic::RPMARCH},
	{"\x02""\xf7\x02", TMagic::DVI},                   {"\x02""\xf7\x59", TMagic::TEXFONT},
        {"\x02""\xf7\x83", TMagic::TEXFONT},               {"\x02""\xf7\xca", TMagic::TEXFONT},
	{"\x06""\xfd\x37zXZ\x00", TMagic::XZIPED},         {"\x02""\xff\xd8", TMagic::JPEG},
	{"\x02""\xff\xe2", TMagic::MPEG3},                 {"\x02""\xff\xe3", TMagic::MPEG3},
	{"\x02""\xff\xf2", TMagic::MPEG3},                 {"\x02""\xff\xf3", TMagic::MPEG3},
	{"\x02""\xff\xf4", TMagic::MPEG2},                 {"\x02""\xff\xf5", TMagic::MPEG2},
	{"\x02""\xff\xf6", TMagic::MPEG1},                 {"\x02""\xff\xf7", TMagic::MPEG1},
	{"\x02""\xff\xfa", TMagic::MPEG3},                 {"\x02""\xff\xfb", TMagic::MPEG3},
	{"\x02""\xff\xfc", TMagic::MPEG2},                 {"\x02""\xff\xfd", TMagic::MPEG2},
	{"\x02""\xff\xfe", TMagic::MPEG1},                 {"\x02""\xff\xff", TMagic::MPEG1}
};

unsigned CMagic::GetTypeByMagic()
{
	// get the first 31 bytes of the file, buf[0] holds the length of the string
	char buf[32];
	memset(buf, 0, 32);
	if (mStreamSize >= 31) mStream.read(buf + 1, 31);
	else mStream.read(buf + 1, mStreamSize); 
	buf[0] = mStream.gcount();
	mStream.clear();	
	if (buf[0] == 0) return TMagic::LOCKED;
	// first we look for magic at file offset 0
	CGlob test;
	test.mPattern = buf;
	const CGlob* end_of_list = mMagic0 + MAGIC0SIZE;
	const CGlob* result = lower_bound(mMagic0, end_of_list, test, LessMag);
	if ((result != end_of_list) && memcmp(test.mPattern + 1, result->mPattern + 1, result->mPattern[0]) == 0) {
		// ok, we found something
		if (result->mPattern[result->mPattern[0] + 1] == 0) return result->mType;
		// some patterns need a special treatment
		switch ((result->mPattern[result->mPattern[0] + 1])) {
		case '\xff': // POSTSCR
			if (memcmp(buf + 16, "EPS", 3) == 0) return TMagic::EPOSTSCR;
			if (memcmp(buf + 3, "PS-AdobeFont-1.", 15) == 0) return TMagic::PS1TFONT;
			if (memcmp(buf + 3, "FontType1-1.", 12) == 0) return TMagic::PS1TFONT;
			return TMagic::POSTSCR;
		case '\xfe': // GZIPED
			if (memcmp(buf + 11, "KOffice", 7) == 0) {
				//KOffice
				if (memcmp(buf + 19, "application/", 12) != 0) return TMagic::GZIPED;
				buf[0] = buf[31];
				if (!mStream.read(buf + 1, 14)) return TMagic::GZIPED;
				if (memcmp(buf, "x-", 2) != 0) return TMagic::GZIPED;
				const CGlob* end_of_list = mKOffice + KOFFICESIZE;
				const CGlob* result = find(mKOffice, end_of_list, buf + 2);
				if (result != end_of_list) return result->mType;
			}
			return TMagic::GZIPED;
		case '\xfd': // APPLIX
			if (memcmp(buf + 7, "SPREADSHEETS", 12) == 0) return TMagic::AXSPREAD;
			if (memcmp(buf + 7, "WORDS", 5) == 0) return TMagic::AXWORD;
			if (memcmp(buf + 7, "GRAPHICS", 8) == 0) return TMagic::AXGRAPH;
			return TMagic::APPLIX;
		case '\xfc': // ARARCH
			if (memcmp(buf + 9, "debian", 6) == 0) return TMagic::DEBIAN;
			return TMagic::ARARCH;
		case '\xfb': // MSBMP
			if (memcmp(buf + 7, "\x00\x00", 2) == 0) return TMagic::MSBMP;
			if ((buf[11] == 12) || (buf[11] == 40) || (buf[11] == 64)) return TMagic::MSBMP;
			break;
		case '\xfa': // UNKIFF
			if (memcmp(buf + 9, "AIFF", 4) == 0) return TMagic::AIFFAU;
			if (memcmp(buf + 9, "AIFC", 4) == 0) return TMagic::AIFFCAU;
			if (memcmp(buf + 9, "8SVX", 4) == 0) return TMagic::SAMPVOX;
			if (memcmp(buf + 9, "SAMP", 4) == 0) return TMagic::SAMPAU;
			if (memcmp(buf + 9, "ILBMBMHD", 8) == 0) return TMagic::ILEAVEIM;
			if (memcmp(buf + 9, "RGBN", 4) == 0) return TMagic::IFFIMAG;
			if (memcmp(buf + 9, "RGB8", 4) == 0) return TMagic::IFFIMAG;
			if (memcmp(buf + 16, "DR2D", 4) == 0) return TMagic::IFF2D;
			if (memcmp(buf + 16, "TDDD", 4) == 0) return TMagic::IFF3D;
			if (memcmp(buf + 16, "FTXT", 4) == 0) return TMagic::IFFTXT;
			return TMagic::UNKIFF;
		case '\xf9': // DOSEXE
			if ((unsigned)buf[25] < 0x40) return TMagic::DOSEXE;
			// Linux kernels with efi support use this format, too!
			if (mStreamSize > (514 + 4 + mOffset) && mStream.seekg(514 + mOffset)) {
				if (mStream.read(buf, 4) && memcmp(buf, "HdrS", 4) == 0) return TMagic::LXKERN;
			}
			if (mStreamSize < (60 + 2 + mOffset) || !mStream.seekg(60 + mOffset)) return TMagic::UNKNOWN;
			if (!mStream.read(buf, 2)) return TMagic::UNKNOWN;
			else {
				ios::pos_type pos = ((unsigned char)buf[1]) * 256 + ((unsigned char)buf[0]);
				if (mStreamSize < (24 + pos + mOffset) || !mStream.seekg(pos + mOffset)) return TMagic::UNKNOWN;
				if (!mStream.read(buf, 24)) return TMagic::UNKNOWN;
				// TODO: Filetype detection of non-PE files 
				if (buf[0] != 'P') return TMagic::WINEXE;
				if (buf[1] != 'E') return TMagic::WINEXE;
				if (buf[23] & 0x20) return TMagic::DLL;
				return TMagic::WINEXE;
			}
		case '\xf8': // ZIPARCH
			if (buf[31] == 'm') {
				if (mStream.read(buf, 19) && memcmp(buf, "imetypeapplication/", 19) == 0) {
					if (mStream.read(buf, 12)) {
						if (memcmp(buf, "vnd.sun.xml.", 12) == 0) {
							// OpenOffice v 1
							if (mStream.read(buf, 7)) {
								const CGlob* end_of_list = mOpenOffice + OPENOFFICESIZE;
								const CGlob* result = find(mOpenOffice, end_of_list, buf);
								if (result != end_of_list) {
									string suffix = GetSuffix();
									if (result->mType == TMagic::OPENCALC) {
										if (suffix == "stc") return TMagic::OPENTMPL;
									} else if (result->mType == TMagic::OPENDRAW) {
										if (suffix == "std") return TMagic::OPENDRAT;
									} else if (result->mType == TMagic::OPENIMPR) {
										if (suffix == "sti") return TMagic::OPENIMPT;
									} else if (result->mType == TMagic::OPENWRIT) {
										if (suffix == "stw") return TMagic::OPENWRTT;
									}
									return result->mType;
								}
							}
						} else if (memcmp(buf, "vnd.oasis.op", 12) == 0) {
							// OpenDocument
							if (mStream.read(buf, 11) && memcmp(buf, "endocument.", 11) == 0) {
								if (mStream.read(buf, 21)) {
									const CGlob* end_of_list = mOpenDoc + OPENDOCSIZE;
									const CGlob* result = find(mOpenDoc, end_of_list, buf);
									if (result != end_of_list) return result->mType;
								}
							}
						}																		
					}
				}
			}
			if (GetSuffix() == "jar") return TMagic::JAVAARCH;
			else return TMagic::ZIPARCH;
		case '\xf7': // UNKRIFF
			if (memcmp(buf + 9, "PAL", 3) == 0) return TMagic::RIFFPAL;
			if (memcmp(buf + 9, "RDIB", 4) == 0) return TMagic::RIFFDIB;
			if (memcmp(buf + 9, "RMID", 4) == 0) return TMagic::RIFFMIDI;
			if (memcmp(buf + 9, "RMMP", 4) == 0) return TMagic::RIFFMOV;
			if (memcmp(buf + 9, "WAVE", 4) == 0) return TMagic::RIFFWAVE;
			if (memcmp(buf + 9, "AVI ", 4) == 0) return TMagic::RIFFAVI;
			if (memcmp(buf + 9, "ACON", 4) == 0) return TMagic::RIFFACUR;
			return TMagic::UNKRIFF;
		case '\xf6': // UNKELF
			if (buf[17] == 1) return TMagic::ELFOBJ;
			if (buf[17] == 2) return TMagic::ELFEXE;
			if (buf[17] == 3) return TMagic::ELFSOBJ;
			if (buf[17] == 4) return TMagic::ELFCORE;
			return TMagic::UNKELF;
		case '\xf5': // HTML
			if (GetSuffix() == "asp") return TMagic::ASP;
			else return TMagic::HTML;
		case '\xf4': // LOTUS123
			if (buf[5] == '\x00') {
				// maybe TARGA
				if (((buf[13] | buf[14]) != 0) && ((buf[15] | buf[16]) != 0)) {
					if (GetSuffix() == "tga") return TMagic::TARGA;
				}
			}
			return TMagic::LOTUS123;
		case '\xf3': // XML
			mStream.ignore(200, '\n');
			mStream.read(buf, 13);
			if (memcmp(buf, "<!DOCTYPE svg", 13) == 0) return TMagic::SVGIMG;
			if (memcmp(buf, "<!doctype svg", 13) == 0) return TMagic::SVGIMG;
			if (memcmp(buf, "<!DOCTYPE SVG", 13) == 0) return TMagic::SVGIMG;
			if (memcmp(buf, "<!doctype SVG", 13) == 0) return TMagic::SVGIMG;
			if (GetSuffix() == "kdevelop") return TMagic::KDEVELOP;
			return TMagic::XML;
		default:
			cxAssert(0);
		}
	}
	if (buf[1] == '#') {
		if (buf[2] == ' ') {
			// weak magic for gettexts .po and .pot files
			string suf = GetSuffix();
			// suffix has to match
			if (suf == "po") return TMagic::GETTEXT;
			else if (suf == "pot") return TMagic::GETTEXTT;
			else if (suf == "pm") return TMagic::PERLMODULE;
		} else if (buf[2] == '#') {
			string suf = GetSuffix();
			if (suf == "pm") return TMagic::PERLMODULE;
		}
	}
	// no match at file offset 0, try other offsets
	// offset = 1
	if (memcmp(buf + 2, "WPC", 3) == 0) return TMagic::WORDPERF;
	if ((unsigned char)buf[2] < 2) {
		if (memcmp(buf + 2, "\x00\x02\x00\x00", 4) == 0) return TMagic::TARGA;
		if (memcmp(buf + 2, "\x00\x0a\x00\x00", 4) == 0) return TMagic::TARGA;
		if (memcmp(buf + 2, "\x00\x03\x00\x00", 4) == 0) return TMagic::TARGA;
		if (memcmp(buf + 2, "\x00\x0b\x00\x00", 4) == 0) return TMagic::TARGA;
		if (memcmp(buf + 2, "\x01\x01\x00\x00", 4) == 0) return TMagic::TARGA;
		if (memcmp(buf + 2, "\x01\x09\x00\x00", 4) == 0) return TMagic::TARGA;
	}
	// offset = 2
	if ((buf[3] == '\x00') && ((buf[4] == '\x11') || (buf[4] == '\x12'))) return TMagic::TEXFONTM;
	if (memcmp(buf + 3, "-l", 2) == 0) {
		if (buf[5] == 'h') {
			switch (buf[6]) {
			case ' ':
			case '0':
			case '1':
			case '2':
			case '3':
			case '5':
			case 'd':
				if (buf[7] == '-') return TMagic::LHAARCH;
				break;
			case '4':
				if (buf[7] == '-') return TMagic::LHAARCH;
				if (memcmp(buf + 7, "0-", 2) == 0) return TMagic::LHAARCH;
			default:
				break;
			}
		}
		if (buf[5] == 'z') {
			switch (buf[6]) {
			case '4':
			case '5':
			case 's':
				if (buf[7] == '-') return TMagic::LHAARCH;
			default:
				break;
			}
		}
	}
	// offset = 3; offset = 4
	if (buf[5] <= 'w' && buf[5] >= 'e') {
		if (buf[5] < 'm') {
			if (memcmp(buf + 4, "Research", 8) == 0) return TMagic::G3FAX;
			if (memcmp(buf + 5, "free", 4) == 0) return TMagic::QUICKTIME;
			if (memcmp(buf + 5, "ftyp", 4) == 0) return TMagic::QUICKTIME;
			if (memcmp(buf + 5, "gtktalog ", 9) == 0) return TMagic::GTKTALOG;
			if (memcmp(buf + 5, "junk", 4) == 0) return TMagic::QUICKTIME;
		} else {
			if (memcmp(buf + 5, "moov", 4) == 0) return TMagic::QUICKTIME;
			if (memcmp(buf + 5, "mdat", 4) == 0) return TMagic::QUICKTIME;
			if (memcmp(buf + 5, "pict", 4) == 0) return TMagic::QUICKTIME;
			if (memcmp(buf + 5, "pnot", 4) == 0) return TMagic::QUICKTIME;
			if (memcmp(buf + 5, "skip", 4) == 0) return TMagic::QUICKTIME;
			if (memcmp(buf + 5, "wide", 4) == 0) return TMagic::QUICKTIME;
		}
	}
	// offset = 6
	if (memcmp(buf + 7, "%!PS-AdobeFont-1.0", 18) == 0) return TMagic::PS1CFONT;
	// offset = 7
	if (memcmp(buf + 8, "\x00\x45\x47\x41", 4) == 0) return TMagic::DOSFONT;
	if (memcmp(buf + 8, "\x00\x56\x49\x44", 4) == 0) return TMagic::DOSFONT;
	// offset = 8
	if (memcmp(buf + 9, "\x13\x7a\x2b", 3) == 0) return TMagic::BDFFONT;
	// offset = 11
	if (memcmp(buf + 12, "must be converted wi", 20) == 0) return TMagic::BINHEX;
	// offset = 12
	if (memcmp(buf + 13, "\x00\x06\x15\x61", 4) == 0) return TMagic::BERKELEYDB;
	if (memcmp(buf + 13, "\x61\x15\x06\x00", 4) == 0) return TMagic::BERKELEYDB;
	if (memcmp(buf + 13, "\x00\x05\x31\x62", 4) == 0) return TMagic::BERKELEYDB;
	if (memcmp(buf + 13, "\x62\x31\x05\x00", 4) == 0) return TMagic::BERKELEYDB;
	if (memcmp(buf + 13, "\x00\x04\x22\x53", 4) == 0) return TMagic::BERKELEYDB;
	if (memcmp(buf + 13, "\x53\x22\x04\x00", 4) == 0) return TMagic::BERKELEYDB;
	// offset = 20
	if (memcmp(buf + 21, "\xdc\xa7\xc4\xfd", 4) == 0) return TMagic::ZOOARCH;
	if (memcmp(buf + 21, "GPAT", 4) == 0) return TMagic::GIMPPAT;
	if (memcmp(buf + 21, "GIMP", 4) == 0) return TMagic::GIMPBRUSH;
	//
	if (mStreamSize < (65 + 4) || !mStream.seekg(mOffset + (ios::pos_type)65)) return TMagic::UNKNOWN;
	if (!mStream.read(buf, 4)) return TMagic::UNKNOWN;
	// offset = 65
	if (memcmp(buf, "LWFN", 4) == 0) return TMagic::FONT;
	if (memcmp(buf, "FFIL", 4) == 0) return TMagic::TTFFONT;
	// the size of a tape archive is a multiple of 512
	if (mStatus.st_size % 512 == 0 || mCompressed) {
		if (mStreamSize < (257 + 8) || !mStream.seekg(mOffset + (ios::pos_type)257)) return TMagic::UNKNOWN;
		if (!mStream.read(buf, 8)) return TMagic::UNKNOWN;
		// offset = 257
		if (memcmp(buf, "ustar", 5) == 0) {
			if (buf[5] == '\x00') return TMagic::POSIXTAR;
			if (memcmp(buf + 5, "  \0", 3) == 0) return TMagic::GNUTAR;
			return TMagic::UNKTAR;
		}
		// try harder... calculate checksum of first block
		// no need to test for eof; the file has at least 512 bytes
		if (mStreamSize >= 512 && mStream.seekg(mOffset)) {
			unsigned i = 0;
			unsigned sum = 0;
			unsigned sum1 = 0;
			char c;
			while (i++ < 148) {
				mStream.get(c);
				sum += (unsigned char)c;
			}
			--i;
			while (i++ < 156) {
				sum += 32;
				mStream.get(c);
				if (c == ' ' || c == 0) continue;
				c -= '0';
				if (c < 0 || c > 7) return TMagic::UNKNOWN;
				sum1 *= 8;
				sum1 += c;
			}
			--i;
			while (i++ < 512) {
				mStream.get(c);
				sum += (unsigned char)c;
			}
			if (sum == sum1) return TMagic::UNKTAR;
		}
	}
	if (mStreamSize < (498 + 20) || !mStream.seekg(mOffset + (ios::pos_type)498)) return TMagic::UNKNOWN;
	if (!mStream.read(buf, 20)) return TMagic::UNKNOWN;
	// offset = 498
	if (buf[8] == '\xff' && (buf[0] == 0 || buf[0] == 1) && buf[9] <= '\xff' && buf[9] >= '\xfd' && memcmp(buf + 16, "HdrS", 4) == 0) return TMagic::LXKERN;
	// no, we cannot determine the file type by magic
	return TMagic::UNKNOWN;
}

unsigned CMagic::GetTypeByGlobbing()
{
	// get file name without path
	string name = cxFileManager->GetFileName(mPath);
	CGlob test;
	test.mPattern = name.c_str();
	// search in mName
	const CGlob* end_of_list = mName + NAMESIZE;
	const CGlob* result = lower_bound(mName, end_of_list, test, Less);
	if ((result != end_of_list) && !strcmp(result->mPattern, test.mPattern)) return result->mType;
	// no, name doesn't match, try suffix
	string suffix = GetSuffix();
	if (suffix.empty()) return TMagic::UNKNOWN;
	test.mPattern = suffix.c_str();
	end_of_list = mEnding + ENDINGSIZE;
	result = lower_bound(mEnding, end_of_list, test, Less);
	if ((result != end_of_list) && !strcmp(result->mPattern, test.mPattern)) return result->mType;
	// no match, give up
	return TMagic::UNKNOWN;
}

unsigned CMagic::GetTextType()
{
	mStream.clear();
	mStream.seekg(mOffset);
	bool ascii(true);
	bool iso(true);
	bool uni(true);
	unsigned unicount(0);
	unsigned char c;
	int max = 512;
	if (mStreamSize < 512) max = mStreamSize;
	for (int i = 0; (i < max) && (mStream != NULL); ++i) {
		mStream.get((char&)c);
		if (c < 0x20) {
			if (unicount > 0) {
				uni = false;
				if (!iso) break;
			}
			if (c != '\n' && c != '\r' && c != '\t') {
				ascii = false;
				iso = false;
				uni = false;
				break;
			}
		} else if (c < 0x7f) {
			if (unicount > 0) {
				uni = false;
				if (!iso) break;
			}
		} else if (c < 0xa0) {
			ascii = false;
			iso = false;
			if (unicount > 0) --unicount;
			else {
				uni = false;
				break;
			}
		} else if (c < 0xc0) {
			ascii = false;
			if (unicount > 0) --unicount;
			else {
				uni = false;
				if (!iso) break;
			}
		} else {
			ascii = false;
			if (unicount == 0) {
				if (c < 0xe0) unicount = 1;
				else if (c < 0xf0) unicount = 2;
				else if (c < 0xf8) unicount = 3;
				else if (c < 0xfc) unicount = 4;
				else if (c < 0xfe) unicount = 5;
				else {
					uni = false;
					if (!iso) break;
				}
			} else {
				uni = false;
				if (!iso) break;
			}
		} 
	}
	if (ascii) return TMagic::ASCII;
	if (uni) return TMagic::UTF8;
	if (iso) return TMagic::TEXT;
	return TMagic::UNKNOWN;
}

// Get the file type via opened filebuf
unsigned CMagic::GetFileType(filebuf& fbuf, const struct stat& st, const string& fn)
{
	mPath = fn;
	mCompressed = false;
	unsigned typ = GetSpecialType(st.st_mode);
	if (typ != TMagic::UNKNOWN) return typ;
	if (st.st_size == 0) return TMagic::EMPTY;
 	mStream.clear();
	mStream.rdbuf(&fbuf);
	mOffset = mStream.tellg();
	mStreamSize = st.st_size;
	memcpy(&mStatus, &st, sizeof(struct stat));
	typ = GetTypeByMagic();
	if (typ != TMagic::UNKNOWN) {
		//ResetFileTimes();
		return typ;
	}
	if (mPath[mPath.size() - 1] == '~') {
		//ResetFileTimes();
		return TMagic::BACKUP;
	}
	if (mPath[mPath.size() - 1] == '%') {
		//ResetFileTimes();
		return TMagic::BACKUP;
	}
	typ = GetTypeByGlobbing();
	if (typ != TMagic::UNKNOWN) {
		//ResetFileTimes();
		return typ;
	}
	typ = GetTextType();
	//ResetFileTimes();
	return typ;
}
	
unsigned CMagic::GetFileType(const string& file)
{
	mCompressed = false;
	mPath = file;
	if (stat(mPath.c_str(), &mStatus)) return TMagic::LOCKED;
	unsigned typ = GetSpecialType(mStatus.st_mode);
	if (typ != TMagic::UNKNOWN) return typ;
	if (mStatus.st_size == 0) return TMagic::EMPTY;
 	mStream.clear();
	mStream.rdbuf(&mFilebuffer);
	mOffset = 0;
	mStreamSize = mStatus.st_size;
 	mFilebuffer.open(file.c_str(), ios::in);
	if (!mStream) return TMagic::LOCKED;
	typ = GetTypeByMagic();
	if (typ == TMagic::GZIPED) mCompressed = true; 
#ifdef CXXX_HAS_BZ2		
	if (typ == TMagic::BZIP2ED) mCompressed = true;
#endif	// CXXX_HAS_BZ2
#ifdef CXXX_HAS_XZ
	if (typ == TMagic::XZIPED) mCompressed = true;
#endif // CXXX_HAS_XZ
	if (mCompressed) {
		unsigned tmptyp = typ;
		CZStreambuf zbuf(mFilebuffer, CZStreambuf::SMALL);
		if (zbuf) mStream.rdbuf(zbuf);
		else {
			ResetFileTimes();
			return tmptyp;
		}		
		mStreamSize = 1000; // hard to determine; assume it's large enough
		typ = GetTypeByMagic();
		if (typ != TMagic::UNKNOWN) {
			if (tmptyp == TMagic::GZIPED) typ |= TMagic::GZCOMPRESSED;
			else if (tmptyp == TMagic::BZIP2ED) typ |= TMagic::BZCOMPRESSED;
			else if (tmptyp == TMagic::XZIPED) typ |= TMagic::XZCOMPRESSED;
			ResetFileTimes();
			return typ;
		}
		typ = GetTextType();
		if (typ == TMagic::UNKNOWN) typ = tmptyp;
		else {
			if (tmptyp == TMagic::GZIPED) typ |= TMagic::GZCOMPRESSED;
			else if (tmptyp == TMagic::BZIP2ED) typ |= TMagic::BZCOMPRESSED;
			else if (tmptyp == TMagic::XZIPED) typ |= TMagic::XZCOMPRESSED;
		}
		ResetFileTimes();
		return typ;
	} 
	if (typ != TMagic::UNKNOWN) {
		ResetFileTimes();
		return typ;
	}
	if (mPath[mPath.size() - 1] == '~') {
		ResetFileTimes();
		return TMagic::BACKUP;
	}
	if (mPath[mPath.size() - 1] == '%') {
		ResetFileTimes();
		return TMagic::BACKUP;
	}
	typ = GetTypeByGlobbing();
	if (typ != TMagic::UNKNOWN) {
		ResetFileTimes();
		return typ;
	}
	typ = GetTextType();
	ResetFileTimes();
	return typ;
}

void CMagic::ResetFileTimes()
{
	mFilebuffer.close();
	timeval tv[2];
	tv[0].tv_sec = mStatus.st_atime;
	tv[1].tv_sec = mStatus.st_mtime;
#ifdef _STATBUF_ST_NSEC
 #ifdef __USE_MISC
	tv[0].tv_usec = mStatus.st_atim.tv_nsec / 1000;
	tv[1].tv_usec = mStatus.st_mtim.tv_nsec / 1000;
 #else // __USE_MISC
	tv[0].tv_usec = mStatus.st_atimensec / 1000;
	tv[1].tv_usec = mStatus.st_mtimensec / 1000;
 #endif // __USE_MISC
#else // _STATBUF_ST_NSEC
	tv[0].tv_usec = 0;
	tv[1].tv_usec = 0;
#endif // _STATBUF_ST_NSEC
	utimes(mPath.c_str(), tv); /* don't care if loses */
}
