/*----------------------------------------------------------------------------
  Project: cxxx <http://cxplorer.sourceforge.net>
  File   : CMenuBar.cpp
  Info   : $Id: CMenuBar.cpp,v 1.18 2012/06/26 08:05:52 gkraeuter Exp $
----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
  Copyright © 2002 - 2012 Gerhard Kräuter <gkraeuter@users.sourceforge.net>

  This  software  is  provided  'as-is',  without  any  express  or  implied
  warranty.  In  no  event  will  the authors be held liable for any damages
  arising from the use of this software.

  Permission  is  granted  to  anyone  to use this software for any purpose,
  including  commercial  applications,  and  to alter it and redistribute it
  freely, subject to the following restrictions:

  1. The  origin  of  this software must not be misrepresented; you must not
     claim  that  you  wrote the original software. If you use this software
     in  a  product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.

----------------------------------------------------------------------------*/

// C++X headers
#include <cxxx/CMenuBar.h>
#include <cxxx/CPopup.h>

// Namespace declarations
using cxxx::cxDpy;

/* virtual */ CMenuBar::~CMenuBar()
{
	GetTopLevelWindow()->ReleaseKey(this);
}

/* virtual */ void CMenuBar::OnExpose(XExposeEvent& e __attribute__ ((unused)))
{
	ClearWindow();
	int x = 0;
	int y = (20 - GetTextHeight()) / 2;
	CColor col;
	for (unsigned i = 0; i < mMenu.size(); ++i) {
		if (i == mSelected) {
			FillRectangle(x, 0, mMenu[i].GetTextWidth() + 20, mHeight, CColor(TC_TITLE));
			col = CColor(TC_TITLE_TEXT);
		} else col = CColor(TC_FRAME_TEXT);
		DrawHotString(x + 10, y, col, mMenu[i].GetText(), mMenu[i].GetHotCharPos(), false, i != mSelected);
		x += mMenu[i].GetTextWidth() + 20;
	}
}

/* virtual */ void CMenuBar::OnLButtonDown(XButtonEvent& e)
{
	Select(GetItemFromPos(e.x));
	XUngrabPointer(cxDpy, CurrentTime);
}

/* virtual */ void CMenuBar::OnEvent(XEvent& e)
{
	switch (e.xany.type) {
	case MotionNotify:
		if (IsSelected()) {
			unsigned i = GetItemFromPos(e.xmotion.x);
			if (i == -1u) return ;
			if (i != mSelected) Select(i);
		}
		return ;
	default:
		break;
	}
}

/* virtual */ void CMenuBar::OnFocus(bool in)
{
	if (!in) Select(-1u);
}

void CMenuBar::Select(unsigned idx)
{
	if (mSelected == idx) return;
	int x;
	int y = (20 - GetTextHeight()) / 2;
	if (mSelected != -1u) {
		x = CalculatePosition(mSelected);
		FillRectangle(x, 0, mMenu[mSelected].GetTextWidth() + 20, mHeight, CColor(TC_FRAME));
		DrawHotString(x + 10, y, CColor(TC_FRAME_TEXT), mMenu[mSelected].GetText(), mMenu[mSelected].GetHotCharPos(), false, true);
	}
	mSelected = idx;
	if (mSelected != -1u) {
		mAttributes.event_mask = ExposureMask | StructureNotifyMask | ButtonPressMask | ButtonReleaseMask | KeyPressMask | LeaveWindowMask | PointerMotionMask;
		ChangeAttributes(CWEventMask);
	} else {
		mAttributes.event_mask = ExposureMask | StructureNotifyMask | ButtonPressMask | ButtonReleaseMask | KeyPressMask;
		ChangeAttributes(CWEventMask);
		return ;
	}
	x = CalculatePosition(mSelected);
	FillRectangle(x, 0, mMenu[mSelected].GetTextWidth() + 20, mHeight, CColor(TC_TITLE));
	DrawHotString(x + 10, y, CColor(TC_TITLE_TEXT), mMenu[mSelected].GetText(), mMenu[mSelected].GetHotCharPos(), false, false);
	cxAssert(!mMenu[mSelected].IsEmpty());
	y = 0;
	TranslateCoordinates(x, y, NULL);
	CPopup* pu = new CPopup(this, mMenu[mSelected]);
	pu->Position(x, y, mMenu[mSelected].GetTextWidth() + 20, 20, CPopup::DOWN);
	pu->Show();
}


/* virtual */ void CMenuBar::OnAccelerator(wchar_t key)
{
	cxAssert(!mMenu.empty());
	for (unsigned i = 0; i < mMenu.size(); ++i) {
		if (mMenu[i].GetAccelerator() == key) {
			SetFocus();
			Select(i);
			return ;
		}
	}
}
