/*----------------------------------------------------------------------------
  Project: cxxx <http://cxplorer.sourceforge.net>
  File   : CMenuBar.h
  Info   : $Id: CMenuBar.h,v 1.10 2012/05/06 18:57:36 gkraeuter Exp $
----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
  Copyright © 2002 - 2012 Gerhard Kräuter <gkraeuter@users.sourceforge.net>

  This  software  is  provided  'as-is',  without  any  express  or  implied
  warranty.  In  no  event  will  the authors be held liable for any damages
  arising from the use of this software.

  Permission  is  granted  to  anyone  to use this software for any purpose,
  including  commercial  applications,  and  to alter it and redistribute it
  freely, subject to the following restrictions:

  1. The  origin  of  this software must not be misrepresented; you must not
     claim  that  you  wrote the original software. If you use this software
     in  a  product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.

----------------------------------------------------------------------------*/

#ifndef CMenuBar_h
#define CMenuBar_h

// C++X headers
#include <cxxx/CTopLevelWindow.h>
#include <cxxx/CMenuItem.h>

// Forward declarations
class CPopup;

class CMenuBar : public CWindow
{
public:
	// Constructor
	CMenuBar(CWindow* parent)
	: CWindow(parent)
	{
		mSelected = -1u;
		mAttributes.event_mask = ExposureMask | StructureNotifyMask | ButtonPressMask | ButtonReleaseMask | KeyPressMask;
		ChangeAttributes(CWEventMask);
		Resize(GetParent()->GetWidth(), 20);
		SetFocusable();
	}

	// Destructor
	virtual ~CMenuBar();

	// X event processing
	virtual void OnExpose(XExposeEvent& e);
	virtual void OnLButtonDown(XButtonEvent& e);
	virtual void OnEvent(XEvent& event);
	virtual void OnFocus(bool in);
	virtual void OnAccelerator(wchar_t key);

	// Adds a menu
	void AddMenu(CMenu& m)
	{
		if (m.GetAccelerator()) GetTopLevelWindow()->BindKey(m.GetAccelerator(), this);
		mMenu.push_back(m);
		mSelected = -1u;
	}

	// Checks the menu entry with message ID msgid
	void Check(unsigned msgid)
	{
		for (std::vector<CMenu>::iterator i = mMenu.begin(); i < mMenu.end(); ++i) i->Check(msgid);
	}

	// Unchecks the menu entry with message ID msgid
	void Uncheck(unsigned msgid)
	{
		for (std::vector<CMenu>::iterator i = mMenu.begin(); i < mMenu.end(); ++i) i->Uncheck(msgid);
	}
	
	// Returns whether a menu entry is selected
	bool IsSelected() const
	{
		return mSelected != -1u;
	}
	
protected:
	// Selects menu with index idx
	void Select(unsigned idx);

	// Returns the screen position of the menu with index idx
	int CalculatePosition(unsigned idx) const
	{
		cxAssert(idx < mMenu.size());
		int x = 0;
		for (unsigned i = 0; i < idx; ++i) x += mMenu[i].GetTextWidth() + 20;
		return x;
	}

	// Returns the index of the menu at screen position x
	unsigned GetItemFromPos(int pos) const
	{
		int x = 0;
		if (pos >= x) {
			for (unsigned i = 0; i < mMenu.size(); ++i) {
				x += mMenu[i].GetTextWidth() + 20;
				if (pos < x) return i;
			}
		}
		return -1u;
	}

protected:
	std::vector<CMenu> mMenu;
	unsigned mSelected;
};

#endif // CMenuBar_h
