/*----------------------------------------------------------------------------
  Project: cxxx <http://cxplorer.sourceforge.net>
  File   : CMenuItem.cpp
  Info   : $Id: CMenuItem.cpp,v 1.20 2012/08/06 17:36:40 gkraeuter Exp $
----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
  Copyright © 2002 - 2012 Gerhard Kräuter <gkraeuter@users.sourceforge.net>

  This  software  is  provided  'as-is',  without  any  express  or  implied
  warranty.  In  no  event  will  the authors be held liable for any damages
  arising from the use of this software.

  Permission  is  granted  to  anyone  to use this software for any purpose,
  including  commercial  applications,  and  to alter it and redistribute it
  freely, subject to the following restrictions:

  1. The  origin  of  this software must not be misrepresented; you must not
     claim  that  you  wrote the original software. If you use this software
     in  a  product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.

----------------------------------------------------------------------------*/

// C++X headers
#include <cxxx/CMenuItem.h>
#include <cxxx/CPopup.h>

// C++ headers
#include <cctype>

// Namespace delarations
using std::string;
using std::wstring;
using std::vector;
using cxxx::cxWiden;

CMenu::CMenu(CWindow* owner, const string& name)
: mOwner(owner),
  mAccelerator(L'\0'),
  mText(name),
  mTextWidth(0)
{
	if (!mText.empty()) {
		std::wstring wtext = cxWiden(mText);
		string::size_type pos = wtext.find(L'&');
		if (pos < wtext.size() - 1) {
			wtext.erase(pos, 1);
			wchar_t hc = wtext[pos];
			if (cxxx::cxRTL) {
				wtext = cxxx::cxBidi->Convert(wtext);
				pos = wtext.rfind(hc);
			} else pos = wtext.find(hc);
			mHotCharPos.mBegin = owner->GetTextWidth(wtext, pos);
			mHotCharPos.mEnd = mHotCharPos.mBegin + owner->GetCharWidth(wtext[pos]) -1;
			mAccelerator = toupper(hc);
			pos = mText.find('&');
			cxAssert(pos != string::npos); 
			mText.erase(pos, 1);
		}
		mTextWidth = owner->GetTextWidth(wtext);
	}
}
			
CMenuItem::CMenuItem(CWindow* owner, unsigned msgid, const string& text, CIcon::TIconName icon, CMenu* child)
: mOwner(owner),
  mChild(child),
  mFlags(0),
  mText(text),
  mAccelerator(L'\0'),
  mMsgID(msgid)
{
	if (!text.empty()) {
		string::size_type sc = mText.find(L'\t');
		if (sc != string::npos) mScutText = mText.substr(sc + 1);
		mText = mText.substr(0, sc);
		std::wstring wtext = cxWiden(mText);
		string::size_type pos = wtext.find(L'&');
		if (pos < wtext.size() - 1) {
			wtext.erase(pos, 1);
			wchar_t hc = wtext[pos];
			if (cxxx::cxRTL) {
				wtext = cxxx::cxBidi->Convert(wtext);
				pos = wtext.rfind(hc);
			} else pos = wtext.find(hc);
			mHotCharPos.mBegin = owner->GetTextWidth(wtext, pos);
			mHotCharPos.mEnd = mHotCharPos.mBegin + owner->GetCharWidth(wtext[pos]) - 1;
			mAccelerator = toupper(hc);
			pos = mText.find('&');
			cxAssert(pos != string::npos); 
			mText.erase(pos, 1);
		}
		mTextWidth = owner->GetTextWidth(wtext);
		mScutTextWidth = owner->GetTextWidth(cxWiden(mScutText));
		if (icon != CIcon::MENU_NO_ICON) {
			if (icon == CIcon::MENU_BULLET) mFlags.set(BULLET);
			else if (icon == CIcon::MENU_CHECK) mFlags.set(CHECK);
			else {
				mIcon.Create(icon, 16);
				if (mIcon.IsValid()) mFlags.set(ICON);
			}
		}
	} else {
		// seperator
		mFlags.set(SEPERATOR);
	}
}

bool CMenuItem::IsEnabled() const
{
	if (mOwner == NULL) return false;
	if (mChild == NULL) return mOwner->OnUpdate(mMsgID);
	else return mChild->IsEnabled();
}
