/*----------------------------------------------------------------------------
  Project: cxxx <http://cxplorer.sourceforge.net>
  File   : CMessageBox.cpp
  Info   : $Id: CMessageBox.cpp,v 1.23 2012/08/06 17:36:40 gkraeuter Exp $
----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
  Copyright © 2002 - 2012 Gerhard Kräuter <gkraeuter@users.sourceforge.net>

  This  software  is  provided  'as-is',  without  any  express  or  implied
  warranty.  In  no  event  will  the authors be held liable for any damages
  arising from the use of this software.

  Permission  is  granted  to  anyone  to use this software for any purpose,
  including  commercial  applications,  and  to alter it and redistribute it
  freely, subject to the following restrictions:

  1. The  origin  of  this software must not be misrepresented; you must not
     claim  that  you  wrote the original software. If you use this software
     in  a  product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.

----------------------------------------------------------------------------*/

// C++X headers
#include <cxxx/CMessageBox.h>
#include <cxxx/CPushButton.h>
#include <cxxx/CLabel.h>

// Namespace declarations
using cxxx::cxMainWin;
using cxxx::cxDpy;
using cxxx::lib::cxgettext;
using cxxx::cxError;
using std::string;

CMessageBox::CMessageBox(const string& text, const string& caption, unsigned style, CIcon::TIconName icon)
: CTopLevelWindow(),
  mText(this),
  mCount(0)
{
	TransientFor(cxMainWin);
	SetCaption(caption);
	Resize(MESSAGEBOX_SIZE_X, MESSAGEBOX_SIZE_Y);
	Move((DisplayWidth(cxDpy, DefaultScreen(cxDpy)) - mWidth) / 2, (DisplayHeight(cxDpy, DefaultScreen(cxDpy)) - mHeight) / 2);
	FixSize();
	mIcon.Create(icon, 32);
	mText.Resize(mWidth - 55, mHeight - 24 - 10);
	mText.SetText(text);
	mText.Move(50, 5);
	for (unsigned i = 0; i < 3; ++i) mButton[i] = NULL;
	if (style & OK) {
		mButton[mCount] = new CPushButton(this, IDOK);
		mButton[mCount]->SetName(_("&OK", 25));
		++mCount;
	}
	if (style & YES) {
		mButton[mCount] = new CPushButton(this, IDYES);
		mButton[mCount]->SetName(_("&Yes", 89));
		++mCount;
	}
	if (style & RETRY) {
		mButton[mCount] = new CPushButton(this, IDRETRY);
		mButton[mCount]->SetName(_("&Retry", 90));
		++mCount;
	}
	if (style & NO) {
		cxAssert(mCount < 3);
		mButton[mCount] = new CPushButton(this, IDNO);
		mButton[mCount]->SetName(_("&No", 91));
		++mCount;
	}
	if (style & IGNORE) {
		cxAssert(mCount < 3);
		mButton[mCount] = new CPushButton(this, IDIGNORE);
		mButton[mCount]->SetName(_("&Ignore", 92));
		++mCount;
	}
	if (style & CANCEL) {
		cxAssert(mCount < 3);
		mButton[mCount] = new CPushButton(this, IDCANCEL);
		mButton[mCount]->SetName(_("&Cancel", 29));
		++mCount;
	}
	switch (mCount) {
	case 1:
		mButton[0]->Move((mWidth - 80) / 2, mHeight - 24 - 5);
		break;
	case 2:
		mButton[0]->Move(mWidth / 2 - 80 - 10, mHeight - 24 - 5);
		mButton[1]->Move(mWidth / 2 + 10, mHeight - 24 - 5);
		break;
	case 3:
		mButton[0]->Move(5, mHeight - 24 - 5);
		mButton[1]->Move((mWidth - 80) / 2, mHeight - 24 - 5);
		mButton[2]->Move(mWidth - 80 - 5, mHeight - 24 - 5);
		break;
	default:
		cxError << _("Too many buttons in CMessageBox", 88) << CError::Fatal;
		break;
	}
	for (unsigned i = 0; i < mCount; ++i) mButton[i]->Show();
	mText.Show();
	Show();
}

/* virtual */ CMessageBox::~CMessageBox()
{
	for (unsigned i = 0; i < mCount; ++i) {
		cxAssert(mButton[i] != NULL);
		delete mButton[i];
	}
}

/* virtual */ void CMessageBox::OnMessage(unsigned msgid, TMessage type, unsigned value __attribute__ ((unused)))
{
	if (msgid >= IDOK && msgid <= IDCANCEL) {
		if (type == MT_RELEASED || type == MT_ACCELERATOR) {
			mReturn = RetValFromID(msgid);
			mShutDown = true;
		}
	}
}

/* virtual */ void CMessageBox::OnQuit()
{
	mReturn = CANCEL;
	mShutDown = true;
}

/* virtual */ void CMessageBox::OnKey(wchar_t key, unsigned state __attribute__ ((unused)))
{
	if (key == L'\x1b') OnQuit();
	else if (key == L'\n' && mFocusWindow != NULL) mFocusWindow->OnAccelerator(L'\n');
}

/* virtual */ void CMessageBox::OnExpose(XExposeEvent& e)
{
	ClearArea(e);
	ClearArea(5, 5, mIcon.GetWidth(), mIcon.GetHeight());
	DrawIcon(5, 5, mIcon);
}
