/*----------------------------------------------------------------------------
  Project: cxxx <http://cxplorer.sourceforge.net>
  File   : CMessageBox.h
  Info   : $Id: CMessageBox.h,v 1.17 2012/08/06 17:36:40 gkraeuter Exp $
----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
  Copyright © 2002 - 2012 Gerhard Kräuter <gkraeuter@users.sourceforge.net>

  This  software  is  provided  'as-is',  without  any  express  or  implied
  warranty.  In  no  event  will  the authors be held liable for any damages
  arising from the use of this software.

  Permission  is  granted  to  anyone  to use this software for any purpose,
  including  commercial  applications,  and  to alter it and redistribute it
  freely, subject to the following restrictions:

  1. The  origin  of  this software must not be misrepresented; you must not
     claim  that  you  wrote the original software. If you use this software
     in  a  product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.

----------------------------------------------------------------------------*/

#ifndef CMessageBox_h
#define CMessageBox_h


// C++X headers
#include <cxxx/CLabel.h>
#include <cxxx/CTopLevelWindow.h>
#include <cxxx/CPushButton.h>

// C++ headers
#include <string>

class CMessageBox : public CTopLevelWindow
{
public:
	enum
	{
		// Styles and return values
		OK = 0x01,
		YES = 0x02,
		NO = 0x04,
		RETRY = 0x08,
		IGNORE = 0x10,
		CANCEL = 0x20
	};

private:
	enum
	{
		// Message IDs
		IDOK = cxxx::msgid::MSGID_CMessageBox,
		IDYES,
		IDNO,
		IDRETRY,
		IDIGNORE,
		IDCANCEL,

		// Geometry
		MESSAGEBOX_SIZE_X = 280,
		MESSAGEBOX_SIZE_Y = 120
	};

public:
	// Constructor
	CMessageBox(const std::string& text, const std::string& caption, unsigned style, CIcon::TIconName icon);
	
	// Destructor
	virtual ~CMessageBox();

	// X event processing
	virtual void OnKey(wchar_t key, unsigned state);
	virtual void OnExpose(XExposeEvent& e);

	// C++X message processing
	virtual void OnMessage(unsigned msgid, TMessage type, unsigned value);

	// Exits the window
	virtual void OnQuit();

	// Focuses button with return value ret
	void FocusButton(unsigned retval)
	{
		for (unsigned i = 0; i < mCount; ++i) {
			if (RetValFromID(mButton[i]->GetMessageID()) == retval) {
				SetFocusWindow(mButton[i]);
				return;
			}
		}
	}

	// Calculates the return value of a button from its message ID
	unsigned RetValFromID(unsigned id) const
	{
		switch (id) {
		case IDOK:
			return OK;
		case IDYES:
			return YES;
		case IDNO:
			return NO;
		case IDRETRY:
			return RETRY;
		case IDIGNORE:
			return IGNORE;
		case IDCANCEL:
			return CANCEL;
		default:
			cxAssert(0);
			return CANCEL;
		}
	}

protected:
	CIcon mIcon;
	CLabel mText;
	CPushButton* mButton[3];
	unsigned mCount;
};

#endif // CMessageBox_h
