/*----------------------------------------------------------------------------
  Project: cxxx <http://cxplorer.sourceforge.net>
  File   : CMessageTransceiver.cpp
  Info   : $Id: CMessageTransceiver.cpp,v 1.8 2012/08/06 17:36:40 gkraeuter Exp $
----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
  Copyright © 2002 - 2012 Gerhard Kräuter <gkraeuter@users.sourceforge.net>

  This  software  is  provided  'as-is',  without  any  express  or  implied
  warranty.  In  no  event  will  the authors be held liable for any damages
  arising from the use of this software.

  Permission  is  granted  to  anyone  to use this software for any purpose,
  including  commercial  applications,  and  to alter it and redistribute it
  freely, subject to the following restrictions:

  1. The  origin  of  this software must not be misrepresented; you must not
     claim  that  you  wrote the original software. If you use this software
     in  a  product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.

----------------------------------------------------------------------------*/

// C++X headers
#include <cxxx/Cxxx.h>
#include <cxxx/CApplication.h>

// Namespace declarations
using cxxx::cxApp;
using cxxx::cxDpy;
using cxxx::cxMainWin;
using std::deque;

// Static variable initialization
deque<XEvent> CMessageTransceiver::mEventQueue;

/* virtual */ CMessageTransceiver::~CMessageTransceiver()
{
	if(cxMainWin == NULL) return;
	cxApp->RemoveIdleTarget(*this);
}

void CMessageTransceiver::PostMessage(TMessage type, unsigned value)
{
	if (mTarget == NULL) return;
	XEvent e;
	while (XPending(cxDpy)) {
		XNextEvent(cxDpy, &e);
		PushEvent(e);
	}
	e.xclient.type = CxxxMessage;
	e.xclient.send_event = True;
	e.xclient.display = cxDpy;
	e.xclient.message_type = type;
	e.xclient.window = mTarget->GetXID();
	e.xclient.format = 32;
	e.xclient.data.l[0] = mMsgID;
	e.xclient.data.l[1] = value;
	// do not pile up scroll messages
	if (type == MT_SCROLL || type == MT_MOTION) {
		for (std::deque<XEvent>::iterator i = mEventQueue.begin(); i != mEventQueue.end(); ++i) {
			if (i->xclient.message_type == (unsigned)type && i->xclient.data.l[0] == (int)mMsgID) {
				mEventQueue.erase(i);
				break;
			}
		}
	}
	PushEvent(e);
}

void CMessageTransceiver::RequestIdle(bool req)
{
	if(req) cxApp->AddIdleTarget(*this);
	else cxApp->RemoveIdleTarget(*this);
}
