/*----------------------------------------------------------------------------
  Project: cxxx <http://cxplorer.sourceforge.net>
  File   : CMessageTransceiver.h
  Info   : $Id: CMessageTransceiver.h,v 1.12 2012/08/06 17:36:40 gkraeuter Exp $
----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
  Copyright © 2002 - 2012 Gerhard Kräuter <gkraeuter@users.sourceforge.net>

  This  software  is  provided  'as-is',  without  any  express  or  implied
  warranty.  In  no  event  will  the authors be held liable for any damages
  arising from the use of this software.

  Permission  is  granted  to  anyone  to use this software for any purpose,
  including  commercial  applications,  and  to alter it and redistribute it
  freely, subject to the following restrictions:

  1. The  origin  of  this software must not be misrepresented; you must not
     claim  that  you  wrote the original software. If you use this software
     in  a  product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.

----------------------------------------------------------------------------*/

#ifndef CMessageTransceiver_h
#define CMessageTransceiver_h

// C++X headers
#include <cxxx/Cxxx.h>

// C++ headers
#include <vector>
#include <algorithm>
#include <deque>

enum
{
	// Additional X event types
	ButtonDblClick = LASTEvent,
	CxxxMessage
};

class CMessageTransceiver
{
public:
	// Constructor
	CMessageTransceiver(CMessageTransceiver* target, unsigned msgid)
	: mXID(None),
	  mMsgID(msgid),
	  mTarget(target)
	{
	}

	// Destructor
	virtual ~CMessageTransceiver();

	// C++X message processing
	virtual void OnMessage(unsigned msgid __attribute__ ((unused)), TMessage type __attribute__ ((unused)), unsigned value __attribute__ ((unused)))
	{
	}
	

	// Feedback message
	virtual bool OnUpdate(unsigned msgid __attribute__ ((unused))) const
	{
		return false;
	}

	// Send a message
	void SendMessage(TMessage type, unsigned value = 0)
	{
		if (mTarget != NULL) mTarget->OnMessage(mMsgID, type, value);
	}

	// Post a message for sending
	void PostMessage(TMessage type, unsigned value = 0);

	// Feedback
	bool GetUpdate()
	{
		if(mTarget != NULL) return mTarget->OnUpdate(mMsgID);
		return false;
	}

	// Request to get idle messages
	void RequestIdle(bool req);
	
	// The X identifier, if applicable
	Window GetXID() const
	{
		return mXID;
	}
	
	// Sets the target for the messages
	void SetMessageTarget(CMessageTransceiver* target)
	{
		mTarget = target;
	}
	
	// Sets the message ID
	void SetMessageID(unsigned msgid)
	{
		mMsgID = msgid;
	}
	
	// Returns the message ID
	unsigned GetMessageID() const
	{
		return mMsgID;
	}
protected:
	// Moves e into the event queue
	void PushEvent(const XEvent& e)
	{
		if (e.type == ConfigureNotify) {
			for (std::deque<XEvent>::iterator i = mEventQueue.begin(); i != mEventQueue.end(); ++i) {
				if (i->type == ConfigureNotify && i->xconfigure.window == e.xconfigure.window) {
					mEventQueue.erase(i);
					break;
				}
			}
		}
		mEventQueue.push_back(e);
	}

	// Retrieves an event from the event queue
	XEvent PopEvent()
	{
		cxAssert(EventsQueued());
		XEvent e = mEventQueue.front();
		mEventQueue.pop_front();
		return e;
	}

	// Returns true if there are events waiting to be processed
	bool EventPending()
	{
		return (!mEventQueue.empty() || XPending(cxxx::cxDpy));
	}

	// Returns true if there are events in the queue
	bool EventsQueued()
	{
		return (!mEventQueue.empty());
	}

protected:
	Window mXID;

private:
	static std::deque<XEvent> mEventQueue;
	unsigned mMsgID;
	CMessageTransceiver* mTarget;
};


#endif // CMessageTransceiver_h
