/*----------------------------------------------------------------------------
  Project: cxxx <http://cxplorer.sourceforge.net>
  File   : CNumEdit.cpp
  Info   : $Id: CNumEdit.cpp,v 1.19 2012/05/06 18:57:36 gkraeuter Exp $
----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
  Copyright © 2005 - 2012 Gerhard Kräuter <gkraeuter@users.sourceforge.net>

  This  software  is  provided  'as-is',  without  any  express  or  implied
  warranty.  In  no  event  will  the authors be held liable for any damages
  arising from the use of this software.

  Permission  is  granted  to  anyone  to use this software for any purpose,
  including  commercial  applications,  and  to alter it and redistribute it
  freely, subject to the following restrictions:

  1. The  origin  of  this software must not be misrepresented; you must not
     claim  that  you  wrote the original software. If you use this software
     in  a  product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.

----------------------------------------------------------------------------*/

// cxxx headers
#include <cxxx/CNumEdit.h>

// X11 headers
#include <X11/cursorfont.h>

// namespace declarations
using std::string;
using std::wstring;


// Constructor
CNumEdit::CNumEdit(CWindow* parent, unsigned msgid)
: CEdit(parent),
  mBuddy(this, IDBUDDY)
{
	SetMessageID(msgid);
	Resize(WIDTH, GetTextHeight() + 2 * BORDER_WIDTH);
	mBuddy.Resize(mBuddy.GetWidth(), mHeight - 2 * BORDER_WIDTH);
	SetStyle(RIGHT | YCENTER | RBUDDY);
	CColor col(TC_BACKGND);
	mAttributes.background_pixel = col.GetPixel();
	ChangeAttributes(CWBackPixel);
	SetFocusable(true);
	mCursor = XCreateFontCursor(cxxx::cxDpy, XC_xterm);
	XDefineCursor(cxxx::cxDpy, GetXID(), mCursor);
	SetValue(0);
	mCaret.SetBlinkRate(500);
	mCaret.HideCaret();
	mBuddy.Show();
	SetCaretPos(0);
}

/* virtual */ void CNumEdit::OnConfigure(XConfigureEvent& e)
{
	if (mStyle & BUDDY) {
		mBuddy.Resize(mBuddy.GetWidth(), mHeight - 2 * BORDER_WIDTH);
		if (mStyle & RBUDDY) mBuddy.Move(e.width - mBuddy.GetWidth() - BORDER_WIDTH, BORDER_WIDTH);
		else mBuddy.Move(BORDER_WIDTH, BORDER_WIDTH);
	}
}

/* virtual */ void CNumEdit::OnMouseWheelUp(XButtonEvent& e)
{
	mBuddy.OnMouseWheelUp(e);
}

/* virtual */ void CNumEdit::OnMouseWheelDown(XButtonEvent& e)
{
	mBuddy.OnMouseWheelDown(e);
}

/* virtual */ void CNumEdit::OnMessage(unsigned msgid, TMessage type, unsigned value)
{
	switch (msgid) {
	case IDBUDDY:
		SetValue(value);
		PostMessage(MT_SCROLL, value);
		break;
	case IDTIMER:
		if (type == MT_TIMER) ScrollText();
		break;
	default:
		break;
	}
}

/* virtual */ void CNumEdit::OnKey(wchar_t key, unsigned mask)
{
	std::string tmp = GetText();
	std::string tmp2;
	CEdit::OnKey(key, mask);
	tmp2 = GetText();
	if (tmp2 != tmp) {
		std::istringstream is(tmp2);
		int val;
		is >> val;
		mBuddy.SetPosition(val);
		PostMessage(MT_SCROLL, GetValue());
	}
}

/* virtual */ void CNumEdit::Enable(bool enable)
{
	if (enable == IsEnabled()) return;
	CWindow::Enable(enable);
	CColor col;
	if (enable) col = CColor(TC_BACKGND);
	else col = CColor(TC_FRAME);
	mAttributes.background_pixel = col.GetPixel();
	ChangeAttributes(CWBackPixel);
	SetFocusable(enable);
	RedrawArea(BORDER_WIDTH, BORDER_WIDTH, mWidth - 2 * BORDER_WIDTH, mHeight - 2 * BORDER_WIDTH);
}
