/*----------------------------------------------------------------------------
  Project: cxxx <http://cxplorer.sourceforge.net>
  File   : CNumEdit.h
  Info   : $Id: CNumEdit.h,v 1.19 2012/08/06 17:36:40 gkraeuter Exp $
----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
  Copyright © 2005 - 2012 Gerhard Kräuter <gkraeuter@users.sourceforge.net>

  This  software  is  provided  'as-is',  without  any  express  or  implied
  warranty.  In  no  event  will  the authors be held liable for any damages
  arising from the use of this software.

  Permission  is  granted  to  anyone  to use this software for any purpose,
  including  commercial  applications,  and  to alter it and redistribute it
  freely, subject to the following restrictions:

  1. The  origin  of  this software must not be misrepresented; you must not
     claim  that  you  wrote the original software. If you use this software
     in  a  product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.

----------------------------------------------------------------------------*/

#ifndef CNumEdit_h
#define CNumEdit_h

// C++X headers
#include <cxxx/CBuddy.h>
#include <cxxx/CCaret.h>
#include <cxxx/CEdit.h>

// C++ headers
#include <sstream>

class CNumEdit : public CEdit
{
private:
	enum
	{
		// Geometry
		WIDTH = 50,
		BORDER_WIDTH = 2,

		// Message IDs
		IDBUDDY = cxxx::msgid::MSGID_CNumEdit,
		IDTIMER
	};

public:
	enum
	{
		// Styles
		LBUDDY  = 1 << 7,
		RBUDDY  = 1 << 8,
		BUDDY	= LBUDDY | RBUDDY
	};

public:
	// Constructor
	CNumEdit(CWindow* parent, unsigned msgid);

	// X event processing
	virtual void OnKey(wchar_t key, unsigned mask);
	virtual void OnConfigure(XConfigureEvent& e);
	virtual void OnMouseWheelUp(XButtonEvent& e);
	virtual void OnMouseWheelDown(XButtonEvent& e);

	// C++X message processing
	virtual void OnMessage(unsigned msgid, TMessage type, unsigned value);

	// Enables or disables the window
	virtual void Enable(bool enable);

	// Sets the text of the field
	void SetValue(int value);
	
	// Returns the value of the textfield
	int GetValue();
		
	// Sets the style of the field
	void SetStyle(unsigned style)
	{
		if (style & RBUDDY) {
			SetMargin(0, mBuddy.GetWidth());
			mBuddy.Move(mWidth - mBuddy.GetWidth() - BORDER_WIDTH, BORDER_WIDTH);
			mBuddy.Show();
		} else if (style & LBUDDY) {
			SetMargin(mBuddy.GetWidth(), 0);
			mBuddy.Move(BORDER_WIDTH, BORDER_WIDTH);
			mBuddy.Show();
		} else mBuddy.Hide();
		CEdit::SetStyle(style | NUMERIC);
	}

	// Sets the range
	void SetRange(int min, int max)
	{
		mBuddy.SetRange(min, max);
 	}

	// Sets the step width
	void SetStep(int step)
	{
		mBuddy.SetStep(step);
	}

protected:
	CBuddy mBuddy;
};

inline void CNumEdit::SetValue(int value)
{
	mBuddy.SetPosition(value);
	mSelMin = mSelMax;
	std::ostringstream os;
	os << GetValue();
	SetText(os.str());
	PostMessage(MT_SCROLL, GetValue());
	RedrawArea(BORDER_WIDTH, BORDER_WIDTH, mWidth - 2 * BORDER_WIDTH, mHeight - 2 * BORDER_WIDTH);
}

inline int CNumEdit::GetValue()
{
	return mBuddy.GetPosition();
}

#endif // CNumEdit_h
