/*----------------------------------------------------------------------------
  Project: cxxx <http://cxplorer.sourceforge.net>
  File   : CPopup.h
  Info   : $Id: CPopup.h,v 1.10 2012/08/06 17:36:40 gkraeuter Exp $
----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
  Copyright © 2002 - 2012 Gerhard Kräuter <gkraeuter@users.sourceforge.net>

  This  software  is  provided  'as-is',  without  any  express  or  implied
  warranty.  In  no  event  will  the authors be held liable for any damages
  arising from the use of this software.

  Permission  is  granted  to  anyone  to use this software for any purpose,
  including  commercial  applications,  and  to alter it and redistribute it
  freely, subject to the following restrictions:

  1. The  origin  of  this software must not be misrepresented; you must not
     claim  that  you  wrote the original software. If you use this software
     in  a  product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.

----------------------------------------------------------------------------*/

#ifndef CPopup_h
#define CPopup_h

// C++X headers
#include <cxxx/CWindow.h>
#include <cxxx/CMenuItem.h>

// Forward declarations
class CMenuBar;
class CTopLevelWindow;

class CPopup : public CWindow
{
private:
	enum
	{
		// Message IDs
		IDTIMER = cxxx::msgid::MSGID_CPopup,

		// Geometry
		BORDER_WIDTH = 24,
		BORDER_HEIGHT = 4,
		SPACE = 4,
		ARROW_WIDTH = 16,
		ICON_WIDTH = 20
    };

public:
    enum
    {
		// Positioning hint
		DOWN,
		LEFT,
		RIGHT
    };

public:
	// Constructor
	CPopup(CWindow* w, CMenu& m);

	// Destructor
	virtual ~CPopup();

	// X event processing
	virtual void OnExpose(XExposeEvent& e);
	virtual void OnLButtonDown(XButtonEvent& e);
	virtual void OnButtonUp(XButtonEvent& e);
	virtual void OnEvent(XEvent& e);
	virtual void OnAccelerator(wchar_t key);

	// C++X message processing
	virtual void OnMessage(unsigned msgid, TMessage type, unsigned value);

	// Draws menu entry with index idx
	void DrawMenu(unsigned idx, bool selected);

	// Positions this popup around rectangle x, y, w, h and use positioning hint ph
	void Position(int x, int y, int w, int h, unsigned ph);
	
	// Selects menu entry with index i
	void Select(unsigned i)
	{
		if (mSelected == i) return ;
		mTimer.Stop();
		if (mChild) {
			delete mChild;
			mChild = NULL;
		}
		if (mSelected != -1u) DrawMenu(mSelected, false);
		mSelected = i;
		if (mSelected != -1u) DrawMenu(mSelected, true);
		if (i != -1u && mMenu[i].IsEnabled() && mMenu[i].HasSubmenu()) mTimer.SetTime(400);
	}
	
protected:
	// Returns the index of the menu entry at screen position pos
	unsigned GetItemFromPos(int pos)
	{
		int y = BORDER_HEIGHT;
		if (pos < y) return -1u;
		for (unsigned i = 0; i < mMenu.GetSize(); ++i) {
			if (mMenu[i].IsSeperator()) y += 4;
			else y += 20;
			if (pos < y) return i;
		}
		return -1u;
	}

	// Calculates the screen position of menu entry with index idx
	int CalculatePosition(unsigned idx)
	{
		int y = BORDER_HEIGHT;
		for (unsigned i = 0; i < idx; ++i) {
			cxAssert(i < mMenu.GetSize());
			if (mMenu[i].IsSeperator()) y += 4;
			else y += 20;
		}
		return y;
	}

protected:
	CMenu& mMenu;
	unsigned mSelected;
	unsigned mTextWidth;
	int mTextYPos;
	unsigned mPosHint;
	Cursor mCursor;
	CIcon mArrow;
	CIcon mBullet;
	CIcon mCheck;
	CTimer mTimer;
	CPopup* mChild;
};

#endif // CPopup_h
