/*----------------------------------------------------------------------------
  Project: cxxx <http://cxplorer.sourceforge.net>
  File   : CProfile.cpp
  Info   : $Id: CProfile.cpp,v 1.25 2012/08/06 17:36:40 gkraeuter Exp $
----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
  Copyright © 2002 - 2012 Gerhard Kräuter <gkraeuter@users.sourceforge.net>

  This  software  is  provided  'as-is',  without  any  express  or  implied
  warranty.  In  no  event  will  the authors be held liable for any damages
  arising from the use of this software.

  Permission  is  granted  to  anyone  to use this software for any purpose,
  including  commercial  applications,  and  to alter it and redistribute it
  freely, subject to the following restrictions:

  1. The  origin  of  this software must not be misrepresented; you must not
     claim  that  you  wrote the original software. If you use this software
     in  a  product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.

----------------------------------------------------------------------------*/

// cxxx headers
#include <cxxx/CProfile.h>
#include <cxxx/CApplication.h>
#include <cxxx/CError.h>

// C++ headers
#include <fstream>
#include <cstdio>

// namespace declarations
using std::ifstream;
using std::ofstream;
using std::string;
using cxxx::cxError;
using cxxx::lib::cxgettext;

void CProfile::Flush()
{
	if (mChanged == false) return ;
	string old = mFile;
	old += ".bak";
	if (rename(mFile.c_str(), old.c_str()))	cxError << _("Failed to rename configuration file {1}.", 96) << mFile << CError::Error;
	ofstream out(mFile.c_str());
	if(!out) {
		cxError << _("Failed to open {1} for output", 93) << mFile << CError::Error;
		return;
	}
	for (TContainer::iterator i = mArray.begin(); i != mArray.end(); ++i) out << *i << "\n";
	mChanged = false;
	if (remove(old.c_str())) cxError << _("Failed to remove old configuration file {1}.", 95) << mFile << CError::Error;
}

void CProfile::ReadFile(const string& file)
{
	Flush();
	mArray.clear();
	mFile = file;
	ifstream in(file.c_str());
	if (!in) {
		cxError << _("Failed to open configuration file {1}.", 94) << file << CError::Error;
		return ;
	}
	string s;
	while(!in.eof()) {
		getline(in, s);
		if (!s.empty() && s[0] != '#' && s[0] != '[') mArray.push_back(s);
	}
}

string CProfile::Read(const string& entry, const string& defval)
{
	TContainer::iterator i = FindEntry(entry);
	if (i == mArray.end()) {
		Write(entry, defval, i);
		return defval;
	}
	return GetValue(i);
}

unsigned CProfile::Read(const string& entry, unsigned defval)
{
	TContainer::iterator i = FindEntry(entry);
	if (i == mArray.end()) {
		Write(entry, ToString(defval), i);
		return defval;
	}

	return ToUint(GetValue(i));
}

void CProfile::Write(const string& entry, const string& value)
{
	TContainer::iterator i = FindEntry(entry);
	if (i != mArray.end() && GetValue(i) == value) return;
	else Write(entry, value, i);
}

void CProfile::Write(const string& entry, unsigned value)
{
	TContainer::iterator i = FindEntry(entry);
	if (i != mArray.end() && ToUint(GetValue(i)) == value) return;
	else Write(entry, ToString(value), i);
}
