/*----------------------------------------------------------------------------
  Project: cxxx <http://cxplorer.sourceforge.net>
  File   : CProfile.h
  Info   : $Id: CProfile.h,v 1.11 2012/08/06 17:36:40 gkraeuter Exp $
----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
  Copyright © 2002 - 2012 Gerhard Kräuter <gkraeuter@users.sourceforge.net>

  This  software  is  provided  'as-is',  without  any  express  or  implied
  warranty.  In  no  event  will  the authors be held liable for any damages
  arising from the use of this software.

  Permission  is  granted  to  anyone  to use this software for any purpose,
  including  commercial  applications,  and  to alter it and redistribute it
  freely, subject to the following restrictions:

  1. The  origin  of  this software must not be misrepresented; you must not
     claim  that  you  wrote the original software. If you use this software
     in  a  product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.

----------------------------------------------------------------------------*/

#ifndef CProfile_h
#define CProfile_h

// C++X headers
#include <cxxx/Cxxx.h>

// C++ headers
#include <vector>

class CProfile
{
public:
	// Type definition
	typedef std::vector<std::string> TContainer;

	// Constructor
	CProfile()
	: mChanged(false)
	{
	}

	CProfile(const std::string& file)
	: mChanged(false)
	{
		cxAssert(!file.empty());
		ReadFile(file);
	}

	// Returns the value of entry
	unsigned Read(const std::string& entry, unsigned def);
	std::string Read(const std::string& entry, const std::string& def);

	// Writes val into entry
	void Write(const std::string& entry, unsigned val);
	void Write(const std::string& entry, const std::string& val);

	// Returns the file name of this profile file
	const std::string& GetFileName() const
	{
		return mFile;
	}

	// Sets the file name of this profile file
	void SetFileName(const std::string& file)
	{
		mFile = file;
	}
	
	// Loads file
	void ReadFile(const std::string& file);
	
	// Returns true if this profile is valid
	bool IsValid() const
	{
		if (mFile.empty()) return false;
		return !mArray.empty();
	}
	
	// Writes all entries to disc
	void Flush();
	
protected:
	// Returns the value of an entry
	std::string GetValue(TContainer::iterator i)
	{
		if (i->empty()) return "";
		std::string::size_type idx = i->find('=');
		if (idx++ > i->size() - 2) return "";
		return i->substr(idx);
	}

	// Searches for entry
	TContainer::iterator FindEntry(const std::string& entry)
	{
		cxAssert(entry.empty() == false);
		TContainer::iterator i;
		for (i = mArray.begin(); i != mArray.end(); ++i) {
			if ((i->size() > entry.size()) && (*i)[entry.size()] == '=' && i->compare(0, entry.size(),entry) == 0) return i;
		}
		return mArray.end();
	}

	// Writes entry to mArray
	void Write(const std::string& entry, const std::string& value, TContainer::iterator iter)
	{
		mChanged = true;
		std::string s = entry + "=" + value;
		if (iter == mArray.end()) mArray.push_back(s);
		else *iter = s;
	}

	// Converts s to unsigned int
	unsigned ToUint(const std::string& s)
	{
		std::istringstream is(s);
		unsigned ret = 0;
		if (s.size() > 2 && s[0] == '0' && (s[1] == 'x' || s[1] == 'X')) is >> std::hex;
		is >> ret;
		return ret;
	}

	// Converts i to string
	std::string ToString(unsigned i)
	{
		std::ostringstream os;
		os.imbue(std::locale::classic());
		os << "0x" << std::hex << i;
		return os.str();
	}
		
protected:
	std::string mFile;
	TContainer mArray;
	bool mChanged;
};

#endif // CProfile_h
