/*----------------------------------------------------------------------------
  Project: cxxx <http://cxplorer.sourceforge.net>
  File   : CPropertyTab.h
  Info   : $Id: CPropertyTab.h,v 1.10 2012/08/06 17:36:40 gkraeuter Exp $
----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
  Copyright © 2002 - 2012 Gerhard Kräuter <gkraeuter@users.sourceforge.net>

  This  software  is  provided  'as-is',  without  any  express  or  implied
  warranty.  In  no  event  will  the authors be held liable for any damages
  arising from the use of this software.

  Permission  is  granted  to  anyone  to use this software for any purpose,
  including  commercial  applications,  and  to alter it and redistribute it
  freely, subject to the following restrictions:

  1. The  origin  of  this software must not be misrepresented; you must not
     claim  that  you  wrote the original software. If you use this software
     in  a  product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.

----------------------------------------------------------------------------*/

#ifndef CPropertyTab_h
#define CPropertyTab_h

// C++X headers
#include <cxxx/CWindow.h>
#include <cxxx/CListItem.h>

class CPropertyTab : public CWindow
{
private:
	// Type definition
	typedef std::vector<CListItem*> TTabs;

public:
	// Constructor
	CPropertyTab(CWindow* parent, unsigned msgid)
	: CWindow(parent, msgid),
	  mSelection(mTab.begin())
	{
		SetFocusable(false);
	}

	// Destructor
	virtual ~CPropertyTab();

	// X event processing
	virtual void OnExpose(XExposeEvent& e);
	virtual void OnLButtonDown(XButtonEvent& e);
	virtual void OnMouseWheelUp(XButtonEvent& e);
	virtual void OnMouseWheelDown(XButtonEvent& e);

	// Adds a tab
	void AddTab(CIcon::TIconName icon, const std::string& text)
	{
		unsigned w = GetTextWidth(cxxx::cxWiden(text));
		CIcon ico;
		ico.Create(icon, 16);
		CListItem* i = new CListItem(ico, text, w, GetTextHeight());
		mTab.push_back(i);
		mSelection = mTab.begin();
	}
	
	void AddTab(const std::string& text)
	{
		unsigned w = GetTextWidth(cxxx::cxWiden(text));
		CIcon ico;
		CListItem* i = new CListItem(ico, text, w, GetTextHeight());
		mTab.push_back(i);
		mSelection = mTab.begin();
	}

	// Returns the index of the current tab
	unsigned GetSelection()
	{
		return mSelection - mTab.begin();
	}

protected:
	TTabs mTab;
	TTabs::iterator mSelection;
};

#endif // CPropertyTab_h
