/*----------------------------------------------------------------------------
  Project: cxxx <http://cxplorer.sourceforge.net>
  File   : CRadioButton.cpp
  Info   : $Id: CRadioButton.cpp,v 1.17 2012/06/26 08:05:52 gkraeuter Exp $
----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
  Copyright © 2002 - 2012 Gerhard Kräuter <gkraeuter@users.sourceforge.net>

  This  software  is  provided  'as-is',  without  any  express  or  implied
  warranty.  In  no  event  will  the authors be held liable for any damages
  arising from the use of this software.

  Permission  is  granted  to  anyone  to use this software for any purpose,
  including  commercial  applications,  and  to alter it and redistribute it
  freely, subject to the following restrictions:

  1. The  origin  of  this software must not be misrepresented; you must not
     claim  that  you  wrote the original software. If you use this software
     in  a  product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.

----------------------------------------------------------------------------*/

// C++X headers
#include <cxxx/CRadioButton.h>

// Namespace declarations
using std::string;

/* virtual */ CRadioButton::~CRadioButton()
{
	mGroup->Unregister(this);
}

/* virtual */ void CRadioButton::OnExpose(XExposeEvent& e __attribute__ ((unused)))
{
	DrawIcon(4, 4, mBright, CColor(TC_BRIGHT));
	DrawIcon(4, 4, mFrame, CColor(TC_FRAME));
	DrawIcon(4, 4, mDark, CColor(TC_DARK));
	DrawIcon(4, 4, mBlack, CColor(TC_BLACK));
	int y = (20 - GetTextHeight()) / 2;
	if (!mName.empty()) DrawHotString(20, y, CColor(TC_FRAME_TEXT), mName, mHotCharPos, !IsEnabled());
	DrawFocusRect();
	UpdateState();
}

/* virtual */ void CRadioButton::OnLButtonDown(XButtonEvent& e __attribute__ ((unused)))
{
	SetCheck(true);
	PostMessage(MT_CHECK);
}

/* virtual */ void CRadioButton::OnFocus(bool in __attribute__ ((unused)))
{
	DrawFocusRect();
}

/* virtual */ void CRadioButton::OnAccelerator(wchar_t key __attribute__ ((unused)))
{
	SetFocus();
	SetCheck(true);
	PostMessage(MT_CHECK);
}

/* virtual */ void CRadioButton::SetName(const string& name)
{
	CButton::SetName(name);
	Resize(25 + GetTextWidth(mName), 20);
	Redraw();
}

/* virtual */ void CRadioButton::UpdateState()
{
	CColor col;
	if (IsEnabled()) col = CColor(TC_BACKGND);
	else col =CColor(TC_FRAME);
	DrawIcon(6, 6, mWhite, col);
	if (IsChecked()) {
		if (IsEnabled()) col = CColor(TC_BACKGND_TEXT);
		else col = CColor(TC_FRAME_TEXT);
		DrawIcon(8, 8, mCheck, col);
	}
}
