/*----------------------------------------------------------------------------
  Project: cxxx <http://cxplorer.sourceforge.net>
  File   : CRadioButton.h
  Info   : $Id: CRadioButton.h,v 1.14 2012/06/08 13:03:40 gkraeuter Exp $
----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
  Copyright © 2002 - 2012 Gerhard Kräuter <gkraeuter@users.sourceforge.net>

  This  software  is  provided  'as-is',  without  any  express  or  implied
  warranty.  In  no  event  will  the authors be held liable for any damages
  arising from the use of this software.

  Permission  is  granted  to  anyone  to use this software for any purpose,
  including  commercial  applications,  and  to alter it and redistribute it
  freely, subject to the following restrictions:

  1. The  origin  of  this software must not be misrepresented; you must not
     claim  that  you  wrote the original software. If you use this software
     in  a  product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.

----------------------------------------------------------------------------*/

#ifndef CRadioButton_h
#define CRadioButton_h

// C++X headers
#include <cxxx/CButton.h>
#include <cxxx/CRadioGroup.h>

class CRadioButton : public CButton
{
public:
	// Constructor
	CRadioButton(CRadioGroup* parent, unsigned msgid)
	: CButton(parent->GetParent(), msgid)
	{
		mChecked = false;
		mGroup = parent;
		mGroup->Register(this);
		mCheck.Create(CIcon::BULLET4, 0);
		mBright.Create(CIcon::RADIOBRIGHT, 0);
		mFrame.Create(CIcon::RADIOFRAME, 0);
		mDark.Create(CIcon::RADIODARK, 0);
		mBlack.Create(CIcon::RADIOBLACK, 0);
		mWhite.Create(CIcon::BULLET8, 0);
		Resize(20, 20);
	}

	// Destructor
	virtual ~CRadioButton();

	// X event processing
	virtual void OnExpose(XExposeEvent& e);
	virtual void OnLButtonDown(XButtonEvent& e);
	virtual void OnFocus(bool in);
	virtual void OnAccelerator(wchar_t key);

	// Sets the name of the button
	virtual void SetName(const std::string& name);

	// Sets check mark to this button
	void SetCheck(bool check)
	{
		if (mChecked == check) return ;
		mChecked = check;
		UpdateState();
		if (mChecked == true) mGroup->OnCheck(this);
	}

	// Returns whether this button is checked
	bool IsChecked() const
	{
		return mChecked;
	}

protected:
	// Updates the state
	virtual void UpdateState();

	// Draws focus rectangle
	void DrawFocusRect()
	{
		if (IsFocused()) DrawDottedRectangle(0, 0, GetWidth() - 1, GetHeight() - 1, CColor(TC_FRAME_TEXT));
		else DrawRectangle(0, 0, GetWidth() - 1, GetHeight() - 1, CColor(TC_FRAME));
	}

protected:
	bool mChecked;
	CIcon mCheck;
	CIcon mBright;
	CIcon mFrame;
	CIcon mDark;
	CIcon mBlack;
	CIcon mWhite;
	CRadioGroup* mGroup;
};

#endif // CRadioButton_h
