/*----------------------------------------------------------------------------
  Project: cxxx <http://cxplorer.sourceforge.net>
  File   : CRadioGroup.h
  Info   : $Id: CRadioGroup.h,v 1.8 2009/12/28 13:27:55 gkraeuter Exp $
----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
  Copyright © 2002 - 2009 Gerhard Kräuter <gkraeuter@users.sourceforge.net>

  This  software  is  provided  'as-is',  without  any  express  or  implied
  warranty.  In  no  event  will  the authors be held liable for any damages
  arising from the use of this software.

  Permission  is  granted  to  anyone  to use this software for any purpose,
  including  commercial  applications,  and  to alter it and redistribute it
  freely, subject to the following restrictions:

  1. The  origin  of  this software must not be misrepresented; you must not
     claim  that  you  wrote the original software. If you use this software
     in  a  product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.

----------------------------------------------------------------------------*/

#ifndef CRadioGroup_h
#define CRadioGroup_h

// C++X headers
#include <cxxx/CMessageTransceiver.h>

// Forward declarations
class CWindow;
class CRadioButton;

class CRadioGroup
{
public:
	// Constructor
    CRadioGroup(CWindow* parent)
	: mSelection(NULL),
	  mParent(parent)
	{
	}

	// Destructor
	~CRadioGroup()
	{
		cxAssert(mButtonArray.empty());
	}

	// Adds b to this group
	void Register(CRadioButton* b)
	{
		mButtonArray.push_back(b);
		mSelection = NULL;
	}

	// Removes b from this group
	void Unregister(CRadioButton* b)
	{
		std::vector<CRadioButton*>::iterator i = std::find(mButtonArray.begin(), mButtonArray.end(), b);
		cxAssert(i != mButtonArray.end());
		mButtonArray.erase(i);
		mSelection = NULL;
	}

	// Returns the parent window of this group
	CWindow* GetParent() const
	{
		return mParent;
	}

	// Sets check mark on b, and removes check mark on all other buttons
	void OnCheck(CRadioButton* b);
protected:
	std::vector<CRadioButton*> mButtonArray ;
	CRadioButton* mSelection;
	CWindow* mParent;
};

#endif // CRadioGroup_h
