/*----------------------------------------------------------------------------
  Project: cxxx <http://cxplorer.sourceforge.net>
  File   : CScrollBar.cpp
  Info   : $Id: CScrollBar.cpp,v 1.16 2012/08/06 17:36:40 gkraeuter Exp $
----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
  Copyright © 2002 - 2012 Gerhard Kräuter <gkraeuter@users.sourceforge.net>

  This  software  is  provided  'as-is',  without  any  express  or  implied
  warranty.  In  no  event  will  the authors be held liable for any damages
  arising from the use of this software.

  Permission  is  granted  to  anyone  to use this software for any purpose,
  including  commercial  applications,  and  to alter it and redistribute it
  freely, subject to the following restrictions:

  1. The  origin  of  this software must not be misrepresented; you must not
     claim  that  you  wrote the original software. If you use this software
     in  a  product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.

----------------------------------------------------------------------------*/

// C++X headers
#include <cxxx/CScrollBar.h>

// Namespace declarations
using cxxx::cxDpy;

CScrollBar::CScrollBar(CWindow* parent, unsigned msgid)
: CWindow(parent, msgid),
  mPage(20),
  mRange(100),
  mPosition(0),
  mTBStart(0),
  mTBEnd(0),
  mTimer(this, IDTIMER),
  mState1(false),
  mState2(false),
  mTracking(false),
  mEnabled(false),
  mOffset(0)
{
	mAttributes.event_mask |= ButtonPressMask | ButtonReleaseMask | LeaveWindowMask;
	ChangeAttributes(CWEventMask);
}

/* virtual */ void CScrollBar::OnMessage(unsigned msgid, TMessage type, unsigned value __attribute__ ((unused)))
{
	switch (msgid) {
	case IDTIMER:
		switch (type) {
		case MT_TIMER:
			if (mState1) {
				if (mPosition == 0) return ;
				mTimer.SetTime(100);
				PosDec();
			} else if (mState2) {
				if (mPosition == mRange - mPage) return ;
				mTimer.SetTime(100);
				PosInc();
			}
			break;
		default:
			break;
		}
		break;
	default:
		break;
	}
}

void CScrollBar::DrawButton(int x, int y, int wh, bool pressed, const CIcon& icon)
{
	TFrameStyle style;
	if (pressed) style = FS_SUNKEN_THICK;
	else style = FS_RAISED_THICK;
	int off = (wh - 7) / 2;
	DrawFrame(x, y, wh, wh, style, true);
	DrawIcon(x + off, y + off, icon, CColor(TC_FRAME_TEXT), !IsEnabled());
}

/* virtual */ void CHScrollBar::OnExpose(XExposeEvent& e __attribute__ ((unused)))
{
	DrawButton(0, 0, mHeight, mState1, mArrow1);
	DrawButton(mWidth - mHeight, 0, mHeight, mState2, mArrow2);
	DrawTrackBar();
}

/* virtual */ void CVScrollBar::OnExpose(XExposeEvent& e __attribute__ ((unused)))
{
	DrawButton(0, 0, mWidth, mState1, mArrow1);
	DrawButton(0, mHeight - mWidth, mWidth, mState2, mArrow2);
	DrawTrackBar();
}

/* virtual */ void CHScrollBar::OnLButtonDown(XButtonEvent& e)
{
	if (mTracking) {
		mTracking = false;
		mAttributes.event_mask = ExposureMask | StructureNotifyMask | ButtonReleaseMask | ButtonPressMask;
		ChangeAttributes(CWEventMask);
	}
	if (e.x < static_cast<int>(mHeight)) {
		mState1 = true;
		DrawFrame(0, 0, mHeight, mHeight, FS_SUNKEN_THICK, false);
		PosDec();
		mTimer.SetTime(500);
		return ;
	}
	if (e.x >= static_cast<int>(mWidth - mHeight)) {
		mState2 = true;
		DrawFrame(mWidth - mHeight, 0, mHeight, mHeight, FS_SUNKEN_THICK, false);
		PosInc();
		mTimer.SetTime(500);
		return ;
	}
	if (e.x < mTBStart) {
		PageDec();
		return ;
	}
	if (e.x >= mTBEnd) {
		PageInc();
		return ;
	}
	mTracking = true;
	mOffset = e.x - mTBStart + mHeight;
	XChangeActivePointerGrab(cxDpy, ButtonReleaseMask | Button1MotionMask, None, CurrentTime);
}

/* virtual */ void CVScrollBar::OnLButtonDown(XButtonEvent& e)
{
	if (mTracking) {
		mTracking = false;
		mAttributes.event_mask = ExposureMask | StructureNotifyMask | ButtonReleaseMask | ButtonPressMask;
		ChangeAttributes(CWEventMask);
	}
	if (e.y < static_cast<int>(mWidth)) {
		mState1 = true;
		DrawFrame(0, 0, mWidth, mWidth, FS_SUNKEN_THICK, false);
		PosDec();
		mTimer.SetTime(500);
		return ;
	}
	if (e.y >= static_cast<int>(mHeight - mWidth)) {
		mState2 = true;
		DrawFrame(0, mHeight - mWidth, mWidth, mWidth, FS_SUNKEN_THICK, false);
		PosInc();
		mTimer.SetTime(500);
		return ;
	}
	if (e.y < mTBStart) {
		PageDec();
		return ;
	}
	if (e.y >= mTBEnd) {
		PageInc();
		return ;
	}
	mTracking = true;
	mOffset = e.y - mTBStart + mWidth;
	XChangeActivePointerGrab(cxDpy, ButtonReleaseMask | Button1MotionMask, None, CurrentTime);
}

/* virtual */ void CVScrollBar::OnButtonUp(XButtonEvent& e)
{
	mTimer.Stop();
	if (e.button != 1) return ;
	if (mState1) {
		mState1 = false;
		DrawFrame(0, 0, mWidth, mWidth, FS_RAISED_THICK, false);
	}
	if (mState2) {
		mState2 = false;
		DrawFrame(0, mHeight - mWidth, mWidth, mWidth, FS_RAISED_THICK, false);
	}
	if (mTracking) {
		mTracking = false;
		int pos = CalculatePosition(e.y - mOffset);
		if (pos != mPosition) SetPosition(mPosition);
		mAttributes.event_mask = ExposureMask | StructureNotifyMask | ButtonReleaseMask | ButtonPressMask;
		ChangeAttributes(CWEventMask);
	}
}

/* virtual */ void CHScrollBar::OnButtonUp(XButtonEvent& e)
{
	mTimer.Stop();
	if (e.button != 1) return ;
	if (mState1) {
		mState1 = false;
		DrawFrame(0, 0, mHeight, mHeight, FS_RAISED_THICK, false);
	}
	if (mState2) {
		mState2 = false;
		DrawFrame(mWidth - mHeight, 0, mHeight, mHeight, FS_RAISED_THICK, false);
	}
	if (mTracking) {
		mTracking = false;
		int pos = CalculatePosition(e.x - mOffset);
		if (pos != mPosition) SetPosition(mPosition);
		mAttributes.event_mask = ExposureMask | StructureNotifyMask | ButtonReleaseMask | ButtonPressMask;
		ChangeAttributes(CWEventMask);
	}
}

/* virtual */ void CHScrollBar::OnEvent(XEvent& e)
{
	if (e.xany.type == MotionNotify) {
		if (!mTracking)	return ;
		int pos = CalculatePosition(e.xmotion.x - mOffset);
		if (pos != mPosition) SetPosition(pos);
		return ;
	}
}

/* virtual */ void CVScrollBar::OnEvent(XEvent& e)
{
	if (e.xany.type == MotionNotify) {
		if (!mTracking)	return ;
		int pos = CalculatePosition(e.xmotion.y - mOffset);
		if (pos != mPosition) SetPosition(pos);
		return ;
	}
}

/* virtual */ void CHScrollBar::DrawTrackBar()
{
	if (!IsVisible()) return ;
	if (( mRange > mPage) && IsEnabled() ) {
		int width = mWidth - 2 * mHeight;
		cxAssert(mRange > 0);
		int tb = (mPage * width) / mRange;
		if (tb < 6) tb = 6;
		width -= tb;
		cxAssert(mRange != mPage);
		int left = (mPosition * width) / (mRange - mPage);
		mTBStart = left + mHeight;
		mTBEnd = mTBStart + tb;
		DrawFrame(mTBStart, 0, tb, mHeight, FS_RAISED_THICK, true);
		FillRectangle(mHeight, 0, left, mHeight, CColor(TC_SCROLL));
		FillRectangle(mTBEnd, 0, width - left, mHeight, CColor(TC_SCROLL));
	} else {
		mTBStart = mHeight;
		mTBEnd = mWidth - mHeight;
		DrawFrame(mTBStart, 0, mTBEnd - mHeight, mHeight, FS_RAISED_THICK, true);
	}
}

/* virtual */ void CVScrollBar::DrawTrackBar()
{
	if (!IsVisible()) return ;
	if ((mRange > mPage) && IsEnabled()) {
		int height = mHeight - 2 * mWidth;
		cxAssert(mRange != 0);
		int tb = (mPage * height) / mRange;
		if (tb < 6) tb = 6;
		height -= tb;
		cxAssert(mRange != mPage);
		int up = (mPosition * height) / (mRange - mPage);
		mTBStart = up + mWidth;
		mTBEnd = mTBStart + tb;
		DrawFrame(0, mTBStart, mWidth, tb, FS_RAISED_THICK, true);
		FillRectangle(0, mWidth, mWidth, up, CColor(TC_SCROLL));
		FillRectangle(0, mTBEnd, mWidth, height - up, CColor(TC_SCROLL));
	} else {
		mTBStart = mWidth;
		mTBEnd = mHeight - mWidth;
		DrawFrame(0, mTBStart, mWidth, mTBEnd - mWidth, FS_RAISED_THICK, true);
	}
}
