/*----------------------------------------------------------------------------
  Project: cxxx <http://cxplorer.sourceforge.net>
  File   : CSelection.h
  Info   : $Id: CSelection.h,v 1.11 2012/08/06 17:36:40 gkraeuter Exp $
----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
  Copyright © 2003 - 2012 Gerhard Kräuter <gkraeuter@users.sourceforge.net>

  This  software  is  provided  'as-is',  without  any  express  or  implied
  warranty.  In  no  event  will  the authors be held liable for any damages
  arising from the use of this software.

  Permission  is  granted  to  anyone  to use this software for any purpose,
  including  commercial  applications,  and  to alter it and redistribute it
  freely, subject to the following restrictions:

  1. The  origin  of  this software must not be misrepresented; you must not
     claim  that  you  wrote the original software. If you use this software
     in  a  product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.

----------------------------------------------------------------------------*/

#ifndef CSelection_h
#define CSelection_h

// C++X headers
#include <cxxx/CTimer.h>

// C++ headers
#include <vector>

// Forward declarations
class CWindow;

class CSelection : public CMessageTransceiver
{
private:
	enum
	{
		// Message IDs
		IDTIMER = cxxx::msgid::MSGID_CSelection
	};

	struct CSelectionItem
	{
		Atom mName;
		Time mStartTime;
		Time mEndTime;
		CWindow* mOwner;
	};

public:
	// Constructor
	CSelection();

	// C++X message processing
	virtual void OnMessage(unsigned msgid, TMessage type, unsigned value);

	// Returns true if grabbing of the selection was successfull
	bool Grab(CWindow* w, Atom sel, Time time);

	// Releases the selection for sel
	void Release(Atom sel, Time time)
	{
		if (time == CurrentTime) time = cxxx::cxServerTime;
		for (std::vector<CSelectionItem>::iterator i = mSelectionList.begin(); i != mSelectionList.end(); ++i) {
			if (i->mName == sel) {
				i->mEndTime = time;
				break;
			}
		}
	}

	// Releases the selection for w
	void Release(CWindow* w)
	{
		for (std::vector<CSelectionItem>::iterator i = mSelectionList.begin(); i != mSelectionList.end(); ++i) {
			if (i->mOwner == w) {
				mSelectionList.erase(i);
				Release(w);
				break;
			}
		}
	}

	// Returns the owner of the selection with sel
	CWindow* GetOwner(Atom sel, Time time)
	{
		if (time == CurrentTime) time = cxxx::cxServerTime;
		for (std::vector<CSelectionItem>::iterator i = mSelectionList.begin(); i != mSelectionList.end(); ++i) {
			if (i ->mName == sel) {
			   int start = time - i->mStartTime;
				int end = i->mEndTime - time;
				if ((start  >= 0) && (end > 0)) return i->mOwner;
				else {
					if (cxxx::cxServerTime - i->mEndTime > 2000) mSelectionList.erase(i);
					break;
				}
			}
		}
		return NULL;
	}

	// X event processing
	void HandleRequest(XSelectionRequestEvent& e);

	void HandleClear(XSelectionClearEvent& e)
	{
		cxAssert(e.time != CurrentTime);
		Release(e.selection, e.time);
	}

	void HandleNotify(XSelectionEvent& e)
	{
		mSelectionNotify = e;
	}

	void HandleProperty(XPropertyEvent& e)
	{
		mPropertyNotify = e;
	}

	void CancelRequest(XSelectionRequestEvent& e)
	{
		XSelectionEvent ret;
		ret.type = SelectionNotify;
		ret.send_event = False;
		ret.display = cxxx::cxDpy;
		ret.requestor = e.requestor;
		ret.selection = e.selection;
		ret.target = e.target;
		ret.property = None;
		ret.time = e.time;
		XSendEvent(cxxx::cxDpy, e.requestor, False, NoEventMask, (XEvent*)&ret);
	}

	void ConfirmRequest(XSelectionRequestEvent& e)
	{
		XSelectionEvent ret;
		ret.type = SelectionNotify;
		ret.send_event = False;
		ret.display = cxxx::cxDpy;
		ret.requestor = e.requestor;
		ret.selection = e.selection;
		ret.target = e.target;
		ret.property = e.property;
		ret.time = e.time;
		XSendEvent(cxxx::cxDpy, e.requestor, False, NoEventMask, (XEvent*)&ret);
	}

	bool Request(CWindow* req, Atom sel, Atom typ, Atom prop, std::vector<unsigned char>& ret);

	// Initialize the selection manager
	bool Init(const std::string& cxxxdir);

	// Returns selection directories
	const std::string& GetClipSelDirectory() const
	{
		return mClipSelDir;
	}

	const std::string& GetDnDSelDirectory() const
	{
		return mDnDSelDir;
	}

protected:
	bool OnRequest(CWindow* owner, XSelectionRequestEvent& e);

public:
	static Atom TARGETS;
	static Atom INCR;
	static Atom MULTIPLE;
	static Atom TIMESTAMP;
	static Atom CLIPBOARD;
private:
	std::vector<CSelectionItem> mSelectionList;
	CTimer mTimer;
	bool mTimeout;
	bool mLocked;
	XSelectionEvent mSelectionNotify;
	XPropertyEvent mPropertyNotify;
	std::string mClipSelDir;
	std::string mDnDSelDir;
};

#endif // CSelection_h
