/*----------------------------------------------------------------------------
  Project: cxxx <http://cxplorer.sourceforge.net>
  File   : CSettings.h
  Info   : $Id: CSettings.h,v 1.12 2012/06/17 08:21:47 gkraeuter Exp $
----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
  Copyright © 2005 - 2012 Gerhard Kräuter <gkraeuter@users.sourceforge.net>

  This  software  is  provided  'as-is',  without  any  express  or  implied
  warranty.  In  no  event  will  the authors be held liable for any damages
  arising from the use of this software.

  Permission  is  granted  to  anyone  to use this software for any purpose,
  including  commercial  applications,  and  to alter it and redistribute it
  freely, subject to the following restrictions:

  1. The  origin  of  this software must not be misrepresented; you must not
     claim  that  you  wrote the original software. If you use this software
     in  a  product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.

----------------------------------------------------------------------------*/

#ifndef CSettings_h
#define CSettings_h

// C++X headers
#include <cxxx/CTopLevelWindow.h>
#include <cxxx/CPropertyTab.h>
#include <cxxx/CPushButton.h>
#include <cxxx/CTimingSheet.h>
#include <cxxx/CColorSheet.h>
#include <cxxx/CFontSheet.h>
#include <cxxx/CThemeSheet.h>
#include <cxxx/CProfile.h>

class CSettings : public CTopLevelWindow
{
public:
	enum 
	{
		// Return values
		OK,
		CANCEL
	};

private:
	enum
	{
		// Message IDs
		IDTAB = cxxx::msgid::MSGID_CSettings,
		IDOK,
		IDCANCEL,
		IDAPPLY
	};

public:
	// Constructor
    CSettings(CWindow* parent);

	// C++X message processing
	virtual void OnMessage(unsigned msgid, TMessage type, unsigned value);

	// Exit function
	virtual void OnQuit()
	{
		mReturn = CANCEL;
		mShutDown = true;
	}

protected:
	// Exit function
	void OnOK()
	{
		if (!mTiming.DoOK()) return;
		if (!mColors.DoOK()) return;
		if (!mFonts.DoOK()) return;
		if (!mThemes.DoOK()) return;
		cxxx::cxProfile->Flush();
		OnApply();
		mReturn = OK;
		mShutDown = true;
	}

	// Applies the settings
	void OnApply()
	{
		mTiming.DoOK();
		mColors.DoOK();
		mFonts.DoOK();
		mThemes.DoOK();
		RecolorWindows();
	}

protected:
	CPropertyTab mTab;
	CPushButton mOK;
	CPushButton mCancel;
	CPushButton mApply;
	CTimingSheet mTiming;
	CColorSheet mColors;
	CFontSheet mFonts;
	CThemeSheet mThemes;
};

#endif // CSettings_h
