/*----------------------------------------------------------------------------
  Project: cxxx <http://cxplorer.sourceforge.net>
  File   : CSlider.h
  Info   : $Id: CSlider.h,v 1.16 2012/08/06 17:36:40 gkraeuter Exp $
----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
  Copyright © 2005 - 2012 Gerhard Kräuter <gkraeuter@users.sourceforge.net>

  This  software  is  provided  'as-is',  without  any  express  or  implied
  warranty.  In  no  event  will  the authors be held liable for any damages
  arising from the use of this software.

  Permission  is  granted  to  anyone  to use this software for any purpose,
  including  commercial  applications,  and  to alter it and redistribute it
  freely, subject to the following restrictions:

  1. The  origin  of  this software must not be misrepresented; you must not
     claim  that  you  wrote the original software. If you use this software
     in  a  product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.

----------------------------------------------------------------------------*/

#ifndef CSlider_h
#define CSlider_h

// C++X headers
#include <cxxx/CWindow.h>

class CSlider : public CWindow
{
private:
	enum
	{
		// Geometry
		WIDTH = 120,
		HEIGHT = 20
	};

public:
	// Constructor
	CSlider(CWindow* parent, unsigned msgid)
	: CWindow(parent, msgid),
	  mTBStart(0),
	  mPage(20),
	  mMin(0),
	  mMax(100),
	  mPosition(0),
	  mTracking(false),
	  mOffset(0),
	  mStep(1)
	{
		mAttributes.event_mask |= ButtonPressMask | ButtonReleaseMask | LeaveWindowMask;
		ChangeAttributes(CWEventMask);
		Resize(WIDTH, HEIGHT);
	}

	// X event processing
	virtual void OnExpose(XExposeEvent& e);
	virtual void OnLButtonDown(XButtonEvent& e);
	virtual void OnButtonUp(XButtonEvent& e);
	virtual void OnMouseWheelUp(XButtonEvent& e);
	virtual void OnMouseWheelDown(XButtonEvent& e);
	virtual void OnEvent(XEvent& e);

	// Sets the range of the slider
	void SetRange(int min, int max)
	{
		mMin = std::min(min, max);
		mMax = std::max(min, max);
	}

	// Sets the page width 
	void SetPage(int page)
	{
		cxAssert(page < (mMax - mMin));
		mPage = page;
	}

	// Sets the step width
	void SetStep(int step)
	{
		cxAssert(step >= 1);
		mStep = step;
	}

	// Sets the position to pos
	void SetPosition(int pos)
	{
		if (pos > mMax)	pos = mMax;
		if (pos < mMin) pos = mMin;
		pos = ((2 * pos + mStep) / (2 * mStep)) * mStep;
		if (mPosition == pos) return;
		mPosition = pos;
		DrawTrackBar();
	}
	
	// Returns the position
	int GetPosition() const
	{
		return mPosition;
	}
	
	// Returns the step width
	int GetStep() const
	{
		return mStep;
	}
	
	// Enables or disables the slider
	void Enable(bool enable)
	{
		if (IsEnabled() == enable) return ;
		CWindow::Enable(enable);
		if (enable) Redraw();
		else mPosition = 0;
	}
	
protected:
	// Decrease the position by one page
	void PageDec()
	{
		SetPosition(mPosition - mPage);
		if (IsEnabled()) PostMessage(MT_SCROLL, mPosition);
	}

	// Increase the position by one page
	void PageInc()
	{
		SetPosition(mPosition + mPage);
		if (IsEnabled()) PostMessage(MT_SCROLL, mPosition);
	}

	// Draws the track bar and the background
	void DrawTrackBar();

	// Calculates the track bar position 
	int CalculatePosition(int x)
	{
		int width = mWidth - 24;
		int pos = (mMax - mMin) * (x - 2);
		pos += (mMax - mMin) / 2;
		return pos / width + mMin;
	}

protected:
	int mTBStart;
	int mPage;
	int mMin;
	int mMax;
	int mPosition;
	bool mTracking;
	int mOffset;
	int mStep;
};

#endif // CSlider_h
