/*----------------------------------------------------------------------------
  Project: cxxx <http://cxplorer.sourceforge.net>
  File   : CStatusBar.h
  Info   : $Id: CStatusBar.h,v 1.15 2012/05/06 18:57:36 gkraeuter Exp $
----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
  Copyright © 2002 - 2012 Gerhard Kräuter <gkraeuter@users.sourceforge.net>

  This  software  is  provided  'as-is',  without  any  express  or  implied
  warranty.  In  no  event  will  the authors be held liable for any damages
  arising from the use of this software.

  Permission  is  granted  to  anyone  to use this software for any purpose,
  including  commercial  applications,  and  to alter it and redistribute it
  freely, subject to the following restrictions:

  1. The  origin  of  this software must not be misrepresented; you must not
     claim  that  you  wrote the original software. If you use this software
     in  a  product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.

----------------------------------------------------------------------------*/

#ifndef CStatusBar_h
#define CStatusBar_h

// C++X headers
#include <cxxx/CWindow.h>

class CStatusField
{
public:
	// Constructor
	CStatusField(char pos, int width)
	: mDefaultWidth(width),
	  mPos(0),
	  mWidth(0)
	{
		if (pos == 'L' || pos == 'l') mLeftAlign = true;
		else mLeftAlign = false;
	}

public:
	std::wstring mText;
	bool mLeftAlign;
	int mDefaultWidth;
	int mPos;
	unsigned mWidth;
};	

class CStatusBar : public CWindow
{
private:
	enum 
	{
		// Geometry
		BORDER_WIDTH = 2,
		TEXT_BORDER = 2,
		HEIGHT = 22
	};

public:
	// Constructor
	CStatusBar(CWindow* parent)
	: CWindow(parent)
	{
		Resize(GetWidth(), HEIGHT);
	}

	// X event processing
	virtual void OnConfigure(XConfigureEvent& e);
	virtual void OnExpose(XExposeEvent& e);

	// Adds a field to the status bar
	unsigned AddField(char pos, int width)
	{
		mFields.push_back(CStatusField(pos, width));
		return mFields.size();
	}

	// Sets the text of the status field with index idx
	void SetText(unsigned idx, const std::string& text)
	{
		cxAssert(idx < mFields.size());
		std::wstring wt = cxxx::cxWiden(text);
		if (mFields[idx].mText != wt) {
			mFields[idx].mText = wt;
			DrawText(idx);
		}
	}

	// Draws the text of the status field with index idx 
	void DrawText(unsigned idx)
	{
			ClearArea(mFields[idx].mPos, 0, mFields[idx].mWidth, GetHeight()); 
			DrawString(mFields[idx].mPos + TEXT_BORDER, (GetHeight() - GetTextHeight()) / 2 + BORDER_WIDTH, CColor(TC_FRAME_TEXT), mFields[idx].mText, true);  
			DrawFrame(mFields[idx].mPos, BORDER_WIDTH, mFields[idx].mWidth, GetHeight() - BORDER_WIDTH, FS_SUNKEN, false);
	}

private:
	std::vector<CStatusField> mFields;
};

#endif // CStatusBar_h
