/*----------------------------------------------------------------------------
  Project: cxxx <http://cxplorer.sourceforge.net>
  File   : CThemeSheet.cpp
  Info   : $Id: CThemeSheet.cpp,v 1.7 2012/06/29 17:03:37 gkraeuter Exp $
----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
  Copyright © 2012 Gerhard Kräuter <gkraeuter@users.sourceforge.net>

  This  software  is  provided  'as-is',  without  any  express  or  implied
  warranty.  In  no  event  will  the authors be held liable for any damages
  arising from the use of this software.

  Permission  is  granted  to  anyone  to use this software for any purpose,
  including  commercial  applications,  and  to alter it and redistribute it
  freely, subject to the following restrictions:

  1. The  origin  of  this software must not be misrepresented; you must not
     claim  that  you  wrote the original software. If you use this software
     in  a  product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.

----------------------------------------------------------------------------*/

// C++X headers
#include <cxxx/CThemeSheet.h>
#include <cxxx/CProfile.h>

// C++ headers
#include <locale>

// C headers
#include <dirent.h>

// Namespace declarations
using cxxx::lib::cxgettext;
using cxxx::lib::cxProfile;
using cxxx::cxHomeDir;
using cxxx::cxFileManager;
using cxxx::cxNarrow;
using cxxx::cxWiden;
using std::set;
using std::string;
using std::wstring;
using std::vector;
using std::ctype;
using std::ctype_base;
using std::use_facet;
using std::locale;

CThemeSheet::CThemeSheet(CWindow* parent)
: CWindow(parent),
  mThemeList(this, IDTHEME),
  mTimer(this, IDTIMER)
{
	CreateThemeList();
	mThemes.insert("C++X");	
	unsigned sel = -1u;
	unsigned count = 0;
	for (set<string>::iterator i = mThemes.begin(); i != mThemes.end(); ++i, ++count) {
		mThemeList.AppendItem(*i);
		if (sel == -1u && *i == "C++X") sel = count;
		if (*i == CIcon::GetTheme()) sel = count;
	}
	mThemeList.Select(sel);
	OnMessage(IDTHEME, MT_SELECTION_CHANGED, 0);
	mThemeList.Move(10, 30);
	mThemeList.Resize(210, 20);
	mThemeList.Show();
}	

/* virtual */ void CThemeSheet::OnExpose(XExposeEvent& e __attribute__ ((unused)))
{
	CColor col(TC_FRAME_TEXT);
	DrawString(10, 20 - GetTextHeight() / 2, col, _("Icon Theme", 383));
	int x = 270;
	int y = 36;
	for (vector<CIcon>::iterator i = mIcon.begin(); i != mIcon.end(); ++i) {
		DrawIcon(x - i->GetWidth() / 2, y - i->GetHeight() / 2, *i);
		x += 52;
		if (x > 322) {
			x = 166;
			y += 52;
		}		
	}
	DrawFrame(0, 0, GetWidth(), GetHeight(), FS_SHEET, false);
}

const CIcon::TIconName CThemeSheet::mIconName[PREVIEW_SIZE + 1] =
{
	CIcon::FOLDER,
	CIcon::FOLDER,
	CIcon::EDITCUT,
	CIcon::EDITCUT,
	CIcon::FOLDER_OPEN,
	CIcon::FOLDER_OPEN,
	CIcon::EDITCOPY,
	CIcon::EDITCOPY,
	CIcon::TXT,
	CIcon::TXT,
	CIcon::EDITPASTE,
	CIcon::EDITPASTE,
	CIcon::EXEC,
	CIcon::EXEC,
	CIcon::TXT
};

const unsigned CThemeSheet::mIconSize[PREVIEW_SIZE + 1] = {16, 32, 16, 32, 16, 32, 16, 32, 16, 32, 16, 32, 16, 32, 0};

/* virtual */ void CThemeSheet::OnMessage(unsigned msgid, TMessage type __attribute__ ((unused)), unsigned value __attribute__ ((unused)))
{
	if (msgid == IDTHEME) {
		mTimer.SetTime(200);
		mIcon.clear();
		Redraw();
	} else if (msgid == IDTIMER) {
		mIcon = CIcon::PreviewCreate(mThemeList.GetText(), mIconName, mIconSize);
		Redraw();
	}
}

/* virtual */ bool CThemeSheet::DoOK()
{
	string theme = mThemeList.GetText();
	cxProfile->Write("Icon Theme", theme);
	CIcon::ForceTheme(theme);
	return true;
}	

const char* const CThemeSheet::mDirNames[DIR_NAME_SIZE] =
{
	"16x16",
	"22X22",
	"32x32",
	"48x48",
	"actions",
	"apps",
	"categories",
	"devices",
	"mimetypes",
	"places",
	"status",
	"stock"
};

void CThemeSheet::CreateThemeList()
{
	vector<string> themedirs;
	string tmp = cxHomeDir + "/.icons";
	themedirs.push_back(tmp);
	themedirs.push_back("/usr/local/share/icons");
	themedirs.push_back("/usr/share/icons");

	struct stat st;
	
	for (vector<string>::iterator i = themedirs.begin(); i != themedirs.end(); ++i) {
		if (stat(i->c_str(), &st) == 0 && S_ISDIR(st.st_mode)) {
			vector<string> themes = cxFileManager->ReadDirectory(*i);
			for (vector<string>::iterator j = themes.begin(); j != themes.end(); ++j) {
				tmp = *i + "/" + *j;
				if (IsValid(tmp)) {
					Capitalize(*j);
					if (mThemes.find(*j) == mThemes.end()) mThemes.insert(*j);
				}
			}
		}
	}
}

bool CThemeSheet::IsValid(const string& dir)
{
	DIR * d = opendir(dir.c_str());
	if (!d) return false;
 	dirent * ent;
	while ((ent = readdir(d))) {
		for (unsigned i = 0; i < DIR_NAME_SIZE; ++i) 
		if (strcmp (ent->d_name, mDirNames[i]) == 0) {
			closedir(d);
			return true;
		}
  	}
	closedir(d); 
	return false;
}

void CThemeSheet::Capitalize(string& s)
{
	wstring w = cxWiden(s);
	const ctype<wchar_t>& ct = use_facet<ctype<wchar_t> >(locale::classic());
	bool up = true;
	for (unsigned i = 0; i < w.size(); ++i) {
		if (!ct.is(ctype_base::alnum, w[i])) up = true;
		else if (up) {
			w[i] = ct.toupper(w[i]);
			up = false;
		} else w[i] = ct.tolower(w[i]);
	}
	s = cxNarrow(w);
}
