/*----------------------------------------------------------------------------
  Project: cxxx <http://cxplorer.sourceforge.net>
  File   : CThemeSheet.h
  Info   : $Id: CThemeSheet.h,v 1.3 2012/08/06 17:36:40 gkraeuter Exp $
----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
  Copyright © 2012 Gerhard Kräuter <gkraeuter@users.sourceforge.net>

  This  software  is  provided  'as-is',  without  any  express  or  implied
  warranty.  In  no  event  will  the authors be held liable for any damages
  arising from the use of this software.

  Permission  is  granted  to  anyone  to use this software for any purpose,
  including  commercial  applications,  and  to alter it and redistribute it
  freely, subject to the following restrictions:

  1. The  origin  of  this software must not be misrepresented; you must not
     claim  that  you  wrote the original software. If you use this software
     in  a  product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.

----------------------------------------------------------------------------*/

#ifndef CThemeSheet_h
#define CThemeSheet_h

// C++X headers
#include <cxxx/CPushButton.h>
#include <cxxx/CDropDownList.h>
#include <cxxx/CLabel.h>

// C++ headers
#include <set>

class CThemeSheet : public CWindow
{
private:
	enum
	{
		IDTHEME = cxxx::msgid::MSGID_CThemeSheet,
		IDTIMER,
		DIR_NAME_SIZE = 12,
		PREVIEW_SIZE = 14
	};
	
public:
	// Constructor
	CThemeSheet(CWindow* parent);
	
	// X event processing
	virtual void OnExpose(XExposeEvent& e);
	
	// C++X message processing
	virtual void OnMessage(unsigned msgid, TMessage type, unsigned value);

	// Prepares window to close down
	bool DoOK();

protected:
	// Lists the fonts into mThemeList
	void CreateThemeList();

	// Capitalizes initial letter(s) of s
	void Capitalize(std::string& s);

	// Returns whether dir contains theme data
	bool IsValid(const std::string& dir);

protected:
	std::set<std::string> mThemes;
	CDropDownList mThemeList;
	std::vector<CIcon> mIcon;
	CTimer mTimer;
	static const CIcon::TIconName mIconName[PREVIEW_SIZE + 1];
	static const unsigned mIconSize[PREVIEW_SIZE + 1];
	
	static const char* const mDirNames[DIR_NAME_SIZE];
};

#endif // CThemeSheet_h
