/*----------------------------------------------------------------------------
  Project: cxxx <http://cxplorer.sourceforge.net>
  File   : CTimer.h
  Info   : $Id: CTimer.h,v 1.12 2012/08/06 17:36:40 gkraeuter Exp $
----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
  Copyright © 2002 - 2012 Gerhard Kräuter <gkraeuter@users.sourceforge.net>

  This  software  is  provided  'as-is',  without  any  express  or  implied
  warranty.  In  no  event  will  the authors be held liable for any damages
  arising from the use of this software.

  Permission  is  granted  to  anyone  to use this software for any purpose,
  including  commercial  applications,  and  to alter it and redistribute it
  freely, subject to the following restrictions:

  1. The  origin  of  this software must not be misrepresented; you must not
     claim  that  you  wrote the original software. If you use this software
     in  a  product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.

----------------------------------------------------------------------------*/

#ifndef CTimer_h
#define CTimer_h

// C++X headers
#include <cxxx/CMessageTransceiver.h>

// C headers
#include <sys/time.h>

class CTimer : public CMessageTransceiver
{
public:
	// Constructor
	CTimer(CMessageTransceiver* target, unsigned msgid)
	: CMessageTransceiver(target, msgid)
	{
	}

	// Destructor
	virtual ~CTimer()
	{
		Stop();
	}

	// Stops the timer
	void Stop()
	{
		if (mTimerList.empty()) return;
		std::vector<CTimer*>::iterator i = std::find(mTimerList.begin(), mTimerList.end(), this);
		if (i != mTimerList.end()) mTimerList.erase(i);
	}

	// Sets the time in milliseconds
	void SetTime(unsigned tim)
	{
		Now();
		mTimeOut = mNow;
		mTimeOut.tv_sec += tim / 1000;
		mTimeOut.tv_usec += (tim % 1000) * 1000;
		if(mTimeOut.tv_usec >= 1000000) {
			mTimeOut.tv_usec -= 1000000;
			++mTimeOut.tv_sec;
		}
		std::vector<CTimer*>::iterator i = std::find(mTimerList.begin(), mTimerList.end(), this);
		if(i == mTimerList.end()) mTimerList.push_back(this);
	}

	// Sends a message
	void SendMessage()
	{
		CMessageTransceiver::SendMessage(MT_TIMER, 0);
	}

	// Tests all timers and returns the next timer expiry in t
    static void Refresh(timeval& t);

protected:
	// Returns the time left before the timer times out 
	int TimeLeft()
	{
		int sec = mTimeOut.tv_sec - mNow.tv_sec;
		int usec = mTimeOut.tv_usec - mNow.tv_usec;
		return sec * 1000 + usec / 1000;
	}

	// Returns the current time
	static void Now()
	{
		gettimeofday(&mNow, NULL);
	}

	// Tests timer 
	void TestTimer()
	{
		if (TimeLeft() <= 0) SendMessage();
		else mTimerList.push_back(this);
	}

protected:
	timeval mTimeOut;
	static timeval mNow;
	static std::vector<CTimer*> mTimerList;
};

#endif // CTimer_h
