/*----------------------------------------------------------------------------
  Project: cxxx <http://cxplorer.sourceforge.net>
  File   : CTimingSheet.cpp
  Info   : $Id: CTimingSheet.cpp,v 1.23 2012/08/06 17:36:40 gkraeuter Exp $
----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
  Copyright © 2005 - 2012 Gerhard Kräuter <gkraeuter@users.sourceforge.net>

  This  software  is  provided  'as-is',  without  any  express  or  implied
  warranty.  In  no  event  will  the authors be held liable for any damages
  arising from the use of this software.

  Permission  is  granted  to  anyone  to use this software for any purpose,
  including  commercial  applications,  and  to alter it and redistribute it
  freely, subject to the following restrictions:

  1. The  origin  of  this software must not be misrepresented; you must not
     claim  that  you  wrote the original software. If you use this software
     in  a  product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.

----------------------------------------------------------------------------*/

// C++X headers
#include <cxxx/CApplication.h>
#include <cxxx/CToolButton.h>
#include <cxxx/CTimingSheet.h>

// Namespace declarations
using cxxx::lib::cxgettext;
using cxxx::lib::cxProfile;
using cxxx::cxApp;

CTimingSheet::CTimingSheet(CWindow* parent)
: CWindow(parent),
  mDblClick(this, IDDBL_CLICK),
  mDelay(this, IDDELAY),
  mDblClickEdit(this, IDDBL_CLICK_EDIT),
  mDelayEdit(this, IDDELAY_EDIT),
  mDelayEnable(this, IDDELAY_ENABLE)
{
	unsigned dblclick = cxProfile->Read("Doubleclick Time", 400);
	unsigned tipdelay = cxProfile->Read("Tooltip Delay", 1000);
	mDblClick.SetRange(200, 2000);
	mDblClick.SetPage(500);
	mDblClick.SetStep(100);
	mDblClick.SetPosition(dblclick);
	mDblClickEdit.SetRange(200, 2000);
	mDblClickEdit.SetStep(100);
	mDblClickEdit.SetValue(dblclick);
	mDelay.SetRange(200, 5000);
	mDelay.SetPage(1000);
	mDelay.SetStep(100);
	mDelay.SetPosition(tipdelay);
	mDelayEdit.SetRange(200, 5000);
	mDelayEdit.SetStep(100);
	mDelayEdit.SetValue(tipdelay);
	mDelayEnable.SetName(_("Enable Tooltips", 413));
	if (CToolButton::GetDelay() != 0) mDelayEnable.SetCheck(true);
	mDblClick.Show();
	mDelay.Show();
	mDblClickEdit.Show();
	mDelayEdit.Show();
	mDelayEnable.Show();
}	

/* virtual */ void CTimingSheet::OnExpose(XExposeEvent& e __attribute__ ((unused)))
{
	unsigned off = (20 - GetTextHeight()) / 2;
	CColor col(TC_FRAME_TEXT);
	DrawString(20, off + 10, col, _("Double click interval [ms]", 411));
	DrawString(20, off + 60, col, _("Tooltip delay [ms]", 412));
	DrawFrame(0, 0, GetWidth(), GetHeight(), FS_SHEET, false);
}

/* virtual */ void CTimingSheet::OnConfigure(XConfigureEvent& e __attribute__ ((unused)))
{
	mDblClick.Resize(145, 20);
	mDblClick.Move(20, 30);
	mDblClickEdit.Resize(65, 20);
	mDblClickEdit.Move(170, 30);
	mDelayEdit.Resize(65, 20);
	mDelayEdit.Move(170, 80);
	mDelay.Resize(145, 20);
	mDelay.Move(20, 80);
	mDelayEnable.Move(20, 110);
}

/* virtual */ void CTimingSheet::OnMessage(unsigned msgid, TMessage type __attribute__ ((unused)), unsigned value)
{
	switch (msgid) {
	case IDDBL_CLICK:
		mDblClickEdit.SetValue(value);
		break;
	case IDDBL_CLICK_EDIT:
		mDblClick.SetPosition(value);
		break;
	case IDDELAY:
		mDelayEdit.SetValue(value);
		break;
	case IDDELAY_EDIT:
		mDelay.SetPosition(value);
		break;
	default:
		break;
	}
}
