/*----------------------------------------------------------------------------
  Project: cxxx <http://cxplorer.sourceforge.net>
  File   : CTimingSheet.h
  Info   : $Id: CTimingSheet.h,v 1.10 2012/08/06 17:36:40 gkraeuter Exp $
----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
  Copyright © 2005 - 2012 Gerhard Kräuter <gkraeuter@users.sourceforge.net>

  This  software  is  provided  'as-is',  without  any  express  or  implied
  warranty.  In  no  event  will  the authors be held liable for any damages
  arising from the use of this software.

  Permission  is  granted  to  anyone  to use this software for any purpose,
  including  commercial  applications,  and  to alter it and redistribute it
  freely, subject to the following restrictions:

  1. The  origin  of  this software must not be misrepresented; you must not
     claim  that  you  wrote the original software. If you use this software
     in  a  product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.

----------------------------------------------------------------------------*/

#ifndef CTimingSheet_h
#define CTimingSheet_h

// C++X headers
#include <cxxx/CSlider.h>
#include <cxxx/CNumEdit.h>
#include <cxxx/CCheckButton.h>
#include <cxxx/CApplication.h>
#include <cxxx/CToolButton.h>

class CTimingSheet : public CWindow
{
private:
	enum
	{
		// Message IDs
		IDDBL_CLICK = cxxx::msgid::MSGID_CTimingSheet,
		IDDBL_CLICK_EDIT,
		IDDELAY,
		IDDELAY_EDIT,
		IDDELAY_ENABLE
	};

public:
	// Constructor
	CTimingSheet(CWindow* parent);

	// X event processing
	virtual void OnExpose(XExposeEvent& e);
	virtual void OnConfigure(XConfigureEvent& e);

	// C++X message processing
	virtual void OnMessage(unsigned msgid, TMessage type, unsigned value);

	// Prepares the window for exit
	bool DoOK()
	{
		cxxx::lib::cxProfile->Write("Doubleclick Time", mDblClickEdit.GetValue());
		cxxx::cxApp->SetDoubleClickTime(mDblClickEdit.GetValue());
		unsigned u = 0;
		if (mDelayEnable.IsChecked()) u = mDelayEdit.GetValue();
		cxxx::lib::cxProfile->Write("Tooltip Delay", u);
		CToolButton::SetDelay(u);
		return true;
	}

	// Returns the double click time
	unsigned GetDblClickTime()
	{
		return mDblClickEdit.GetValue();
	}

	// Returns the tool tip time
	unsigned GetToolTipTime()
	{
		if (mDelayEnable.IsChecked()) return mDelayEdit.GetValue();
		else return 0;
	}
	
protected:
	CSlider mDblClick;
	CSlider mDelay;
	CNumEdit mDblClickEdit;
	CNumEdit mDelayEdit;
	CCheckButton mDelayEnable;
};

#endif // CTimingSheet_h
