/*----------------------------------------------------------------------------
  Project: cxxx <http://cxplorer.sourceforge.net>
  File   : CToolButton.h
  Info   : $Id: CToolButton.h,v 1.13 2012/06/08 13:03:40 gkraeuter Exp $
----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
  Copyright © 2002 - 2012 Gerhard Kräuter <gkraeuter@users.sourceforge.net>
  
  This  software  is  provided  'as-is',  without  any  express  or  implied
  warranty.  In  no  event  will  the authors be held liable for any damages
  arising from the use of this software.

  Permission  is  granted  to  anyone  to use this software for any purpose,
  including  commercial  applications,  and  to alter it and redistribute it
  freely, subject to the following restrictions:

  1. The  origin  of  this software must not be misrepresented; you must not
     claim  that  you  wrote the original software. If you use this software
     in  a  product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.

----------------------------------------------------------------------------*/

#ifndef CToolButton_h
#define CToolButton_h

// C++X headers
#include <cxxx/CWindow.h>
#include <cxxx/CToolTip.h>

class CToolButton : public CWindow
{
enum
{
	// Mesage IDs
	IDTIMER = cxxx::msgid::MSGID_CToolButton,
	// Geometry
	TOOLBUTTON_SIZE_X  = 22,
	TOOLBUTTON_SIZE_Y  = 22
};

public:
	// Constructor
	CToolButton(CWindow* parent, unsigned msgid, CIcon::TIconName icon, const std::string& tip);
	
	// Destructor
	virtual ~CToolButton();

	// X event processing
	virtual void OnExpose(XExposeEvent& e);
	virtual void OnLButtonDown(XButtonEvent& e);
	virtual void OnButtonUp(XButtonEvent& e);
	virtual void OnEvent(XEvent& e);

	// C++X message processing
	virtual void OnMessage(unsigned msgid, TMessage type, unsigned value);

	// Updates the button
	void Update()
	{
		if (GetUpdate() == IsEnabled()) return;
		Enable(!IsEnabled());
		Redraw();
	}

	// Closes the tool tip
	void CloseToolTip()
	{
		mTimer.Stop();
		if (mToolTip == NULL) return ;
		delete mToolTip;
		mToolTip = NULL;
	}

	// Openes the tool tip
	void OpenToolTip()
	{
		if (mToolTipText.empty()) return ;
		mToolTip = new CToolTip(this, mToolTipText);
	}
	
	// Sets the delay before opening the tool tip
	static void SetDelay(unsigned d)
	{
		mDelay = d;
	}
	
	// Reads the delay before opening the tool tip
	static unsigned GetDelay()
	{
		return mDelay;
	}
	
	// Change the displayed icon
	void SetIcon(CIcon::TIconName icon)
	{
		mIcon.Create(icon, mIconSize);
	}

protected:
	CIcon mIcon;
	unsigned mIconSize;
	CToolTip* mToolTip;
	std::string mToolTipText;
	bool mPressed;
	CTimer mTimer;
	static unsigned mDelay;
};



#endif // CToolButton_h
