/*----------------------------------------------------------------------------
  Project: cxxx <http://cxplorer.sourceforge.net>
  File   : CTreeControl.cpp
  Info   : $Id: CTreeControl.cpp,v 1.34 2012/08/06 17:36:40 gkraeuter Exp $
----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
  Copyright © 2002 - 2012 Gerhard Kräuter <gkraeuter@users.sourceforge.net>

  This  software  is  provided  'as-is',  without  any  express  or  implied
  warranty.  In  no  event  will  the authors be held liable for any damages
  arising from the use of this software.

  Permission  is  granted  to  anyone  to use this software for any purpose,
  including  commercial  applications,  and  to alter it and redistribute it
  freely, subject to the following restrictions:

  1. The  origin  of  this software must not be misrepresented; you must not
     claim  that  you  wrote the original software. If you use this software
     in  a  product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.

----------------------------------------------------------------------------*/

// C++X Headers
#include <cxxx/CTreeControl.h>

// Namespace declarations
using std::string;
using std::wstring;
using std::vector;
using cxxx::cxWiden;

/* virtual */ void CTreeControl::OnExpose(XExposeEvent& e)
{
	if (!mList.empty()) {
		int x = e.x;
		int y = e.y;
		ScreenToWindow(x, y);
		unsigned rl = GetRow(y);
		if (rl != -1u) {
			y += e.height;
			unsigned rh = GetRow(y);
			if (rh >= mList.size()) rh = mList.size() - 1;
			for (unsigned idx = rl; idx <= rh; ++idx) DrawItem(idx);
		}
	}
	DrawFrame();
}

/* virtual */ void CTreeControl::DrawItem(unsigned idx)
{
	if(!IsVisible()) return;
	CTreeItem& item(GetItem(idx));
	int x = 0;
	int y = idx * mIconHeight;
	unsigned w = item.GetIndentation() * INDENT_STEP + item.GetWidth() + INDENT_STEP;
	unsigned h = mIconHeight;
	WindowToScreen(x, y);
	ClearArea(x, y, w, h);
	x += item.GetIndentation() * INDENT_STEP + (INDENT_STEP + 1) / 2;
	CTreeItem itmp(item);
	if (!item.IsTree()) itmp = itmp.GetParent();
	int x1 = x;
	CColor col(TC_BACKGND_TEXT);
	while (!itmp.IsTree()) {
		x1 -= INDENT_STEP;
		if (itmp.GetParent().GetChildren().back() !=  itmp) DrawLine(x1, y, x1, y + h - 1, col);
		itmp = itmp.GetParent();
	}
	x1 = x + INDENT_STEP - (INDENT_STEP + 1) / 2;
	int y1 = y;
	y += mIconHeight / 2;
	if (item.GetChildren().empty() == false) {
		if (item.IsOpened()) DrawIcon(x - 4, y - 4, mMinus, col);
		else DrawIcon(x - 4, y - 4, mPlus, col);
		if (!item.IsTree()) DrawLine(x, y1, x, y - 5, col);
		DrawLine(x + 5, y, x1 - 1, y, col);
		if (!item.IsTree() && item.GetParent().GetChildren().back() !=  item) DrawLine(x, y + 5, x, y1 + h - 1, col);
	} else {
		if (!item.IsTree()) DrawLine(x, y1, x, y, col);
		DrawLine(x, y, x1 - 1, y, col);
		if (!item.IsTree() && item.GetParent().GetChildren().back() !=  item) DrawLine(x, y, x, y1 + h - 1, col);
	}
	DrawIcon(x1 + ICON_BORDER, y1 + ICON_BORDER, item.GetIcon());
	x = x1 + mIconWidth;
	y = y1 +  (mIconHeight - GetTextHeight()) / 2;
	if ( item.IsSelected()) {
		if (IsFocused()) {
			FillRectangle(x, y - TEXT_BORDER, item.GetWidth() - mIconWidth, GetTextHeight() + 2 * TEXT_BORDER, CColor(TC_TITLE));
			col = CColor(TC_TITLE_TEXT);
		} else {
			DrawDottedRectangle (x, y - TEXT_BORDER, item.GetWidth() - mIconWidth - 1, GetTextHeight() + 2 * TEXT_BORDER - 1, col);
		}
	}
	if (!item.GetText().empty()) DrawString(x + TEXT_BORDER, y, col, item.GetText(), false);
	x = 0;
	y = idx * mIconHeight;
	WindowToScreen(x, y);
}

/* virtual */ void CTreeControl::OnLButtonDown(XButtonEvent& e)
{
	if (IsDragging()) CancelDrag();
	int x(e.x);
	int y(e.y);
	ScreenToWindow(x, y);
  	unsigned idx = GetRow(y);
	bool cancel = false;
	bool openclose = false;
	if (idx < mList.size()) {
		x -= GetItem(idx).GetIndentation() * INDENT_STEP;
		if (x < 0) cancel = true;
		else {
			x -= INDENT_STEP;
			if (x < 0) openclose = true;
			else {
				x -= GetItem(idx).GetWidth();
				if (x >= 0) cancel = true;
			}
		}
	} else cancel = true;
	if (cancel) {
		if (GetSelCount() != 0) {
			DeselectAll();
   			PostMessage(MT_SELECTION_CHANGED);
		}
		StopDrag(e);
		mLastSel = -1u;
		return ;
   	}
	if (openclose) {
		if (GetItem(idx).IsOpened()) Close(idx);
		else Open(idx);
	} else if (!GetItem(idx).IsSelected()) {
		DeselectAll();
		Select(idx);
		PostMessage(MT_SELECTION_CHANGED);
	}
	StartDrag(e);
	mLastSel = idx;
}

/* virtual */ void CTreeControl::OnRButtonDown(XButtonEvent& e)
{
	if (IsDragging()) CancelDrag();
  	unsigned idx = ItemFromPos(e.x, e.y);
	if (idx >= mList.size()) {
   		DeselectAll();
   		PostMessage(MT_SELECTION_CHANGED);
		StopDrag(e);
		mLastSel = -1u;
		return ;
	}
	if (!mList[idx].IsSelected()) {
		DeselectAll();
		Select(idx);
		PostMessage(MT_SELECTION_CHANGED);
	}
	StartDrag(e);
	mLastSel = idx;
}

/* virtual */ void CTreeControl::CalculateDocSize()
{
	mDocHeight = mList.size() * mIconHeight;
	mRow.push_back(mDocHeight);
	mDocWidth = 0;
	unsigned tmp;
	for (unsigned i = 0; i < mList.size(); ++i) {
		tmp = GetItem(i).GetWidth() + GetItem(i).GetIndentation() * INDENT_STEP + INDENT_STEP;
		if (tmp > mDocWidth) mDocWidth = tmp;
	}
}

/* virtual */ void CTreeControl::Open(unsigned idx)
{
	GetItem(idx).Open();
	vector<CControlItem> tmp;
	std::copy(mList.begin() + idx + 1, mList.end(), back_inserter(tmp));
	unsigned sel = -1u;
	if (GetSelCount() != 0 && GetSelection().front() > idx) {
		sel = GetSelection().front();
		DeselectAll();
	}
	mList.resize(idx + 1);
	Sync(GetItem(idx));
	if (sel != -1u) sel += mList.size() - idx - 1;
	std::copy(tmp.begin(), tmp.end(), back_inserter(mList));
	// we cannot use Select(sel) because that would scroll contents
	if (sel != -1u) {
		mList[sel].Select(true);
		mSelection.push_back(sel);
	}
	Update();
}

/* virtual */ void CTreeControl::Close(unsigned idx)
{
	GetItem(idx).Close();
	unsigned sz = GetListSize(GetItem(idx));
	unsigned sel = -1u;
	if (GetSelCount() != 0) {
		sel = GetSelection().front();
		if (sel > idx) {
			sel -= sz;
			if ((int)sel < (int)idx) sel = idx;
			DeselectAll();
		} else sel = -1u;
	}
	std::vector<CControlItem>::iterator beg = mList.begin() + idx + 1;
	mList.erase(beg, beg + sz);
	if (sel >= idx && sel != -1u) {
		// we cannot use Select(sel) because that would scroll contents
		mList[sel].Select(true);
		mSelection.push_back(sel);
		if (sel == idx) PostMessage(MT_SELECTION_CHANGED);
	}
	Update();
}

unsigned CTreeControl::GetListSize(CTreeItem& item) const
{
	unsigned ret = 0;
	for (vector<CTreeItem>::iterator i = item.GetChildren().begin(); i < item.GetChildren().end(); ++i) {
		++ret;
		if (i->IsOpened()) ret += GetListSize(*i);
	}
	return ret;
}

void CTreeControl::Sync(CTreeItem& item)
{
	for (vector<CTreeItem>::iterator i = item.GetChildren().begin(); i < item.GetChildren().end(); ++i) {
		mList.push_back(*i);
		if (i->IsOpened()) Sync(*i);
	}
}

void CTreeControl::GetBounds(int& x, int& y, unsigned& w, unsigned& h)
{
	ScreenToWindow(x, y);
	unsigned idx = GetRow(y);
	if (idx == -1u) {
		x = 0;
		y = mRow[0] * mIconHeight;
		w = mDocWidth + mViewWidth;
		h = mDocHeight + mViewHeight;
	} else {
		cxAssert(idx < mList.size());
		int xsave(x);
		CTreeItem i(GetItem(idx));
		x = i.GetIndentation() * INDENT_STEP + INDENT_STEP;
		y = idx * mIconHeight;
		w = i.GetWidth();
		h = mIconHeight;
		if (xsave < x) {
			w = x;
			x = 0;
		} else if (xsave > x + (int)w) {
			x += w;
			w = mDocWidth + mViewWidth;
		}
	}
   	WindowToScreen(x, y);
	if (x < BORDER_WIDTH) {
		w += x - BORDER_WIDTH;
		x = BORDER_WIDTH;
	}
	if (y < BORDER_WIDTH) {
		h += y - BORDER_WIDTH;
		y = BORDER_WIDTH;
	}
	if ((x + w) > mViewWidth + BORDER_WIDTH) w = mViewWidth + BORDER_WIDTH - x;
	if ((y + h) > mViewHeight + BORDER_WIDTH) h = mViewHeight + BORDER_WIDTH - y;
}

/* virtual */ void CTreeControl::Append(CTreeItem& parent, CIcon::TIconName closed, CIcon::TIconName opened, const string& text)
{
	unsigned w = GetTextWidth(cxWiden(text)) + mIconWidth + 2 * TEXT_BORDER;
	parent.GetChildren().push_back(CTreeItem(parent, closed, opened, mIconSize, text, w, mIconHeight));
}

/* virtual */ void CTreeControl::ScrollIn(unsigned idx)
{
	int xi = GetItem(idx).GetIndentation() * INDENT_STEP + INDENT_STEP + GetItem(idx).GetWidth();
	int yi = idx * mIconHeight;
	int xw = BORDER_WIDTH + mViewWidth;
	int yw = BORDER_WIDTH;
	int scrlx = 0;
	int scrly = 0;
	WindowToScreen(xi, yi);
	if (yi < yw) scrly = (yi - yw - SCROLL_STEP + 1) / SCROLL_STEP;
	else {
		yi += mIconHeight;
		yw += mViewHeight;
		if (yi > yw) scrly = (yi - yw + SCROLL_STEP - 1) / SCROLL_STEP;
	}
	if (xi > xw) scrlx = (xi - xw + SCROLL_STEP - 1) / SCROLL_STEP;
	xi -= GetItem(idx).GetWidth() + INDENT_STEP + scrlx * SCROLL_STEP;
	xw -= mViewWidth;
	if (xi < xw) scrlx += (xi - xw - SCROLL_STEP + 1) / SCROLL_STEP;
	if (scrlx != 0) mHScrollBar.SetPosition(mHScrollBar.GetPosition() + scrlx);
	if (scrly != 0) mVScrollBar.SetPosition(mVScrollBar.GetPosition() + scrly);
}
