/*----------------------------------------------------------------------------
  Project: cxxx <http://cxplorer.sourceforge.net>
  File   : CTreeControl.h
  Info   : $Id: CTreeControl.h,v 1.26 2012/06/08 13:03:40 gkraeuter Exp $
----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
  Copyright © 2002 - 2012 Gerhard Kräuter <gkraeuter@users.sourceforge.net>

  This  software  is  provided  'as-is',  without  any  express  or  implied
  warranty.  In  no  event  will  the authors be held liable for any damages
  arising from the use of this software.

  Permission  is  granted  to  anyone  to use this software for any purpose,
  including  commercial  applications,  and  to alter it and redistribute it
  freely, subject to the following restrictions:

  1. The  origin  of  this software must not be misrepresented; you must not
     claim  that  you  wrote the original software. If you use this software
     in  a  product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.

----------------------------------------------------------------------------*/

#ifndef CTreeControl_h
#define CTreeControl_h

// C++X headers
#include <cxxx/CControl.h>
#include <cxxx/CTreeItem.h>

// Forward declarations

class CTreeControl : public CControl
{
protected:
	enum
	{
		// Geometry
		INDENT_STEP = 15,
		ICON_BORDER = 2,
		TEXT_BORDER = 2
	};

public:
	// Constructor
	CTreeControl(CWindow* parent, unsigned msgid)
	: CControl(parent, msgid)
	{
		mPlus.Create(CIcon::TREEPLUS, 0);
		mMinus.Create(CIcon::TREEMINUS, 0);
	}

	// X event processing
	virtual void OnExpose(XExposeEvent& e);
	virtual void OnLButtonDown(XButtonEvent& e);
	virtual void OnRButtonDown(XButtonEvent& e);

	// Opens item with index idx
	virtual void Open(unsigned idx);

	// Closes item with index idx	
	virtual void Close(unsigned idx);

	// Sets the root of the tree
	void SetTree(CIcon::TIconName closed, CIcon::TIconName opened, const std::string& text)
	{
		unsigned w = GetTextWidth(cxxx::cxWiden(text)) + mIconWidth + 2 * TEXT_BORDER;;
		mTree = CTreeItem(closed, opened, mIconSize, text, w, mIconHeight);
	}

	// Inserts an item into the tree
	virtual void Append(CTreeItem& parent, CIcon::TIconName closed, CIcon::TIconName opened, const std::string& text);
	
	// Access function
	CTreeItem& GetItem(unsigned idx) const
	{
		cxAssert(idx < mList.size());
		return (CTreeItem&)mList[idx];
	}
	
protected:
	// Calculates the size of the document
	virtual void CalculateDocSize();

	// Returns the index to the item at screen position x, y
	virtual unsigned ItemFromPos(int x, int y)
	{
		ScreenToWindow(x, y);
		unsigned idx = GetRow(y);
		if (idx >= mList.size()) return -1u;
		CTreeItem item = GetItem(idx);
		int tmp = item.GetIndentation() * INDENT_STEP + INDENT_STEP;
		if (x < tmp) return -1u;
		tmp += item.GetWidth();
		if(x > tmp) return -1u;
		return idx;
	}

	// Draws item with index idx
	virtual void DrawItem(unsigned idx);

	// Brings item with index idx into view
	virtual void ScrollIn(unsigned idx);

	// Returns the item count of the list
	unsigned GetListSize(CTreeItem& item) const;

	// Returns the row of screen position y
	unsigned GetRow(int y) const
	{
		if (y < 0) return 0;
		unsigned r = y / mIconHeight;
		if (r >= mList.size()) r = -1u;
		return r;
	}

	// Synchronize the subtree of item
	void Sync(CTreeItem& item);

	// Returns the index of item
	unsigned GetIndex(const CTreeItem& item) const
	{
		for (unsigned i = 0; i < mList.size(); ++i) if ((const CControlItem&)item == mList[i]) return i;
		return -1u;
	}
	
	// Calculates the bounding rectangle at screen position x, y
	void GetBounds(int& x, int& y, unsigned& w, unsigned& h);
	
protected:
	CIcon mPlus;
	CIcon mMinus;
	CTreeItem mTree;
};

#endif // CTreeControl_h
