/*----------------------------------------------------------------------------
  Project: cxxx <http://cxplorer.sourceforge.net>
  File   : CXZStreambuf.h
  Info   : $Id: CXZStreambuf.h,v 1.1 2012/05/06 12:09:49 gkraeuter Exp $
----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
  Copyright © 2012 Gerhard Kräuter <gkraeuter@users.sourceforge.net>

  This  software  is  provided  'as-is',  without  any  express  or  implied
  warranty.  In  no  event  will  the authors be held liable for any damages
  arising from the use of this software.

  Permission  is  granted  to  anyone  to use this software for any purpose,
  including  commercial  applications,  and  to alter it and redistribute it
  freely, subject to the following restrictions:

  1. The  origin  of  this software must not be misrepresented; you must not
     claim  that  you  wrote the original software. If you use this software
     in  a  product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.

----------------------------------------------------------------------------*/

#ifndef CXZStreambuf_h
#define CXZStreambuf_h

// C++X headers
#include <cxxx/CxxxFeatures.h>

#ifdef CXXX_HAS_XZ

// C++ headers
#include <streambuf>
#include <cstring>
#include <cstdio>

// xz header
#include <lzma.h>

class CXZStreambuf : public std::streambuf
{
public:
	typedef enum
	{
		UNINITIALIZED,
		GOOD,
		FINISH,
		END,
		FAULT
	} TStreamState;

public:
	// Constructor
	CXZStreambuf(std::streambuf& in, unsigned size);

	// Destructor
	virtual ~CXZStreambuf();
	
protected:
	// Inherited functions from streambuf
	virtual int overflow(int c = EOF);
	virtual std::streampos seekoff(std::streamoff off, std::ios_base::seekdir way, std::ios_base::openmode which = std::ios_base::in | std::ios_base::out);
	virtual std::streampos seekpos(std::streampos pos, std::ios_base::openmode which = std::ios_base::in | std::ios_base::out);
	virtual std::streamsize showmanyc();
	virtual int underflow();
	virtual std::streamsize xsgetn(char* s, std::streamsize n);

private:
	// Initializes the compression library
	bool DecompressInit();

	// Closes the compression library
	void DecompressEnd()
	{
		if (mState != UNINITIALIZED) lzma_end(&mXZ);
		mState = UNINITIALIZED;
	}

	// Decompresses a maximum of size characters, returns true if successfull
	bool Decompress(unsigned size);

private:
	std::streambuf& mIn;
	lzma_stream mXZ;
	TStreamState mState;
	unsigned mBufferSize;
	char* mInBuffer;
	char* mOutBuffer;
};

#endif // CXXX_HAS_XZ

#endif // CXZStreambuf_h
