/*----------------------------------------------------------------------------
  Project: cxxx <http://cxplorer.sourceforge.net>
  File   : Cxxx.h
  Info   : $Id: Cxxx.h,v 1.75 2012/09/12 17:05:11 gkraeuter Exp $
----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
  Copyright © 2002 - 2012 Gerhard Kräuter <gkraeuter@users.sourceforge.net>

  This  software  is  provided  'as-is',  without  any  express  or  implied
  warranty.  In  no  event  will  the authors be held liable for any damages
  arising from the use of this software.

  Permission  is  granted  to  anyone  to use this software for any purpose,
  including  commercial  applications,  and  to alter it and redistribute it
  freely, subject to the following restrictions:

  1. The  origin  of  this software must not be misrepresented; you must not
     claim  that  you  wrote the original software. If you use this software
     in  a  product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.

----------------------------------------------------------------------------*/

#ifndef Cxxx_h
#define Cxxx_h

#include <locale>
#include <cstdlib>
#include <climits>

#include <sys/stat.h> // for mode_t

#include <cxxx/CxxxFeatures.h>
#include <cxxx/CIconv.h>
#include <intlize/i5e.hpp>
#include <cxxx/CError.h>

#include <X11/Xlib.h> // for Display
#include <X11/Xatom.h>
#include <X11/extensions/Xrender.h>

#define _(String, index) cxgettext(index)

#ifndef XA_UTF8_STRING
 #define XA_UTF8_STRING cxxx::cxUtf8Atom
#endif // XA_UTF8_STRING
 
class CApplication;
class CBidi;
class CDnDManager;
class CFileHandler;
class CFont;
class CProfile;
class CSelection;
class CTopLevelWindow;

#define CK_Linefeed   L'\xff0a'
#define CK_Home       L'\xff50'
#define CK_Left       L'\xff51'
#define CK_Up         L'\xff52'
#define CK_Right      L'\xff53'
#define CK_Down       L'\xff54'
#define CK_Page_Up    L'\xff55'
#define CK_Page_Down  L'\xff56'
#define CK_End        L'\xff57'
#define CK_Begin      L'\xff58'
#define CKP_Seperator L'\xffac'
#define CKP_Decimal   L'\xffae'


//message types
typedef enum
{
	MT_TIMER             = 1 << 0,
	MT_IDLE              = 1 << 1,
	MT_ERROR             = 1 << 2,
	MT_ACCELERATOR       = 1 << 3,
	MT_CHECK             = 1 << 4,
	MT_UNCHECK           = 1 << 5,
	MT_CONFIRM_SELECTION = 1 << 6,
	MT_SELECTION_CHANGED = 1 << 7,
	MT_SCROLL            = 1 << 8,
	MT_PRESSED           = 1 << 9,
	MT_RELEASED          = 1 << 10,
	MT_MOTION            = 1 << 11
} TMessage;

// Colors
typedef enum {
	TC_FRAME,
	TC_DARK,
	TC_BRIGHT,
	TC_TITLE,
	TC_TOOLTIP,
	TC_SCROLL,
	TC_BACKGND,
	TC_TEXT,
	TC_FRAME_TEXT = TC_TEXT,
	TC_DARK_TEXT,
	TC_BRIGHT_TEXT,
	TC_TITLE_TEXT,
	TC_TOOLTIP_TEXT,
	TC_SCROLL_TEXT,
	TC_BACKGND_TEXT,
	TC_BLACK,
	TC_WHITE,
	TC_COLORCOUNT
} TColor;


struct CHotCharPos
{
	unsigned mBegin;
	unsigned mEnd;
	
	CHotCharPos()
	: mBegin(0),
	  mEnd(0)
	{
	}
};

namespace cxxx
{
	namespace filesort
	{
		enum
		{
			NAME      = 1 << 0,
			SIZE      = 1 << 2,
			TIME      = 1 << 3,
			TYPE      = 1 << 4,
			ASCENDING = 1 << 6,
			DIRFIRST = 1 << 7,
			HIDDEN = 1 << 8
		};
	}

	namespace msgid 
	{
		enum {
			MSGID_Step = 0x100000,
			MSGID_CApplication		= MSGID_Step * 0x01,
			MSGID_CButton			= MSGID_Step * 0x02,
			MSGID_CCaret			= MSGID_Step * 0x03,
			MSGID_CCheckButton		= MSGID_Step * 0x03,
			MSGID_CCxxxAbout		= MSGID_Step * 0x04,
			MSGID_CDnDAskDialog		= MSGID_Step * 0x05,
			MSGID_CDnDManager		= MSGID_Step * 0x06,
			MSGID_CDnDWindow		= MSGID_Step * 0x07,
			MSGID_CDropDownList	 	= MSGID_Step * 0x08,
			MSGID_CDropDownPopup	= MSGID_Step * 0x09,
			MSGID_CDupFNDialog		= MSGID_Step * 0x0a,
			MSGID_CEdit				= MSGID_Step * 0x0b,
			MSGID_CFileListControl	= MSGID_Step * 0x0c,
			MSGID_CFileTree			= MSGID_Step * 0x0d,
			MSGID_CHSplitterWindow	= MSGID_Step * 0x0e,
			MSGID_CLabel			= MSGID_Step * 0x0f,
			MSGID_CListControl		= MSGID_Step * 0x10,
			MSGID_CMenuBar			= MSGID_Step * 0x11,
			MSGID_CMenuItem			= MSGID_Step * 0x12,
			MSGID_CMessageBox		= MSGID_Step * 0x13,
			MSGID_CPopup			= MSGID_Step * 0x14,
			MSGID_CPropertyTab		= MSGID_Step * 0x15,
			MSGID_CPushButton		= MSGID_Step * 0x16,
			MSGID_CRadioButton		= MSGID_Step * 0x17,
			MSGID_CScrollBar		= MSGID_Step * 0x18,
			MSGID_CSelection		= MSGID_Step * 0x19,
			MSGID_CStatusBar		= MSGID_Step * 0x1a,
			MSGID_CTimer			= MSGID_Step * 0x1b,
			MSGID_CToolButton		= MSGID_Step * 0x1c,
			MSGID_CToolTip			= MSGID_Step * 0x1d,
			MSGID_CTopLevelWindow	= MSGID_Step * 0x1e,
			MSGID_CTreeControl		= MSGID_Step * 0x1f,
			MSGID_CTreeView			= MSGID_Step * 0x20,
			MSGID_CWindow			= MSGID_Step * 0x21,
			MSGID_CControl			= MSGID_Step * 0x22,
			MSGID_CFileOpenDialog	= MSGID_Step * 0x23,
			MSGID_CColorDialog		= MSGID_Step * 0x24,
			MSGID_CNumEdit			= MSGID_Step * 0x25,
			MSGID_CBuddy			= MSGID_Step * 0x26,
			MSGID_CSettings			= MSGID_Step * 0x27,
			MSGID_CTimingSheet		= MSGID_Step * 0x28,
			MSGID_CColorSheet		= MSGID_Step * 0x29,
			MSGID_CFontSheet		= MSGID_Step * 0x2a,
			MSGID_CThemeSheet		= MSGID_Step * 0x2b,
			MSGID_Last				= MSGID_Step * 0x2c
		};
	}
	// essentials
	extern Display*         cxDpy;
	extern CApplication*    cxApp;
	extern CTopLevelWindow* cxMainWin;
	// fonts
	extern CFont*           cxTextFont;
	extern CFont*           cxTitleFont;
	extern Atom             cxUtf8Atom;
	// helpers
	extern i5e              cxCxxxIntl;
	extern i5e              cxAppIntl;
	extern CIconv           cxIconv;
	extern CDnDManager*     cxDnDManager;
	extern CError           cxError;
	extern CFileHandler*	cxFileManager;
	extern CSelection*      cxSelManager;
	extern Time             cxServerTime;
	// settings
	extern CProfile*        cxProfile;
	namespace lib
	{
		extern CProfile* cxProfile;
	}
	// rendering formats
	extern XRenderPictFormat* cxAlphaFormat;	
	extern XRenderPictFormat* cxWindowFormat;	
	extern XRenderPictFormat* cxARGBFormat;
	// rendering pictures
	extern Picture            cxDefaultPic;
	extern Picture            cxGrayPic;
	extern Picture            cxStipplePic;
	// 32 bit GC
	extern GC                 cxGC32;
	// misc
	extern std::locale        cxLocale;
	extern std::string        cxSysIconDir;
	extern std::string        cxHomeDir;
	extern std::string        cxCxxxLocalePath;
	extern std::string        cxAppLocalePath;
	extern unsigned char      cxBitSwap[256];
	extern const std::string  cxEmptyString;
	extern CBidi*             cxBidi;
	extern std::string        cxStartupID;
	
	/*** internationalisation ***/
	namespace lib
	{
		inline const char* cxgettext(int idx)
		{
			return 	::cxxx::cxCxxxIntl.getmsg(idx);
		}
	}

	namespace noop
	{
		inline int cxgettext(int idx)
		{
			return idx;
		}
	}	

	inline const char* cxgettext(int idx)
	{
		return ::cxxx::cxAppIntl.getmsg(idx);
	}
	
	inline ::std::wstring cxWiden(const ::std::string& s)
	{
		wchar_t c;
		::std::wstring ret;
		::std::string::size_type pos(0);
		int state;
		while (pos < s.size()) {
			state = ::std::mbtowc(&c, &s[pos], s.size() - pos);
			if (state == -1) {
				++pos;
				ret.push_back(L'?');
			} else if (state) {
				pos += state;
				ret.push_back(c);
			// avoid infinite loops
			} else ++pos;
		}
		return ret;
	}
	
	inline ::std::string cxNarrow(const ::std::wstring& s)
	{
		char c[MB_LEN_MAX + 1];
		::std::string ret;
		int len;
		for (unsigned i = 0; i < s.size(); ++i) {
			len = ::std::wctomb(c, s[i]);
			if (len < 0 ) ret += '?';
			else {
				c[len] = '\0';
				ret += c;
			}
		}
		return ret;
	}

	inline void cxUpdateServerTime(Time& t)
	{
		if (t && (int)(t - cxServerTime) > 0) cxServerTime = t;
		else t = cxServerTime;
	}
}

#endif // Cxxx_h
