/*----------------------------------------------------------------------------
  Project: libintlize <http://intlize.sourceforge.net>
  File   : i5e.h
  Info   : $Id: i5e.h,v 1.3 2012/03/03 14:22:36 gkraeuter Exp $
----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
  Copyright © 2010 - 2012 Gerhard Kräuter <gkraeuter@users.sourceforge.net>

  This  software  is  provided  'as-is',  without  any  express  or  implied
  warranty.  In  no  event  will  the authors be held liable for any damages
  arising from the use of this software.

  Permission  is  granted  to  anyone  to use this software for any purpose,
  including  commercial  applications,  and  to alter it and redistribute it
  freely, subject to the following restrictions:

  1. The  origin  of  this software must not be misrepresented; you must not
     claim  that  you  wrote the original software. If you use this software
     in  a  product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.

----------------------------------------------------------------------------*/

#ifndef i5e_h
#define i5e_h

#ifdef __cplusplus
 #ifdef included_from_i5e_hpp
  #undef included_from_i5e_hpp
  extern "C" {
 #else
  #error This is the C header file. Use i5e.hpp for C++.
 #endif
#endif

#include <stdlib.h>
	 
typedef struct
{
	char **msg;
	int msg_cnt;
	int charset;
	char *id;
	char *buf;
	char *buf_end;
	void *converter;
} i5e_catalog_struct_t;

typedef i5e_catalog_struct_t *i5e_catalog_t;	 

i5e_catalog_t i5e_create_catalog();
void i5e_free_catalog(i5e_catalog_t catalog);
int i5e_open(i5e_catalog_t catalog, const char *lang, const char *path, const char *charset);
void i5e_close(i5e_catalog_t catalog);
const char* i5e_get_converted_msg(i5e_catalog_t catalog, int msgnum);
const char* i5e_error_text(int errnum);
const char* i5e_getid(i5e_catalog_t catalog);

/* i5e_getmsg 
   arguments: catalog: the value returned by i5e_create_catalog;
              msgnum: the number of the requested message
   return value: a char* pointer to the message.
   
   The return value is zero terminated. It is allocated on the heap and must not be freed,
   this is done automatically when i5e_close is called. If charset conversion is requested,
   it is done on the fly. The result gets cached to optimize speed.
*/
   
static inline const char* i5e_getmsg(i5e_catalog_t catalog, int msgnum)
{
	if (msgnum >= catalog->msg_cnt) {
		if (catalog->msg_cnt) msgnum = 0;
		else return i5e_error_text(10);
	}
	if (catalog->converter == NULL || catalog->msg[msgnum] < catalog->buf || catalog->msg[msgnum] > catalog->buf_end) return catalog->msg[msgnum];
	return i5e_get_converted_msg(catalog, msgnum);
}	
		  
#ifdef __cplusplus
}
#endif

#endif
