/*----------------------------------------------------------------------------
  Project: Xplorer <http://cxplorer.sourceforge.net>
  File   : CFileAssoc.h
  Info   : $Id: CFileAssoc.h,v 1.17 2012/07/17 12:19:27 gkraeuter Exp $
----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
  Copyright © 2005 - 2012 Gerhard Kräuter <gkraeuter@users.sourceforge.net>

  This  program  is  free software: you can redistribute it and/or modify it
  under  the  terms  of  the  GNU General Public License as published by the
  Free Software Foundation,  either  version  3  of the License, or (at your
  option) any later version.

  This  program  is  distributed  in  the  hope  that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  or  FITNESS FOR A PARTICULAR PURPOSE.  See the  GNU General Public License
  for more details.

  You  should  have  received a copy of the GNU General Public License along
  with this program. If not, see  <http://www.gnu.org/licenses/>.
----------------------------------------------------------------------------*/

#ifndef CFileAssoc_h
#define CFileAssoc_h

// Application headers
#include "Xplorer.h"

// C++X headers
#include <cxxx/CTopLevelWindow.h>
#include <cxxx/CRadioButton.h>
#include <cxxx/CEdit.h>
#include <cxxx/CToolButton.h>
#include <cxxx/CCheckButton.h>
#include <cxxx/CPushButton.h>
#include <cxxx/TMagic.h>

class CFileAssoc : public CTopLevelWindow
{
public:
	enum
	{
		// Return values
		OK,
		CANCEL
	};

private:
	enum
	{
		// Message IDs
		IDOK = xplorer::MSGID_CFileAssoc,
		IDCANCEL,
		IDEXEC,
		IDDOCUMENT,
		IDBROWSE,
		IDTERM,
		IDREMEMBER
	};

public:
	// Constructor
	CFileAssoc(CWindow* parent, const std::string& file, unsigned type, bool remember);
	
	// X event processing
	virtual void OnExpose(XExposeEvent& e);
	virtual void OnKey(wchar_t key, unsigned mask);

	// C++X message processing
	virtual void OnMessage(unsigned msgid, TMessage type, unsigned value);
	virtual bool OnUpdate(unsigned msgid) const;

	// Exits this window
	virtual void OnQuit();

	// Access functions
	bool RememberAssoc() const
	{
		return mRemember.IsChecked();
	}
	
	bool NeedTerminal() const
	{
		return mTerm.IsChecked();
	}

	bool IsExecutable() const
	{
		return mExec.IsChecked();
	}

	std::string GetPath() const
	{
		return mPath.GetText();
	}

	std::string GetArguments() const
	{
		return mArguments.GetText();
	}

	// Sets the search path for binaries
	void SetBinPath(const std::string& path)
	{
		mExecdir = path;
	}
	
private:
	// Opens a browser window
	void OnBrowse();

private:
	CIcon mIcon;
	std::string mName;
	std::string mType;
	std::string mExecdir;
	CRadioGroup mRadio;
	CRadioButton mExec;
	CRadioButton mDocument;
	CEdit mPath;
	CEdit mArguments;
	CToolButton mBrowse;
	CCheckButton mTerm;
	CCheckButton mRemember;
	CPushButton mOK;
	CPushButton mCancel;
};

#endif // CFileAssoc_h
