/*----------------------------------------------------------------------------
  Project: Xplorer <http://cxplorer.sourceforge.net>
  File   : CGeneralSheet.h
  Info   : $Id: CGeneralSheet.h,v 1.16 2012/08/06 17:36:40 gkraeuter Exp $
----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
  Copyright © 2002 - 2012 Gerhard Kräuter <gkraeuter@users.sourceforge.net>

  This  program  is  free software: you can redistribute it and/or modify it
  under  the  terms  of  the  GNU General Public License as published by the
  Free Software Foundation,  either  version  3  of the License, or (at your
  option) any later version.

  This  program  is  distributed  in  the  hope  that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  or  FITNESS FOR A PARTICULAR PURPOSE.  See the  GNU General Public License
  for more details.

  You  should  have  received a copy of the GNU General Public License along
  with this program. If not, see  <http://www.gnu.org/licenses/>.
----------------------------------------------------------------------------*/

#ifndef CGeneralSheet_h
#define CGeneralSheet_h

// C++X headers
#include <cxxx/CMagic.h>
#include <cxxx/CEdit.h>
#include <cxxx/CLabel.h>
#include <cxxx/CApplication.h>

// C headers
#include <sys/stat.h>

// Forward declarations
class CTapeArchive;

class CGeneralSheet : public CWindow
{
public:
	// Constructor
	CGeneralSheet(CWindow* parent, CTapeArchive* arch, const std::string& path);

	// Destructor
	virtual ~CGeneralSheet()
	{
		if (mStatus) delete mStatus;
	}

	// X event processing
	virtual void OnExpose(XExposeEvent& e);
	virtual void OnMap(XMapEvent& e);
	virtual void OnFocus(bool in);

	// Prepares the window for shut down
	bool DoOK();

private:
	// Returns the size of a file or directory
	std::uint64_t GetSize(const std::string& file, const struct stat& st)
	{
		if (EventPending()) cxxx::cxApp->MainLoop();
		if (S_ISDIR(st.st_mode)) return GetDirSize(file);
		else return st.st_size;
	}
		
	// Returns the size of all files in a directory
	std::uint64_t GetDirSize(const std::string& file);
	
	// Draws the size to the screen
	void UpdateSize(std::uint64_t u);
	
private:
	CEdit mEdit;
	std::wstring mLocation;
	std::string mTo;
	std::string mType;
	std::string mSize;
	std::string mATime;
	std::string mMTime;
	std::string mMime;
	CIcon mIcon;
	CMagic mMagic;
	std::string mPath;
	struct stat* mStatus;
	std::string mName;
	bool mCalculate;
};

#endif // CGeneralSheet_h
