/*----------------------------------------------------------------------------
  Project: Xplorer <http://cxplorer.sourceforge.net>
  File   : CNewDialog.cpp
  Info   : $Id: CNewDialog.cpp,v 1.19 2012/08/06 17:36:40 gkraeuter Exp $
----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
  Copyright © 2002 - 2012 Gerhard Kräuter <gkraeuter@users.sourceforge.net>

  This  program  is  free software: you can redistribute it and/or modify it
  under  the  terms  of  the  GNU General Public License as published by the
  Free Software Foundation,  either  version  3  of the License, or (at your
  option) any later version.

  This  program  is  distributed  in  the  hope  that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  or  FITNESS FOR A PARTICULAR PURPOSE.  See the  GNU General Public License
  for more details.

  You  should  have  received a copy of the GNU General Public License along
  with this program. If not, see  <http://www.gnu.org/licenses/>.
----------------------------------------------------------------------------*/

// Application headers
#include "CNewDialog.h"
#include "CXplorerFrame.h"

// C++ headers
#include <cerrno>
#include <fstream>

// Namespace declarations
using std::string;
using cxxx::cxError;
using cxxx::cxgettext;

CNewDialog::CNewDialog(CXplorerFrame *parent, unsigned type, const string& cwd)
: CTopLevelWindow(),
  mFrameWin(parent),
  mEdit(this),
  mStatic(this),
  mButtonClear(this, IDCLEAR),
  mButtonOK(this, IDOK),
  mButtonCancel(this, IDCANCEL),
  mType(type),
  mCWD(cwd)
{
	TransientFor(parent);
	Resize(270, 100);
	FixSize();
	SetCaption(_("Create", 19));
	mStatic.Resize(250, 20);
	mStatic.Move(10, 5);
	mEdit.Resize(250, 20);
	mEdit.Move(10, 25);
	switch (type) {
	case DIRECTORY:
		mStatic.SetText( _("New Directory", 20));
		mEdit.SetText(_("directory", 21));
		break;
	case TEXT_FILE:
		mStatic.SetText(_("New Text File", 22));
		mEdit.SetText(_("textfile", 23));
		break;
	default:
		cxError << _("CNewDialog: unknown file type", 24) << CError::Fatal;
    }
	mEdit.SelectAll();
	mButtonClear.SetName(_("C&lear", 25));
	mButtonClear.Move(10, 60);
	mButtonOK.SetName(_("&OK", 3));
	mButtonOK.Move(95, 60);
	mButtonCancel.SetName(_("&Cancel", 4));
	mButtonCancel.Move(180, 60);
	mStatic.Show();
	mEdit.Show();
	mButtonOK.Show();
	mButtonClear.Show();
	mButtonCancel.Show();
	SetFocusWindow(&mEdit);
}

/* virtual */ void CNewDialog::OnFocus(bool in)
{
	if (in) SetFocusWindow(&mEdit);
	else SetFocusWindow(NULL);
}

/* virtual */ void CNewDialog::OnKey(wchar_t key, unsigned state)
{
	if (key == L'\n') OnOK();
	else if (key == '\x1b') OnQuit();
	else CTopLevelWindow::OnKey(key, state);
	if (mEdit.GetText().empty()) {
		mButtonClear.Enable(false);
		mButtonOK.Enable(false);
	} else {
		mButtonClear.Enable(true);
		mButtonOK.Enable(true);
	}
}

/* virtual */ void CNewDialog::OnMessage(unsigned msgid, TMessage type __attribute__ ((unused)), unsigned value __attribute__ ((unused)))
{
	switch (msgid) {
	case IDCLEAR:
		mEdit.SetText("");
		mButtonClear.Enable(false);
		mButtonOK.Enable(false);
		break;
	case IDOK:
		OnOK();
		break;
	case IDCANCEL:
		OnQuit();
		break;
	default:
		break;
	}
}

void CNewDialog::OnOK()
{
	if (mCWD.empty() || (mCWD[mCWD.size() - 1] != '/')) mCWD += '/';
	mCWD += mEdit.GetText();
	unsigned len = mCWD.size();
	bool ok = true;
	if (mType == TEXT_FILE) {
		std::ofstream o;
		o.open(mCWD.c_str());
		ok = o;
		if (ok) ok = o << "\n";
	} else if (mType == DIRECTORY) {
		if (mkdir(mCWD.c_str(), S_IRWXU | S_IRGRP | S_IXGRP | S_IROTH | S_IXOTH)) ok = false;
	} else assert(0);
	if (!ok) {
		cxError << _("Could not create {1}:\n{2}", 26) << mCWD << strerror(errno) << CError::Error;
		if (errno == EEXIST) {
			mCWD.resize(len);
			return ;
		}
	}
	mShutDown = true;
}
