/*----------------------------------------------------------------------------
  Project: Xplorer <http://cxplorer.sourceforge.net>
  File   : CPreviewDialog.cpp
  Info   : $Id: CPreviewDialog.cpp,v 1.20 2012/09/12 13:14:49 gkraeuter Exp $
----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
  Copyright © 2005 - 2012 Gerhard Kräuter <gkraeuter@users.sourceforge.net>

  This  program  is  free software: you can redistribute it and/or modify it
  under  the  terms  of  the  GNU General Public License as published by the
  Free Software Foundation,  either  version  3  of the License, or (at your
  option) any later version.

  This  program  is  distributed  in  the  hope  that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  or  FITNESS FOR A PARTICULAR PURPOSE.  See the  GNU General Public License
  for more details.

  You  should  have  received a copy of the GNU General Public License along
  with this program. If not, see  <http://www.gnu.org/licenses/>.
----------------------------------------------------------------------------*/

// Application headers
#include "CPreviewDialog.h"
#include "CXplorerFrame.h"

// Namespace declarations
using cxxx::cxgettext;
using cxxx::cxMainWin;
using cxxx::cxWiden;

CPreviewDialog::CPreviewDialog(unsigned flags)
: CTopLevelWindow(),
  mDesktop(this, IDDESKTOP),
  mPPM(this, IDPPM),
  mXPM(this, IDXPM),
  mBMP(this, IDBMP),
#ifdef CXXX_HAS_GIF
  mGIF(this, IDGIF),
#endif // CXXX_HAS_GIF
#ifdef CXXX_HAS_JPEG
  mJPEG(this, IDJPEG),
#endif // CXXX_HAS_JPEG
  mPNG(this, IDPNG),
#ifdef CXXX_HAS_TIFF
  mTIFF(this, IDTIFF),
#endif // CXXX_HAS_TIFF
  mFileSize(this),
  mSize10k(&mFileSize, IDSIZE10K),
  mSize30k(&mFileSize, IDSIZE30K),
  mSize100k(&mFileSize, IDSIZE100K),
  mSize300k(&mFileSize, IDSIZE300K),
  mSize1M(&mFileSize, IDSIZE1M),
  mSizeAll(&mFileSize, IDSIZEALL),
  mOK(this, IDOK),
  mCancel(this, IDCANCEL),
  mStatic(this)
{
	SetCaption(_("File preview", 137));
	unsigned y = 10;
	mDesktop.SetName(_("Desktop Configuration File", 106));
	mDesktop.Move(10, y);
	y += 20;
	mPPM.SetName(_("Netpbm formats (PAM, PBM, PGM, PPM)", 135));
	mPPM.Move(10, y);
	y += 20;
	mXPM.SetName(_("X Pixmap (XPM) and X Cursor", 94));
	mXPM.Move(10, y);
	y += 20;
	mBMP.SetName(_("Bitmap (BMP)", 68));
	mBMP.Move(10, y);
	y += 20;
	mPNG.SetName(_("Portaple Network Graphics (PNG)", 133));
	mPNG.Move(10, y);
	y += 20;
#ifdef CXXX_HAS_GIF
	mGIF.SetName(_("Graphics Interchange Format (GIF)", 105));
	mGIF.Move(10, y);
	y += 20;
#endif // CXXX_HAS_GIF
#ifdef CXXX_HAS_JPEG
	mJPEG.SetName(_("JPEG Image (JPEG)", 55));
	mJPEG.Move(10, y);
	y += 20;
#endif //CXXX_HAS_JPEG
#ifdef CXXX_HAS_TIFF
	mTIFF.SetName(_("Tagged Image File (TIFF)", 111));
	mTIFF.Move(10, y);
#endif // CXXX_HAS_TIFF
	mStatic.SetText(_("Maximum size", 93));
	mStatic.Resize(mStatic.GetTextWidth(cxWiden(mStatic.GetText())) + 1, mStatic.GetTextHeight());
	mStatic.Move(280, 20 - GetTextHeight() / 2);
	mSize10k.SetName(_("10 kBytes", 130));
	mSize10k.Move(280, 30);
	mSize30k.SetName(_("30 kBytes", 131));
	mSize30k.Move(280, 50);
	mSize100k.SetName(_("100 kBytes", 132));
	mSize100k.Move(280, 70);
	mSize300k.SetName(_("300 kBytes", 92));
	mSize300k.Move(280, 90);
	mSize1M.SetName(_("1000 kBytes", 107));
	mSize1M.Move(280, 110);
	mSizeAll.SetName(_("Unlimited", 136));
	mSizeAll.Move(280, 130);
	mOK.SetName(_("&OK", 3));
	mOK.Move(10, 175);
	mCancel.SetName(_("&Cancel", 4));
	mCancel.Move(310, 175); 
	TransientFor(cxMainWin);
	mReturn = 0;
	if (flags & CXplorerFrame::VIEWDESKTOP) {
		mReturn |= CXplorerFrame::VIEWDESKTOP;
		mDesktop.SetCheck(true);
	}
	if (flags & CXplorerFrame::VIEWXPM) {
		mReturn |= CXplorerFrame::VIEWXPM;
		mXPM.SetCheck(true);
	}
	if (flags & CXplorerFrame::VIEWPPM) {
		mReturn |= CXplorerFrame::VIEWPPM;
		mPPM.SetCheck(true);
	}
	if (flags & CXplorerFrame::VIEWBMP) {
		mReturn |= CXplorerFrame::VIEWBMP;
		mBMP.SetCheck(true);
	}
	if (flags & CXplorerFrame::VIEWPNG) {
		mReturn |= CXplorerFrame::VIEWPNG;
		mPNG.SetCheck(true);
	}
#ifdef CXXX_HAS_GIF
	if (flags & CXplorerFrame::VIEWGIF) {
		mReturn |= CXplorerFrame::VIEWGIF;
		mGIF.SetCheck(true);
	}
#endif // CXXX_HAS_GIF
#ifdef CXXX_HAS_JPEG
	if (flags & CXplorerFrame::VIEWJPG) {
		mReturn |= CXplorerFrame::VIEWJPG;
		mJPEG.SetCheck(true);
	}
#endif // CXXX_HAS_JPEG
#ifdef CXXX_HAS_TIFF
	if (flags & CXplorerFrame::VIEWTIFF) {
		mReturn |= CXplorerFrame::VIEWTIFF;
		mTIFF.SetCheck(true);
	}
#endif // CXXX_HAS_TIFF
	switch (flags & CXplorerFrame::VIEWSIZE) {
	case CXplorerFrame::VIEW10K:
		mSize10k.SetCheck(true);
		break;
	case CXplorerFrame::VIEW30K:
		mSize30k.SetCheck(true);
		break;
	case CXplorerFrame::VIEW100K:
		mSize100k.SetCheck(true);
		break;
	case CXplorerFrame::VIEW300K:
		mSize300k.SetCheck(true);
		break;
	case CXplorerFrame::VIEW1M:
		mSize1M.SetCheck(true);
		break;
	default:
		mSizeAll.SetCheck(true);
		flags &= ~CXplorerFrame::VIEWSIZE;
		flags |= CXplorerFrame::VIEWALL;
	}
	mReturn = flags;
	Resize(400, 210);
	FixSize();
	mDesktop.Show();
	mPPM.Show();
	mXPM.Show();
	mBMP.Show();
	mPNG.Show();
#ifdef CXXX_HAS_GIF
	mGIF.Show();
#endif // CXXX_HAS_GIF
#ifdef CXXX_HAS_JPEG
	mJPEG.Show();
#endif // CXXX_HAS_JPEG
#ifdef CXXX_HAS_TIFF
	mTIFF.Show();
#endif // CXXX_HAS_TIFF
	mSize10k.Show();
	mSize30k.Show();
	mSize100k.Show();
	mSize300k.Show();
	mSize1M.Show();
	mSizeAll.Show();
	mStatic.Show();
	mOK.Show();
	mCancel.Show();
}

/* virtual */ void CPreviewDialog::OnExpose(XExposeEvent& e __attribute__ ((unused)))
{
	DrawFrame(270, 20, 120, 140, FS_ENGRAVED, false);
}

/* virtual */ void CPreviewDialog::OnMessage(unsigned msgid, TMessage type, unsigned value __attribute__ ((unused)))
{
	switch (msgid) {
	case IDDESKTOP:
		if (type == MT_CHECK) mReturn |= CXplorerFrame::VIEWDESKTOP;
		else if (type == MT_UNCHECK) mReturn &= ~CXplorerFrame::VIEWDESKTOP;
		break;
	case IDPPM:
		if (type == MT_CHECK) mReturn |= CXplorerFrame::VIEWPPM;
		else if (type == MT_UNCHECK) mReturn &= ~CXplorerFrame::VIEWPPM;
		break;
	case IDXPM:
		if (type == MT_CHECK) mReturn |= CXplorerFrame::VIEWXPM;
		else if (type == MT_UNCHECK) mReturn &= ~CXplorerFrame::VIEWXPM;
		break;
	case IDBMP:
		if (type == MT_CHECK) mReturn |= CXplorerFrame::VIEWBMP;
		else if (type == MT_UNCHECK) mReturn &= ~CXplorerFrame::VIEWBMP;
		break;
	case IDPNG:
		if (type == MT_CHECK) mReturn |= CXplorerFrame::VIEWPNG;
		else if (type == MT_UNCHECK) mReturn &= ~CXplorerFrame::VIEWPNG;
		break;
	case IDGIF:
		if (type == MT_CHECK) mReturn |= CXplorerFrame::VIEWGIF;
		else if (type == MT_UNCHECK) mReturn &= ~CXplorerFrame::VIEWGIF;
		break;
	case IDJPEG:
		if (type == MT_CHECK) mReturn |= CXplorerFrame::VIEWJPG;
		else if (type == MT_UNCHECK) mReturn &= ~CXplorerFrame::VIEWJPG;
		break;
	case IDTIFF:
		if (type == MT_CHECK) mReturn |= CXplorerFrame::VIEWTIFF;
		else if (type == MT_UNCHECK) mReturn &= ~CXplorerFrame::VIEWTIFF;
		break;
	case IDSIZE10K:
		if (type == MT_CHECK) {
			mReturn &= ~CXplorerFrame::VIEWSIZE;
			mReturn |= CXplorerFrame::VIEW10K;
		}
		break;
	case IDSIZE30K:
		if (type == MT_CHECK) {
			mReturn &= ~CXplorerFrame::VIEWSIZE;
			mReturn |= CXplorerFrame::VIEW30K;
		}
		break;
	case IDSIZE100K:
		if (type == MT_CHECK) {
			mReturn &= ~CXplorerFrame::VIEWSIZE;
			mReturn |= CXplorerFrame::VIEW100K;
		}
		break;
	case IDSIZE300K:
		if (type == MT_CHECK) {
			mReturn &= ~CXplorerFrame::VIEWSIZE;
			mReturn |= CXplorerFrame::VIEW300K;
		}
		break;
	case IDSIZE1M:
		if (type == MT_CHECK) {
			mReturn &= ~CXplorerFrame::VIEWSIZE;
			mReturn |= CXplorerFrame::VIEW1M;
		}
		break;
	case IDSIZEALL:
		if (type == MT_CHECK) {
			mReturn &= ~CXplorerFrame::VIEWSIZE;
			mReturn |= CXplorerFrame::VIEWALL;
		}
		break;
	case IDOK:
		mShutDown = true;
		break;
	case IDCANCEL:
		OnQuit();
		break;
	default:
		break;
	}
}
