/*----------------------------------------------------------------------------
  Project: Xplorer <http://cxplorer.sourceforge.net>
  File   : CPropertyDialog.cpp
  Info   : $Id: CPropertyDialog.cpp,v 1.22 2012/08/06 17:36:40 gkraeuter Exp $
----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
  Copyright © 2002 - 2012 Gerhard Kräuter <gkraeuter@users.sourceforge.net>

  This  program  is  free software: you can redistribute it and/or modify it
  under  the  terms  of  the  GNU General Public License as published by the
  Free Software Foundation,  either  version  3  of the License, or (at your
  option) any later version.

  This  program  is  distributed  in  the  hope  that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  or  FITNESS FOR A PARTICULAR PURPOSE.  See the  GNU General Public License
  for more details.

  You  should  have  received a copy of the GNU General Public License along
  with this program. If not, see  <http://www.gnu.org/licenses/>.
----------------------------------------------------------------------------*/

// Application headers
#include "CXplorerList.h"
#include "CPropertyDialog.h"
#include "CGeneralSheet.h"
#include "CPermissionSheet.h"

// Namespace declarations
using std::string;
using cxxx::cxgettext;

CPropertyDialog::CPropertyDialog(CWindow* parent, CTapeArchive* arch, const string& file)
: CTopLevelWindow(),
  mTab(this, IDTAB),
  mGeneral(this, arch, file),
  mPermission(this, arch, file),
  mButtonOK(this, IDOK),
  mButtonCancel(this, IDCANCEL)
{
	TransientFor(parent->GetTopLevelWindow());
	Resize(400, 300);
	FixSize();
	SetCaption(_("Properties", 44));
	mTab.Resize(390, 25);
	mTab.Move(5, 5);
	mTab.AddTab(_("General", 45));
	mTab.AddTab(_("Permissions", 46));
	mGeneral.Resize(390, 225);
	mGeneral.Move(5, 30);
	mPermission.Resize(390, 225);
	mPermission.Move(5, 30);
	mButtonOK.SetName(_("&OK", 3));
	mButtonOK.Move(225, 265);
	mButtonCancel.SetName(_("&Cancel", 4));
	mButtonCancel.Move(315, 265);
	mTab.Show();
	mGeneral.Show();
	mButtonOK.Show();
	mButtonCancel.Show();
}

/* virtual */ void CPropertyDialog::OnFocus(bool in)
{
	if (in) {
		if (mPermission.IsVisible()) mPermission.SetFocus();
		else if (mGeneral.IsVisible()) mGeneral.SetFocus();
	}
}

/* virtual */ void CPropertyDialog::OnMessage(unsigned msgid, TMessage type, unsigned value)
{
	switch (msgid) {
	case IDTAB:
		if (MT_SELECTION_CHANGED) {
			if (mTab.GetSelection() == 0) OnGeneral();
			else OnPermission();
		}
		break;
	case IDOK:
		OnOK();
		break;
	case IDCANCEL:
		OnQuit();
		break;
	default:
		CTopLevelWindow::OnMessage(msgid, type, value);
		break;
	}
}

/* virtual */ void CPropertyDialog::OnKey(wchar_t key, unsigned state)
{
	if (mFocusWindow) {
		mFocusWindow->OnKey(key, state);
		if (key == L'\n') OnOK();
	}
}
