/*----------------------------------------------------------------------------
  Project: Xplorer <http://cxplorer.sourceforge.net>
  File   : CPropertyDialog.h
  Info   : $Id: CPropertyDialog.h,v 1.13 2012/06/26 19:52:23 gkraeuter Exp $
----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
  Copyright © 2002 - 2012 Gerhard Kräuter <gkraeuter@users.sourceforge.net>

  This  program  is  free software: you can redistribute it and/or modify it
  under  the  terms  of  the  GNU General Public License as published by the
  Free Software Foundation,  either  version  3  of the License, or (at your
  option) any later version.

  This  program  is  distributed  in  the  hope  that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  or  FITNESS FOR A PARTICULAR PURPOSE.  See the  GNU General Public License
  for more details.

  You  should  have  received a copy of the GNU General Public License along
  with this program. If not, see  <http://www.gnu.org/licenses/>.
----------------------------------------------------------------------------*/

#ifndef CPropertyDialog_h
#define CPropertyDialog_h

// Application headers
#include "Xplorer.h"
#include "CPermissionSheet.h"
#include "CGeneralSheet.h"

// C++X headers
#include <cxxx/CTopLevelWindow.h>
#include <cxxx/CPropertyTab.h>
#include <cxxx/CPushButton.h>

// Forward declarations
class CXplorerList;

class CPropertyDialog : public CTopLevelWindow
{
private:
	enum
	{
		// Message IDs
		IDTAB = xplorer::MSGID_CPropertyDialog,
		IDOK,
		IDCANCEL
	};

public:
	// Constructor
	CPropertyDialog(CWindow* parent, CTapeArchive* arch, const std::string& file);

	// X event processing
	virtual void OnFocus(bool in);

	// C++X message processing
	virtual void OnMessage(unsigned msgid, TMessage type, unsigned value);

	// Exit function
	virtual void OnQuit()
	{
		mShutDown = true;
	}

private:
	// Opens general sheet
	void OnGeneral()
	{
		mPermission.Hide();
		mGeneral.Show();
	}

	// Opens permission sheet
	void OnPermission()
	{
		mGeneral.Hide();
		mPermission.Show();
	}

	// Exit function
	void OnOK()
	{
		if (!mPermission.DoOK()) return ;
		if (!mGeneral.DoOK()) return ;
		OnQuit();
	}

	// Function override to exit on return
	virtual void OnKey(wchar_t key, unsigned state);
	
private:
	CPropertyTab mTab;
	CGeneralSheet mGeneral;
	CPermissionSheet mPermission;
	CPushButton mButtonOK;
	CPushButton mButtonCancel;
};

#endif // CPropertyDialog_h
