/*----------------------------------------------------------------------------
  Project: Xplorer <http://cxplorer.sourceforge.net>
  File   : CSortDialog.cpp
  Info   : $Id: CSortDialog.cpp,v 1.20 2012/06/26 08:05:52 gkraeuter Exp $
----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
  Copyright © 2002 - 2012 Gerhard Kräuter <gkraeuter@users.sourceforge.net>

  This  program  is  free software: you can redistribute it and/or modify it
  under  the  terms  of  the  GNU General Public License as published by the
  Free Software Foundation,  either  version  3  of the License, or (at your
  option) any later version.

  This  program  is  distributed  in  the  hope  that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  or  FITNESS FOR A PARTICULAR PURPOSE.  See the  GNU General Public License
  for more details.

  You  should  have  received a copy of the GNU General Public License along
  with this program. If not, see  <http://www.gnu.org/licenses/>.
----------------------------------------------------------------------------*/

// Application headers
#include "CSortDialog.h"

// Namespace declarations
using cxxx::cxMainWin;
using cxxx::filesort::DIRFIRST;
using cxxx::filesort::ASCENDING;
using cxxx::filesort::NAME;
using cxxx::filesort::SIZE;
using cxxx::filesort::TIME;
using cxxx::filesort::TYPE;
using cxxx::cxgettext;
using cxxx::cxWiden;

CSortDialog::CSortDialog(unsigned flags)
: CTopLevelWindow(),
  mSortGroup (this),
  mDirFirst(this, IDDIRFIRST),
  mAscending(this, IDASCENDING),
  mAlpha(&mSortGroup, IDNAME),
  mSize(&mSortGroup, IDSIZE),
  mTime(&mSortGroup, IDTIME),
  mType(&mSortGroup, IDTYPE),
  mStatic1(this),
  mStatic2(this),
  mPushOK(this, IDOK),
  mPushDef(this, IDDEFAULT),
  mPushCancel(this, IDCANCEL)
{
	SetCaption(_("File sorting", 47));
	Resize(360, 200);
	int y = 10 - GetTextHeight() / 2;
	mStatic1.SetText(_("Sort key", 48));
	mStatic1.Resize(mStatic1.GetTextWidth(cxWiden(mStatic1.GetText())), mStatic1.GetTextHeight());
	mStatic1.Move(20, y);
	mStatic2.SetText(_("Options", 49));
	mStatic2.Resize(mStatic2.GetTextWidth(cxWiden(mStatic2.GetText())), mStatic2.GetTextHeight());
	mStatic2.Move(200, y);
	mAlpha.SetName(_("&Alphabetic", 50));
	mAlpha.Move(20, 20);
	mTime.SetName(_("By &Modification Time", 51));
	mTime.Move(20, 40);
	mSize.SetName(_("By &Size", 52));
	mSize.Move(20, 60);
	mType.SetName(_("By &Type", 53));
	mType.Move(20, 80);
	mDirFirst.SetName(_("&Directories first", 54));
	mDirFirst.Move(200, 20);
	mAscending.SetName(_("Asc&ending order", 56));
	mAscending.Move(200, 40);
	mPushOK.Resize(100, 24);
	mPushOK.SetName(_("&OK", 3));
	mPushOK.Move(10, 160);
	mPushDef.Resize(100, 24);
	mPushDef.SetName(_("De&fault", 57));
	mPushDef.Move(130, 160);
	mPushCancel.Resize(100, 24);
	mPushCancel.SetName(_("&Cancel", 4));
	mPushCancel.Move(250, 160);
	TransientFor(cxMainWin);
	mReturn = flags;
	mDirFirst.SetCheck(flags & DIRFIRST);
	mAscending.SetCheck(flags & ASCENDING);
	flags &= NAME | SIZE | TIME | TYPE;
	if (flags & NAME) mAlpha.SetCheck(true);
	else if(flags & SIZE) mSize.SetCheck(true);
	else if(flags & TIME) mTime.SetCheck(true);
	else if(flags & TYPE) mType.SetCheck(true);
	else mAlpha.SetCheck(true);
	FixSize();
	mDirFirst.Show();
	mAscending.Show();
	mAlpha.Show();
	mSize.Show();
	mTime.Show();
	mType.Show();
	mStatic1.Show();
	mStatic2.Show();
	mPushOK.Show();
	mPushDef.Show();
	mPushCancel.Show();
}

/* virtual */ void CSortDialog::OnExpose(XExposeEvent& e __attribute__ ((unused)))
{
	DrawFrame(10, 10, 170, 140, FS_ENGRAVED, false);
	DrawFrame(190, 10, 160, 60, FS_ENGRAVED, false);
}

/* virtual */ void CSortDialog::OnMessage(unsigned msgid, TMessage type, unsigned value __attribute__ ((unused)))
{
	switch (msgid) {
	case IDOK:
		mReturn |= OK;
		mShutDown = true;
		break;
	case IDDEFAULT:
		mAlpha.SetCheck(true);
		mReturn |= NAME;
		mDirFirst.SetCheck(true);
		mReturn |= DIRFIRST;
		mAscending.SetCheck(true);
		mReturn |= ASCENDING;
		break;
	case IDCANCEL:
		OnQuit();
		break ;
	case IDNAME:
		if (type == MT_CHECK) mReturn |= NAME;
		else if (type ==  MT_UNCHECK) mReturn &= ~NAME;
		break;
	case IDASCENDING:
		if (type ==  MT_CHECK) mReturn |= ASCENDING;
		else if (type ==  MT_UNCHECK) mReturn &= ~ASCENDING;
		break;
	case IDDIRFIRST:
		if (type ==  MT_CHECK) mReturn |= DIRFIRST;
		else if (type ==  MT_UNCHECK) mReturn &= ~DIRFIRST;
		break;
	case IDSIZE:
		if (type ==  MT_CHECK) mReturn |= SIZE;
		else if (type ==  MT_UNCHECK) mReturn &= ~SIZE;
		break;
	case IDTIME:
		if (type ==  MT_CHECK) mReturn |= TIME;
		else if (type ==  MT_UNCHECK) mReturn &= ~TIME;
		break;
	case IDTYPE:
		if (type ==  MT_CHECK) mReturn |= TYPE;
		else if (type ==  MT_UNCHECK) mReturn &= ~TYPE;
		break;
	default:
		break;
	}
}
