/*----------------------------------------------------------------------------
  Project: Xplorer <http://cxplorer.sourceforge.net>
  File   : CTermConfig.h
  Info   : $Id: CTermConfig.h,v 1.2 2012/03/23 16:56:42 gkraeuter Exp $
----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
  Copyright © 2012 Gerhard Kräuter <gkraeuter@users.sourceforge.net>

  This  program  is  free software: you can redistribute it and/or modify it
  under  the  terms  of  the  GNU General Public License as published by the
  Free Software Foundation,  either  version  3  of the License, or (at your
  option) any later version.

  This  program  is  distributed  in  the  hope  that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  or  FITNESS FOR A PARTICULAR PURPOSE.  See the  GNU General Public License
  for more details.

  You  should  have  received a copy of the GNU General Public License along
  with this program. If not, see  <http://www.gnu.org/licenses/>.
----------------------------------------------------------------------------*/

#ifndef CTermConfig_h
#define CTermConfig_h

// Application headers
#include "Xplorer.h"

// C++X headers
#include <cxxx/CTopLevelWindow.h>
#include <cxxx/CEdit.h>
#include <cxxx/CToolButton.h>
#include <cxxx/CPushButton.h>

class CTermConfig : public CTopLevelWindow
{
public:
	enum
	{
		// Return values
		OK,
		CANCEL
	};

private:
	enum
	{
		// Message IDs
		IDOK = xplorer::MSGID_CTermConfig,
		IDCANCEL,
		IDBROWSE
	};

public:
	// Constructor
	CTermConfig(CWindow* parent, const std::string& cmd, const std::string& opt);
	
	// X event processing
	virtual void OnExpose(XExposeEvent& e);
	virtual void OnKey(wchar_t key, unsigned mask);

	// C++X message processing
	virtual void OnMessage(unsigned msgid, TMessage type, unsigned value);
	virtual bool OnUpdate(unsigned msgid) const;

	// Exits this window
	virtual void OnQuit();

	// Access functions

	std::string GetTerminalCommand() const
	{
		return mPath.GetText();
	}

	std::string GetTerminalArgs() const
	{
		return mArguments.GetText();
	}

	// Sets the search path for binaries
	void SetBinPath(const std::string& path)
	{
		mExecdir = path;
	}
	
private:
	// Opens a browser window
	void OnBrowse();
	// Helper function to detect executable files
	bool IsExecutable(const std::string& file) const;
private:
	std::string mExecdir;
	CEdit mPath;
	CEdit mArguments;
	CToolButton mBrowse;
	CPushButton mOK;
	CPushButton mCancel;
};

#endif // CTermConfig_h
