/*----------------------------------------------------------------------------
  Project: Xplorer <http://cxplorer.sourceforge.net>
  File   : CUntrashDialog.cpp
  Info   : $Id: CUntrashDialog.cpp,v 1.21 2012/07/17 12:16:43 gkraeuter Exp $
----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
  Copyright © 2007 - 2012 Gerhard Kräuter <gkraeuter@users.sourceforge.net>

  This  program  is  free software: you can redistribute it and/or modify it
  under  the  terms  of  the  GNU General Public License as published by the
  Free Software Foundation,  either  version  3  of the License, or (at your
  option) any later version.

  This  program  is  distributed  in  the  hope  that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  or  FITNESS FOR A PARTICULAR PURPOSE.  See the  GNU General Public License
  for more details.

  You  should  have  received a copy of the GNU General Public License along
  with this program. If not, see  <http://www.gnu.org/licenses/>.
----------------------------------------------------------------------------*/

// Application headers
#include "CUntrashDialog.h"
#include "CFileAssoc.h"

// C++X headers
#include <cxxx/CProfile.h>
#include <cxxx/CFileOpenDialog.h>

// C++ headers
#include <cassert>

// Namespace declarations
using cxxx::cxgettext;
using cxxx::cxFileManager;
using cxxx::cxProfile;
using cxxx::cxWiden;
using cxxx::cxNarrow;
using std::string;
using std::vector;

CUntrashDialog::CUntrashDialog(CWindow* parent)
: CTopLevelWindow(),
  mFileList(this, IDFILELIST),
  mClear(this, IDCLEAR),
  mRestore(this, IDRESTORE),
  mClose(this, IDCLOSE),
  mBrowse(this, IDBROWSE, CIcon::BROWSE, "Browse"),
  mPath(this)
{
	TransientFor(parent->GetTopLevelWindow());
	SetCaption(_("Restore File", 154));
	Resize(400, 300);
	FixSize();
	mFileList.SetDirectory(cxFileManager->GetTrashDir());
	mClear.SetName(_("&Empty Trashcan", 153));
	mRestore.SetName(_("&Restore", 156));
	mClose.SetName(_("&Close", 58));
	mFileList.Show();
	if (mFileList.GetItemCount() == 0) mClear.Enable(false);
	mClear.Show();
	mRestore.Enable(false);
	mRestore.Show();
	mClose.Show();
	mPath.Enable(false);
	mPath.Show();
	mBrowse.Show();
}

/* virtual */ void CUntrashDialog::OnConfigure(XConfigureEvent& e __attribute__ ((unused)))
{
	mFileList.Resize(390, 230);
	mFileList.Move(5, 5);
	int x = GetTextWidth(cxWiden(_("Restore to", 158))) + 10;
	mPath.Resize(390 - mBrowse.GetWidth() - x, mPath.GetHeight());
	mPath.Move(x, 240);
	mClear.Resize(126, mClear.GetHeight());
	mClear.Move(5, 270);
	mRestore.Resize(125, mRestore.GetHeight());
	mRestore.Move(138, 270);
	mClose.Resize(125, mRestore.GetHeight());
	mClose.Move(270, 270);
	mBrowse.Move(373, 238);
}
	

/* virtual */ void CUntrashDialog::OnExpose(XExposeEvent& e __attribute__ ((unused)))
{
	DrawString(5, 240 + (mPath.GetHeight() - GetTextHeight()) / 2, CColor(TC_FRAME_TEXT), _("Restore to", 158));
}

/* virtual */ void CUntrashDialog::OnMessage(unsigned msgid, TMessage type, unsigned value __attribute__ ((unused)))
{
	switch (msgid) {
	case IDCLEAR:
		if (mFileList.GetItemCount() != 0) OnEmptyTrashcan();		
		break;
	case IDRESTORE:
		if (mFileList.GetSelCount()	!= 0) {
			const char* txt = mFileList.GetItem(mFileList.GetSelection()[0]).GetText().c_str();
			txt += strlen(txt) + 1;
			cxFileManager->Untrash(txt, mPath.GetText());
			mFileList.Reload();
			OnSelectionChanged();
		}
		break;
	case IDFILELIST:
		if (type == MT_SELECTION_CHANGED) OnSelectionChanged();
		break;
	case IDCLOSE:
		OnQuit();
		break;
	case IDEDIT_DEL:
		OnDelete();
		break;
	case IDOPEN_WITH:
		OnOpenWith();
		break;
	case IDLISTPROP:
		mFileList.OnProperties();
		break;
	case IDBROWSE:
		OnBrowse();
		break;
	default:
		assert(0);
		break;
	}
}

/* virtual */ bool CUntrashDialog::OnUpdate(unsigned msgid) const
{
	switch (msgid) {
	case IDEDIT_DEL:
	case IDOPEN_WITH:
	case IDLISTPROP:
	case IDBROWSE:
		return mFileList.GetSelCount() == 1;
	default:
		return false;
		break;
	}
}


void CUntrashDialog::OnSelectionChanged()
{
	if (mFileList.GetSelCount() == 0) {
		if (mRestore.IsEnabled()) mRestore.Enable(false);
		mPath.SetText("");
		mPath.Enable(false);
	} else {
		if (!mRestore.IsEnabled()) mRestore.Enable(true);
		const char* txt = mFileList.GetItem(mFileList.GetSelection()[0]).GetText().c_str();
		txt += strlen(txt) + 1;
		txt += strlen(txt) + 1;
		mPath.SetText(txt);
		mPath.Enable(true);
	}
	if (mFileList.GetItemCount() == 0) {
		if (mClear.IsEnabled()) mClear.Enable(false);
	} else {
		if (!mClear.IsEnabled()) mClear.Enable(true);
	}					
}

void CUntrashDialog::OnDelete()
{
	const char* name = mFileList.GetItem(mFileList.GetSelection()[0]).GetText().c_str();
	string text = _("Really delete file \'%s\'?\n\nThere is no way to restore the file!", 59);
	string::size_type pos = text.find("%s");
	if (pos != string::npos) text.replace(pos, 2, name);
	string caption = _("Delete File", 60);
	CMessageBox mb (text, caption, CMessageBox::OK | CMessageBox::CANCEL, CIcon::WARNING);
	mb.FocusButton(CMessageBox::CANCEL);
	if (mb.DoModal() ==	CMessageBox::OK) {
		name += strlen(name) + 1;
		cxFileManager->Delete(cxFileManager->GetAbsolutePath(name, cxFileManager->GetTrashDir()));
	}
	mFileList.Reload();
}

void CUntrashDialog::OnOpenWith()
{
	if (mFileList.GetSelCount() == 1) {
		unsigned idx = mFileList.GetSelection().front();
		CFileListItem sel = mFileList.GetItem(idx);
		string cmd; // = GetFileAssoc(mList.GetPath(idx).c_str(), sel.GetType());
		int x = 10;
		int y = 10;
		TranslateCoordinates(x, y, NULL);
		CFileAssoc dlg(this, sel.GetText().c_str(), sel.GetType(), false);
		dlg.SetBinPath(cxProfile->Read("Executables Path", "/usr/bin"));
		dlg.Move(x, y);
		dlg.Show();
		if (dlg.DoModal() == CFileAssoc::OK) {
			const char* fn = sel.GetText().c_str();
			fn += strlen(fn) + 1;
			string::size_type pos = 0;
			string filename = cxFileManager->GetAbsolutePath(fn, cxFileManager->GetTrashDir());
			while ((pos = filename.find(' ', pos)) < filename.size()) {
				filename.replace(pos, 1, "\\ ");
				pos += 2;
			}
			if (dlg.NeedTerminal()) {
				cmd = cxProfile->Read("Terminal Command", "xterm");
				cmd += " -hold -e ";
			}
			if (dlg.IsExecutable()) {
				cmd += filename;
				if (!dlg.GetArguments().empty()) cmd += " " + dlg.GetArguments();
			} else {
				string execpath = dlg.GetPath();
				string::size_type pos = 0;
				while ((pos = execpath.find(' ', pos)) < execpath.size()) {
					execpath.replace(pos, 1, "\\ ");
					pos += 2;
				}
				cmd += execpath;
				if (!dlg.GetArguments().empty()) cmd += " " + dlg.GetArguments();
				cmd	+= " " + filename;
			}
			cxFileManager->Execute(cmd, cxFileManager->GetTrashDir());
		} 
	}
}

void CUntrashDialog::OnBrowse()
{
	string dir;
	if (!mPath.GetText().empty()) {
		dir = mPath.GetText();
		struct stat st;
		if (stat(dir.c_str(), &st) != 0 || !S_ISDIR(st.st_mode)) {
			string::size_type pos = dir.rfind('/');
			if (pos != string::npos) dir.resize(pos);
			else dir = "/";
		}
	} else dir = "/";
	mPath.SetText(dir);
	vector<unsigned> filter;
	filter.push_back(TMagic::DIRECTORY);
	CFileOpenDialog dlg(this, dir, filter);
	int x = 10;
	int y = 10;
	TranslateCoordinates(x, y, NULL);
	dlg.Move(x, y);
	dlg.Show();
	if (dlg.DoModal() == CFileOpenDialog::OK) mPath.SetText(dlg.GetPath());
}
