/*----------------------------------------------------------------------------
  Project: Xplorer <http://cxplorer.sourceforge.net>
  File   : CUntrashDialog.h
  Info   : $Id: CUntrashDialog.h,v 1.12 2012/06/08 13:03:40 gkraeuter Exp $
----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
  Copyright © 2007 - 2012 Gerhard Kräuter <gkraeuter@users.sourceforge.net>

  This  program  is  free software: you can redistribute it and/or modify it
  under  the  terms  of  the  GNU General Public License as published by the
  Free Software Foundation,  either  version  3  of the License, or (at your
  option) any later version.

  This  program  is  distributed  in  the  hope  that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  or  FITNESS FOR A PARTICULAR PURPOSE.  See the  GNU General Public License
  for more details.

  You  should  have  received a copy of the GNU General Public License along
  with this program. If not, see  <http://www.gnu.org/licenses/>.
----------------------------------------------------------------------------*/

#ifndef CUntrashDialog_h
#define CUntrashDialog_h

// Application headers
#include "Xplorer.h"
#include "CTrashListControl.h"

// C++X headers
#include <cxxx/CTopLevelWindow.h>
#include <cxxx/CPushButton.h>
#include <cxxx/CToolButton.h>
#include <cxxx/CEdit.h>
#include <cxxx/CMessageBox.h>
#include <cxxx/CFileHandler.h>

class CUntrashDialog : public CTopLevelWindow
{
public:
	enum
	{
		// Message IDs
		IDCLEAR = xplorer::MSGID_CUntrashDialog,
		IDFILELIST,
		IDRESTORE,
		IDCLOSE,
		IDEDIT_DEL,
		IDOPEN_WITH,
		IDLISTPROP,
		IDBROWSE
	};

public:
	// Constructor
	CUntrashDialog(CWindow* parent);

	// X event processing
	virtual void OnConfigure(XConfigureEvent& e);
	virtual void OnExpose(XExposeEvent& e);

	// C++X message processing
	virtual void OnMessage(unsigned msgid, TMessage type, unsigned value);
	virtual bool OnUpdate(unsigned msgid) const;

	// Exit function
	virtual void OnQuit()
	{
		mReturn = 0;
		mShutDown = true;
	}

private:
	// Deletes all trashed files
	void OnEmptyTrashcan()
	{
		CMessageBox mb(cxxx::_("Really empty trashcan? There is no way to restore the deleted files.", 157), cxxx::_("Empty Trashcan", 155), CMessageBox::OK | CMessageBox::CANCEL, CIcon::WARNING);
		mb.FocusButton(CMessageBox::CANCEL);
		if (mb.DoModal() == CMessageBox::OK) {
			cxxx::cxFileManager->ClearTrash();
			mFileList.Reload();
			OnSelectionChanged();
		}
	}

	// Helper functions
	void OnSelectionChanged();
	void OnDelete();
	void OnOpenWith();
	void OnBrowse();

private:
	CTrashListControl mFileList;
	CPushButton mClear;
	CPushButton mRestore;
	CPushButton mClose;
	CToolButton mBrowse;
	CEdit mPath;
};

#endif // CUntrashDialog_h
