/*----------------------------------------------------------------------------
  Project: Xplorer <http://cxplorer.sourceforge.net>
  File   : CXplorerFrame.h
  Info   : $Id: CXplorerFrame.h,v 1.50 2012/08/06 17:36:40 gkraeuter Exp $
----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
  Copyright © 2002 - 2012 Gerhard Kräuter <gkraeuter@users.sourceforge.net>

  This  program  is  free software: you can redistribute it and/or modify it
  under  the  terms  of  the  GNU General Public License as published by the
  Free Software Foundation,  either  version  3  of the License, or (at your
  option) any later version.

  This  program  is  distributed  in  the  hope  that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  or  FITNESS FOR A PARTICULAR PURPOSE.  See the  GNU General Public License
  for more details.

  You  should  have  received a copy of the GNU General Public License along
  with this program. If not, see  <http://www.gnu.org/licenses/>.
----------------------------------------------------------------------------*/

#ifndef CXplorerFrame_h
#define CXplorerFrame_h

// Application headers
#include "CXplorerHistory.h"
#include "CXplorerTree.h"
#include "CXplorerList.h"
#include "CXplorerMenu.h"
#include "CXplorerTool.h"
#include "Xplorer.h"
#include "CXplorerAbout.h"
#include "CSortDialog.h"
#include "CNewDialog.h"
#include "CPreviewDialog.h"

// C++X headers
#include <cxxx/CTopLevelWindow.h>
#include <cxxx/CHSplitterWindow.h>
#include <cxxx/CStatusBar.h>
#include <cxxx/CFileHandler.h>
#include <cxxx/CCxxxAbout.h>
#include <cxxx/CFileOpenDialog.h>
#include <cxxx/CSettings.h>

// Forward declarations
class CProfile;
class CTreeItem;

class CXplorerFrame : public CTopLevelWindow
{
private:
	enum
	{
		// Geometry
		MIN_CONTROL_WIDTH = 100
	};
public:
	// Message IDs
	enum
	{
		IDTREE = xplorer::MSGID_CXplorerFrame,
		IDLIST,
		IDTOOL,
		IDSPLITTER,
		IDSTATUS,
		IDLIST_BIG,
		IDLIST_SMALL,
		IDLIST_LIST,
		IDLIST_DETAILS,
		IDFIND_FILE,
		IDSHOW_HIDDEN,
		IDVIEW_REFRESH,
		IDVIEW_SORT,
		IDVIEW_PREVIEW,
		IDVIEW_MULI_COLUMN,
		IDVIEW_ICON,
		IDOPEN_TERM,
		IDCONFIGURE_TERM,
		IDNEW_DIR,
		IDNEW_TEXT,
		IDNEW_LINK,
		IDNEW_KDE,
		IDNAV_UP,
		IDNAV_FORWARD,
		IDNAV_BACK,
		IDNAV_HOME,
		IDNAV_RECENT,
		IDFILE_EXIT = IDNAV_RECENT + CRecentMenu::RECENTSIZE,
		IDFILE_OPEN,
		IDEDIT_CUT,
		IDEDIT_COPY,
		IDEDIT_PASTE,
		IDEDIT_DEL,
		IDEDIT_TRASH,
		IDEDIT_SELA,
		IDEDIT_SELR,
		IDEDIT_UNDO,
		IDLISTPROP,
		IDSETTINGS,
		IDTREEPROP,
		IDTRASH,
		IDABOUT_XPLORER,
		IDABOUT_CXXX,
		IDOPEN_WITH,

		// Flags
		TOOLBAR = 1 << 0,
		STATUSBAR = 1 << 1,
		VIEWDESKTOP = 1 << 2,
		VIEWXPM = 1 << 3,
		VIEWPNG = 1 << 4,
		VIEWPPM = 1 << 5,
		VIEWGIF = 1 << 6,
		VIEWJPG = 1 << 7,
		VIEWTIFF = 1 << 8,
		VIEWBMP = 1 << 9,
		VIEW10K = 1 << 10,
		VIEW30K = 2 << 10,
		VIEW100K = 3 << 10,
		VIEW300K = 4 << 10,
		VIEW1M = 5 << 10,
		VIEWALL = 6 << 10,
		VIEWSIZE = 7 << 10,
		VIEWFLAGS = VIEWDESKTOP | VIEWXPM | VIEWPNG | VIEWPPM | VIEWGIF | VIEWJPG | VIEWTIFF | VIEWBMP | VIEWSIZE,
		VIEW_MULTICOLUMN = 1 << 13
	};

public:
	// Constructor
	CXplorerFrame();

	// Destructor
	virtual ~CXplorerFrame();

	// X event processing
	virtual void OnConfigure(XConfigureEvent& e __attribute__ ((unused)))
	{
		Configure();
	}

	virtual void OnShortcut(wchar_t key, unsigned state);
	virtual void OnKey(wchar_t key, unsigned state);

	// C++X message processing
	virtual void OnMessage(unsigned msgid, TMessage type, unsigned value);
	virtual bool OnUpdate(unsigned msgid) const;

	// Reloads alls views
    void Reload()
	{
		if (mFlags & STATUSBAR) mStatusBar.SetText(0, cxxx::_("Reloading...", 122));
		mList.Reload();
		mTree.Reload();
		UpdateStatusBar();
	}

	// Sets the current directory to dir
	void SetDirectory(std::string dir);

	// Selects filename within the list field
	void Select(const std::string& filename);

private:
	// Helper functions
	void OnAboutCxxx()
	{
		int x = 10;
		int y = 10;
		mTree.TranslateCoordinates(x, y, NULL);
		CCxxxAbout cx(this);
		cx.Move(x, y);
		cx.Show();
		cx.DoModal();
	}

	void OnAboutXplorer()
	{
		int x = 10;
		int y = 10;
		mTree.TranslateCoordinates(x, y, NULL);
		CXplorerAbout cx(this);
		cx.Move(x, y);
		cx.Show();
		cx.DoModal();
	}

	void OnListSelConfirmed()
	{
		if (mList.GetSelCount() != 1) return;
		int idx = mList.GetSelection().front();
		OpenFile(mList.GetPath(idx), mList.GetItem(idx).GetType());
	}

	void OnSortDialog()
	{
		int x = 10;
		int y = 10;
		mTree.TranslateCoordinates(x, y, NULL);
		CSortDialog sd(mSortFlags);
		sd.Move(x, y);
		sd.Show();
		unsigned ret = sd.DoModal();
		if (ret & CSortDialog::OK) {
			ret &= ~CSortDialog::OK;
			if (ret != mSortFlags) SetSortKey(ret);
		}
	}

	void OnNewDialog(unsigned type)
	{
		int x = 10;
		int y = 10;
		mTree.TranslateCoordinates(x, y, NULL);
		CNewDialog dlg(this, type, mCurrentDir);
		dlg.Move(x, y);
		dlg.Show();
		dlg.DoModal();
		Reload();
	}

	void OnOpenTerminal()
	{
		cxxx::cxFileManager->Execute(GetTerminalCommand(), mCurrentDir);
	}

	void OnOpenWith();

	void OnPreviewDialog()
	{
		int x = 10;
		int y = 10;
		mTree.TranslateCoordinates(x, y, NULL);
		CPreviewDialog pd(mFlags & VIEWFLAGS);
		pd.Move(x, y);
		pd.Show();
		unsigned ret = pd.DoModal();
		if (ret & VIEWSIZE) {
			mFlags &= ~VIEWFLAGS;
			SetFlags(mFlags | ret);
		}
	}

	void OnOpenFile()
	{
		int x = 10;
		int y = 10;
		mTree.TranslateCoordinates(x, y, NULL);
		std::vector<unsigned> dummy;
		std::string dir;
		if (mIsArchive) dir = cxxx::cxFileManager->GetFilePath(mArchive.GetName());
		else dir = mCurrentDir;
		CFileOpenDialog dlg(this, dir, dummy);
		dlg.Move(x, y);
		dlg.Show();
		if (dlg.DoModal() == CFileOpenDialog::OK) {
			CMagic mag;
			mIsArchive = false;
			OpenFile(dlg.GetPath(), mag.GetFileType(dlg.GetPath()));
		}
	}

	void OnSettings()
	{
		CSettings sd(this);
		sd.DoModal();
	}

	bool ShowHidden();

    void OnPaste()
	{
		std::vector<std::string> list;
		if (!GetSelection(list)) return;
		if (list.empty()) return;
		cxxx::cxFileManager->Copy(list, mCurrentDir);
		Reload();
	}

    void OnDelete();
    void OnCopy();

    void OnTrash()
	{
		cxxx::cxFileManager->Trash(GetListSelection());
		Reload();
	}

    void Configure();
    void OnCut();
	void UpdateStatusBar();
	void SetSortKey(unsigned key)
	{
		mSortFlags = key;
		mList.SetSortKey(key);
	
		mTree.SetSortKey(key);
	}

	void SetFlags(unsigned flags)
	{
		mFlags = flags;
		mList.SetFlags(flags & VIEWFLAGS);
	}

	void OnConfigureTerminal();

	void OpenFile(std::string path, unsigned type);
	const std::string& GetTerminalCommand();
	std::string GetFileAssoc(const std::string& name, unsigned type);

	// Selection processing
	virtual bool ConvertSelection(Atom sel, Atom typ, std::vector<unsigned char>& ret);
	bool GetSelection(std::vector<std::string>& ret);
	std::vector<std::string> GetListSelection();

public:
	static Atom TEXT_URI_LIST;
	static Atom DND_USERNAME;
	static Atom USERNAME;
	static Atom HOSTNAME;

protected:
	CXplorerTree mTree;
	CXplorerList mList;
	CXplorerMenu mMenu;
	CXplorerTool mTool;
	CHSplitterWindow mSplitter;
	CXplorerHistory mHistory;
	CStatusBar mStatusBar;
	unsigned mFlags;
	unsigned mSortFlags;
	unsigned mTreeWidth;
	CProfile* mMagicFile;
	std::string mCurrentDir;
	std::string mCaption;
	std::vector<std::string> mSelection;
	std::string mExecPath;
	std::string mTerminalCmd;
	std::string mTerminalArgs;
	CTapeArchive mArchive;
	bool mIsArchive;
};

#endif // CXplorerFrame_h
