/*----------------------------------------------------------------------------
  Project: Xplorer <http://cxplorer.sourceforge.net>
  File   : CXplorerHistory.h
  Info   : $Id: CXplorerHistory.h,v 1.8 2012/08/06 17:36:40 gkraeuter Exp $
----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
  Copyright © 2002 - 2012 Gerhard Kräuter <gkraeuter@users.sourceforge.net>

  This  program  is  free software: you can redistribute it and/or modify it
  under  the  terms  of  the  GNU General Public License as published by the
  Free Software Foundation,  either  version  3  of the License, or (at your
  option) any later version.

  This  program  is  distributed  in  the  hope  that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  or  FITNESS FOR A PARTICULAR PURPOSE.  See the  GNU General Public License
  for more details.

  You  should  have  received a copy of the GNU General Public License along
  with this program. If not, see  <http://www.gnu.org/licenses/>.
----------------------------------------------------------------------------*/

#ifndef CXplorerHistory_h
#define CXplorerHistory_h

// C++ headers
#include <string>

class CXplorerHistory
{
private:
	enum
	{
		// Settings
		HISTSIZE = 50
	};

public:
	// Constructor
	CXplorerHistory()
	: mCurrent(0),
	  mFirst(0),
	  mLast(0)
	{
		mArray[0] = "/";
	}

	// Sets the current directory
	void SetCurrent(const std::string& s)
	{
		if (s == GetCurrent()) return;
		if (++mCurrent == HISTSIZE) mCurrent = 0;
		mLast = mCurrent;
		if (mFirst == mLast) {
			if (++mFirst == HISTSIZE) mFirst = 0;
		}
		mArray[mCurrent] = s;
	}

	// Returns the previous directory
	const std::string& GetPrev()
	{
		if (CanBack()) {
			if (!mCurrent) mCurrent = HISTSIZE;
			--mCurrent;
		}
		return mArray[mCurrent];
	}

	// Returns the next directory
	const std::string& GetNext()
	{
		if (CanForward()) {
			if (++mCurrent == HISTSIZE) mCurrent = 0;
		}
		return mArray[mCurrent];
	}

	// Returns the current directory
	const std::string& GetCurrent() const
	{
		return mArray[mCurrent];
	}

	// Returns true if this is not the last directory
	bool CanForward() const
	{
		return (mCurrent != mLast);
	}

	// Returns true if this is not the first directory
	bool CanBack() const
	{
		return (mCurrent != mFirst);
	}
	
private:
	std::string mArray[HISTSIZE];
	unsigned mCurrent;
	unsigned mFirst;
	unsigned mLast;
};

#endif // CXplorerHistory_h
