/*----------------------------------------------------------------------------
  Project: Xplorer <http://cxplorer.sourceforge.net>
  File   : CXplorerList.h
  Info   : $Id: CXplorerList.h,v 1.23 2012/08/06 17:36:40 gkraeuter Exp $
----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
  Copyright © 2001 - 2012 Gerhard Kräuter <gkraeuter@users.sourceforge.net>

  This  program  is  free software: you can redistribute it and/or modify it
  under  the  terms  of  the  GNU General Public License as published by the
  Free Software Foundation,  either  version  3  of the License, or (at your
  option) any later version.

  This  program  is  distributed  in  the  hope  that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  or  FITNESS FOR A PARTICULAR PURPOSE.  See the  GNU General Public License
  for more details.

  You  should  have  received a copy of the GNU General Public License along
  with this program. If not, see  <http://www.gnu.org/licenses/>.
----------------------------------------------------------------------------*/

#ifndef CXplorerList_h
#define CXplorerList_h

// Application headers
#include "Xplorer.h"
#include "CTapeArchive.h"
#include "CPropertyDialog.h"

// C++X headers
#include <cxxx/CFileListControl.h>
#include <cxxx/CMenuItem.h>

class CXplorerList : public CFileListControl
{
private:
	enum
	{
		// Message IDs
		IDPOSTLOAD = xplorer::MSGID_CXplorerList,
		// Settings
		PREVIEWCHUNK = 20000
	};

public:
	// Constructor
	CXplorerList(CWindow* parent, unsigned msgid);

	// Opens the Context menu
	virtual void OnContext()
	{
		Popup(mContext);
	}

	// C++X message processing
	virtual void OnMessage(unsigned msgid, TMessage type, unsigned value);

	// Appends an item to the list
	virtual void Append(const std::string& path, const std::string& name);
    
	// Converts the selection
	virtual bool ConvertSelection(Atom sel, Atom typ, std::vector<unsigned char>& ret);

	// Displays the properties dialog
	void OnProperties()
	{
		if (GetSelCount() > 1) return;
		int x(20);
		int y(20);
		unsigned sel(-1u);
		GetParent()->TranslateCoordinates(x, y, NULL);
		if (GetSelCount() > 0) sel = GetSelection().front();
		CPropertyDialog dlg(this, mArchive, GetPath(sel));
		dlg.Move(x, y);
		dlg.Show();
		dlg.DoModal();
	}
	
	// Drag 'n' drop related stuff
	virtual bool OnDragInit(unsigned button, Atom& action);
	virtual Atom GetDnDAction(int x, int y, Atom action);
	virtual bool OnDrop(Atom action);
	virtual bool OnDropInit(std::vector<Atom> mime);
	virtual bool CanAcceptAction(Atom action);
	virtual void OnDropFinished();
	bool GetActionList(std::vector<Atom>& actions, std::vector<std::string>& desc);

	bool CanPerformAction(Atom action)
	{
		return (action == DND_MOVE || action == DND_COPY || action == DND_LINK);
	}
	
	// Returns true if pasting into the current directory is allowed
	bool CanPaste() const
	{
		return mWriteAccess;
	}

	// Settings
	void SetFlags(unsigned flags);
	virtual void SetStyle(unsigned style, unsigned wico, unsigned hico);

	// Makes dir the current directory
	void SetDirectory(const std::string& dir, CTapeArchive* arch)
	{
		mArchive = arch;
		CFileListControl::SetDirectory(dir);
	}

protected:
	// Reads the directory
	virtual void ReadDirectory();

	// Reads an archive
	void ReadArchive();

private:
	// Returns the user
	std::string GetDnDUser();

	// Displays thumbnails
	void OnPostLoad();

	// Retrieves the drag 'n' drop files
	bool GetDnDFiles(std::vector<std::string>& ret);

	// Creates the context menu
	void CreateContextMenu();

private:
	CMenu mContext;
	CMenu mContextCreate;
	std::vector<std::string> mDnDFiles;
	unsigned mDnDSelection;
	CIcon mOpenDir;
	CIcon mActiveExe;
	CIcon mDnDIcon;
	CTimer mPostLoadTimer;
	std::vector<CFileListItem> mPostLoadList;
	unsigned mXplorerFlags;
	unsigned mMaxPreviewSize;
	CTapeArchive* mArchive;
};

#endif // CXplorerList_h
