/*----------------------------------------------------------------------------
  Project: Xplorer <http://cxplorer.sourceforge.net>
  File   : CXplorerMenu.cpp
  Info   : $Id: CXplorerMenu.cpp,v 1.36 2012/06/08 13:03:40 gkraeuter Exp $
----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
  Copyright © 2002 - 2012 Gerhard Kräuter <gkraeuter@users.sourceforge.net>

  This  program  is  free software: you can redistribute it and/or modify it
  under  the  terms  of  the  GNU General Public License as published by the
  Free Software Foundation,  either  version  3  of the License, or (at your
  option) any later version.

  This  program  is  distributed  in  the  hope  that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  or  FITNESS FOR A PARTICULAR PURPOSE.  See the  GNU General Public License
  for more details.

  You  should  have  received a copy of the GNU General Public License along
  with this program. If not, see  <http://www.gnu.org/licenses/>.
----------------------------------------------------------------------------*/

// application headers
#include "Xplorer.h"
#include "CXplorerFrame.h"
#include "CXplorerMenu.h"
#include "CSortDialog.h"

// cxxx headers
#include <cxxx/CCxxxAbout.h>

// namespace declarations
using cxxx::cxgettext;
using std::string;
using cxxx::cxNarrow;

CXplorerMenu::CXplorerMenu(CWindow* parent)
: CMenuBar(parent),
  mMnuFile(GetTopLevelWindow(), _("&File", 82)),
  mMnuFileNew(GetTopLevelWindow(), ""),
  mMnuFileRecent(GetTopLevelWindow(), "", CXplorerFrame::IDNAV_RECENT),
  mMnuEdit(GetTopLevelWindow(), _("&Edit", 83)),
  mMnuView(GetTopLevelWindow(), _("&View", 84)),
  mMnuTools(GetTopLevelWindow(), _("&Tools", 85)),
  mMnuHelp(GetTopLevelWindow(), _("&Help", 86))
{
	// --- FILE ---
	mMnuFile.Entry(CXplorerFrame::IDFILE_OPEN, _("&Open...", 87), CIcon::FILEOPEN);
	mMnuFile.Seperator();
	mMnuFile.Entry(mMnuFileNew, _("&New", 88), CIcon::FILENEW);
	mMnuFile.Entry(CXplorerFrame::IDFILE_EXIT, _("E&xit\tCtrl+Q", 89), CIcon::EXIT);
	mMnuFile.Seperator();
	mMnuFile.Entry(CXplorerFrame::IDNAV_UP, _("Up\tAlt+Up", 161), CIcon::UP);
	mMnuFile.Entry(CXplorerFrame::IDNAV_BACK, _("Back\tAlt+Left", 162), CIcon::BACK);
	mMnuFile.Entry(CXplorerFrame::IDNAV_FORWARD, _("Forward\tAlt+Right", 159), CIcon::FORWARD);
	mMnuFile.Seperator();
	mMnuFile.Entry(mMnuFileRecent, _("&Recently visited", 160));

	// --- FILE --- NEW ---
	mMnuFileNew.Entry(CXplorerFrame::IDNEW_DIR, _("&Directory", 90), CIcon::FOLDERNEW);
	mMnuFileNew.Entry(CXplorerFrame::IDNEW_TEXT, _("Text F&ile", 91), CIcon::FILENEW);
	
	// --- FILE --- RECENT
	// This menu is changed by CXplorerFrame
	
	// --- EDIT ---
	mMnuEdit.Entry(CXplorerFrame::IDEDIT_CUT, _("C&ut\tCtrl+X", 95), CIcon::EDITCUT);
	mMnuEdit.Entry(CXplorerFrame::IDEDIT_COPY, _("&Copy\tCtrl+C", 96), CIcon::EDITCOPY);
	mMnuEdit.Entry(CXplorerFrame::IDEDIT_PASTE, _("&Paste\tCtrl+V", 97), CIcon::EDITPASTE);
	mMnuEdit.Seperator();
	mMnuEdit.Entry(CXplorerFrame::IDEDIT_DEL, _("&Delete\tShift+DEL", 98), CIcon::EDITDELETE);
	mMnuEdit.Entry(CXplorerFrame::IDEDIT_TRASH, _("&Move to trash\tDEL", 99), CIcon::TRASHCAN_EMPTY);
	mMnuEdit.Seperator();
	mMnuEdit.Entry(CXplorerFrame::IDEDIT_SELA, _("Select &all\tCtrl+A", 100));
	mMnuEdit.Entry(CXplorerFrame::IDEDIT_SELR, _("&Invert Selection", 101));
	mMnuEdit.Seperator();
	mMnuEdit.Entry(CXplorerFrame::IDSETTINGS, _("&Settings...", 104), CIcon::CONFIGURE),

	// --- VIEW ---
	mMnuView.Entry(CXplorerFrame::IDTOOL, _("T&oolbar", 102), CIcon::MENU_CHECK);
	mMnuView.Entry(CXplorerFrame::IDSTATUS, _("Status&bar", 103), CIcon::MENU_CHECK);
	mMnuView.Seperator();
	mMnuView.Entry(CXplorerFrame::IDVIEW_MULI_COLUMN, _("&Multi column view", 149), CIcon::MENU_BULLET);
	mMnuView.Entry(CXplorerFrame::IDVIEW_ICON, _("I&con view", 150), CIcon::MENU_BULLET);
	mMnuView.Seperator();
	mMnuView.Entry(CXplorerFrame::IDSHOW_HIDDEN, _("Sho&w hidden files\tCtrl+H", 108), CIcon::MENU_CHECK);
	mMnuView.Seperator();
	mMnuView.Entry(CXplorerFrame::IDVIEW_SORT, _("Sort f&iles...", 109));
	mMnuView.Seperator();
	mMnuView.Entry(CXplorerFrame::IDVIEW_PREVIEW, _("File &preview...", 134));

	// --- TOOLS ---
	mMnuTools.Entry(CXplorerFrame::IDOPEN_TERM, _("&Open Terminal...\tStrg+T", 110), CIcon::OPENTERM);
	mMnuTools.Entry(CXplorerFrame::IDCONFIGURE_TERM, _("Configure Ter&minal...", 175), CIcon::TERMCONFIG);
	mMnuTools.Seperator();
	mMnuTools.Entry(CXplorerFrame::IDTRASH, _("Trash&can...", 152), CIcon::TRASHCAN_EMPTY);
	
	// --- HELP ---
	mMnuHelp.Entry(CXplorerFrame::IDABOUT_XPLORER, _("About &Xplorer", 112), CIcon::APP);
	mMnuHelp.Entry(CXplorerFrame::IDABOUT_CXXX, _("About &C++X", 113), CIcon::CXXX);

	AddMenu(mMnuFile);
	AddMenu(mMnuEdit);
	AddMenu(mMnuView);
	AddMenu(mMnuTools);
	AddMenu(mMnuHelp);
}

void CXplorerMenu::SetDirList(const string& list)
{
	if (list.empty()) return;
	string::size_type beg = 0;
	string::size_type end = 0;
	unsigned id = CXplorerFrame::IDNAV_RECENT;
	while (end != string::npos) {
		end = list.find(':', beg);
		mMnuFileRecent.Entry(id++, list.substr(beg, end - beg));
		beg = end + 1;
	}
}
