/*----------------------------------------------------------------------------
  Project: Xplorer <http://cxplorer.sourceforge.net>
  File   : CXplorerTree.h
  Info   : $Id: CXplorerTree.h,v 1.21 2012/08/06 17:36:40 gkraeuter Exp $
----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
  Copyright © 2002 - 2012 Gerhard Kräuter <gkraeuter@users.sourceforge.net>

  This  program  is  free software: you can redistribute it and/or modify it
  under  the  terms  of  the  GNU General Public License as published by the
  Free Software Foundation,  either  version  3  of the License, or (at your
  option) any later version.

  This  program  is  distributed  in  the  hope  that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  or  FITNESS FOR A PARTICULAR PURPOSE.  See the  GNU General Public License
  for more details.

  You  should  have  received a copy of the GNU General Public License along
  with this program. If not, see  <http://www.gnu.org/licenses/>.
----------------------------------------------------------------------------*/

#ifndef CXplorerTree_h
#define CXplorerTree_h

// Application headers
#include "CPropertyDialog.h"

// C++X headers
#include <cxxx/CFileTree.h>
#include <cxxx/CMenuItem.h>

// Forward declarations
class CTapeArchive;

class CXplorerTree : public CFileTree
{
public:
	// Constructor
	CXplorerTree(CWindow* parent, unsigned msgid)
	: CFileTree(parent, msgid),
	  mContextMenu(GetTopLevelWindow(), ""),
	  mDnDSelection(-1u),
	  mArchive(NULL)
	{
		mOpenDir.Create(CIcon::FOLDER_OPEN, mIconSize);
		CreateContextMenu();
		SetupDnD();
	}

	// Opens a context menu
	virtual void OnContext();

	// Drag 'n' drop related stuff
	virtual bool OnDragInit(unsigned button, Atom& action);
	virtual bool OnDrop(Atom action);
	virtual bool OnDropInit(std::vector<Atom> mime);
	virtual bool CanAcceptAction(Atom action);
	virtual bool CanPerformAction(Atom action);
	virtual Atom GetDnDAction(int x, int y, Atom Action);
	virtual void OnDropFinished();
	virtual bool GetActionList(std::vector<Atom>& actions, std::vector<std::string>& desc);
	virtual bool ConvertSelection(Atom sel, Atom type, std::vector<unsigned char>& ret);

	// Displays the properties dialog
	void OnProperties()
	{
		if (GetSelCount() > 1) return;
		int x = 20;
		int y = 20;
		GetParent()->TranslateCoordinates(x, y, NULL);
		unsigned sel(-1u);
		if (GetSelCount() > 0) sel = GetSelection().front();
		CPropertyDialog dlg(this, mArchive, GetPath(sel));
		dlg.Move(x, y);
		dlg.Show();
		dlg.DoModal();
	}	
	
	// Sets the current directory
	void SetDirectory(const std::string& dir, CTapeArchive* arch)
	{
		mArchive = arch;
		CFileTree::SetDirectory(dir);
	}
	
private:
	bool GetDnDFiles(std::vector<std::string>& ret);
	void CreateContextMenu();
	std::string GetDnDUser();
private:
	CMenu mContextMenu;
	CIcon mDnDIcon;
	CIcon mOpenDir;
	unsigned mDnDSelection;
	std::vector<std::string> mDnDFiles;
	CTapeArchive* mArchive;
};

#endif // CXplorerTree_h
